/*
 * ***** BEGIN LICENSE BLOCK *****
 * Copyright (C) 2011-2016 ZeXtras
 *
 * The contents of this file are subject to the ZeXtras EULA;
 * you may not use this file except in compliance with the EULA.
 * You may obtain a copy of the EULA at
 * http://www.zextras.com/zextras-eula.html
 * ***** END LICENSE BLOCK *****
 */

/**
 * ZeXtras main view ID for the list
 * @type {number}
 */
ZaZimbraAdmin._ZEXTRAS_GROUP_LIST = ZaZimbraAdmin.VIEW_INDEX++;
/**
 * ZeXtras main view ID for the group
 * @type {number}
 */
ZaZimbraAdmin._ZEXTRAS_GROUP_VIEW = ZaZimbraAdmin.VIEW_INDEX++;

if (ZaMsg) {
  /**
   * Title for ZeXtras main view.
   * @type {string}
   */
  ZaMsg.ZXGroups_view_title = 'ZeXtras';
}

if (ZaItem) {
  /**
   * Tag name forZeXtras
   * @type {string}
   */
  ZaItem.ZEXTRAS = 'ZeXtras';
}

/**
 * ZeXtras group list view name
 * @type {string}
 */
ZaSettings.ZEXTRAS_GROUP_LIST_VIEW = 'zextrasGroupsListView';
ZaSettings.ALL_UI_COMPONENTS.push(
  { value: ZaSettings.ZEXTRAS_GROUP_LIST_VIEW, label: 'ZeXtras' }
);

/**
 * Constructor for ZeXtras Zimlet item.
 * All methods contained in this class can be called from the 'ZeXtras' global object.
 * @constructor
 */
function ZeXtrasItem(legacyConnector, itemList) {
  ZaItem.call(this, 'ZeXtras');
  this.type = ZaItem.ZEXTRAS;
  this._created_menu = false;
  this._versionInfo = {};
  this._serverPoolIsOnline = {};
  this._mainTree = void 0;
  this.mLegacyConnector = legacyConnector;

  var serverData = {},
    singleData = {},
    serverList = new ZaItemList(ZaServer),
    dummyList = new ZaItemList(ZaServer),
    server = new ZaServer(),
    i,
    resp,
    serverName,
    vendor,
    sysType;

  try {
    this._versionInfo = this.checkVersion();
  } catch (err) {
    if (err.code && err.code === 'MISC_1') {
      var details = err.details[ZxAdminRequest.TARGET_LOCAL];

      ZeXtrasItem.showMessageDialog(
        zxMsg('MSG_Mig_and_Core_installed_please_remove_once',
          [(details.systemType === 'ZeXtras') ? 'ZxMig' : 'ZeXtras']),
        '',
        DwtMessageDialog.WARNING_STYLE
      );

      this._versionInfo = err.details;

    } else {
      throw err;
    }
  }

  for (serverName in this._versionInfo) {
    if (!this._versionInfo.hasOwnProperty(serverName)) continue;
    vendor = this._versionInfo[serverName].vendor;
    sysType = this._versionInfo[serverName].systemType;
  }

  /*
   * On the first thing, get the actions white list from the server.
   */

  resp = ZxAdminRequest.sendSyncRequest(ZxAdminRequest.TARGET_LOCAL, 'ZxCore', 'getUnlicensedCommands');
  ZxUtils.log(resp, 'Retrieved actions white list');
  ZxAdminRequest._whiteList = resp[ZxAdminRequest.TARGET_LOCAL].response;
  ZxAdminRequest._receivedWL = true;

  this._license = new ZxLicense(new AjxCallback(this, this._onLicenseInfo));
  this._status = {};
  this.attrs = {};
  this._itemLists = itemList;

  this.__label = zxMsg('label');

  this._operationsManager = new ZxOperationsManager();

  ZxUtils.log((ZeXtrasItem.isUserAdmin() ? 'Yep' : 'Nope'), 'Are you an Administrator?');

  this._accountModifier = new ZxAccountFormModifier();
  this._cosModifier = new ZxCOSFormModifier();
  this._domainModifier = new ZxDomainFormModifier();

  server.id = '00000000-0000-0000-0000-000000000000';
  server.name = 'localhost';
  server.show_volumes = false;
  server.type = 'server';
  server.zimbraServiceHostname = 'delegated.admin.foo';
  dummyList.add(server, 0);

  if (ZeXtrasItem.isUserAdmin()) {
    serverList = ZaServer.getAllMBSs();
  } else {
    serverList = dummyList;
  }
  serverData.server = serverList.getArray();
  singleData.server = [serverList.getArray()[0]];

  // Do not load modules not designed for delegated admins.
  if (!ZeXtrasItem.isUserAdmin()) {
    for (i in this._itemLists) {
      if (!this._itemLists.hasOwnProperty(i)) { continue; }

      if (!window[i + ZxModuleViewConstructor._ITEM_DELEGATED]) {
        delete this._itemLists[i];
      }
    }
  }

  // Initialize all the items
  for (i in this._itemLists) {
    if (!this._itemLists.hasOwnProperty(i)) { continue; }
    if (ZxUtils.indexOf(ZxModule.SINGLES, i) > -1) {
      this._itemLists[i].loadFromJS(singleData);
    } else {
      if (i === 'ZxNotifications') {
        this._notificationsDialog = new ZxNotificationsDialog(serverData.server.length);
      }
      this._itemLists[i].loadFromJS(serverData);
    }
  }
  for (i in this._itemLists) {
    if (!this._itemLists.hasOwnProperty(i)) { continue; }
    var tmpArr = this._itemLists[i].getArray(),
      j;
    for (j = 0; j < tmpArr.length; j++) {
      tmpArr[j].injectConstructors(this.mLegacyConnector);
    }
  }
}

ZeXtrasItem.prototype = new ZaItem();

/**
 * Constructor redefined to use ours.
 * @type {function}
 */
ZeXtrasItem.prototype.constructor = ZeXtrasItem;
ZeXtrasItem.PROD_ZEXTRAS = 'zextras';
ZeXtrasItem.PROD_MIG = 'zxmig';
ZeXtrasItem.ENABLED_VALUE = "Enabled";

ZeXtrasItem.COMMIT = 'f05460b2622f19a08c3c5aef60d87e3fd37bff13';

ZeXtrasItem.zimbra_version_regexp = /(\d+)\.(\d+)\.(\d+)/;

ZeXtrasItem._MAIN_VIEW_NAME = 'ZeXtras_Main';
ZeXtrasItem._SERVER_LIST_NAME = 'ZeXtras_Server_List';

/**
 * Return if the user is an administrator.
 * @return {boolean}
 */
ZeXtrasItem.isUserAdmin = function() {
  return ZaZimbraAdmin.currentAdminAccount.attrs.zimbraIsAdminAccount ?
      ZaZimbraAdmin.currentAdminAccount.attrs.zimbraIsAdminAccount === 'TRUE' : false;
};

ZeXtrasItem.prototype.isUserAdmin = function() {
  return ZeXtrasItem.isUserAdmin();
};

/**
 * Get if the current server is ZxMig
 * @return {boolean}
 */
ZeXtrasItem.prototype.isMigZimlet = function() {
  return this._versionInfo[ZxAdminRequest.TARGET_LOCAL].systemType === 'ZxMig';
};

/**
 * Check if running ZeXtras Suite
 * @return {boolean}
 */
ZeXtrasItem.prototype.isZeXtrasSuite = function() {
  return this._versionInfo[ZxAdminRequest.TARGET_LOCAL].vendor === 'zextras';
};

/**
 * Return if the ZeXtras product is ZxMig (Migration tool).
 * @param {string} server
 * @return {boolean}
 */
ZeXtrasItem.prototype.isZxMig = function(server) {
  return ZxLicense.LICENSE_MIG === this._license.getType(server);
};

/**
 * Return the Module item associated to a server.
 * If server parameter is undefined, will be return the first Item found for the specified module, useful for a
 *   single/server setup.
 * @param {ZxAdmin|ZxBackup|ZxChat|ZxCore|ZxMig|ZxNotifications|ZxMobile|ZxPowerstore|string} moduleClass
 *         The class of needed module.
 * @param {String=} server name or server id.
 * @return {ZxModule|undefined}
 */
ZeXtrasItem.prototype.getModuleItem = function(moduleClass, server) {
  var list,
    i,
    array,
    modName;

  try {
    modName = ZxUtils.getFunctionName(moduleClass);
    list = this.getModuleItemList(modName);
  } catch (err) {
    throw new Error('Unable to get module name');
  }

  if (!list) {
    return;
  }
  array = list.getArray();
  for (i = 0; i < array.length; i = i + 1) {
    if (!server) {
      return array[i];
    }
    if (array[i].name === server || array[i].id === server) {
      return array[i];
    }
  }
};

/**
 * @param {string} moduleClass
 *         The class of needed module.
 * @return {ZaItemList|undefined}
 */
ZeXtrasItem.prototype.getModuleItemList = function(moduleClass) {
  if (typeof moduleClass !== 'string') { throw new Error('[moduleClass] must be a string.'); }

  try {
    if (typeof moduleClass === 'string') {
      return this._itemLists[moduleClass];
    }
  } catch (err) {
    var modName;
    if (typeof moduleClass === 'string') {
      modName = moduleClass;
    }
    ZxUtils.log(modName, 'Module Does Not Exist', ZxUtils.LOG_WARNING);
  }
};

/**
 * Get the lists of the Items of the modules.
 * @return {{}}
 */
ZeXtrasItem.prototype.getModuleListsItems = function() {
  return this._itemLists;
};

/**
 * Get the account tab modifier.
 * @return {ZxAccountFormModifier}
 */
ZeXtrasItem.prototype.getAccountModifier = function() {
  return this._accountModifier;
};

/**
 * Get the account tab modifier.
 * @return {ZxCOSFormModifier}
 */
ZeXtrasItem.prototype.getCOSModifier = function() {
  return this._cosModifier;
};

/**
 * Get the account tab modifier.
 * @return {ZxDomainFormModifier}
 */
ZeXtrasItem.prototype.getDomainModifier = function() {
  return this._domainModifier;
};

/**
 * Check if the version of the zimlet match with the core version.
 * Warning, this method use Synchronous SOAP Call.
 * @return {{}}.
 */
ZeXtrasItem.prototype.checkVersion = function() {
  var zimletCoreVersion = this.getZimletVersion('com_zextras_zextras'),
    zimletMigVersion = this.getZimletVersion('com_zextras_zxmig'),
    installedCore = false,
    installedMig = false,
    versionInfo = {},
    server,
    resp;

  try {
    resp = ZxAdminRequest.sendSyncRequest(ZxAdminRequest.TARGET_LOCAL, 'ZxCore', 'getVersion');
  } catch (err) {
    // Maybe the core is missing or does not respond.
    ZxUtils.log(err, ZxUtils.ERROR_CODES['CONN_1']);
    var error = new Error(ZxUtils.ERROR_CODES['CONN_1']);
    error.code = 'CONN_1';
    error.details = err;
    throw error;
  }

  for (server in resp) {

    if (!resp.hasOwnProperty(server)) { continue; }
    if (!resp[server].ok) {
      versionInfo[server] = {
        updateStatus: 0,
        commit: 'na',
        libCommit: 'na',
        version: 'na',
        libVersion: 'na',
        systemType: 'na',
        vendor: 'na',
        zimletVersion: 'na'
      };
      continue;
    }

    if (!ZeXtrasItem.isUserAdmin()) {
      var adminCheckResponse = ZxAdminRequest.sendSyncRequest(ZxAdminRequest.TARGET_LOCAL, 'ZxAdmin', 'isZxDelegatedAdmin');
      if (!adminCheckResponse[ZxAdminRequest.TARGET_LOCAL].ok) {
        throw new Error('You are not a valid Delegated Admin');
      }
    }

    var updateStatus = 0,
      coreVersion = resp[server].response.version,
      zimletVersionMigTrimmed,
      zimletVersionCoreTrimmed,
      coreVersionTrimmed;

    if (!installedCore && resp[server].response.systemType === 'ZeXtras') {
      installedCore = true;
    }
    if (!installedMig && resp[server].response.systemType === 'ZxMig') {
      installedMig = true;
    }

    // Zimlet MUST ignore the micro version, the minor version has compatible APIs
    coreVersionTrimmed = coreVersion.substring(0, coreVersion.lastIndexOf('.'));
    zimletVersionCoreTrimmed = zimletCoreVersion.substring(0, zimletCoreVersion.lastIndexOf('.'));
    zimletVersionMigTrimmed = zimletMigVersion.substring(0, zimletMigVersion.lastIndexOf('.'));

    if (resp[server].response.systemType === 'ZxMig') {
      updateStatus = ZxUtils.compareVersions(zimletVersionMigTrimmed, coreVersionTrimmed);
    } else {
      updateStatus = ZxUtils.compareVersions(zimletVersionCoreTrimmed, coreVersionTrimmed);
    }

    versionInfo[server] = {
      updateStatus: updateStatus,
      commit: resp[server].response.commit,
      vendor: resp[server].response.vendor,
      libCommit: resp[server].response.libCommit,
      version: coreVersion,
      libVersion: resp[server].response.libVersion,
      systemType: resp[server].response.systemType,
      zimletVersion: (resp[server].response.systemType === 'ZxMig') ? zimletMigVersion : zimletCoreVersion
    };

  }

  if (zimletMigVersion === '0.0.0' && zimletCoreVersion === '0.0.0') {
    var err = new Error(ZxUtils.ERROR_CODES.MISC_4);
    err.code = 'MISC_4';
    err.details = versionInfo;
    err.zimletVersion = zimletCoreVersion;
    err.zimletMigVersion = zimletMigVersion;
    throw err;
  }

  if (zimletMigVersion !== '0.0.0' && zimletCoreVersion !== '0.0.0') {
    var err = new Error(ZxUtils.ERROR_CODES.MISC_1);
    err.code = 'MISC_1';
    err.details = versionInfo;
    throw err;
  }

  if ((installedCore && zimletMigVersion !== '0.0.0') || (installedMig && zimletCoreVersion !== '0.0.0')) {
    var err = new Error(ZxUtils.ERROR_CODES.MISC_3);
    err.code = 'MISC_3';
    err.details = versionInfo;
    throw err;
  }

  if (updateStatus !== 0) {
    var err = new ZxError(ZxErrorCode.ZIMLET_CORE_VERISON_MISMATCH);
    err.details = versionInfo;
    throw err;
  }

  return versionInfo;
};

/**
 * Get the ZeXtras License.
 * @return {ZxLicense}
 */
ZeXtrasItem.prototype.getLicense = function() {
  return this._license;
};

/**
 * Return if the current server is part of a multiserver
 * @return {boolean}
 */
ZeXtrasItem.prototype.isMultiServer = function() {
  return ZaApp.getInstance().getMbsList().size() > 1;
};

/**
 * Callback invoked when the license informations change.
 * @param {ZxLicense} license
 */
ZeXtrasItem.prototype._onLicenseInfo = function(license) {
  ZxUtils.log(license, 'New License info');

  var modName, mod, itemArray, i;
  for (modName in this._itemLists) {
    if (!this._itemLists.hasOwnProperty(modName)) { continue; }
    mod = this._itemLists[modName];
    itemArray = mod.getArray();
    for (i = 0; i < itemArray.length; i += 1) {
      itemArray[i]._notifyLicenseChanges(license);
    }
  }
};

/**
 * Process the version informations.
 * @param {{}} versionInfo
 */
ZeXtrasItem.prototype._onVersionInfo = function(versionInfo) {
  this._versionInfo = versionInfo;
  ZxUtils.log(this._versionInfo, 'Version informations');
};

/**
 * Get the instance operations manager.
 * @return {ZxOperationsManager}
 */
ZeXtrasItem.prototype.getOperationsManager = function() {
  return this._operationsManager;
};

/**
 * Declare to the system that the server is online.
 * @param {string} server
 */
ZeXtrasItem.prototype.declareServerOnline = function(server) {
  if (this._serverPoolIsOnline[server] === false) {
    if (ZaActionStatusView) {
      ZaZimbraAdmin.getInstance().setActionStatusMsg({
        msg: zxMsg('MSG_TST_Server_online', [server]),
        level: 1 // = ZaActionStatusView.LEVEL_INFO
      });
    }
    ZxUtils.log(server, 'Server ONLINE');
  }
  this._serverPoolIsOnline[server] = true;
};

/**
 * Declare to the system that the server is offline.
 * @param {string} server
 */
ZeXtrasItem.prototype.declareServerOffline = function(server) {
  if (!!this._serverPoolIsOnline[server]) {
    if (ZaActionStatusView) {
      ZaZimbraAdmin.getInstance().setActionStatusMsg({
        msg: zxMsg('MSG_TST_Server_offline', [server]),
        level: 3 // = ZaActionStatusView.LEVEL_CRITICAL
      });
    }
    ZxUtils.log(server, 'Server OFFLINE');
  }
  this._serverPoolIsOnline[server] = false;
};

 /**
  * Return server is online.
  * If we don't know if the server is online or not, ssume is online to display the default values in the module view.
  * @param {string} server
  * @return {boolean}
  */
ZeXtrasItem.prototype.isServerOnline = function(server) {
  if (!this._serverPoolIsOnline.hasOwnProperty(server)) {
    // Fix for ZX-2804, we don't know if the server is online or not
    // Assume is online to display the default values in the module view.
    return true;
  }
  return this._serverPoolIsOnline[server] === true;
};

/**
 * Return server is offline.
 * @param {string} server
 * @return {boolean}
 */
ZeXtrasItem.prototype.isServerOffline = function(server) {
  return !this._serverPoolIsOnline.hasOwnProperty(server) || this._serverPoolIsOnline[server] === false;
};

/**
 * Get the main notification dialog, used to show notifications during ZeXtras startup.
 * @return {ZxNotificationsDialog|undefined}
 */
ZeXtrasItem.prototype.getNotificationsDialog = function() {
  return this._notificationsDialog;
};

/**
 * Get the version of a zimet, by name.
 * @param {string} zimletName
 * @returns {string} version or '0.0.0' if not found
 */
ZeXtrasItem.prototype.getZimletVersion = function(zimletName) {
  var indexOf = [].indexOf || function(item) { for (var i = 0, l = this.length; i < l; i++) { if (i in this && this[i] === item) return i; } return -1;},
    ref, i, j, len, len1, zimletNames = [], zimlet, zimlet1;

  ref = ZaZimlet.getAll().getArray();
  for (i = 0, len = ref.length; i < len; i++) {
    zimlet = ref[i];
    zimletNames.push(zimlet[ZaZimlet.A_name]);
  }
  if (indexOf.call(zimletNames, zimletName) >= 0) {
    for (j = 0, len1 = ref.length; j < len1; j++) {
      zimlet = ref[j];
      if (zimletName === zimlet[ZaZimlet.A_name]) {
        zimlet1 = zimlet;
      }
    }
    return zimlet1.attrs[ZaZimlet.A_zimbraZimletVersion];
  } else {
    return '0.0.0';
  }
};

/**
 * Show a warning dialog, if the context controller is not available,
 * wait 500ms and retry to display the warning dialog.
 * @param {string} message
 * @param {string=} title
 * @param {string=} style must be one of (@see <code>DwtMessageDialog.*_STYLE</code> constants)
 */
ZeXtrasItem.showMessageDialog = function(message, title, style) {
  var actxt = ZaApp.getInstance().getAppCtxt();

  title = title || ZxUtils.getTitleWithIcon('ZeXtras', zxMsg('LBL_ZeXtras_Message'));
  style = style || DwtMessageDialog.INFO_STYLE;

  if (!actxt) {
    Q.delay(500).then(
      AjxCallback.simpleClosure(
        ZeXtrasItem.showMessageDialog,
        actxt,
        message,
        title,
        style
      )
    );
  } else {
    var msgDialog = new DwtMessageDialog({
        buttons: [DwtDialog.DISMISS_BUTTON],
        title: '',
        parent: actxt.getShell()
      });

    msgDialog.setMessage(message, style, title);
    msgDialog.registerCallback(
      DwtDialog.DISMISS_BUTTON,
      new AjxCallback(
        msgDialog,
        msgDialog.popdown
      )
    );

    msgDialog.popup();
  }
};

ZeXtrasItem.prototype.setMainTree = function(tree) {
  this._mainTree = tree;
};

ZeXtrasItem.prototype.getMainMenuItemDataById = function(id /* string */) {
  if (typeof this._mainTree !== 'undefined' && typeof this._mainTree._rootData !== 'undefined') {
    return this._recGetTreeItemDataById(this._mainTree._rootData.childrenData.getArray(), id);
  }
};

ZeXtrasItem.prototype._recGetTreeItemDataById = function(array /* ZaTreeItemData[] */, id /* string */) {
  var i = 0,
    child = void 0;
  for (i = 0; i < array.length; i++) {
    if (array[i].id === id) { return array[i]; }
    child = this._recGetTreeItemDataById(array[i].childrenData.getArray(), id);
    if (typeof child !== 'undefined') {
      return child;
    }
  }
};

ZeXtrasItem.prototype.getMainMenuItemById = function(id /* string */, startFrom) {
  if (typeof startFrom !== 'undefined') {
    return this._recGetTreeItemById(startFrom.getChildren(), id);
  }
};

ZeXtrasItem.prototype._recGetTreeItemById = function(array /* ZaTreeItem[] */, id /* string */) {
  var i = 0,
    child = void 0,
    labelFallback = '',
    tmpId;
  for (i = 0; i < array.length; i++) {
    if (typeof array[i].getData === 'undefined') continue;
    if (typeof array[i].getData('dataItem') !== 'undefined') {
      tmpId = array[i].getData('dataItem').id;
      if (id === tmpId) { return array[i]; }
    } else {
      tmpId = array[i].getText();
      labelFallback = this._getMainMenuItemLabelById(id);
      if (tmpId === labelFallback) { return array[i]; }
    }
    child = this._recGetTreeItemById(array[i].getChildren(), id);
    if (typeof child !== 'undefined') {
      return child;
    }
  }
};

ZeXtrasItem.prototype.followMenuItems = function(menuIds /* string[] */, startFrom) {
  if (menuIds.length < 1) return;
  var menuItem = this.jumpToView(menuIds.shift(), startFrom || this._mainTree);
  setTimeout(
    (function(_this, _menuIds, _menuItem) {
      return function() {
        var startFrom = (ZxUtils.isZ8Up()) ? _this._mainTree : _menuItem;
        _this.followMenuItems(_menuIds, startFrom);
      }
    })(this, menuIds, menuItem),
    500
  );
};

ZeXtrasItem.prototype.jumpToView = function(menuId /* string */, startFrom) {
  var menuItemData = this.getMainMenuItemDataById(menuId),
    item = this.getMainMenuItemById(menuId, startFrom);
  if (typeof item === 'undefined' && typeof menuItemData !== 'undefined') {
    item = this._mainTree._buildNodeItem(menuItemData);
  }
  if (typeof item !== 'undefined') {
    this._mainTree._itemClicked(item, new DwtMouseEvent());
    return item;
  }
};

ZeXtrasItem.prototype._getMainMenuItemLabelById = function(id) {
  if (id === ZxMenuIds.CONFIGURE_GLOBAL) {
    return ZaMsg.OVP_global;
  } else if (id === ZxMenuIds.CONFIGURE_GLOBAL_ZXCHAT) {
    return zxMsg('LBL_ZxChat');
  } else if (id === ZxMenuIds.ZEXTRAS) {
    return zxMsg('label');
  } else if (id === ZxMenuIds.ZEXTRAS_NOTIFICATION) {
    return zxMsg('LBL_ZxNotifications');
  }
};

ZeXtrasItem.isZeXtrasSuiteFcn = function() {
  return ZeXtras.isZeXtrasSuite();
};

function ZxMenuIds() {}
//ZxMenuIds.MONITOR = 'zti__AppAdmin__Home__monHV';
//ZxMenuIds.MONITOR_OV_STATUS = 'zti__AppAdmin__monHV__overviewStatusHV';
//ZxMenuIds.MONITOR_ADV_STATS = 'zti__AppAdmin__Home__monHV__1';
//ZxMenuIds.MONITOR_MSG_COUNT = 'zti__AppAdmin__Home__monHV__2';
//ZxMenuIds.MONITOR_MSG_VOLUME = 'zti__AppAdmin__Home__monHV__3';
//ZxMenuIds.MONITOR_SPAM_VIRUS = 'zti__AppAdmin__Home__monHV__4';
//ZxMenuIds.MONITOR_SERVER_STATS = 'zti__AppAdmin__monHV__serverListForStatisticsHV';
//ZxMenuIds.MONITOR_MSG_QUEUE = 'zti__AppAdmin__monHV__postQueueHV';
//ZxMenuIds.ACCOUNTS = 'zti__AppAdmin__Home__manActHV';
//ZxMenuIds.ACCOUNTS_ACC = 'zti__AppAdmin__Home__actLstHV';
//ZxMenuIds.ACCOUNTS_ALIAS = 'zti__AppAdmin__Home__aliaLstHV';
//ZxMenuIds.ACCOUNTS_DL = 'zti__AppAdmin__Home__dlLstHV';
//ZxMenuIds.ACCOUNTS_RES = 'zti__AppAdmin__Home__resLstHV';
//ZxMenuIds.CONFIGURE = 'zti__AppAdmin__Home__adminHV';
//ZxMenuIds.CONFIGURE_COS = 'zti__AppAdmin__CONFIGURATION__COS';
//ZxMenuIds.CONFIGURE_DOMAINS = 'zti__AppAdmin__CONFIGURATION__DOMAINS';
//ZxMenuIds.CONFIGURE_SERVER = 'zti__AppAdmin__CONFIGURATION__serverHV';
ZxMenuIds.CONFIGURE_GLOBAL = 'zti__AppAdmin__CONFIGURATION__GSET';
ZxMenuIds.CONFIGURE_GLOBAL_ZXCHAT = 'zti__AppAdmin__CONFIGURATION__GSET__13';
//ZxMenuIds.CONFIGURE_ZIMLET = 'zti__AppAdmin__CONFIGURATION__ZIMLETS';
//ZxMenuIds.CONFIGURE_ADMIN_EXT = 'zti__AppAdmin__CONFIGURATION__AEMINEXT';
//ZxMenuIds.CONFIGURE_CERTS = 'zti__AppAdmin__CONFIGURATION__CertHV';
// Not Yet listed
ZxMenuIds.MIGRATION_AND_TOOLS = 'zti__AppAdmin__Home__magHV';
ZxMenuIds.SEARCH = 'zti__AppAdmin__Home__searchHV';
// Useless to list
ZxMenuIds.ZEXTRAS = 'zti__AppAdmin__Home__ZeXtras';
ZxMenuIds.ZEXTRAS_MOBILE = 'zti__AppAdmin__ZeXtras__ZxMobile';
ZxMenuIds.ZEXTRAS_NOTIFICATION = 'zti__AppAdmin__ZeXtras__ZxNotifications';
