// Generated by CoffeeScript 1.10.0
var ParallelRequestManager, ParallelRequestStat, PoolRequest, Q, ZxAdminRequest, ZxError, ZxErrorCode, ZxSOAPRequest, exports,
  indexOf = [].indexOf || function(item) { for (var i = 0, l = this.length; i < l; i++) { if (i in this && this[i] === item) return i; } return -1; },
  hasProp = {}.hasOwnProperty;

if (typeof require !== "undefined" && require !== null) {
  ZxError = require('./zxJsLibs/ZxError.coffee');
  ZxErrorCode = require('./zxJsLibs/ZxErrorCode.coffee');
  Q = require('q');
} else {
  ZxError = window.ZxError;
  Q = window.Q;
  ZxSOAPRequest = window.ZxSOAPRequest;
  ZxErrorCode = window.ZxErrorCode;
}

ParallelRequestManager = (function() {
  ParallelRequestManager.PARALLEL_TIMEOUT = 100;

  function ParallelRequestManager() {
    this.currentPool = null;
    this.statistics = [];
  }

  ParallelRequestManager.prototype.queueRequest = function(request) {
    if ((this.currentPool == null) || this.currentPool.isExecuted()) {
      this.currentPool = new PoolRequest(this);
    }
    return this.currentPool.queueRequest(request);
  };

  ParallelRequestManager.prototype.storeStatistics = function(stats) {
    return this.statistics.push(stats);
  };

  ParallelRequestManager.prototype.printStatistics = function() {
    var j, len, ref, requestsDone, stats, totalFailures, totalRequests, totalTime;
    totalRequests = 0;
    totalFailures = 0;
    totalTime = 0;
    requestsDone = 0;
    ref = this.statistics;
    for (j = 0, len = ref.length; j < len; j++) {
      stats = ref[j];
      requestsDone += 1;
      totalRequests += stats.countTotal;
      totalFailures += stats.countFailed;
      totalTime += stats.getTimeDelta();
    }
    return "Saved: " + (totalRequests - requestsDone) + " requests ( " + ((((totalRequests - requestsDone) / totalRequests) * 100).toFixed(1)) + " % )\n Done: " + totalRequests + " requests, grouped in " + requestsDone + " requests\n       with " + totalFailures + " failures, mean processing time of " + ((totalTime / totalRequests).toFixed()) + " ms";
  };

  return ParallelRequestManager;

})();

PoolRequest = (function() {
  function PoolRequest(requestManager) {
    this.requestQueue = [];
    this.requestManager = requestManager;
    this.executed = false;
    this.statistics = new ParallelRequestStat();
    this.creationDate = new Date();
    Q.delay(ParallelRequestManager.PARALLEL_TIMEOUT).done((function(_this) {
      return function() {
        return _this._sendRequest();
      };
    })(this));
  }

  PoolRequest.prototype.isExecuted = function() {
    return this.executed;
  };

  PoolRequest.prototype.queueRequest = function(request) {
    request.setId(this.requestQueue.length + 1);
    this.requestQueue.push(request);
    return request.getPromise();
  };

  PoolRequest.prototype._sendRequest = function() {
    var j, len, mainRequest, ref, request, requestParams, requestsObjects;
    this.executed = true;
    this.statistics.setCount(this.requestQueue.length);
    mainRequest = new ZxSOAPRequest(ZxSOAPRequest.ADMINISTRATION);
    mainRequest.set("module", "ZxCore");
    mainRequest.set("action", "parallelRequest");
    requestsObjects = [];
    ref = this.requestQueue;
    for (j = 0, len = ref.length; j < len; j++) {
      request = ref[j];
      requestParams = request.getRequestParameters();
      ZxUtils.log(requestParams, "Processing a request");
      requestsObjects.push(requestParams);
    }
    mainRequest.set("requests", requestsObjects);
    return mainRequest.sendRequest().then((function(_this) {
      return function(response) {
        var error, error1, parsedResponse;
        try {
          parsedResponse = JSON.parse(response.content);
          if ((parsedResponse != null) && (parsedResponse.ok != null) && parsedResponse.ok) {
            return _this._processResponses(parsedResponse.response.responses);
          } else {
            if ((parsedResponse.exception != null) || (parsedResponse.error != null)) {
              error = new ZxSOAPError(ZxError.fromResponse(parsedResponse));
            } else {
              error = new ZxError(parsedResponse);
            }
            return _this._processError(error);
          }
        } catch (error1) {
          error = error1;
          return _this._processError(error);
        }
      };
    })(this), (function(_this) {
      return function(error) {
        return _this._processError(error);
      };
    })(this));
  };

  PoolRequest.prototype._processResponses = function(responses) {
    var completedCounter, j, len, response;
    if (responses == null) {
      responses = [];
    }
    ZxUtils.log(responses, "Processing parallels responses");
    this.statistics.start();
    completedCounter = 0;
    for (j = 0, len = responses.length; j < len; j++) {
      response = responses[j];
      if (this._processSingleResponse(response)) {
        completedCounter += 1;
      }
    }
    this.statistics.stop();
    this.statistics.setFailures(this.requestQueue.length - completedCounter);
    return this.requestManager.storeStatistics(this.statistics);
  };

  PoolRequest.prototype._processSingleResponse = function(response) {
    var action, candidateRequest, checkedResponse, deferred, everythingOk, modName, module, parameters, request, server;
    candidateRequest = (function() {
      var j, len, ref, results;
      ref = this.requestQueue;
      results = [];
      for (j = 0, len = ref.length; j < len; j++) {
        request = ref[j];
        if (("" + request.id) === ("" + response.id)) {
          results.push(request);
        }
      }
      return results;
    }).call(this);
    everythingOk = false;
    if (candidateRequest.length !== 1) {
      throw new Error('Unable to determine the request for a response');
    }
    request = candidateRequest[0];
    parameters = request.getRequestParameters();
    deferred = request.getDeferred();
    ZxUtils.log({
      request: request,
      response: response
    }, "Processing a single response");
    if ((response != null) && (response.ok != null) && response.ok) {
      server = parameters.targetServers;
      module = parameters.module;
      action = parameters.action;
      ZxAdminRequest._cacheResponse(server, module, action, parameters, response.response);
      modName = module;
      if (action === "getAllNotifications" || action === "setNotificationAttr") {
        modName = "ZxNotifications";
      }
      if ((typeof ZeXtras !== "undefined" && ZeXtras !== null) && ZeXtras.isZxMig(server) && modName === "ZxBackup") {
        modName = "ZxMig";
      }
      checkedResponse = ZxAdminRequest._checkModuleResponse(server, modName, response);
      if (checkedResponse instanceof ZxError) {
        everythingOk = false;
        deferred.reject(checkedResponse);
      } else {
        everythingOk = true;
        deferred.resolve(checkedResponse);
      }
    }
    return everythingOk;
  };

  PoolRequest.prototype._processError = function(response) {
    var action, candidateRequest, checkedResponse, deferred, everythingOk, module, parameters, request, server;
    ZxUtils.log(response, "Processing an error");
    candidateRequest = (function() {
      var j, len, ref, results;
      ref = this.requestQueue;
      results = [];
      for (j = 0, len = ref.length; j < len; j++) {
        request = ref[j];
        if (("" + request.id) === ("" + response.id)) {
          results.push(request);
        }
      }
      return results;
    }).call(this);
    everythingOk = false;
    if (candidateRequest.length !== 1) {
      throw new Error('Unable to determine the request for a response');
    }
    request = candidateRequest[0];
    parameters = request.getRequestParameters();
    deferred = request.getDeferred();
    server = parameters.targetServers;
    module = parameters.module;
    action = parameters.action;
    if (action === "getAllNotifications" || action === "setNotificationAttr") {
      module = "ZxNotifications";
    }
    if ((typeof ZeXtras !== "undefined" && ZeXtras !== null) && ZeXtras.isZxMig(server) && module === "ZxBackup") {
      module = "ZxMig";
    }
    checkedResponse = ZxAdminRequest._checkModuleResponse(server, module, response);
    if (checkedResponse instanceof ZxError) {
      everythingOk = false;
      deferred.reject(checkedResponse);
    }
    return false;
  };

  return PoolRequest;

})();

ParallelRequestStat = (function() {
  function ParallelRequestStat(countTotal, countFailed) {
    this.countTotal = countTotal != null ? countTotal : 0;
    this.countFailed = countFailed != null ? countFailed : 0;
    this.timeStart = 0;
    this.timeStop = 0;
  }

  ParallelRequestStat.prototype.setCount = function(countTotal) {
    this.countTotal = countTotal;
  };

  ParallelRequestStat.prototype.setFailures = function(countFailed) {
    this.countFailed = countFailed;
  };

  ParallelRequestStat.prototype.start = function() {
    return this.timeStart = new Date();
  };

  ParallelRequestStat.prototype.stop = function() {
    return this.timeStop = new Date();
  };

  ParallelRequestStat.prototype.getTimeDelta = function() {
    return this.timeStop.getTime() - this.timeStart.getTime();
  };

  return ParallelRequestStat;

})();

ZxAdminRequest = (function() {
  function ZxAdminRequest() {}

  ZxAdminRequest._cache = {};

  ZxAdminRequest._receivedWL = false;

  ZxAdminRequest._whiteList = {
    ZxCore: ["getUnlicensedCommands", "getLicenseInfo", "getVersion"]
  };

  ZxAdminRequest._parallelRequestManager = new ParallelRequestManager();

  ZxAdminRequest.printStatistics = function() {
    return this._parallelRequestManager.printStatistics();
  };

  ZxAdminRequest._cacheRules = {
    ZxCore: {},
    ZxAdmin: {}
  };

  ZxAdminRequest.TARGET_ALL = "all_servers";

  ZxAdminRequest.TARGET_LOCAL = "localhost";

  ZxAdminRequest.isRequestWhiteListed = function(module, request) {
    if (ZxAdminRequest._whiteList[module] != null) {
      return indexOf.call(ZxAdminRequest._whiteList[module], request) >= 0;
    }
    return false;
  };

  ZxAdminRequest._responsesCount = function(response) {
    var i, key, value;
    i = 0;
    for (key in response) {
      if (!hasProp.call(response, key)) continue;
      value = response[key];
      i += 1;
    }
    return i;
  };

  ZxAdminRequest._getCachedResponse = function(server, module, action, params) {
    var found, now, stringParams;
    if (params == null) {
      params = {};
    }
    if (typeof server !== "string") {
      throw new TypeError("[server] must be a 'string'");
    }
    if (typeof module !== "string") {
      throw new TypeError("[module] must be a 'string'");
    }
    if (typeof action !== "string") {
      throw new TypeError("[action] must be a 'string'");
    }
    stringParams = JSON.stringify(params);
    found = false;
    now = new Date().getTime();
    if ((ZxAdminRequest._cacheRules[module] != null) && (ZxAdminRequest._cacheRules[module][action] != null) && (ZxAdminRequest._cache[module] != null) && (ZxAdminRequest._cache[module][action] != null) && (ZxAdminRequest._cache[module][action][stringParams] != null) && (ZxAdminRequest._cache[module][action][stringParams].date != null) && ((now - ZxAdminRequest._cache[module][action][stringParams].date.getTime()) < ZxAdminRequest._cacheRules[module][action])) {
      found = true;
    }
    ZxUtils.log(found ? ZxAdminRequest._cache[module][action][stringParams].response : null, "Cache " + (found ? "HIT" : "MISS") + " for " + module + "." + action);
    if (found) {
      return ZxAdminRequest._cache[module][action][stringParams].response;
    }
  };

  ZxAdminRequest._cacheResponse = function(server, module, action, params, response) {
    var stringParams;
    if (params == null) {
      params = {};
    }
    if (response == null) {
      response = null;
    }
    if (typeof server !== "string") {
      throw new TypeError("[server] must be a 'string'");
    }
    if (typeof module !== "string") {
      throw new TypeError("[module] must be a 'string'");
    }
    if (typeof action !== "string") {
      throw new TypeError("[action] must be a 'string'");
    }
    stringParams = JSON.stringify(params);
    if (ZxAdminRequest._cache[module] == null) {
      ZxAdminRequest._cache[module] = {};
    }
    if (ZxAdminRequest._cache[module][action] == null) {
      ZxAdminRequest._cache[module][action] = {};
    }
    if (ZxAdminRequest._cache[module][action][stringParams] == null) {
      ZxAdminRequest._cache[module][action][stringParams] = {};
    }
    ZxAdminRequest._cache[module][action][stringParams].date = new Date();
    ZxAdminRequest._cache[module][action][stringParams].response = response;
    return ZxUtils.log(response, "Cached " + module + "." + action + " response");
  };

  ZxAdminRequest._prepareRequest = function(module, action, params) {
    var key, request, value;
    if (params == null) {
      params = {};
    }
    if (typeof module !== "string") {
      throw new TypeError("[module] must be a 'string'");
    }
    if (typeof action !== "string") {
      throw new TypeError("[action] must be a 'string'");
    }
    request = new ZxSOAPRequest(ZxSOAPRequest.ADMINISTRATION);
    request.set("module", module);
    request.set("action", action);
    for (key in params) {
      if (!hasProp.call(params, key)) continue;
      value = params[key];
      request.set(key, value);
    }
    return request;
  };

  ZxAdminRequest._checkModuleResponse = function(server, module, response) {
    var error1, error2, getModuleError, moduleItem, responseToReturn, serv, servResponse;
    if ((response.ok == null) || !response.ok) {
      response = ZxError.fromResponse(response);
    } else {
      response = response.response;
    }
    if (typeof ZeXtras === "undefined" || ZeXtras === null) {
      return response;
    }
    try {
      moduleItem = ZeXtras.getModuleItem(module, server);
      if ((moduleItem != null) && response instanceof ZxError) {
        moduleItem.handleServerError(response);
      } else {
        if (moduleItem != null) {
          moduleItem.handleServerOnline();
        }
        ZeXtras.declareServerOnline(server);
      }
    } catch (error1) {
      getModuleError = error1;
    }
    if (server !== ZxAdminRequest.TARGET_ALL && (response[server] != null)) {
      responseToReturn = response[server];
      if ((responseToReturn.ok == null) || !responseToReturn.ok) {
        return ZxError.fromResponse(responseToReturn);
      } else {
        return responseToReturn;
      }
    } else {
      for (serv in response) {
        if (!hasProp.call(response, serv)) continue;
        servResponse = response[serv];
        if ((servResponse.ok == null) || !servResponse.ok) {
          response[serv] = ZxError.fromResponse(servResponse);
          try {
            moduleItem = ZeXtras.getModuleItem(module, serv);
            if ((moduleItem != null) && response[serv] instanceof ZxError) {
              moduleItem.handleServerError(response[serv]);
            } else {
              if (moduleItem != null) {
                moduleItem.handleServerOnline();
              }
              ZeXtras.declareServerOnline(serv);
            }
          } catch (error2) {
            getModuleError = error2;
          }
        }
      }
      return response;
    }
  };

  ZxAdminRequest.sendSyncRequest = function(server, module, action, params) {
    var cache, err, error, modName, request, responseReq;
    if (params == null) {
      params = {};
    }
    if (typeof server !== "string") {
      throw new TypeError("[server] must be a 'string'");
    }
    if (typeof module !== "string") {
      throw new TypeError("[module] must be a 'string'");
    }
    if (typeof action !== "string") {
      throw new TypeError("[action] must be a 'string'");
    }
    params.targetServers = server;
    cache = ZxAdminRequest._getCachedResponse(server, module, action, params);
    if (cache != null) {
      return cache;
    }
    if ((server !== ZxAdminRequest.TARGET_LOCAL && server !== ZxAdminRequest.TARGET_ALL) && !ZxServer.isMailbox(server) && ZxUtils.amIAdmin()) {
      error = new ZxError(ZxErrorCode.UNABLE_TO_QUERY_NON_MBOX_SERVERS);
      error.setDetail("server", server);
      throw error;
    }
    request = ZxAdminRequest._prepareRequest(module, action, params);
    request.setSynchronous();
    responseReq = request.sendRequest();
    if ((responseReq != null) && (responseReq.ok != null) && (responseReq.response != null) && responseReq.ok) {
      ZxAdminRequest._cacheResponse(server, module, action, params, responseReq.response);
      if (typeof ZeXtras !== "undefined" && ZeXtras !== null) {
        modName = module;
        if (action === "getAllNotifications" || action === "setNotificationAttr") {
          modName = "ZxNotifications";
        }
        if (ZeXtras.isZxMig(server) && modName === "ZxBackup") {
          modName = "ZxMig";
        }
        ZxAdminRequest._checkModuleResponse(server, modName, responseReq);
        if (server !== ZxAdminRequest.TARGET_ALL && ZxAdminRequest._responsesCount(responseReq.response) === 1) {
          if (responseReq.response instanceof ZxError) {
            throw response;
          }
        }
        if (ZxAdminRequest._responsesCount(responseReq.response) > 1 || server === ZxAdminRequest.TARGET_ALL) {
          return responseReq.response;
        }
      }
    } else {
      if ((responseReq.exception != null) || (responseReq.error != null)) {
        err = new ZxSOAPError(ZxError.fromResponse(responseReq));
      } else {
        err = new ZxError(responseReq);
      }
      throw err;
    }
    return responseReq.response;
  };

  ZxAdminRequest.sendRequest = function(server, module, action, params, defer) {
    var cache, deferred, errString, error, request, requestPromise;
    if (params == null) {
      params = {};
    }
    if (typeof server !== "string") {
      throw new TypeError("[server] must be a 'string'");
    }
    if (typeof module !== "string") {
      throw new TypeError("[module] must be a 'string'");
    }
    if (typeof action !== "string") {
      throw new TypeError("[action] must be a 'string'");
    }
    params.targetServers = server;
    deferred = defer != null ? defer : Q.defer();
    if (!ZxAdminRequest.isRequestWhiteListed(module, action) && !ZxLicense.isModuleLicensed(module, server)) {
      errString = "Module '" + module + "' not licensed for server '" + server + "'(" + action + ")";
      if (ZxAdminRequest._receivedWL != null) {
        ZxUtils.log(errString, "Module action not licensed", ZxUtils.LOG_ERROR);
        throw new Error(errString);
      } else {
        ZxUtils.log({
          server: server,
          module: module,
          action: action
        }, "Deferring request");
        Q.delay(ZxSOAPRequest.RPC_CACHE_INTERVAL_RETRY).then(function() {
          return ZxAdminRequest.sendRequest(server, module, action, params, defer);
        });
        return deferred.promise;
      }
    }
    if ((server !== ZxAdminRequest.TARGET_LOCAL && server !== ZxAdminRequest.TARGET_ALL) && !ZxServer.isMailbox(server) && ZxUtils.amIAdmin()) {
      error = new ZxError(ZxErrorCode.UNABLE_TO_QUERY_NON_MBOX_SERVERS);
      error.setDetail("server", server);
      deferred.reject(error);
      return deferred.promise;
    }
    cache = ZxAdminRequest._getCachedResponse(server, module, action, params);
    if (cache != null) {
      deferred.resolve(cache);
    } else {
      request = ZxAdminRequest._prepareRequest(module, action, params);
      requestPromise = ZxAdminRequest._parallelRequestManager.queueRequest(request);
      requestPromise.then(function(response) {
        return deferred.resolve(response);
      }, function(error) {
        ZxUtils.log(error, 'Request Error');
        return deferred.reject(error);
      });
    }
    return deferred.promise;
  };

  return ZxAdminRequest;

})();

exports = exports != null ? exports : this;

if (exports.ZxAdminRequest == null) {
  exports.ZxAdminRequest = ZxAdminRequest;
}

if ((typeof module !== "undefined" && module !== null) && (module.exports != null)) {
  module.exports = ZxAdminRequest;
}

//# sourceMappingURL=ZxAdminRequest.js.map
