/*
 * ***** BEGIN LICENSE BLOCK *****
 * Copyright (C) 2011-2016 ZeXtras
 *
 * The contents of this file are subject to the ZeXtras EULA;
 * you may not use this file except in compliance with the EULA.
 * You may obtain a copy of the EULA at
 * http://www.zextras.com/zextras-eula.html
 * ***** END LICENSE BLOCK *****
 */

function loadZxACLManager(context, modifiers, loadMethods, modifyMethods, viewMethods) {
  if (!ZxUtils.isZ8Up()) { return; }
  if (!modifiers || !loadMethods || !modifyMethods || !viewMethods) { return; }

  function ZxACLManager() {}
  ZxACLManager.ACCOUNT_ENTRY_POINT = 'account_form_notes_group';
  ZxACLManager.DL_ENTRY_POINT = 'dl_properties';
  ZxACLManager.A_target_img = 'target_img';
  ZxACLManager.A_target = 'target';
  ZxACLManager.A_target_id = 'target_id';
  ZxACLManager.A_target_type = 'target_type';
  ZxACLManager.A_target_alias = 'target_alias';
  ZxACLManager.ZX_grantsList = 'zxGrantsList';
  ZxACLManager.A2_grantsListSelectedItems = 'grantsListSelectedItems';
  ZxACLManager.ZX_grantsListSelectedItems = 'ZXgrantsListSelectedItems';
  ZxACLManager.A_readableGrant = 'rightReadable';
  ZxACLManager.A_grantee_img = 'grantee_img';
  ZxACLManager.A_grantee = 'grantee';
  ZxACLManager.A_grantee_id = 'grantee_id';
  ZxACLManager.A_grantee_type = 'grantee_type';
  ZxACLManager.A_right = 'right';
  ZxACLManager.A_deny = 'deny';
  ZxACLManager.A_allow = 'allow';
  ZxACLManager.A_canDelegate = 'canDelegate';
  ZxACLManager.A_canInheritBySubDomain = 'subDomain';
  ZxACLManager.A_disinheritSubGroups = 'disinheritSubGroups';
  ZxACLManager.G_sendAs = 'sendAs';
  ZxACLManager.G_sendOnBehalfOf = 'sendOnBehalfOf';
  ZxACLManager.G_sendAsDistList = 'sendAsDistList';
  ZxACLManager.G_sendOnBehalfOfDistList = 'sendOnBehalfOfDistList';
  ZxACLManager.A2_isAdmin = 'isAdmin';
  ZxACLManager.A_right_type = 'right_type';

  ZxACLManager.DeniedAliasList = 'zxDeniedAliasList';
  ZxACLManager.GrantedAliasList = 'zxGrantedAliasList';
  ZxACLManager.AliasList = 'zxAliasList';

  ZxACLManager.inlineTargetType = [
    ZaItem.ACCOUNT, ZaItem.DL, ZaItem.RESOURCE, ZaItem.DOMAIN,
    ZaItem.COS, ZaItem.SERVER, ZaItem.UC_SERVICE, ZaItem.GLOBAL_CONFIG
  ];
  ZxACLManager.RIGHT_TYPE_CHOICES = [
    {value: 'system', label: zxMsg('LBL_system_right')},
    {value: 'inline', label: zxMsg('LBL_inline_right')}
  ];
  ZxACLManager.targetType = ZxACLManager.inlineTargetType.concat([ZaItem.ZIMLET, ZaItem.GLOBAL_GRANT]);

  ZxACLManager.castRightTypeForDistributionList = function(right) {
    if (right === ZxACLManager.G_sendAs) {
      return ZxACLManager.G_sendAsDistList;
    }
    if (right === ZxACLManager.G_sendOnBehalfOf) {
      return ZxACLManager.G_sendOnBehalfOfDistList;
    }
    return right;
  };

  ZxACLManager._XFORM = {
    type: _TOP_GROUPER_,
    label: '<div class="ImgZeXtras" style="float:left; padding-right:8px;"></div>' + zxMsg('LBL_ACL_Header'),
    visibilityChecks: [function() {return typeof ZeXtras !== 'undefined'; }],
    colSizes: [275, '*'],
    colSpan: '*',
    numCols: 2,
    items: [
      {
        type: _ZAWIZ_TOP_GROUPER_,
        label: zxMsg('LBL_ACL_Rights'),
        visibilityChecks: [function() {
          return typeof ZeXtras !== 'undefined';
        }],
        colSpan: 2,
        numCols: 1,
        colSizes: ['*'],
        items: [
          {
            type: _ZX_LIST_VIEW_,
            ref: 'attrs/' + ZxACLManager.ZX_grantsList,
            colSpan: 2,
            multiselect: false,
            widgetClass: ZxACLListView,
            headerList: ZxACLListView._getHeaderList(),
            optionalButtons: ZxACLListView._getOptButtons(),
            onSelection: ZxACLListView.onSelectionListener
          }
        ]
      },
      {
        type: _ZAWIZ_TOP_GROUPER_,
        label: zxMsg('LBL_ACL_aliases'),
        visibilityChecks: [function() {return typeof ZeXtras !== 'undefined'; }],
        colSpan: 2,
        numCols: 3,
        colSizes: ['*', '100', '*'],
        items: [
          {
            type: _ZX_ALERT_,
            colSpan: 3,
            visibilityChecks: [function() {return typeof ZeXtras !== 'undefined'; }],
            style: DwtAlert.INFORMATION,
            items: [
              {
                type: _OUTPUT_,
                value: zxMsg('MSG_ACL_Main_address_always_shared', [
                  '<a href="' + ZxLinks.HELP_ACL + '"  target="_blank">', '</a>'
                ])
              }
            ]
          },
          {
            type: _ZAWIZ_TOP_GROUPER_,
            label: zxMsg('LBL_ACL_Enabled_aliases'),
            valign: _TOP_,
            colSizes: ['*'],
            numCols: 1,
            items: [
              {
                type: _ZX_LIST_VIEW_,
                id: ZxACLManager.GrantedAliasList,
                ref: 'attrs/' + ZxACLManager.GrantedAliasList,
                height: 150,
                bmolsnr: true
              }
            ]
          },
          {
            type: _GROUP_,
            colSizes: ['*'],
            numCols: 1,
            cssClass: 'ZxListCenterButtons',
            valign: _MIDDLE_,
            items: [
              {
                type: _DWT_BUTTON_,
                label: zxMsg('LBL_Deny') + ' &gt;',
                enableDisableChangeEventSources: ['attrs/' + ZxACLManager.GrantedAliasList],
                enableDisableChecks: [
                  function() {
                    var currentName = this.getInstance().name,
                      grantedList = this.getInstanceValue('attrs/' + ZxACLManager.GrantedAliasList);
                    if (!currentName || !grantedList) { return false; }
                    return !(grantedList.length === 1 && grantedList[0] === currentName);
                  }
                ],
                onActivate: function(ev) {
                  var list,
                    obj = this.getInstance();
                  try {
                    list = this.getForm().getItemsById(ZxACLManager.GrantedAliasList);
                  } catch (err) {
                  }
                  if (list && list[0]) {
                    ZxACLManager.doAllowDenyAlias(obj, list[0].getSelection(), false);
                  }
                }
              },
              {
                type: _DWT_BUTTON_,
                cssStyle: 'margin-top: 10px',
                label: '&lt; ' + zxMsg('LBL_Allow'),
                onActivate: function(ev) {
                  var list,
                    obj = this.getInstance();
                  try {
                    list = this.getForm().getItemsById(ZxACLManager.DeniedAliasList);
                  } catch (err) {
                  }
                  if (list && list[0]) {
                    ZxACLManager.doAllowDenyAlias(obj, list[0].getSelection(), true);
                  }
                }
              }
            ]
          },
          {
            type: _ZAWIZ_TOP_GROUPER_,
            label: zxMsg('LBL_ACL_Disabled_aliases'),
            valign: _TOP_,
            colSizes: ['*'],
            numCols: 1,
            items: [
              {
                type: _ZX_LIST_VIEW_,
                height: 150,
                id: ZxACLManager.DeniedAliasList,
                ref: 'attrs/' + ZxACLManager.DeniedAliasList
              }
            ]
          }
        ]
      }
    ]
  };

  /**
   * Allow or deny an aliases if can be used.
   * @param {{}} instance
   * @param {String[]} aliases
   * @param {Boolean} allow
   */
  ZxACLManager.doAllowDenyAlias = function(instance, aliases, allow) {
    ZxUtils.log({allow: allow, aliases: aliases}, 'Allow or deny aliases to be used');
    if (aliases.length < 1) { return; }

    var command;
    if (instance.type === ZaItem.ACCOUNT) {
      command = allow ? 'doAddAllowedAliasesForAccount' : 'doRemoveAllowedAliasesForAccount';
    }
    if (instance.type === ZaItem.DL) {
      command = allow ? 'doAddAllowedAliasesForDistributionList' : 'doRemoveAllowedAliasesForDistributionList';
    }
    if (!command) {
      return;
    }

    ZxAdminRequest.sendRequest(
      ZxAdminRequest.TARGET_LOCAL,
      'ZxCore',
      command,
      {
        targetID: instance.id,
        aliases: aliases
      }
    ).then(
      AjxCallback.simpleClosure(
        function(instance, result) {
          ZxACLManager.AccountAndDistributionListLoadMethod.call(instance, 'id', instance.id);
        },
        this,
        instance
      ),
      ZxUtils.catchError
    );
  };

  ZxACLManager.AccountXFormModifier = function(xFormObject, entry) {
    var entryPoint;

    entryPoint = ZxUtils.searchXFormInjectionPoint(xFormObject, ZxACLManager.ACCOUNT_ENTRY_POINT);
    if (!entryPoint) { return; }
    entryPoint.splice((entryPoint.length > 0) ? entryPoint.length - 1 : 0, 0, ZxACLManager._XFORM);
  };

  ZxACLManager.DLXFormModifier = function(xFormObject, entry) {
    var entryPoint;

    entryPoint = ZxUtils.searchXFormInjectionPoint(xFormObject, ZxACLManager.DL_ENTRY_POINT);
    if (!entryPoint) { return; }
    entryPoint.push(
      {
        type: _ZX_ALERT_,
        style: DwtAlert.WARNING,
        visibilityChangeEventSources: ['dlType'],
        visibilityChecks: [function() {
          var instance = this.getInstance();
          return (instance.dlType === '1');
        }],
        items: [
          {
            type: _OUTPUT_,
            value: zxMsg('delegateRightsNotEnabled')
          }
        ]
      }
    );
    entryPoint.push(ZxACLManager._XFORM);
  };

  /**
   *
   * @param {string} by
   * @param {string} val
   * @param {*} withCos
   * @return {Array|undefined}
   * @this ZaAccount
   */
  ZxACLManager.AccountAndDistributionListLoadMethod = function(by, val, withCos) {
    ZxAdminRequest.sendRequest(
      ZxAdminRequest.TARGET_LOCAL,
      'ZxCore',
      'getAllowDelegatedAddress',
      {
        targetID: val,
        type: this.type,
        by: 'name'
      }
    ).then(
      AjxCallback.simpleClosure(
        function(resp) {
          var grants = resp.grants,
            grantList = [];

          if (grants != null) {
            for (var i = 0; i < grants.length; i++) {
              if (grants[i][ZxACLManager.G_sendAs] === 'false' && grants[i][ZxACLManager.G_sendOnBehalfOf] === 'false') {
                // Patch for ZX-3225 unknown grants will be skipped.
                continue;
              }
              var grant = {};   // ZaGrant object
              grant[ZxACLManager.A_target_img] = grants[i].targetImg;
              grant[ZxACLManager.A_target] = grants[i].targetName;
              grant[ZxACLManager.A_target_id] = grants[i].targetID;
              grant[ZxACLManager.A_target_type] = grants[i].targetType;

              grant[ZxACLManager.A_grantee_img] = grants[i].granteeImg;
              grant[ZxACLManager.A_grantee] = grants[i].granteeName;
              grant[ZxACLManager.A_grantee_type] = grants[i].granteeType;
              if (grant[ZxACLManager.A_grantee_type] === 'egp') {
                grant[ZxACLManager.A_grantee] = grant[ZxACLManager.A_target] + ':' + grant[ZxACLManager.A_grantee];
              }
              grant[ZxACLManager.A_grantee_id] = grants[i].granteeID;

              grant[ZxACLManager.G_sendAs] = (grants[i][ZxACLManager.G_sendAs] === 'true');
              grant[ZxACLManager.G_sendOnBehalfOf] = (grants[i][ZxACLManager.G_sendOnBehalfOf] === 'true');

              if (grant[ZxACLManager.G_sendAs] && grant[ZxACLManager.G_sendOnBehalfOf]) {
                grant[ZxACLManager.A_readableGrant] =
                  zxMsg('sendAs') + ' ' + zxMsg('LBL_and') + ' ' + zxMsg('sendOnBehalfOf');
              } else if (grant[ZxACLManager.G_sendAs]) {
                grant[ZxACLManager.A_readableGrant] = zxMsg('sendAs');
              } else if (grant[ZxACLManager.G_sendOnBehalfOf]) {
                grant[ZxACLManager.A_readableGrant] = zxMsg('sendOnBehalfOf');
              }

              grantList.push(grant);
            }

            if (this.type === ZaItem.GLOBAL_GRANT) {
              return grantList;
            } else {
              this.attrs[ZxACLManager.ZX_grantsList] = grantList;
              ZxFormModifier.setInstanceValue(this, grantList, 'attrs/' + ZxACLManager.ZX_grantsList, true);
            }
          }
        },
        this
      )
    );

    ZxAdminRequest.sendRequest(
      ZxAdminRequest.TARGET_LOCAL,
      'ZxCore',
      'getDelegatedAliasAddress',
      {
        targetID: val,
        type: this.type
      }
    ).then(
      AjxCallback.simpleClosure(
        function(resp) {
          var availableList = [],
            i;

          this.attrs[ZxACLManager.GrantedAliasList] = resp[ZxACLManager.GrantedAliasList];
          ZxFormModifier.setInstanceValue(this, this.attrs[ZxACLManager.GrantedAliasList],
              'attrs/' + ZxACLManager.GrantedAliasList, true);

          this.attrs[ZxACLManager.AliasList] = resp[ZaAccount.A_zimbraMailAlias];
          ZxFormModifier.setInstanceValue(this, this.attrs[ZxACLManager.AliasList],
              'attrs/' + ZxACLManager.AliasList, true);

          for (i = 0; i < this.attrs[ZxACLManager.AliasList].length; i += 1) {
            if (ZxUtils.indexOf(resp[ZxACLManager.GrantedAliasList], this.attrs[ZxACLManager.AliasList][i]) < 0) {
              availableList.push(this.attrs[ZxACLManager.AliasList][i]);
            }
          }

          this.attrs[ZxACLManager.DeniedAliasList] = availableList;
          ZxFormModifier.setInstanceValue(this, availableList,
              'attrs/' + ZxACLManager.DeniedAliasList, true);
        },
        this
      )
    );
  };

  ZxACLManager.AccountAndDistributionListModifyMethod = function(mods, tmpObj) {
    ZxUtils.log(mods, 'Called modify method');
    if (mods) {
      if (mods[ZxACLManager.ZX_grantsList]) {
        delete mods[ZxACLManager.ZX_grantsList];
      }
      if (mods[ZxACLManager.DeniedAliasList]) {
        delete mods[ZxACLManager.DeniedAliasList];
      }
      if (mods[ZxACLManager.GrantedAliasList]) {
        delete mods[ZxACLManager.GrantedAliasList];
      }
      if (mods[ZxACLManager.AliasList]) {
        delete mods[ZxACLManager.AliasList];
      }
    }
    if (tmpObj) {
      if (tmpObj.attrs[ZxACLManager.ZX_grantsList]) {
        delete tmpObj.attrs[ZxACLManager.ZX_grantsList];
      }
      if (tmpObj.attrs[ZxACLManager.DeniedAliasList]) {
        delete tmpObj.attrs[ZxACLManager.DeniedAliasList];
      }
      if (tmpObj.attrs[ZxACLManager.GrantedAliasList]) {
        delete tmpObj.attrs[ZxACLManager.GrantedAliasList];
      }
      if (tmpObj.attrs[ZxACLManager.AliasList]) {
        delete tmpObj.attrs[ZxACLManager.AliasList];
      }
    }
  };

  ZxACLManager.grantListItem = {
    id: ZxACLManager.ZX_grantsList,
    ref: 'attrs/' + ZxACLManager.ZX_grantsList,
    type: _LIST_,
    listItems: {
      type: _OBJECT_,
      items: [
        {id: ZxACLManager.A_target, type: _STRING_, ref: ZxACLManager.A_target },
        {id: ZxACLManager.A_target_type, ref: ZxACLManager.A_target_type, type: _STRING_,
          choices: ZxACLManager.targetType },
        {id: ZxACLManager.A_grantee, type: _STRING_, ref: ZxACLManager.A_grantee, required: true },
        {id: ZxACLManager.A_grantee_type, type: _LIST_, ref: ZxACLManager.A_grantee_type, required: true
          /* choices: ZaGrant.GRANT_TYPE */},
        {id: ZxACLManager.A_canDelegate, type: _ENUM_, ref: ZxACLManager.A_canDelegate,
          choices: ZaModel.BOOLEAN_CHOICES2 },
        {id: ZxACLManager.A_deny, type: _ENUM_, ref: ZxACLManager.A_allow, choices: ZaModel.BOOLEAN_CHOICES2 },
        {id: ZxACLManager.A_deny, type: _ENUM_, ref: ZxACLManager.A_deny, choices: ZaModel.BOOLEAN_CHOICES2 },
        {id: ZxACLManager.A_readableGrant, type: _STRING_, ref: ZxACLManager.A_readableGrant, required: true }
      ]
    }
  };

  ZxACLManager.grantListSelectItem = {
    ref: ZxACLManager.A2_grantsListSelectedItems,
    id: ZxACLManager.A2_grantsListSelectedItems,
    type: _LIST_
  };

  if (ZaAccount) {
    ZaAccount.myXModel.items.push(ZxACLManager.grantListItem);
    ZaAccount.myXModel.items.push(ZxACLManager.grantListSelectItem);
  }
  if (modifiers && modifiers.ZaAccountXFormView) {
    modifiers.ZaAccountXFormView.push(ZxACLManager.AccountXFormModifier);
  }
  if (loadMethods && loadMethods.ZaAccount) {
    loadMethods.ZaAccount.push(ZxACLManager.AccountAndDistributionListLoadMethod);
  }
  if (modifyMethods && modifyMethods.ZaAccount) {
    modifyMethods.ZaAccount.splice(0, 0, ZxACLManager.AccountAndDistributionListModifyMethod);
  }

  if (ZaDistributionList) {
    ZaDistributionList.myXModel.items.push(ZxACLManager.grantListItem);
    ZaDistributionList.myXModel.items.push(ZxACLManager.grantListSelectItem);
  }
  if (modifiers && modifiers.ZaDLXFormView) {
    modifiers.ZaDLXFormView.push(ZxACLManager.DLXFormModifier);
  }
  if (loadMethods && loadMethods.ZaDistributionList) {
    loadMethods.ZaDistributionList.push(ZxACLManager.AccountAndDistributionListLoadMethod);
  }
  if (modifyMethods && modifyMethods.ZaDistributionList) {
    modifyMethods.ZaDistributionList.splice(0, 0, ZxACLManager.AccountAndDistributionListModifyMethod);
  }

  if (ZaSearch && !ZaSearch.prototype.dynSelectGrantees) {
    ZaSearch.prototype.dynSelectGrantees = function(callArgs) {
      try {
        var value = callArgs.value,
          event = callArgs.event,
          callback = callArgs.callback,
          form = callArgs.form,
          params = {},
          dataCallback;

        params.types = [ZaSearch.ACCOUNTS, ZaSearch.DLS]; //ZaSearch.ALIASES.
       /* if (form.instance[ZxACLManager.A_target_type] === ZaItem.ACCOUNT) {
          params.types.push(ZaSearch.DLS);
        }*/

        var query = ZaSearch.getSearchByNameQuery(value, params.types),
          controller = ZaApp.getInstance().getCurrentController(),
          adminrequired = ['(|',
            '(', ZaDistributionList.A_isAdminGroup, '=TRUE)',
            '(', ZaAccount.A_zimbraIsAdminAccount, '=TRUE)',
            '(', ZaAccount.A_zimbraIsDelegatedAdminAccount, '=TRUE)',
            ')'].join('');

        if (form.instance[ZxACLManager.A_target_type] !== ZaItem.ACCOUNT &&
            form.instance[ZxACLManager.A_target_type] !== ZaItem.DOMAIN &&
            form.instance[ZxACLManager.A_target_type] !== ZaItem.GLOBAL_GRANT &&
            form.instance[ZxACLManager.A_target_type] !== ZaItem.DL) {
          query = '(&' + query + adminrequired + ')';

        } else if (form.instance[ZxACLManager.A_target_type] === ZaItem.DOMAIN ||
            form.instance[ZxACLManager.A_target_type] === ZaItem.ACCOUNT ||
            form.instance[ZxACLManager.A_target_type] === ZaItem.GLOBAL_GRANT ||
            form.instance[ZxACLManager.A_target_type] === ZaItem.DL) {
          value = ZaSearch.escapeLdapQuery(value);
          query = '(|' + query + '(' + ZaDomain.A_domainName + '=*' + value + '*))';
          params.types.push(ZaSearch.DOMAINS);

        }

        dataCallback = new AjxCallback(this,
          this.dynSelectGranteeCallback,
          {callback: callback, controller: controller});
        params.callback = dataCallback;
        params.sortBy = ZaAccount.A_name;
        params.query = query;
        params.controller = controller;
        ZaSearch.searchDirectory(params);
      } catch (ex) {
        ZaApp.getInstance().getCurrentController()._handleException(ex, 'ZaSearch.prototype.dynSelectDataFetcher');
      }
    };

    if (ZaSearch && !ZaSearch.prototype.dynSelectGranteeCallback) {
      ZaSearch.prototype.dynSelectGranteeCallback = function(params, resp) {
        var callback = params.callback;
        var controller = params.controller;
        if (!callback)
          return;
        try {
          if (!resp) {
            throw (new AjxException(ZaMsg.ERROR_EMPTY_RESPONSE_ARG,
              AjxException.UNKNOWN, 'ZaListViewController.prototype.dynSelectDataCallback'));
          }
          if (resp.isException()) {
            throw (resp.getException());
          } else {
            var response = resp.getResponse().Body.SearchDirectoryResponse;
            var list = new ZaItemList(null);
            list.loadFromJS(response);
            if (controller instanceof ZaDLController) {
              controller.granteeList = list.getArray();
            }

            callback.run(list.getArray(), response.more, response.searchTotal);
          }
        } catch (ex) {
          ZaApp.getInstance().getCurrentController()._handleException(ex, 'ZaSearch.prototype.dynSelectDataCallback');
        }
      };
    }
  }

  context.ZxACLManager = ZxACLManager;
}

loadZxACLManager(
  window,
  ZaTabView.XFormModifiers,
  ZaItem.loadMethods,
  ZaItem.modifyMethods,
  ZaController.setViewMethods
);
