/*
 * ***** BEGIN LICENSE BLOCK *****
 * Copyright (C) 2011-2016 ZeXtras
 *
 * The contents of this file are subject to the ZeXtras EULA;
 * you may not use this file except in compliance with the EULA.
 * You may obtain a copy of the EULA at
 * http://www.zextras.com/zextras-eula.html
 * ***** END LICENSE BLOCK *****
*/

function ZxACLEditDialog(currentObject, grant, editMode) {
  this._original = ZxUtils.cloneObj(grant);
  this._account = currentObject;
  this._editMode = editMode;
  this._sendAs = grant[ZxACLEditDialog.A_right];
  this.targetNameChoices = new XFormChoices([], XFormChoices.OBJECT_LIST, 'name', 'name');
  this._original[ZxACLEditDialog.TARGET_CHANGED] = false;

  var helpButton,
    closeButton,
    saveButton,
    helpUrl = ZxLinks.HELP_ACL_Dialog_edit_new,
    icon = 'ZeXtras',
    title;

  title = ZxUtils.getTitleWithIcon(icon, zxMsg('LBL_Delegate_Permission'));

  helpButton = new DwtDialog_ButtonDescriptor(
    ZaXDialog.HELP_BUTTON,
    ZaMsg.TBB_Help,
    DwtDialog.ALIGN_LEFT,
    new AjxCallback(
      this,
      ZxUtils.openURL,
      [helpUrl]
    )
  );

  closeButton = new DwtDialog_ButtonDescriptor(
    ZxACLEditDialog.ID_BTN_CLOSE,
    ZaMsg.TBB_Close,
    DwtDialog.ALIGN_RIGHT,
    new AjxCallback(
      this,
      function() {
        this.popdown();
      }
    )
  );

  saveButton = new DwtDialog_ButtonDescriptor(
    ZxACLEditDialog.ID_BTN_SAVE,
    ZaMsg.TBB_Save,
    DwtDialog.ALIGN_RIGHT,
    new AjxCallback(
      this,
      this._saveCallback
    )
  );

  this._standardButtons = [DwtDialog.NO_BUTTONS];
  this._extraButtons = [helpButton, saveButton, closeButton];
  this._rightsChoices = new XFormChoices([], XFormChoices.SIMPLE_LIST);

  ZaXDialog.call(
    this,
    ZaApp.getInstance().getAppCtxt().getShell(),
    null,
    title,
    '400px',
    '120px',
    null,
    null
  );
  this.initForm(ZxACLEditDialog.MODEL, this.getMyXForm());
  if (!this._editMode) {
    this.getButton(ZxACLEditDialog.ID_BTN_SAVE).setEnabled(false);
  }
  this.setObject(grant);

  this._helpURL = helpUrl;
}
ZxACLEditDialog.prototype = new ZaXDialog();
ZxACLEditDialog.prototype.constructor = ZxACLEditDialog;
ZxACLEditDialog.prototype.supportMinimize = false;

ZxACLEditDialog.ID_BTN_CLOSE = 'closeBTN';
ZxACLEditDialog.ID_BTN_SAVE = 'saveBTN';
ZxACLEditDialog.TARGET_CHANGED = 'targetChanged';

ZxACLEditDialog.ENTRY_POINT = 'account_form_notes_group';
ZxACLEditDialog.A_target = 'target';
ZxACLEditDialog.A_target_id = 'target_id';
ZxACLEditDialog.A_target_type = 'target_type';
ZxACLEditDialog.ZX_grantsList = 'zxGrantsList';
ZxACLEditDialog.A2_grantsListSelectedItems = 'grantsListSelectedItems';
ZxACLEditDialog.ZX_grantsListSelectedItems = 'ZXgrantsListSelectedItems';
ZxACLEditDialog.A_readableGrant = 'rightReadable';
ZxACLEditDialog.A_grantee = 'grantee';
ZxACLEditDialog.A_grantee_id = 'grantee_id';
ZxACLEditDialog.A_grantee_type = 'grantee_type';
ZxACLEditDialog.A_right = 'right';
ZxACLEditDialog.A_deny = 'deny';
ZxACLEditDialog.A_allow = 'allow';
ZxACLEditDialog.A_canDelegate = 'canDelegate';
ZxACLEditDialog.A_canInheritBySubDomain = 'subDomain';
ZxACLEditDialog.A_disinheritSubGroups = 'disinheritSubGroups';
ZxACLEditDialog.G_sendAs = 'sendAs';
ZxACLEditDialog.G_sendAsDistList = 'sendAsDistList';
ZxACLEditDialog.G_sendOnBehalfOf = 'sendOnBehalfOf';
ZxACLEditDialog.G_sendOnBehalfOfDistList = 'sendOnBehalfOfDistList';
ZxACLEditDialog.A2_isAdmin = 'isAdmin';
ZxACLEditDialog.A_right_type = 'right_type';
ZxACLEditDialog.inlineTargetType = [
  ZaItem.ACCOUNT, ZaItem.DL, ZaItem.RESOURCE, ZaItem.DOMAIN,
  ZaItem.COS, ZaItem.SERVER, ZaItem.UC_SERVICE, ZaItem.GLOBAL_CONFIG
];
ZxACLEditDialog.RIGHT_TYPE_CHOICES = [
  {value: 'system', label: zxMsg('LBL_system_right')},
  {value: 'inline', label: zxMsg('LBL_inline_right')}
];
ZxACLEditDialog.targetType = ZxACLEditDialog.inlineTargetType.concat([ZaItem.ZIMLET, ZaItem.GLOBAL_GRANT]);

ZxACLEditDialog.MODEL = [
  {type: _STRING_, id: ZxACLEditDialog.A_target,
    ref: ZxACLEditDialog.A_target},
  {type: _STRING_, id: ZxACLEditDialog.A_grantee,
    ref: ZxACLEditDialog.A_grantee},
  {type: _ENUM_, id: ZxACLEditDialog.G_sendAs, choices: ZaModel.BOOLEAN_CHOICES1,
    ref: ZxACLEditDialog.G_sendAs},
  {type: _ENUM_, id: ZxACLEditDialog.G_sendOnBehalfOf, choices: ZaModel.BOOLEAN_CHOICES1,
    ref: ZxACLEditDialog.G_sendOnBehalfOf},
  {type: _ENUM_, id: ZxACLEditDialog.A2_isAdmin, choices: ZaModel.BOOLEAN_CHOICES2,
    ref: ZxACLEditDialog.A2_isAdmin},
  {type: _ENUM_, id: ZxACLEditDialog.A_right_type, choices: ZxACLEditDialog.RIGHT_TYPE_CHOICES,
    ref: ZxACLEditDialog.A_right_type},
  {type: _STRING_, id: ZxACLEditDialog.A_grantee_type,
    ref: ZxACLEditDialog.A_grantee_type}
];

ZxACLEditDialog.prototype.getShell = function() {
  return this._shell;
};

ZxACLEditDialog.prototype._saveCallback = function() {
  var diff = ZxUtils.objDiff(this._original, this.getObject()),
    newACL = this.getObject(),
    oldACL = this._original,
    rightsToRemove = [];

  if (ZxUtils.isObjectEmpty(diff)) {
    this.popdown();
    return;
  }

  if (!oldACL[ZxACLEditDialog.G_sendAs] && newACL[ZxACLEditDialog.G_sendAs]) {
    this._grantRight(newACL, ZxACLEditDialog.G_sendAs);
  }
  if (!oldACL[ZxACLEditDialog.G_sendOnBehalfOf] && newACL[ZxACLEditDialog.G_sendOnBehalfOf]) {
    this._grantRight(newACL, ZxACLEditDialog.G_sendOnBehalfOf);
  }

  if (oldACL[ZxACLEditDialog.G_sendAs] && !newACL[ZxACLEditDialog.G_sendAs]) {
    rightsToRemove.push(ZxACLEditDialog.G_sendAs);
  }
  if (oldACL[ZxACLEditDialog.G_sendOnBehalfOf] && !newACL[ZxACLEditDialog.G_sendOnBehalfOf]) {
    rightsToRemove.push(ZxACLEditDialog.G_sendOnBehalfOf);
  }
  if (rightsToRemove.length > 0) {
    this.popupRevokeRights(oldACL, rightsToRemove);
  }

  this.popdown();
};

ZxACLEditDialog.prototype._revokeRightsCallback = function(account, grant, rightsToRemove) {
  rightsToRemove = rightsToRemove || [ZxACLManager.G_sendAs, ZxACLManager.G_sendOnBehalfOf];
  var i = 0;

  for (i = 0; i < rightsToRemove.length; i += 1) {
    if (grant[rightsToRemove[i]]) {
      this._revokeSingleRight(account, grant, rightsToRemove[i]);
    }
  }
};

/**
 * Remove a set of rights from the grant object.
 * @param {{}} grant
 * @param {String[]} rightsToRemove
 */
ZxACLEditDialog.prototype.popupRevokeRights = function(grant, rightsToRemove) {
  rightsToRemove = rightsToRemove || [ZxACLManager.G_sendAs, ZxACLManager.G_sendOnBehalfOf];
  // By default remove all grants

  var messageString = [zxMsg('MSG_Are_you_sure_you_want_to_remove_acls'), ''],
    dialog = new DwtMessageDialog({
      parent: this.getShell(),
      buttons: [DwtDialog.YES_BUTTON, DwtDialog.NO_BUTTON]
    }),
    i,
    appendAnd = false,
    stringBuffer = [];

  for (i = 0; i < rightsToRemove.length; i += 1) {
    if (appendAnd) {
      stringBuffer.push(zxMsg('LBL_and'));
      appendAnd = false;
    }
    if (rightsToRemove[i] === ZxACLManager.G_sendAs && grant[ZxACLManager.G_sendAs]) {
      stringBuffer.push(zxMsg('sendAs'));
      appendAnd = true;
    }
    if (rightsToRemove[i] === ZxACLManager.G_sendOnBehalfOf && grant[ZxACLManager.G_sendOnBehalfOf]) {
      stringBuffer.push(zxMsg('sendOnBehalfOf'));
      appendAnd = true;
    }
  }

  messageString.push('<b>' + zxMsg('LBL_Target_Name') + '</b>: ' + grant[ZxACLEditDialog.A_target]);
  messageString.push('<b>' + zxMsg('LBL_Grantee_Name') + '</b>: ' + grant[ZxACLEditDialog.A_grantee]);
  messageString.push('<b>' + ZaMsg.DLV_Type_col + '</b>: ' + stringBuffer.join(' '));

  dialog.setMessage(
    messageString.join('<br>'),
    DwtMessageDialog.WARNING_STYLE,
    ZxUtils.getTitleWithIcon('ZeXtras', zxMsg('LBL_Delegate_Permission'))
  );

  dialog.registerCallback(
    DwtDialog.YES_BUTTON,
    new AjxCallback(
      this,
      function(dlg, grants) {
        this._revokeRightsCallback(this._account, grants, rightsToRemove);
        dlg.popdown();
      },
      [dialog, grant]
    )
  );

  dialog.popup();
};

ZxACLEditDialog.prototype._grantRight = function(grant, right) {
  if (!grant) { return; }
  /*  <GrantRightRequest xmlns="urn:zimbraAdmin">
      <target by="name" type="account">admin@example.com</target>
      <grantee by="name" type="usr">asd@example.com</grantee>
      <right>sendAs</right>
    </GrantRightRequest> */
  var params = {
    targetID: grant[ZxACLEditDialog.A_target_id],
    targetEmail: grant[ZxACLEditDialog.A_target],
    type: grant[ZxACLEditDialog.A_target_type],
    by: 'name',
    granteeEmail: grant[ZxACLEditDialog.A_grantee],
    granteeType: grant[ZxACLEditDialog.A_grantee_type]
  },
    command = '';

  if (grant[ZxACLEditDialog.A_target_type] === 'account') {
    command = 'doAddAllowAddressForDelegatedSender';
  }

  if (grant[ZxACLEditDialog.A_target_type] === 'dl') {
    command = 'doAddAllowAddressForDistributionList';
    right = ZxACLManager.castRightTypeForDistributionList(right);
  }

  if (command === '') {
    return false;
  }

  params.right = right;

  try {
    ZxAdminRequest.sendSyncRequest(
      ZxAdminRequest.TARGET_LOCAL,
      'ZxCore',
      command,
      params
    );

    ZxACLManager.AccountAndDistributionListLoadMethod.call(this._account, 'id', this._account.id);
  } catch (error) {
    ZxUtils.catchError(error);
  }

  return true;
};

/**
 * Revoke a right sending the request.
 * @param {{}|ZaGrant} grant
 * @return {boolean}
 */
ZxACLEditDialog.prototype._revokeSingleRight = function(account, grant, right) {
  if (!grant) { return; }
/*  <RevokeRightRequest xmlns="urn:zimbraAdmin">
      <target by="id" type="account">xxxxxxxxx</target>
      <grantee by="id" type="usr">xxxxxxxxx</grantee>
      <right>sendAs</right>
    </RevokeRightRequest> */
  try {
    var params = {},
      command = '';

    if (grant[ZxACLEditDialog.A_target_type] === 'account') {
      command = 'doRemoveDelegatedSenderAddress';
    }

    if (grant[ZxACLEditDialog.A_target_type] === 'dl') {
      command = 'doRemoveDelegatedDistributionList';
      right = ZxACLManager.castRightTypeForDistributionList(right);
    }

    if (command === '') {
      return false;
    }

    params = {
      targetID: grant[ZxACLEditDialog.A_target_id],
      targetEmail: grant[ZxACLEditDialog.A_target],
      type: grant[ZxACLEditDialog.A_target_type],
      by: 'name',
      granteeEmail: grant[ZxACLEditDialog.A_grantee],
      granteeType: grant[ZxACLEditDialog.A_grantee_type],
      right: right
    };

    try {
      ZxAdminRequest.sendSyncRequest(
        ZxAdminRequest.TARGET_LOCAL,
        'ZxCore',
        command,
        params
      );
      ZxACLManager.AccountAndDistributionListLoadMethod.call(this._account, 'id', this._account.id);
    } catch (error) {
      ZxUtils.catchError(error);
    }

    return true;
  } catch (ex) {
    ZxUtils.catchError(ex);

    if (grant[ZxACLEditDialog.A_target_type] === 'account' ||
        grant[ZxACLEditDialog.A_target_type] === 'dl') {
      ZxACLManager.AccountAndDistributionListLoadMethod.call(this._account, 'id', this._account.id);
    }
    return false;
  }
};


ZxACLEditDialog.setGranteeChanged = function(value, event, form) {
  var instance = this.getInstance(),
    newEvent;

  this.setInstanceValue(value);
  this.setInstanceValue(value, ZxACLEditDialog.A_grantee);

  if (!value && instance[ZxACLEditDialog.A_target_type] === ZaItem.DL) {
    return;
  }

  if (
      ((value.lastIndexOf('@') === ZxUtils.indexOf(value, '@')) && (ZxUtils.indexOf(value, '@') > 0)) ||
      instance[ZxACLEditDialog.A_target_type] === ZaItem.DOMAIN ||
      instance[ZxACLEditDialog.A_target_type] === ZaItem.DL ||
      instance[ZxACLEditDialog.A_target_type] === ZaItem.ACCOUNT ||
      instance[ZxACLEditDialog.A_target_type] === ZaItem.GLOBAL_GRANT
  ) {
    form.parent.updateGranteeType(value, this);

  } else {
    this.setError(AjxMessageFormat.format(AjxMsg.invalidEmailAddr));
    newEvent = new DwtXFormsEvent(form, this, value);
    form.notifyListeners(DwtEvent.XFORMS_VALUE_ERROR, newEvent);
    form.parent.getButton(ZxACLEditDialog.ID_BTN_SAVE).setEnabled(false);
  }
};

ZxACLEditDialog.prototype.updateGranteeType = function(grantee, xform) {
  try {
    var params = {},
      query,
      dataCallback;

    grantee = ZaSearch.escapeLdapQuery(grantee);
    query = [
      '(|',
      '(', ZaAccount.A_mail, '=', grantee, ')',
      '(', ZaAccount.A_zimbraMailAlias, '=', grantee, ')',
      '(', ZaDomain.A_domainName, '=', grantee, ')',
      ')'].join('');
    dataCallback = new AjxCallback(this, this.setGranteeType, {xform: xform});
    params.types = [ZaSearch.ACCOUNTS, ZaSearch.DLS, ZaSearch.DOMAINS];
    params.callback = dataCallback;
    params.query = query;
    params.controller = ZaApp.getInstance().getCurrentController();
    ZaSearch.searchDirectory(params);
  } catch (ex) {
    ZxUtils.catchError(ex);
  }
};

ZxACLEditDialog.prototype.setGranteeType = function(params, resp) {
  try {
    if (!resp) {
      throw (new AjxException(ZaMsg.ERROR_EMPTY_RESPONSE_ARG, AjxException.UNKNOWN, 'ZxACLEditDialog.setGranteeType'));
    }
    if (resp.isException()) {
      throw (resp.getException());
    } else {
      var response = resp.getResponse().Body.SearchDirectoryResponse,
        list = new ZaItemList(null),
        grantee;

      list.loadFromJS(response);
      grantee = list.getArray();
      if (grantee.length !== 1) {
        //either grantee doesn't exist or not unique.
        if (params.xform) {
          var xform = params.xform;
          xform.setError(ZaMsg.ERROR_INVALID_VALUE);
          var event = new DwtXFormsEvent(this._localXForm, xform);
          this._localXForm.notifyListeners(DwtEvent.XFORMS_VALUE_ERROR, event);
          this.getButton(ZxACLEditDialog.ID_BTN_SAVE).setEnabled(false);
        }
      } else {
        var type = grantee[0].type,
          granteeType = '',
          oldValue = this._localXForm.getInstanceValue(ZxACLEditDialog.A2_isAdmin),
          emptyChoises = new XFormChoices([], XFormChoices.OBJECT_LIST, 'name', 'name');
        if (type === ZaItem.ACCOUNT) {
          granteeType = 'usr';
          this._localXForm.setInstanceValue('0', ZxACLEditDialog.A2_isAdmin);
        } else if (type === ZaItem.DL) {
          granteeType = 'grp';
          if (oldValue != '0') {
            this._localXForm.setInstanceValue('0', ZxACLEditDialog.A2_isAdmin);
          }
        } else if (type === ZaItem.DOMAIN) {
          granteeType = 'dom';
          if (oldValue != '0') {
            this._localXForm.setInstanceValue('0', ZxACLEditDialog.A2_isAdmin);
          }
        }
        if (this._localXForm.getInstanceValue(ZxACLEditDialog.A2_isAdmin) !== oldValue) {
          if (this._localXForm.getInstanceValue(ZxACLEditDialog.A2_isAdmin) == '0') {
            this._localXForm.setInstanceValue('system', ZxACLEditDialog.A_right_type);
          }
        }
        if (this._localXForm.getInstanceValue(ZxACLEditDialog.A2_isAdmin) == '0') {
          this._rightsChoices.setChoices(emptyChoises);
          this._rightsChoices.dirtyChoices();
          this.getButton(ZxACLEditDialog.ID_BTN_SAVE).setEnabled(true);
        }

        this._localXForm.setInstanceValue(granteeType, ZxACLEditDialog.A_grantee_type);
//        this._localXForm.getItemsById(ZxACLEditDialog.A_grantee_type)[0].updateElement(granteeType);
      }
    }
  } catch (ex) {
    this.getButton(ZxACLEditDialog.ID_BTN_SAVE).setEnabled(false);
    ZxUtils.catchError(ex);
  }
};

ZxACLEditDialog.setARight = function(currentValue) {
  var instance = this.getInstance();
  instance[ZxACLEditDialog.A_right] = currentValue;
};

ZxACLEditDialog.prototype.getMyXForm = function() {
  var xFormObject;

  xFormObject = {
    numCols: 1,
    colSizes: ['*'],
    cssStyle: 'padding: 5px',
    items: [
      {
        type: _GROUP_,
        numCols: 2,
        cssStyle: 'overflow: hidden;',
        colSizes: [110, '*'],
        items: [
          {
            type: _OUTPUT_,
            ref: ZxACLEditDialog.A_target,
            id: ZxACLEditDialog.A_target,
            label: zxMsg('LBL_Target_Name') + ': '
          },
          {
            ref: ZxACLEditDialog.A_grantee,
            id: ZxACLEditDialog.A_grantee,
            type: _DYNSELECT_,
            label: zxMsg('LBL_Grantee_Name') + ': ',
            inputWidth: 200,
            visibilityChecks: [function() { return true; }],
            enableDisableChecks: [AjxCallback.simpleClosure(function() {
              return !this._editMode;
            }, this)],
            choices: this.targetNameChoices,
            onChange: ZxACLEditDialog.setGranteeChanged,
            dataFetcherClass: ZaSearch,
            dataFetcherMethod: ZaSearch.prototype.dynSelectGrantees,
            keyUp: function(value, event, form) {
              if (value instanceof ZaItem) {
                this.setInstanceValue(value.name);
              } else {
                this.setInstanceValue(value);
              }
            },
            editable: true
          },
          {
            type: _CHECKBOX_,
            labelCssClass: 'ZxCheckboxLabel',
            visibilityChecks: [function() { return true; }],
            enableDisableChecks: [function() { return true; }],
            label: zxMsg('sendAs') + ': ',
            ref: ZxACLEditDialog.G_sendAs,
            id: ZxACLEditDialog.G_sendAs
          },
          {
            type: _CHECKBOX_,
            labelCssClass: 'ZxCheckboxLabel',
            visibilityChecks: [function() { return true; }],
            enableDisableChecks: [function() { return true; }],
            label: zxMsg('sendOnBehalfOf') + ': ',
            ref: ZxACLEditDialog.G_sendOnBehalfOf,
            id: ZxACLEditDialog.G_sendOnBehalfOf
          }
        ]
      }
    ]
  };

  return xFormObject;
};
