/*
 * ***** BEGIN LICENSE BLOCK *****
 * Copyright (C) 2011-2016 ZeXtras
 *
 * The contents of this file are subject to the ZeXtras EULA;
 * you may not use this file except in compliance with the EULA.
 * You may obtain a copy of the EULA at
 * http://www.zextras.com/zextras-eula.html
 * ***** END LICENSE BLOCK *****
*/

function ZxACLListView(parent, className, posStyle, headerList) {
  if (arguments.length === 0) { return; }
  className = className || null;
  posStyle = posStyle || DwtControl.STATIC_STYLE;
  headerList = headerList || ZxACLListView._getHeaderList();

  ZaListView.call(this, parent, className, posStyle, headerList);
  this._appCtxt = this.shell.getData(ZaAppCtxt.LABEL);
  this.setMultiSelect(false);
}

ZxACLListView.prototype = new ZaListView();
ZxACLListView.prototype.constructor = ZxACLListView;

ZxACLListView.A_target = 'target';
ZxACLListView.A_target_id = 'target_id';
ZxACLListView.A_target_type = 'target_type';
ZxACLListView.A_target_img = 'target_img';

ZxACLListView.G_sendAs = 'sendAs';
ZxACLListView.G_sendOnBehalfOf = 'sendOnBehalfOf';

ZxACLListView.A_right = 'right';
ZxACLListView.A_grantee_img = 'grantee_img';
ZxACLListView.A_grantee = 'grantee';
ZxACLListView.A_grantee_id = 'grantee_id';
ZxACLListView.A_readableGrant = 'rightReadable';
ZxACLListView.ZX_grantsList = 'zxGrantsList';
ZxACLListView.A2_grantsListSelectedItems = 'grantsListSelectedItems';


ZxACLListView.prototype.toString = function() {
  return 'ZxACLListView';
};

ZxACLListView.prototype.getTitle = function() {
  return zxMsg('LBL_Manage_Global_Grants');
};

ZxACLListView.prototype.getTabIcon = function() {
  return 'GlobalPermission';
};

ZxACLListView.prototype._createItemHtml = function(grant, now, isDragProxy) {
  if (grant === null) { return; }

  var html = [],
    div = document.createElement('div'),
    idx = 0;

  div[DwtListView._STYLE_CLASS] = 'Row';
  div[DwtListView._SELECTED_STYLE_CLASS] = div[DwtListView._STYLE_CLASS] + '-' + DwtCssStyle.SELECTED;
  div.className = div[DwtListView._STYLE_CLASS];
  this.associateItemWithElement(grant, div, DwtListView.TYPE_LIST_ITEM);

  html[idx++] = '<table width="100%" cellspacing="0" cellpadding="0">';
  html[idx++] = '<tr>';
  var cnt = this._headerList.length;
  for (var i = 0; i < cnt; i++) {
   var field = this._headerList[i]._field;
   var cellWidth = this._getCellWidth(i, {});
   html[idx++] = '<td align="left" width=' + cellWidth + '><nobr>';
   if (field == ZxACLListView.A_right) {
     html[idx++] = grant[field];
   } else if (field != null) {

     var value = grant[field];

     if (field === ZxACLListView.A_grantee) {
       if (value == null || value.length < 0) {
         value = grant[ZxACLListView.A_grantee_id];
       }
     }
     if (field === ZxACLListView.A_target) {
       if (value == null || value.length < 0) {
         value = grant[ZxACLListView.A_target];
       }
     }
     if (field === ZxACLListView.A_grantee_img || field === ZxACLListView.A_target_img) {
       html[idx++] = AjxImg.getImageHtml(value);
       continue;
     }

     html[idx++] = AjxStringUtil.htmlEncode(value);
   }

    html[idx++] = '</nobr></td>';
  }
  html[idx++] = '</tr></table>';
  div.innerHTML = html.join('');
  return div;
};

ZxACLListView._getHeaderList = function() {
  var headerList = [];

  headerList.push(new ZaListHeaderItem(ZxACLListView.A_grantee_img, '', null,
    '16px', null, ZxACLListView.A_grantee_img, true, false));
  headerList.push(new ZaListHeaderItem(ZxACLListView.A_grantee, 'Grantee', null,
    '250px', true, ZxACLListView.A_grantee, true, true));
  headerList.push(new ZaListHeaderItem(ZxACLListView.A_readableGrant, 'Right', null,
    '*', true, ZxACLListView.A_readableGrant, true, true));

  return headerList;
};

ZxACLListView._getOptButtons = function() {
  var array = [];

  array.push(new ZxListViewButton(
    'Add',
    ZaMsg.NAD_Add,
    ZxACLListView.addButtonListener
  ));

  array.push(new ZxListViewButton(
    'Edit',
    ZaMsg.Edit_perms_title,
    ZxACLListView.editButtonListener
  ));

  array.push(new ZxListViewButton(
    'Delete',
    ZaMsg.TBB_Delete,
    ZxACLListView.deleteButtonListener
  ));

  array.push(new ZxListViewButton(
    'help',
    ZaMsg.TBB_Help,
    ZxACLListView.helpButtonListener
  ));

  return array;
};

ZxACLListView.isDeleteEnabled = function() {
  var grantListItem = this.getForm().getItemsById(ZxACLListView.ZX_grantsList)[0];
  if (typeof ZeXtras !== 'undefined' &&
      grantListItem && grantListItem.getSelection().length > 0) {
    return true;
  } else {
    return false;
  }
};

ZxACLListView.isEditEnabled = ZxACLListView.isDeleteEnabled;

ZxACLListView.onSelectionListener = function(ev) {
  this.getModel().setInstanceValue(
    this.getForm().getInstance(),
    ZxACLListView.A2_grantsListSelectedItems,
    this.getSelection()
  );
};

ZxACLListView.addButtonListener = function(ev, selection) {
  var newGrant = {},
    dialog;

  newGrant[ZxACLListView.A_target] = this.getInstance().name;
  newGrant[ZxACLListView.A_target_id] = this.getInstance().id;
  newGrant[ZxACLListView.A_target_type] = this.getInstance().type;
  newGrant[ZxACLListView.G_sendAs] = false;
  newGrant[ZxACLListView.G_sendOnBehalfOf] = false;

  dialog = new ZxACLEditDialog(
    this.getInstance(),
    newGrant,
    false
  );
  dialog.popup();
};

ZxACLListView.editButtonListener = function(ev, selection) {
  if (!selection || selection.length !== 1) { return; }

  var dialog = new ZxACLEditDialog(
    this.getInstance(),
    selection[0],
    true
  );
  dialog.popup();
};

ZxACLListView.deleteButtonListener = function(ev, selection) {
  var dialog,
    rightsToRemove = [];

  if (selection.length !== 1) { return; }

  dialog = new ZxACLEditDialog(
    this.getInstance(),
    selection[0],
    true
  );
  if (selection[0][ZxACLListView.G_sendAs]) {
    rightsToRemove.push(ZxACLListView.G_sendAs);
  }
  if (selection[0][ZxACLListView.G_sendOnBehalfOf]) {
    rightsToRemove.push(ZxACLListView.G_sendOnBehalfOf);
  }
  dialog.popupRevokeRights(selection[0], rightsToRemove);
};

ZxACLListView.helpButtonListener = function() {
  ZxUtils.openURL(ZxLinks.HELP_ACL);
};

ZxACLListView.prototype._sortColumn = function(columnItem, bSortAsc) {
  this._list.sort(
    AjxCallback.simpleClosure(
      function(col, asc, a, b) {
        if (!a.hasOwnProperty(col) || !b.hasOwnProperty(col)) {
          return 0;
        }
        if (a[col] < b[col]) {
          return (asc) ? -1 : 1;
        }
        if (a[col] > b[col]) {
          return (asc) ? 1 : -1;
        }
        return 0;
      },
      this,
      columnItem.getSortField(),
      bSortAsc
    )
  );
  this.setUI();
};
