/*
 * ***** BEGIN LICENSE BLOCK *****
 * Copyright (C) 2011-2016 ZeXtras
 *
 * The contents of this file are subject to the ZeXtras EULA;
 * you may not use this file except in compliance with the EULA.
 * You may obtain a copy of the EULA at
 * http://www.zextras.com/zextras-eula.html
 * ***** END LICENSE BLOCK *****
 */

function ZxModule(modID, modName, icon) {
  if (arguments.length < 1) { return; }
  ZaItem.call(this, modID);
  this._init();
  this._modID = modID;
  this._publicProperties = {};
  this.attrs = {};
  this.attrs.properties = {};
  this.attrs[ZxModule.KEY_LICENSED] = ZxLicense.isModuleLicensed(this._modID, this.name);
  this.attrs[ZxModule.KEY_ENABLED_ON_STARTUP] = true;
  this.attrs[ZxModule.KEY_RUNNING] = true;

  this._controllers = {};
  this._restartMap = {};

  this._modName = modName;
  this._icon = icon;

  this._isOnline = true;
  this._isZxInstalled = true;
  this._isDifferentVersion = false;

  this._viewID = ZaZimbraAdmin.VIEW_INDEX++;
  this._hyperCombo = false;

  this.__errors = [];
  this.__lastErrorTime = new Date(0);

  this.registerLoadMethod(this.__loadModuleStatus, 'ZxModuleLoad');

  this._helpURL = ZxLinks.HELP_COR_Core; // Default value for help url of a module.
}

ZxModule.prototype = new ZaItem();
ZxModule.prototype.constructor = ZxModule;

/**
 * If an error appear with multiple errors, show the first error for n milliseconds.
 * @type {number}
 * @private
 */
ZxModule._ERROR_SILENCE_INTERVAL = 3 * 1000;

ZxModule.INDEX_MENU_ID = 'INDEX_MENU_ID';
ZxModule._OBJ_NAME = '_OBJ_NAME';
ZxModule._MOD_ID = '_MOD_ID';

ZxModule._INTERNAL_ID_LICENSE_ALERT = "alert_warning";
ZxModule._INTERNAL_ID_RUNNING_ALERT = "running_warning";
ZxModule._INTERNAL_ID_ENABLED_ON_STARTUP_ALERT = "enabled_on_startup_warning";

ZxModule.KEY_LICENSED = 'licensed';
ZxModule.MODEL_LICENSED = {type: _ENUM_, id: ZxModule.KEY_LICENSED, isZX: true,
  ref: 'attrs/' + ZxModule.KEY_LICENSED, choices: ZaModel.BOOLEAN_CHOICES1};
ZxModule.KEY_INTO_GRACE_LIMIT = 'isWithinGraceInterval';
ZxModule.MODEL_INTO_GRACE_LIMIT  = {type: _ENUM_, id: ZxModule.KEY_INTO_GRACE_LIMIT, isZX: true,
  ref: 'attrs/' + ZxModule.KEY_INTO_GRACE_LIMIT, choices: ZaModel.BOOLEAN_CHOICES1};
ZxModule.KEY_ENABLED_ON_STARTUP = 'ModuleEnabledAtStartup';
ZxModule.MODEL_ENABLED_ON_STARTUP = {type: _ENUM_, id: ZxModule.KEY_ENABLED_ON_STARTUP, isZX: true,
  ref: 'attrs/' + ZxModule.KEY_ENABLED_ON_STARTUP, choices: ZaModel.BOOLEAN_CHOICES1};
ZxModule.KEY_RUNNING = 'running';
ZxModule.MODEL_RUNNING = {type: _ENUM_, id: ZxModule.KEY_RUNNING, isZX: true,
  ref: 'attrs/' + ZxModule.KEY_RUNNING, choices: ZaModel.BOOLEAN_CHOICES1};

ZxModule._registeredLoadMethods = {};
ZxModule._registeredModifyMethods = {};

/**
 * Array that contains the names of the modules that will be associated with a single dummy server item (localhost).
 * @type {string[]}
 */
ZxModule.SINGLES = [];

/**
 * Array that contains the names of the modules that are incompatible with Zimbra NE.
 * @type {string[]}
 */
ZxModule.NE_INCOMPATIBLE = [];

/**
 * Handle the selection of the module by the menu.
 * ( With HyperComboTechnology ;) )
 * Hypercombo enable/disable special or secret functions, to enable/disable it click on the
 * menu item pressing 'ctrl-alt-shift'.
 * @param {DwtSelectionEvent} ev
 * @this ZaOverviewPanelController
 */
ZxModule.treeClickHandler = function(ev) {
  ZxUtils.log(ev, 'Clicked on module menu voice');
  var hyperCombo,
    itemID,
    itemName,
    item,
    modController,
    msg = '';

  itemName = ev.item.getData(ZxModule._OBJ_NAME);
  itemID = ev.item.getData(ZaOverviewPanelController._OBJ_ID);

  if (itemName === ZeXtrasItem._MAIN_VIEW_NAME) {
    // Clicked the main menu ZeXtras Voice


  } else if (itemName === ZeXtrasItem._SERVER_LIST_NAME) {
    // Clicked on ZeXtras menu voice, but we need to choose the server
    if (ZaApp.getInstance().getCurrentController()) {
      modController = new ZxServersListController(
        this._appCtxt,
        this._appCtxt.getShell(),
        ZeXtrasItem._SERVER_LIST_NAME,
        itemID
      );
    }

  } else {
    // Clicked Module menu voice
    hyperCombo = (ev.altKey && ev.ctrlKey && ev.shiftKey);
    item = ZeXtras.getModuleItemList(itemName).getItemById(itemID);

    if (ZaApp.getInstance().getCurrentController()) {
      if (ZeXtras.isMultiServer()) {
        modController = item.getController(
          item.getViewID()
        );
      } else {
        // If is not a multi server, load the first (and i hope the unique) server.
        modController = item.getController(
          item.getViewID(),
          ZaApp.getInstance().getMbsList().getArray()[0]
        );
      }
    }
  }

  if (!item && !modController) {
    ZxUtils.log(['Item', itemName, '(', itemID, ') not found.'].join(' '));
    return;
  }

  if (item && hyperCombo) {
    item._hyperCombo = !item._hyperCombo;
    ZxUtils.log(
      [((item._hyperCombo) ? 'Enabled' : 'Disabled'), 'for', itemName, '(', itemID, ')'].join(' '),
      'HyperCombo Mode'
    );
  }

  if (typeof item !== 'undefined' &&
      !(ZeXtras.isServerOnline(item.getServerName()) && item.isServerSameZxVersion() && item.isZxInstalled())) {

    if (!ZeXtras.isServerOnline(item.getServerName())) {
      msg = zxMsg('MSG_DLG_Server_Offline');
    } else if (!item.isZxInstalled()) {
      msg = zxMsg('MSG_DLG_Server_ZeXtras_not_installed');
    } else if (!item.isServerSameZxVersion()) {
      var list = ['<ul>'],
        itmList = ZeXtras.getModuleItemList(item.getID()).getArray(),
        i,
        itm;
      for (i = 0; i < itmList.length; i += 1) {
        itm = itmList[i];
        if (itm.isServerSameZxVersion()) {
          list.push('<li>' + itm.getServerName() + '</li>');
        }
      }
      list.push('</ul>');
      msg = zxMsg('MSG_DLG_Server_ZxVersion_Mismatch', ['<b>' + item.getServerName() + '</b>', list.join('')]);
    }

    ZaApp.getInstance().getCurrentController().popupErrorDialog(msg);
    return;
  }
  if (ZaApp.getInstance().getCurrentController()) {
    ZaApp.getInstance().getCurrentController().switchToNextView(
      modController,
      modController.show,
      item || {}
    );
  }
};

ZxModule.prototype.injectConstructors = function(legacyConnector) {
  this.mLegacyConnector = legacyConnector;
};

ZxModule.prototype.getLegacyConnector = function() {
  return this.mLegacyConnector;
};

/**
 * If a module has defined a function to be called on init @see ZxModule.prototype.onInit()
 * will be called when the module item is initialized.
 * @param {{}} obj Item informations
 * @override ZaItem.prototype.initFromJS
 */
ZxModule.prototype.initFromJS = function(obj) {
  if (!obj) { return; }
  ZaItem.prototype.initFromJS.call(this, obj);

  if (typeof this.onInit === 'function') {
    this.onInit(obj);
  }
};

/**
 * Dummy function which need to be overridden from the modules which need an initliazation.
 * @param {{}} obj Item informations
 */
ZxModule.prototype.onInit = function(obj) { /* I'm a potato. */ };

/**
 * Map used to select the display order, @see ZxModule.sortFunction
 */
ZxModule._sortOrder = {
  ZxMig: 0,
  ZxCore: 0,
  ZxBackup: 1,
  ZxMobile: 2,
  ZxPowerstore: 3,
  ZxAdmin: 4,
  ZxChat: 5,
  ZxNotifications: 99
};

/**
 * Sort function used to select the diplay order of the menu voices of
 * the modules.
 * @param {ZxModule} a
 * @param {ZxModule} b
 * @return {number}
 */
ZxModule.sortFunction = function(a, b) {
  return (ZxModule._sortOrder[a] < ZxModule._sortOrder[b]) ?
      -1 : ((ZxModule._sortOrder[a] > ZxModule._sortOrder[b]) ? 1 : 0);
};

/**
 * Manage an error occurred during the SOAP communication.
 * @param {ZxError} err
 */
ZxModule.prototype.handleServerError = function(err) {
  ZxUtils.log(err, 'Error on request [' + this.getName() + ']', ZxUtils.LOG_WARNING);

  switch (err.getCode()) {
    case ZxErrorCode.UNABLE_TO_COMMUNICATE_WITH_SERVER:
    case ZxErrorCode.UNABLE_TO_PROXY_REQUEST:
      // Different Version
      this._isDifferentVersion = true;
      break;

    case ZxErrorCode.UNABLE_TO_CONNECT_TO_SERVER:
      // Server offline
      var server = ZxUtils.getServerIDByName(this.getServerName());
      if (!ZxUtils.isZimbraRunning(server)) {
        this._isOnline = false;
        if (ZeXtras) {
          ZeXtras.declareServerOffline(this.name);
        }
      } else {
        this._isZxInstalled = false;
      }
      break;
  }
};

/**
 * Notify the correct SOAP communication.
 */
ZxModule.prototype.handleServerOnline = function() {
//  ZxUtils.log('Request OK.', 'Server online on request [' + this.getName() + ']');
  this._isDifferentVersion = false;
  this._isOnline = true;
  this._isZxInstalled = true;
  if (ZeXtras) {
    ZeXtras.declareServerOnline(this.name);
  }
};

/**
 * Get if the server associated to the module instance is online.
 * @return {boolean}
 */
ZxModule.prototype.isServerOnline = function() {
  return this._isOnline;
};

/**
 * Get if the server associated to the module has the same version of current server.
 * @return {boolean}
 */
ZxModule.prototype.isServerSameZxVersion = function() {
  var itemLists = ZeXtras.getModuleListsItems(),
    modName,
    modItem,
    versionDiffer = this._isDifferentVersion;

  for (modName in itemLists) {
    if (!itemLists.hasOwnProperty(modName)) { continue; }
    modItem = ZeXtras.getModuleItem(modName, this.name);
    if (!modItem) { continue; }
    versionDiffer = versionDiffer || modItem._isDifferentVersion;
  }

  return !versionDiffer;
};

/**
 * Get if the ZeXtras or Mig is installed.
 * @return {boolean}
 */
ZxModule.prototype.isZxInstalled = function() {
  return this._isZxInstalled;
};

/**
 * Cheks used to enable or disable the module view, based on license information this check is pushed into the
 * @see XFormItem.enableDisableChecks array.
 * @return {boolean}
 * @this Group_XFormItem
 */
ZxModule.licenseCheck = function() {
  return this.getInstanceValue('attrs/' + ZxModule.KEY_LICENSED);
};

/**
 * Cheks used to enable or disable the module view, based on enabled this check is pushed into the
 * @see XFormItem.enableDisableChecks array.
 * @return {boolean}
 * @this Group_XFormItem
 */
ZxModule.isModuleEnabledOnStartupCheck = function() {
  return this.getInstanceValue('attrs/' + ZxModule.KEY_ENABLED_ON_STARTUP);
};

/**
 * Cheks used to enable or disable the module view, based on enabled this check is pushed into the
 * @see XFormItem.enableDisableChecks array.
 * @return {boolean}
 * @this Group_XFormItem
 */
ZxModule.isModuleRunningCheck = function() {
  return this.getInstanceValue('attrs/' + ZxModule.KEY_RUNNING);
};

/**
 * Get the module name.
 * @return {string}
 */
ZxModule.prototype.getName = function() {
  return this._modName;
};

/**
 * Get the module icon.
 * @return {string}
 */
ZxModule.prototype.getIcon = function() {
  return this._icon;
};

/**
 * Get the module ID.
 * @return {string}
 */
ZxModule.prototype.getID = function() {
  return this._modID;
};

/**
 * Get the module view ID.
 * @return {string|number}
 */
ZxModule.prototype.getViewID = function() {
  return this._viewID;
};

/**
 * Register a function which will be called on loading attributes.
 * @param {function} fcn
 */
ZxModule.prototype.registerLoadMethod = function(fcn, stage) {
  if (typeof fcn !== 'function') { throw new TypeError('[fcn] must be a function'); }
  if (!ZaItem.loadMethods.hasOwnProperty(this._modID)) {
    ZaItem.loadMethods[this._modID] = [];
  }

  if (!ZxModule._registeredLoadMethods.hasOwnProperty(this._modID)) {
    ZxModule._registeredLoadMethods[this._modID] = {};
  }
  if (!ZxModule._registeredLoadMethods[this._modID].hasOwnProperty(stage)) {
    ZaItem.loadMethods[this._modID].push(fcn);
    ZxModule._registeredLoadMethods[this._modID][stage] = true;
  }
};

/**
 * Trigger the load methods to be executed.
 */
ZxModule.prototype.fireLoadMethods = function() {
  this.load(null, null, !ZeXtrasItem.isUserAdmin());
};

/**
 * Register a function which will be called on modify attributes.
 * @param {function} fcn
 */
ZxModule.prototype.registerModifyMethod = function(fcn, stage) {
  if (typeof fcn !== 'function') { throw new TypeError('[fcn] must be a function'); }
  if (!ZaItem.modifyMethods.hasOwnProperty(this._modID)) {
    ZaItem.modifyMethods[this._modID] = [];
  }

  if (!ZxModule._registeredModifyMethods.hasOwnProperty(this._modID)) {
    ZxModule._registeredModifyMethods[this._modID] = {};
  }
  if (!ZxModule._registeredModifyMethods[this._modID].hasOwnProperty(stage)) {
    ZaItem.modifyMethods[this._modID].push(fcn);
    ZxModule._registeredModifyMethods[this._modID][stage] = true;
  }
};

/**
 * Register a function which will be called on loading attributes.
 * @param {function} fcn
 */
ZxModule.prototype.registerExtraToolbar = function(fcn) {
  if (typeof fcn !== 'function') { throw new TypeError('[fcn] must be a function'); }

  if (!ZxUtils.isZ8Up()) {
    if (!ZaController.initToolbarMethods[this._modID]) {
      ZaController.initToolbarMethods[this._modID] = [];
    }
    ZaController.initToolbarMethods[this._modID].push(fcn);
  }
};

/**
 * Register a function which will be called on loading attributes.
 * @param {function} fcn
 */
ZxModule.prototype.registerPopupMenu = function(fcn) {
  if (typeof fcn !== 'function') { throw new TypeError('[fcn] must be a function'); }
  if (!ZaController.initPopupMenuMethods[this._modID]) {
    ZaController.initPopupMenuMethods[this._modID] = [];
  }
  ZaController.initPopupMenuMethods[this._modID].push(fcn);
};

/**
 * Get the DWT Shell of the current app.
 * @return {DwtShell}
 */
ZxModule.prototype.getShell = function() {
  return ZaApp.getInstance().getAppCtxt().getShell();
};

/**
 * No need to load effective rights for real
 * @param {string} by
 * @param {string} val
 * @param {boolean} expandDefaults
 */
ZxModule.prototype.loadEffectiveRights = function(by, val, expandDefaults) {
};

/**
 * Get the current appCtxt.
 * @return {ZaAppCtxt}
 */
ZxModule.prototype.getAppCtxt = function() {
  return ZaApp.getInstance().getAppCtxt();
};

/**
 * Callback invoked when are available
 * @param {ZxLicense} license License.
 */
ZxModule.prototype._notifyLicenseChanges = function(license) {
  ZxUtils.log(
    license,
    'Notifying License informations to ' + this.getID() + ' on ' + this.getServerName()
  );

  var controller = this.getCurrentController();

  this.attrs[ZxModule.KEY_LICENSED] = license.isModuleLicensed(this.getID(), this.getCurrentServer());
  this.attrs[ZxModule.KEY_INTO_GRACE_LIMIT] = license.isWithinGraceInterval(this.getCurrentServer());

  if (controller) {
    controller.setInstanceValue(
      'attrs/' + ZxModule.KEY_LICENSED,
      this.attrs[ZxModule.KEY_LICENSED]
    );
    controller.setInstanceValue(
        'attrs/' + ZxModule.KEY_INTO_GRACE_LIMIT,
      this.attrs[ZxModule.KEY_INTO_GRACE_LIMIT]
    );
  }

  if (this._onLicenseInfo) {
    this._onLicenseInfo(license);
  }
};

/**
 * Get the controller associated to the view of the module.
 * @param {number} viewId
 * @return {ZxModuleController}
 */
ZxModule.prototype.getController = function(viewId) {
  if (viewId && this._controllers[viewId]) {
    return this._controllers[viewId];
  } else {
    this._controllers[viewId] = new ZxModuleController(this.getAppCtxt(), this.getAppCtxt().getShell(), this);
    //c.addChangeListener(new AjxListener(this.getMTAListController(), ZxModuleController.prototype.handleMTAChange));
    //c.addChangeListener(new AjxListener(c, ZaMTAController.prototype.handleMTAChange));
    return this._controllers[viewId];
  }
};

/**
 * Get the controller of the current view.
 * @return {ZxModuleController}
 */
ZxModule.prototype.getCurrentController = function() {
  return this.getController(this.getViewID());
};

/**
 * Return the XForm item associated to the main view of the
 * module.
 * @return {{}}
 */
ZxModule.prototype.getMyXFormModule = function() {
  var xForm = {
      items: []
    },
    pushWarningLicense = true,
    pushWarningRunning = true,
    pushWarningEnabledAtStartup = true,
    node,
    xFormType,
    xFormName,
    getLicenseTextClosure,
    getStoppedOrDisabledTextClosure,
    getStoppedOrDisabledShowCoreButtonClosure,
    jumpToCoreViewClosure,
    isFreeModule,
    i;

  if (ZeXtrasItem.isUserAdmin()) {
    xFormType = ZxModuleViewConstructor._ITEM;
  } else {
    xFormType = ZxModuleViewConstructor._ITEM_DELEGATED;
  }
  xFormName = this.getID() + xFormType;

  if (window[xFormName] && this._xForm) {
    return this._xForm;
  }

  if (!window[xFormName]) {
    ZxUtils.log('Please define ' + xFormName, 'getMyXform Error', ZxUtils.LOG_ERROR);
  } else {
    xForm.items = window[xFormName];
  }

  for (i = 0; i < xForm.items.length; i += 1) {
    node = xForm.items[i];
    if (!node.hasOwnProperty('_zxInternalId')) { continue; }

    switch (node._zxInternalId) {
      case ZxModule._INTERNAL_ID_LICENSE_ALERT:
        pushWarningLicense = false;
        break;
      case ZxModule._INTERNAL_ID_RUNNING_ALERT:
        pushWarningRunning = false;
        break;
      case ZxModule._INTERNAL_ID_ENABLED_ON_STARTUP_ALERT:
        pushWarningEnabledAtStartup = false;
        break;
    }
  }

  if (!pushWarningLicense && !pushWarningRunning && !pushWarningEnabledAtStartup) {
    this._xForm = xForm;
    return xForm;
  }

  getLicenseTextClosure = AjxCallback.simpleClosure(
    function() {
      if (ZeXtrasItem.isUserAdmin()) {
        return ZeXtras.getLicense().getWarningText(this.getServerName(), this.getID(), this.getName());
      } else {
        if (!this.attrs[ZxModule.KEY_LICENSED]) {
          return zxMsg('MSG_LIC_Unlicensed_delegated');
        } else {
          return '';
        }
      }
    },
    this
  );
  getStoppedOrDisabledTextClosure = AjxCallback.simpleClosure(
    function() {
      var completeMessage = [];

      if (ZxUtils.indexOf(ZxModule.NE_INCOMPATIBLE, this.getID()) > -1 && ZaSettings.isNetworkVersion()) {
        return zxMsg(
          'MSG_MOD_Disabled_Newtork',
          [
            this.getName(),
            '<a href="' + ZxLinks.MSG_Module_Disabled_on_NE + '" target="_blank">' +
            ZxLinks.MSG_Module_Disabled_on_NE + '</a>'
          ]
        );
      }

      if (!(this.attrs[ZxModule.KEY_RUNNING] === true)) {
        if (ZxUtils.amIDelegated()) {
          return zxMsg('MSG_MOD_Not_running_for_delegated', [this.getName()]);
        } else {
          completeMessage.push(zxMsg('MSG_MOD_Not_running'));
        }
      }
      if (!(this.attrs[ZxModule.KEY_ENABLED_ON_STARTUP] === true)) {
        completeMessage.push(zxMsg('MSG_MOD_Disabled_Preferences'));
      }

      return zxMsg('MSG_MOD_Status_base', [
        this.getName(),
        completeMessage.join(' ' + zxMsg('LBL_and') + ' ')
      ]);
    },
    this
  );
  getStoppedOrDisabledShowCoreButtonClosure = AjxCallback.simpleClosure(
    function() {
      if (ZxUtils.amIDelegated()) {
        return false;
      }
      if (ZxUtils.indexOf(ZxModule.NE_INCOMPATIBLE, this.getID()) > -1 && ZaSettings.isNetworkVersion()) {
        return false;
      }
      return true;
    },
    this
  );
  jumpToCoreViewClosure = AjxCallback.simpleClosure(
    function(ev) {
      var modController, item;

      item = ZeXtras.getModuleItem('ZxCore', this.getServerName());
      if (!item) { return; }
      modController = item.getCurrentController();
      if (!item) { return; }

      ZaApp.getInstance().getCurrentController().switchToNextView(
        modController,
        modController.show,
        item || {}
      );
    },
    this
  );

  if (ZeXtras.isMigZimlet()) {
    isFreeModule = function() { return true; };
  }
  else {
    isFreeModule = AjxCallback.simpleClosure(
      function(ev) {
        return this.getID() === ZxCore.MOD_ID ||
          this.getID() === ZxCore.KEY_LICENSE_ZXCHAT;
      },
      this
    );
  }

  for (i = 0; i < xForm.items.length; i += 1) {
    // Push the license check inside of the components of the view.

    if (i === 0) {
      // The first thing in the view is the header, allow some buttons to be always enabled,
      // Check if the property 'availableIfModuleDisabled' is true.
      this._searchAndEditDisableChecksOnButtons(xForm.items[i]);
    } else {
      this._alterFormItemDisableCheck(xForm.items[i]);
    }

  }
  // Push the warning block for License into the view.
  if (pushWarningLicense) {
    xForm.items.splice(
      1,
      0,
      {
        type: _ZX_ALERT_,
        _zxInternalId: ZxModule._INTERNAL_ID_LICENSE_ALERT,
        style: DwtAlert.WARNING,
        visibilityChangeEventSources: ['attrs/' + ZxModule.KEY_LICENSED],
        visibilityChecks: [function() {
          return !isFreeModule() && getLicenseTextClosure() !== '';
        }],
        items: [
          {
            type: _OUTPUT_,
            getDisplayValue: getLicenseTextClosure
          }
        ]
      }
    );
  }
  if (pushWarningRunning) {
    xForm.items.splice(
      1,
      0,
      {
        type: _ZX_ALERT_,
        _zxInternalId: ZxModule._INTERNAL_ID_RUNNING_ALERT,
        style: DwtAlert.WARNING,
        visibilityChangeEventSources: ['attrs/' + ZxModule.KEY_LICENSED, 'attrs/' + ZxModule.KEY_RUNNING, 'attrs/' + ZxModule.KEY_ENABLED_ON_STARTUP],
        visibilityChecks: [function() {
          return ZxModule.licenseCheck.call(this) && (!ZxModule.isModuleRunningCheck.call(this) || !ZxModule.isModuleEnabledOnStartupCheck.call(this));
        }],
        numCols: 2,
        items: [
          {
            type: _OUTPUT_,
            getDisplayValue: getStoppedOrDisabledTextClosure
          },
          {
            type: _DWT_BUTTON_,
            width: 100,
            icon: 'ZeXtras',
            visibilityChangeEventSources: ['attrs/' + ZxModule.KEY_RUNNING, 'attrs/' + ZxModule.KEY_ENABLED_ON_STARTUP],
            visibilityChecks: [getStoppedOrDisabledShowCoreButtonClosure],
            label: zxMsg('LBL_here'),
            onActivate: jumpToCoreViewClosure
          }
        ]
      }
    );
  }

  this._xForm = xForm;

  return xForm;
};

ZxModule.prototype._searchAndEditDisableChecksOnButtons = function(item) {
  if (item.type === _DWT_BUTTON_) {
    if (item.hasOwnProperty('availableIfModuleDisabled') && item.availableIfModuleDisabled === true) {
      return;
    }
    this._alterFormItemDisableCheck(item);
  }

  if (item.type === _GROUP_) {
    var i;
    for (i = 0; i < item.items.length; i += 1) {
      this._searchAndEditDisableChecksOnButtons(item.items[i]);
    }
  }
};

ZxModule.prototype._alterFormItemDisableCheck = function(item) {
  if (item.enableDisableChangeEventSources) {
    item.enableDisableChangeEventSources.push('attrs/' + ZxModule.KEY_LICENSED);
  }
  else {
    item.enableDisableChangeEventSources = ['attrs/' + ZxModule.KEY_LICENSED];
  }

  if (item.enableDisableChecks) {
    item.enableDisableChecks.push(ZxModule.licenseCheck);
  }
  else {
    item.enableDisableChecks = [ZxModule.licenseCheck];
  }
};

/**
 * Return the data model associated to the module.
 * @return {{}}
 */
ZxModule.prototype.getDataModelModule = function() {
  var dataModel = {};

  if (!window[this.getID() + 'Model']) {
    ZxUtils.log('Please define ' + this.getID() + 'Model');
    throw new Error(this.getID() + ' data model not defined');
  } else {
    dataModel = window[this.getID() + 'Model'];
    dataModel.push(ZxModule.MODEL_LICENSED);
    dataModel.push(ZxModule.MODEL_INTO_GRACE_LIMIT);
    dataModel.push(ZxModule.MODEL_ENABLED_ON_STARTUP);
    dataModel.push(ZxModule.MODEL_RUNNING);
  }
  return dataModel;
};

/**
 * Switch to the server view for this module.
 * @param {ZaServer} server
 */
ZxModule.prototype.switchToNextView = function(server) {
  var modController = this.getController(
    this.getViewID(),
    server
  );

  ZaApp.getInstance().getCurrentController().switchToNextView(
    modController,
    modController.show,
    this
  );
};

/**
 * Get the server name associated to the item.
 * @return {string}
 */
ZxModule.prototype.getServerName = function() {
  return this.name;
};

/**
 * Get the current server.
 * @return {string}
 */
ZxModule.prototype.getCurrentServer = function() {
  return this.name;
};

/**
 * Enable or disable the module (setting the property).
 * @param {boolean} enable
 * @param {AjxCallback.simpleClosure} callback
 */
ZxModule.prototype.__doEnableDisableModuleOnStartup = function(enable, callback) {
  var opts = ZxUtils.createObjectFromRef('properties/' + this._modID + '_ModuleEnabledAtStartup', enable),
    dialog,
    enableDisableCallback,
    closeDialogCallback;

  dialog = new DwtMessageDialog({
    parent: this.getShell(),
    buttons: [DwtDialog.YES_BUTTON, DwtDialog.NO_BUTTON]
  });
  dialog.setMessage(
    zxMsg('MSG_MOD_Sure_want_disable_module', [this.getName()]),
    DwtMessageDialog.WARNING_STYLE,
    ZxUtils.getTitleWithIcon(this.getName())
  );

  closeDialogCallback = AjxCallback.simpleClosure(
    function(enable, dialog, callback, resp) {
      this.fireLoadMethods();
      if (callback) {
        callback();
      }
      if (!enable) {
        dialog.popdown();
      }
    },
    this,
    enable,
    dialog,
    callback
  );

  enableDisableCallback = AjxCallback.simpleClosure(
    function() {
      ZxAdminRequest.sendRequest(
        this.getCurrentServer(),
        this._modID,
        'setPublicProperties',
        opts
      ).then(
        closeDialogCallback,
        ZxUtils.catchError
      );
    },
    this
  );

  dialog.registerCallback(
    DwtDialog.YES_BUTTON,
    new AjxCallback(
      this,
      enableDisableCallback
    ),
    []
  );

  dialog.registerCallback(
    DwtDialog.NO_BUTTON,
    new AjxCallback(
      this,
      closeDialogCallback
    ),
    []
  );

  if (!enable) {
    dialog.popup();
  } else {
    enableDisableCallback();
  }
};

/**
 * Start or stop the module.
 * @param {boolean} start
 * @param {AjxCallback.simpleClosure} callback
 */
ZxModule.prototype.__doStartStopModule = function(start, callback) {
  var dialog,
    enableDisableCallback,
    command = start ? 'doStartService' : 'doStopService';

  dialog = new DwtMessageDialog({
    parent: this.getShell(),
    buttons: [DwtDialog.YES_BUTTON, DwtDialog.NO_BUTTON]
  });
  dialog.setMessage(
    zxMsg('MSG_MOD_Sure_want_stop_module', [this.getName()]),
    DwtMessageDialog.WARNING_STYLE,
    ZxUtils.getTitleWithIcon(this.getName())
  );

  enableDisableCallback = AjxCallback.simpleClosure(
    function(command, enable, dialog, callback) {
      ZxAdminRequest.sendRequest(
        this.getCurrentServer(),
        this._modID,
        command,
        {
          service_name: 'module'
        }
      ).then(
        AjxCallback.simpleClosure(
          function(start, dialog, callback, resp) {
            this.fireLoadMethods();
            if (!start && dialog) {
              dialog.popdown();
            }
            if (callback) {
              callback();
            }
          },
          this,
          enable,
          dialog,
          callback
        ),
        AjxCallback.simpleClosure(
          function(callback, err) {
            this.fireLoadMethods();
            if (callback) {
              callback();
            }
            dialog = new DwtMessageDialog({
              parent: this.getShell()
            });
            if (err.getCode() === ZxErrorCode.UNABLE_TO_START_UNNECESSARY_SERVICE) {
              dialog.setMessage(
                err.getMessage(),
                DwtMessageDialog.WARNING_STYLE,
                ZxUtils.getTitleWithIcon(this.getName())
              );
              dialog.popup();
            } else if (err.getCode() === ZxErrorCode.UNABLE_TO_START_MANDATORY_SERVICE) {
              ZxUtils.catchError(err);
            } else if (err.getCode() === ZxErrorCode.UNABLE_TO_START_SERVER_MODULE) {
              ZxUtils.catchError(err);
            } else {
              ZxUtils.catchError(err);
            }
          },
          this,
          callback
        )
      );
    },
    this,
    command,
    start,
    dialog,
    callback
  );

  dialog.registerCallback(
    DwtDialog.YES_BUTTON,
    new AjxCallback(
      this,
      enableDisableCallback
    ),
    []
  );

  if (!start) {
    dialog.popup();
  } else {
    enableDisableCallback(command, start);
  }
};

/**
 * Restart a service of the module.
 * @param {string} service
 * @return {Q.promise}
 */
ZxModule.prototype.__doRestartService = function(service) {
  return ZxAdminRequest.sendRequest(
    this.getCurrentServer(),
    this._modID,
    'doRestartService',
    {
      service_name: service
    }
  );
};

ZxModule.prototype.__loadModuleStatus = function(by, val) {
  if (this._modID !== 'ZxNotifications' && this._modID !== 'ZxMig') {
    ZxAdminRequest.sendRequest(
      this.getCurrentServer(),
      this._modID,
      'getServices'
    ).then(
      AjxCallback.simpleClosure(
        function(resp) {
          var services = resp.response.services,
            modStatus = services.module,
            controller = this.getCurrentController();

          this.attrs[ZxModule.KEY_RUNNING] = modStatus.running;
          if (this._modID === 'ZxCore' || this._modID === 'ZxMig' || this._modID === 'ZxNotifications') {
            // The Core/Mig/Notifications are always enabled
            this.attrs[ZxModule.KEY_RUNNING] = true;
          }

          if (controller) {
            controller.setInstanceValue(
              'attrs/' + ZxModule.KEY_RUNNING,
              this.attrs[ZxModule.KEY_RUNNING]
            );
          }
        },
        this
      ),
      this._getDefaultCatchErrorFcn()
    );

    ZxAdminRequest.sendRequest(
      this.getCurrentServer(),
      this._modID,
      'getPublicProperties'
    ).then(
      AjxCallback.simpleClosure(
        function(resp) {
          if (resp.response && resp.response.properties) {
            var controller = this.getCurrentController();

            this._publicProperties = resp.response.properties;
            if (resp.response.relatedServices) {
              this._restartMap = resp.response.relatedServices;
            }

            this.attrs[ZxModule.KEY_ENABLED_ON_STARTUP] =
              this._publicProperties[this._modID + '_ModuleEnabledAtStartup'] === 'true';
            if (this._modID === 'ZxCore' || this._modID === 'ZxMig' || this._modID === 'ZxNotifications') {
              // The Core/Mig/Notifications are always enabled
              this.attrs[ZxModule.KEY_ENABLED_ON_STARTUP] = true;
            }

            if (controller) {
              controller.setInstanceValue(
                'attrs/' + ZxModule.KEY_ENABLED_ON_STARTUP,
                this.attrs[ZxModule.KEY_ENABLED_ON_STARTUP]
              );
            }
          }
        },
        this
      ),
      this._getDefaultCatchErrorFcn()
    );
  }
};

/**
 * Get if a module is running.
 * @return {Boolean}
 */
ZxModule.prototype.isModuleRunning = function() {
  return this.attrs[ZxModule.KEY_RUNNING];
};

/**
 * Get if a module is enabled on startup.
 * @return {Boolean}
 */
ZxModule.prototype.isModuleEnabledOnStartup = function() {
  return this.attrs[ZxModule.KEY_ENABLED_ON_STARTUP];
};

/**
 * Method invoked when is asked to save some modified values.
 * @param {{}} mods
 * @this ZxModule
 */
ZxModule.prototype.modifyMethod = function(mods) {
  if (ZxUtils.isObjectEmpty(mods)) {
    return;
  }

  if (typeof mods.properties !== 'undefined') {
    var params = {}, props = [], tmp;
    params.properties = ZxUtils.cloneObj(mods.properties);

    for (tmp in params.properties) {
      if (!params.properties.hasOwnProperty(tmp)) { continue; }
      props.push(tmp);
//      props.push(tmp.substring(ZxUtils.indexOf(tmp, '_') + 1));
    }

    ZxAdminRequest.sendRequest(
      this.getCurrentServer(),
      (this._modID === 'ZxMig') ? 'ZxCore' : this._modID,
      'setPublicProperties',
      params
    ).then(
      AjxCallback.simpleClosure(
        function(props, resp) {
          var i, tmpArr, fcn;

          fcn = AjxCallback.simpleClosure(
            function(fcn, arr) {
              if (!arr || arr.length <= 0) { return; }

              this.__doRestartService(arr.shift()).then(
                AjxCallback.simpleClosure(
                  function(fcn, arr, resp) {
                    this.fireLoadMethods();
                    fcn(arr);
                  },
                  this,
                  fcn,
                  arr
                ),
                ZxUtils.catchError
              );
            },
            this
          );

          for (i = 0; i < props.length; i += 1) {
            if (this._restartMap.hasOwnProperty(props[i])) {
              tmpArr = this._restartMap[props[i]];
              fcn(fcn, tmpArr);
            }
          }

        },
        this,
        props
      ),
      ZxUtils.catchError
    );
  }
};

/**
 * Refresh the view of the module.
 */
ZxModule.prototype.refreshView = function() {
  this.fireLoadMethods();
};

/**
 * Get the default catch error function used in the requests.
 * @return {function}
 */
ZxModule.prototype._getDefaultCatchErrorFcn = function() {
  return AjxCallback.simpleClosure(
    this._catchLoadErrors,
    this
  );
};

/**
 * Basic function made to handle request error made by the module.
 * @param {ZxError|Error} err
 * @return {boolean} (true)
 */
ZxModule.prototype._catchLoadErrors = function(err) {
  if ((err.code && err.code === AjxException.NETWORK_ERROR) ||
      (typeof err.getCode === 'function' &&
        err.getCode() === ZxErrorCode.ZM_CSFE_EXCEPTION &&
        typeof err.getDetail === 'function' &&
        err.getDetail(ZxSOAPError.KEY_CODE) &&
        err.getDetail(ZxSOAPError.KEY_CODE) === ZmCsfeException.EMPTY_RESPONSE)) {

    // Ignore Empty response for ZxModule._ERROR_SILENCE_INTERVAL ms.
    if ((new Date() - this.__lastErrorTime) > ZxModule._ERROR_SILENCE_INTERVAL) {
      this.__lastErrorTime = new Date();
      this.__errors.push(err);
      ZaApp.getInstance().getCurrentController().popupWarningDialog(
        zxMsg('MSG_DLG_Server_Offline')
      );
    }

  } else {
    ZxUtils.log(err, 'Error during loading Module Values');
    this.__errors.push(err);
    ZxUtils.catchError(err);
  }
  return true;
};

ZxModule.hyperComboActivatedCheck = function() {
  var item = ZeXtras.getModuleItem(ZxCore, this.getForm().getInstance().id);
  return item._hyperCombo;
};
