/*
 * ***** BEGIN LICENSE BLOCK *****
 * Copyright (C) 2011-2016 ZeXtras
 *
 * The contents of this file are subject to the ZeXtras EULA;
 * you may not use this file except in compliance with the EULA.
 * You may obtain a copy of the EULA at
 * http://www.zextras.com/zextras-eula.html
 * ***** END LICENSE BLOCK *****
 */

function ZxModuleController(appCtxt, container, item) {
  ZaXFormViewController.call(this, appCtxt, container);

  this.tabConstructor = ZxModuleViewConstructor;
  this._toolbarOperations = [];
  this._popupOperations = [];
  this._iKeyName = item.getID();
  this._container = container;
  this.formChangeListener = new AjxListener(this, ZaXFormViewController.prototype.handleXFormChange, []);

  if (!ZaController.changeActionsStateMethods.hasOwnProperty(item.getID())) {
    ZaController.changeActionsStateMethods[item.getID()] = [];
  }
  if (!ZaController.initToolbarMethods.hasOwnProperty(item.getID())) {
    ZaController.initToolbarMethods[item.getID()] = [];
  }
  if (!ZaController.initPopupMenuMethods.hasOwnProperty(item.getID())) {
    ZaController.initPopupMenuMethods[item.getID()] = [];
  }
  if (!ZaController.setViewMethods.hasOwnProperty(item.getID())) {
    ZaController.setViewMethods[item.getID()] = [this.setViewMethod];
  }
  if (typeof ZaController.saveChangeCheckMethods !== 'undefined' &&
      !ZaController.saveChangeCheckMethods.hasOwnProperty(item.getID())) {
    ZaController.saveChangeCheckMethods[item.getID()] = [];
  }
  if (typeof ZaController.postChangeMethods !== 'undefined' &&
      !ZaController.postChangeMethods.hasOwnProperty(item.getID())) {
    ZaController.postChangeMethods[item.getID()] = [];
  }

  this._helpURL = item._helpURL;
}

ZxModuleController.prototype = new ZaXFormViewController();
ZxModuleController.prototype.constructor = ZxModuleController;

ZxModuleController.KEY_SERVER = 'server';

/**
 * Item to show
 * @param {ZxModule} item
 */
ZxModuleController.prototype.show = function(item) {
  this._item = item;
  this._setView(item, !ZxUtils.isZ8Up(), false);
};

ZxModuleController.prototype.setViewMethod = function(openInNewTab, skipRefresh) {
  this._item.load('id', this._item.id, true);
  if (!this._UICreated) {
    this._createUI();
  }
  this._currentObject = this._item;
  //ZaApp.getInstance().pushView(ZaZimbraAdmin._POSTQ_BY_SERVER_VIEW);
  ZaApp.getInstance().pushView(this.getContentViewId());
  this._view.setObject(this._item);
  this._view.setDirty(false);
  //setObject is delayed to be called after pushView in order to avoid jumping of the view
};

ZxModuleController.prototype._createUI = function() {
  var elements = {};
  this._contentView = this._view = new this.tabConstructor(this._container, this._item);

  this._view._localXForm.addListener(
    DwtEvent.XFORMS_VALUE_CHANGED,
    this.formChangeListener
  );

  if (typeof appNewUI === 'undefined' || !appNewUI) {
    // Zimbra 7
    this._initToolbar();

    this._toolbarOperations[ZaOperation.NONE] = new ZaOperation(ZaOperation.NONE);
    this._toolbarOperations[ZaOperation.HELP] = new ZaOperation(ZaOperation.HELP,
      ZaMsg.TBB_Help, ZaMsg.TBB_Help_tt,
      'Help', 'Help',
      new AjxListener(this, this._helpButtonListener));
    this._toolbarOrder.push(ZaOperation.NONE);
    this._toolbarOrder.push(ZaOperation.HELP);

    this._toolbarOperations[ZaOperation.REFRESH] =
      new ZaOperation(ZaOperation.REFRESH,
        'Refresh', 'Refresh',
        'Refresh', 'Refresh',
        new AjxListener(this, function() {
          this._item.load(null, null, !ZeXtrasItem.isUserAdmin());
        }));

    this._toolbarOperations[ZaOperation.SAVE] =
      new ZaOperation(ZaOperation.SAVE,
        'Save', 'Save',
        'Save', 'Save',
        new AjxListener(this, this.saveButtonListener));

    this._toolbarOrder.push(ZaOperation.SAVE);
    this._toolbarOrder.push(ZaOperation.REFRESH);

//    ZaController.initToolbarMethods[this.getContentViewId()].push(GenericTab.initToolbarMethod);

    this._toolbar = new ZaToolBar(this._container, this._toolbarOperations, this._toolbarOrder);
    elements[ZaAppViewMgr.C_TOOLBAR_TOP] = this._toolbar;
  } else {
    // Zimbra 8
    this._view._localXForm.addListener(
      DwtEvent.XFORMS_VALUE_CHANGED, new AjxListener(this, function(ev) {
        ZaZimbraAdmin.getInstance().getCurrentAppBar().enableButton(ZaOperation.SAVE, true);
      })
    );

    this._initPopupMenu();
  }

  elements[ZaAppViewMgr.C_APP_CONTENT] = this._view;

  if (typeof appNewUI !== 'undefined' && !appNewUI) {
    // Zimbra 8
    var openNewTab = ZeXtras.isMultiServer(),
      tabParams = {
        openInNewTab: openNewTab,
        tabId: this.getContentViewId(),
        tab: this.getMainTab()
      };
    /*
    view: ZaApp._currentModule.name
    */
    ZaApp.getInstance().createView(this.getContentViewId(), elements, tabParams);
  } else {
    // Zimbra 7
    ZaApp.getInstance().getAppViewMgr().createView(this.getContentViewId(), elements);
  }
  this._UICreated = true;
  ZaApp.getInstance()._controllers[this.getContentViewId()] = this;
};

/**
 * Set a value that force to update the XForm Item associated.
 * When a value i setted, the save button will be disabled.
 * ATTENTION: If the ref does not start with "attrs/", prefix will be
 * added!
 * @param {string} ref
 * @param {*} value
 */
ZxModuleController.prototype.setInstanceValue = function(ref, value) {
  if (ZxUtils.indexOf(ref, 'attrs/') === -1) {
    ref = ['attrs', ref].join('/');
  }

  var obj = ZxUtils.createObjectFromRef(ref, value),
    xformView;

  if (typeof this._item !== 'undefined') {
    this._item.attrs = ZxUtils.mergeObjects(this._item.attrs, obj.attrs);

    this._item.oldAttrs = ZxUtils.cloneObj(this._item.attrs);
  }

  if (typeof this._view !== 'undefined') {
    xformView = this._view._localXForm;
    xformView.setInstanceValue(value, ref);
    xformView.updateElementStates();
    this._view.setDirty(false);
    if (typeof appNewUI === 'undefined' || !appNewUI) {
      // Zimbra 7
      if (this._toolbarOperations[ZaOperation.SAVE]) {
        this._toolbarOperations[ZaOperation.SAVE].enabled = false;
      }
    } else {
      // Zimbra 8
      ZaZimbraAdmin.getInstance().getCurrentAppBar().enableButton(ZaOperation.SAVE, false);
    }
  }
};

/**
 * Method invoked by zimbra when is requested to save the modifications.
 * @return {boolean}
 */
ZxModuleController.prototype._saveChanges = function() {
  var item = this._currentObject,
    diffs = ZxUtils.mergeObjects(
      ZxUtils.objDiff(this._view._containedObject.attrs, this._currentObject.attrs),
      ZxUtils.objDiff(this._currentObject.attrs, this._currentObject.oldAttrs)
    );
  item.modify(diffs);
  return true;
};

/**
 * Return the shell of the controller view.
 * @return {DwtShell}
 */
ZxModuleController.prototype.getShell = function() {
  return this._shell;
};

/**
 * Get the item handled by the controller.
 * @return {ZxModule}
 */
ZxModuleController.prototype.getItem = function() {
  return this._item;
};
