/*
 * ***** BEGIN LICENSE BLOCK *****
 * Copyright (C) 2011-2016 ZeXtras
 *
 * The contents of this file are subject to the ZeXtras EULA;
 * you may not use this file except in compliance with the EULA.
 * You may obtain a copy of the EULA at
 * http://www.zextras.com/zextras-eula.html
 * ***** END LICENSE BLOCK *****
 */

function ZxServer() {}

ZxServer._SERVER_CACHE = {};
ZxServer._LAST_UPDATE = 0;

/**
 * Cache time in minutes.
 * @type {number}
 */
ZxServer.CACHE_TIME = 30;

/**
 * Return the server list, the information is cached {@see ZxServer.CACHE_TIME}
 * @return {ZaItemList}
 */
ZxServer.getAll = function() {
  var expired = (ZxServer._LAST_UPDATE + (ZxServer.CACHE_TIME * 60 * 1000)) < new Date();

  if (expired) {
    ZxServer._SERVER_CACHE = ZaServer.getAll();
    ZxServer._LAST_UPDATE = new Date();
  }

  return ZxServer._SERVER_CACHE;
};

/**
 * Select a server by name.
 * @param {string} name
 * @return {ZaServer|undefined}
 */
ZxServer.getServerByName = function(name) {
  var all = ZxServer.getAll().getArray(),
    i = 0;

  for (i = 0; i < all.length; i = i + 1) {
    if (all[i].name === name) {
      return all[i];
    }
  }
};

/**
 * Return if a server has the mailbox service active.
 * @param {string} serverName
 * @return {boolean}
 */
ZxServer.isMailbox = function(serverName) {
  var server = ZxServer.getServerByName(serverName);
  if (!server) { return false; }
  return ZxUtils.indexOf(server.attrs[ZaServer.A_zimbraServiceEnabled], 'mailbox', 0, true) > -1;
};
