/*
 * ***** BEGIN LICENSE BLOCK *****
 * Copyright (C) 2011-2016 ZeXtras
 *
 * The contents of this file are subject to the ZeXtras EULA;
 * you may not use this file except in compliance with the EULA.
 * You may obtain a copy of the EULA at
 * http://www.zextras.com/zextras-eula.html
 * ***** END LICENSE BLOCK *****
 */

/**
 *
 * @param {DwtShell} parent
 * @param {ZxServersListController} controller
 * @constructor
 */
function ZxServersListView(parent, controller) {
  if (arguments.length === 0) { return; }

  ZaListView.call(this, {
    parent: parent,
    className: 'ZxServersListView',
    posStyle: DwtControl.ABSOLUTE_STYLE,
    id: ZxServersListView.TAB_ID_ZX_SERVERS,
    headerList: this._getHeaderList()
  });

  this._appCtxt = this.shell.getData(ZaAppCtxt.LABEL);
  this._controller = controller;
}

ZxServersListView.prototype = new ZaListView();
ZxServersListView.prototype.constructor = ZxServersListView;

ZxServersListView.STYLE_CLASS = 'Row';
ZxServersListView.SELECTED_STYLE_CLASS = 'Row-' + DwtCssStyle.SELECTED;

ZxServersListView.KEY_ICON = 'icon';
ZxServersListView.KEY_MODULE = 'module';
ZxServersListView.KEY_SERVERS = 'server_array';

ZxServersListView.TAB_ID_ZX_SERVERS = 'ZxServerList';
ZxServersListView.PRFX_Server = 'server';
ZxServersListView.PRFX_Notifications = 'notifications';
ZxServersListView.PRFX_LicenseInfo = 'license';

ZxServersListView.MIN_ROW_HEIGHT = 58;

ZxServersListView.WIDTH_ICON = ZxServersListView.MIN_ROW_HEIGHT;
ZxServersListView.WIDTH_SERVER = 250;
ZxServersListView.WIDTH_NOTIFICATIONS = 250;
ZxServersListView.WIDTH_LICENSE = 'auto';

ZxServersListView.prototype.getController = function() {
  return this._controller;
};

/**
 *
 * @param {ZaServer} item
 * @return {number}
 * @this ZxServersListView
 */
ZxServersListView.prototype._createItemHtml = function(item, now, wtf, idx) {
  var module = ZeXtras.getModuleItem(this.getController().getModule(), item.attrs[ZaServer.A_name]),
    licenseInfo = ZeXtras.getLicense().getServerLicenseInfo(item.attrs[ZaServer.A_name]),
    modNot = ZeXtras.getModuleItem('ZxNotifications', item.attrs[ZaServer.A_name]),
    isOnline = ZeXtras.isServerOnline(item.attrs[ZaServer.A_name]) &&
      module.isServerSameZxVersion() &&
      module.isZxInstalled(),
    html = [],
//    idx = 0,
    i,
    div = document.createElement('div'),
    cnt = this._headerList.length;

  if (!AjxDateUtil._wordyDate) {
    AjxDateUtil._wordyDate = new AjxDateFormat(AjxMsg.formatWordyDate);
  }

  div[DwtListView._STYLE_CLASS] = 'Row ZxServerListViewRow';
  div[DwtListView._SELECTED_STYLE_CLASS] = div[DwtListView._STYLE_CLASS] + '-' + DwtCssStyle.SELECTED;
  div.className = div[DwtListView._STYLE_CLASS];
  this.associateItemWithElement(item, div, DwtListView.TYPE_LIST_ITEM);

  html[idx++] = '<table width="100%">';
  html[idx++] = '<tr>';
  for (i = 0; i < cnt; i += 1) {
    var field = this._headerList[i]._field;
    if (field === ZxServersListView.KEY_ICON) {
      // Icon
      html[idx++] = '<td width="' + this._headerList[i]._width + '">' +
        AjxImg.getImageHtml(isOnline ? 'ZxServerOnline' : 'ZxServerOffline') + '</td>';

    } else if (field === ZxServersListView.PRFX_Server) {
      // Info
      html[idx++] = '<td style="vertical-align:middle;" align="center" width=' + this._headerList[i]._width + '>';
      html[idx++] = '<b>' + AjxStringUtil.htmlEncode(item.attrs[ZaServer.A_ServiceHostname]) + '</b>';
      if (ZaItem.getDescriptionValue(item.attrs[ZaServer.A_description])) {
        html[idx++] = '<br>';
        html[idx++] = AjxStringUtil.htmlEncode(
          AjxStringUtil.clip(
            ZaItem.getDescriptionValue(item.attrs[ZaServer.A_description]),
            230
          )
        );
      }
      html[idx++] = '</td>';

    } else if (field === ZxServersListView.PRFX_Notifications) {
      // Notifications
      html[idx++] = '<td align="left" style="vertical-align:middle;" width="' + this._headerList[i]._width + '">';

      if (!isOnline) {
        if (ZeXtras.isServerOffline(item.attrs[ZaServer.A_name])) {
          html[idx++] = '<span style="color: #A00000; font-weight:bold;">' + zxMsg('MSG_Server_Offline') + '</span>';
        } else if (!module.isZxInstalled()) {
          html[idx++] = '<span style="color: #A00000; font-weight:bold;">' +
            zxMsg('MSG_Server_ZeXtras_not_installed') + '</span>';
        } else if (!module.isServerSameZxVersion()) {
          html[idx++] = '<span style="color: #A00000; font-weight:bold;">' + zxMsg('MSG_Server_ZxVersion_Mismatch') +
            '</span>';
        }
      } else {
        var redCount = modNot.getCountByLevel(ZxNotifications.SEVERITY_TO_CODE.Critical) +
            modNot.getCountByLevel(ZxNotifications.SEVERITY_TO_CODE.Error),
          redUnreadCount = modNot.getCountUnreadByLevel(ZxNotifications.SEVERITY_TO_CODE.Critical) +
            modNot.getCountUnreadByLevel(ZxNotifications.SEVERITY_TO_CODE.Error),
          yellowCount = modNot.getCountByLevel(ZxNotifications.SEVERITY_TO_CODE.Warning),
          yellowUnreadCount = modNot.getCountUnreadByLevel(ZxNotifications.SEVERITY_TO_CODE.Warning),
          blueCount = modNot.getCountByLevel(ZxNotifications.SEVERITY_TO_CODE.Information) +
            modNot.getCountByLevel(ZxNotifications.SEVERITY_TO_CODE.Debug),
          blueUnreadCount = modNot.getCountUnreadByLevel(ZxNotifications.SEVERITY_TO_CODE.Information) +
            modNot.getCountUnreadByLevel(ZxNotifications.SEVERITY_TO_CODE.Debug);

        html[idx++] = '<table><tbody>';

        html[idx++] = '<tr><td><div class="ImgCritical"></div></td><td  style="vertical-align: bottom;">';
        if (redUnreadCount > 0) {
          html[idx++] = zxMsg('MSG_Count_with_unread', [redCount, redUnreadCount]);
        } else {
          html[idx++] = '<span>' + redCount + '</span>';
        }
        html[idx++] = '</td></tr>';

        html[idx++] = '<tr><td><div class="ImgWarning"></div></td><td style="vertical-align: bottom;">';
        if (yellowUnreadCount > 0) {
          html[idx++] = zxMsg('MSG_Count_with_unread', [yellowCount, yellowUnreadCount]);
        } else {
          html[idx++] = '<span>' + yellowCount + '</span>';
        }
        html[idx++] = '</td></tr>';

        html[idx++] = '<tr><td><div class="ImgInformation"></div></td><td style="vertical-align: bottom;">';
        if (blueUnreadCount > 0) {
          html[idx++] = zxMsg('MSG_Count_with_unread', [blueCount, blueUnreadCount]);
        } else {
          html[idx++] = '<span>' + blueCount + '</span>';
        }
        html[idx++] = '</td></tr>';

        html[idx++] = '</tbody></table>';
      }
      html[idx++] = '</td>';

    } else if (field === ZxServersListView.PRFX_LicenseInfo) {
      // License
      html[idx++] = '<td align="left" style="line-height:150%;" width="' + this._headerList[i]._width + '">';
      if (isOnline && licenseInfo) {
        if (licenseInfo.notYetValid) {
          html[idx++] = '<span style="color: #A00000">' + zxMsg('LBL_NotYetValid') + '</span>';
        } else if (!licenseInfo.expired) {
          html[idx++] = '<span style="color: #00A000">' + zxMsg('LBL_Valid') + '</span>';
          if (!!licenseInfo.dateEnd) {
            html[idx++] = ' (' + zxMsg('LBL_Until') +
              ' ' + AjxDateUtil._wordyDate.format(new Date(licenseInfo.dateEnd)) + ')';
          }
        } else {
          html[idx++] = '<span style="color: #A00000">' + zxMsg('LBL_Expired') + '</span>';
          if (!!licenseInfo.dateEnd) {
            html[idx++] = ' (' + zxMsg('LBL_Since') +
              ' ' + AjxDateUtil._wordyDate.format(new Date(licenseInfo.dateEnd)) + ')';
          }
        }
        html[idx++] = '<br>';
        html[idx++] = zxMsg('LBL_COR_license_Type') + ': ' + licenseInfo.type;
        html[idx++] = '<br>';
        if (!!licenseInfo.accountCount && !!licenseInfo.licensedUsers) {
          html[idx++] = zxMsg('LBL_Accounts') + ': ' +
            licenseInfo.accountCount + ' / ' + licenseInfo.licensedUsers;
        }
      }
      html[idx++] = '</td>';
    }
  }
  html[idx++] = '</tr></table>';
  div.innerHTML = html.join('');
  div.style.height = ZxServersListView.MIN_ROW_HEIGHT;
  return div;
};

/**
 * Get the headers list.
 * @return {ZaListHeaderItem[]}
 */
ZxServersListView.prototype._getHeaderList = function() {
  var headerList = [];
  headerList.push(new ZaListHeaderItem(ZxServersListView.KEY_ICON, '', null,
    ZxServersListView.WIDTH_ICON, false, null, false, true));
  headerList.push(new ZaListHeaderItem(ZxServersListView.PRFX_Server, zxMsg('LBL_SERVER_LIST_Server'), null,
    ZxServersListView.WIDTH_SERVER, null, null, true, true));
  headerList.push(new ZaListHeaderItem(ZxServersListView.PRFX_Notifications, zxMsg('LBL_SERVER_LIST_Notifications'),
    null, ZxServersListView.WIDTH_NOTIFICATIONS, null, null, true, true));
  headerList.push(new ZaListHeaderItem(ZxServersListView.PRFX_LicenseInfo, zxMsg('LBL_SERVER_LIST_License_info'), null,
    ZxServersListView.WIDTH_LICENSE, null, null, true, true));
  return headerList;
};
