/*
 * ***** BEGIN LICENSE BLOCK *****
 * Copyright (C) 2011-2016 ZeXtras
 *
 * The contents of this file are subject to the ZeXtras EULA;
 * you may not use this file except in compliance with the EULA.
 * You may obtain a copy of the EULA at
 * http://www.zextras.com/zextras-eula.html
 * ***** END LICENSE BLOCK *****
 */
/**
 *
 * @param {DwtShell} parent
 * @param {ZxOperation} operation
 * @constructor
 */
function ZxOperationDialog(parent, operation) {
  this.opData = operation;

  var helpButton,
    closeButton,
    stopButton,
    helpUrl = ZxLinks.HELP_NOT_Dialog_OpInfo,
    title;

  this._icon = operation.getModuleName();

  if (ZeXtras.isZxMig(operation.getServerName())) {
    this._icon = 'ZxMig';
  }

  title = ZxUtils.getTitleWithIcon(this._icon, zxMsg('LBL_OP_Details'));

  helpButton = new DwtDialog_ButtonDescriptor(
    ZaXDialog.HELP_BUTTON,
    ZaMsg.TBB_Help,
    DwtDialog.ALIGN_LEFT,
    new AjxCallback(
      this,
      ZxUtils.openURL,
      [helpUrl]
    )
  );

  stopButton = new DwtDialog_ButtonDescriptor(
    ZxOperationDialog.ID_BTN_STOP,
    zxMsg('LBL_Stop_Operation'),
    DwtDialog.ALIGN_RIGHT,
    new AjxCallback(
      this,
      this.stopOperation
    )
  );

  closeButton = new DwtDialog_ButtonDescriptor(
    ZxOperationDialog.ID_BTN_CLOSE,
    ZaMsg.TBB_Close,
    DwtDialog.ALIGN_RIGHT,
    new AjxCallback(
      this,
      this.popdown
    )
  );

  this._standardButtons = [DwtDialog.NO_BUTTONS];
  this._extraButtons = [helpButton, stopButton, closeButton];

  ZaXDialog.call(this, parent, null, title, '480px', '200px', null, null);
  this.initForm(ZxOperationDialog.MODEL, this.getMyXForm());
  this._setAllowSelection();

  this._helpURL = helpUrl;
}
ZxOperationDialog.prototype = new ZaXDialog();
ZxOperationDialog.prototype.constructor = ZxOperationDialog;

ZxOperationDialog.ID_BTN_CLOSE = 'closeBTN';
ZxOperationDialog.ID_BTN_STOP = 'stopBTN';

ZxOperationDialog.MODEL = [
  {type: _STRING_, id: ZxOperation.KEY_DESC_NAME,
    ref: 'opData/' + ZxOperation.KEY_DESC_NAME},
  {type: _STRING_, id: ZxOperation.KEY_START_TIME,
    ref: 'opData/' + ZxOperation.KEY_START_TIME},
  {type: _STRING_, id: ZxOperation.KEY_QUEUED_TIME,
    ref: 'opData/' + ZxOperation.KEY_QUEUED_TIME},
  {type: _STRING_, id: ZxOperation.KEY_STATE,
    ref: 'opData/' + ZxOperation.KEY_STATE},
  {type: _STRING_, id: ZxOperation.KEY_HOST,
    ref: 'opData/' + ZxOperation.KEY_HOST},
  {type: _STRING_, id: ZxOperation.KEY_MODULE,
    ref: 'opData/' + ZxOperation.KEY_MODULE},
  {type: _STRING_, id: ZxOperation.KEY_NAME,
    ref: 'opData/' + ZxOperation.KEY_NAME},
  {type: _STRING_, id: ZxOperation.KEY_ID,
    ref: 'opData/' + ZxOperation.KEY_ID}
];

/**
 * Return the Operation displayed into the dialog.
 * @return {ZxOperation}
 */
ZxOperationDialog.prototype.getOpData = function() {
  return this.opData;
};

/**
 * Stop the operation.
 */
ZxOperationDialog.prototype.stopOperation = function() {
  ZxUtils.log('Stopping operation');
  var op = this.getOpData(),
    msgDialog;

  msgDialog = new DwtMessageDialog({
    buttons: [DwtDialog.YES_BUTTON, DwtDialog.NO_BUTTON],
    title: '',
    parent: ZaApp.getInstance().getAppCtxt().getShell()
  });

  msgDialog.setMessage(
    zxMsg('MSG_Stop_Operation'),
    DwtMessageDialog.WARNING_STYLE,
    ZxUtils.getTitleWithIcon(this._icon, zxMsg('LBL_Stop_Operation'))
  );

  msgDialog.registerCallback(
    DwtDialog.YES_BUTTON,
    new AjxCallback(
      this,
      function(operation, dialog) {
        ZxAdminRequest.sendRequest(
          operation.getServerName(),
          operation.getModuleName(),
          'doStopOperation',
          {
            operationId: operation.getID()
          }
        ).then(
          AjxCallback.simpleClosure(
            function(dialog, response) {
              dialog.popdown();
              this.popdown();
              ZeXtras.getOperationsManager().requestUpdate(true);
            },
            this,
            dialog
          ),
          AjxCallback.simpleClosure(
            function(dialog, error) {
              if (error.getCode() !== ZxErrorCode.UNABLE_TO_FIND_OPERATION) {
                ZxUtils.catchError(error);
              }
              dialog.popdown();
              this.popdown();
              ZeXtras.getOperationsManager().requestUpdate(true);
            },
            this,
            dialog
          )
        );
      },
      [op, msgDialog]
    )
  );

  msgDialog.registerCallback(
    DwtDialog.NO_BUTTON,
    new AjxCallback(
      msgDialog,
      msgDialog.popdown
    )
  );

  msgDialog.popup();
};

ZxOperationDialog.prototype.getMyXForm = function() {
  var xFormObject,
    isMig = ZeXtras.isMigZimlet();

  xFormObject = {
    numCols: 1,
    colSizes: ['*'],
    cssStyle: 'padding: 10px',
    items: [
      {
        type: _GROUP_,
        numCols: 2,
        colSizes: [100, '*'],
        items: [
          {
            type: _OUTPUT_,
            label: zxMsg('LBL_OP_Description') + ': ',
            ref: 'opData/' + ZxOperation.KEY_DESC_NAME
          },
          {
            type: _OUTPUT_,
            label: zxMsg('LBL_OP_Queued_Time') + ': ',
            ref: 'opData/' + ZxOperation.KEY_QUEUED_TIME
          },
          {
            type: _OUTPUT_,
            label: zxMsg('LBL_OP_Start_Time') + ': ',
            ref: 'opData/' + ZxOperation.KEY_START_TIME
          },
          {
            type: _OUTPUT_,
            label: zxMsg('LBL_OP_Status') + ': ',
            ref: 'opData/' + ZxOperation.KEY_STATE
          },
          {
            type: _OUTPUT_,
            label: zxMsg('LBL_OP_Host') + ': ',
            ref: 'opData/' + ZxOperation.KEY_HOST
          },
          {
            type: _OUTPUT_,
            label: zxMsg('LBL_OP_Module') + ': ',
            ref: 'opData/' + ZxOperation.KEY_MODULE
          },
          {
            type: _OUTPUT_,
            label: zxMsg('LBL_OP_Name') + ': ',
            ref: 'opData/' + ZxOperation.KEY_NAME
          },
          {
            type: _OUTPUT_,
            label: zxMsg('LBL_OP_Id') + ': ',
            ref: 'opData/' + ZxOperation.KEY_ID
          },
          {
            type: _OUTPUT_,
            colSpan: 2,
            getDisplayValue: function(val) {
              var op = this.getForm().parent.opData;
              return zxMsg('LBL_OP_Monitor_Info',
                [
                  isMig ? 'zxmig' : 'zxsuite',
                  isMig ? 'migration' : op.module.substring(2).toLowerCase(),
                  op.uuid
                ]);
            }
          }
        ]
      }
    ]
  };

  return xFormObject;
};
