/*
 * ***** BEGIN LICENSE BLOCK *****
 * Copyright (C) 2011-2016 ZeXtras
 *
 * The contents of this file are subject to the ZeXtras EULA;
 * you may not use this file except in compliance with the EULA.
 * You may obtain a copy of the EULA at
 * http://www.zextras.com/zextras-eula.html
 * ***** END LICENSE BLOCK *****
 */

function ZxWaitingDialog(parent, w, h, title, helpUrl) {

  var helpButton, closeButton;

  helpButton = new DwtDialog_ButtonDescriptor(
    ZaXDialog.HELP_BUTTON,
    ZaMsg.TBB_Help,
    DwtDialog.ALIGN_LEFT,
    new AjxCallback(
      this,
      ZxUtils.openURL,
      [helpUrl]
    )
  );

  closeButton = new DwtDialog_ButtonDescriptor(
    ZxWaitingDialog.ID_BTN_CLOSE,
    ZaMsg.TBB_Close,
    DwtDialog.ALIGN_RIGHT,
    new AjxCallback(
      this,
      function() {
        this.popdown();
      }
    )
  );

  this._standardButtons = [DwtDialog.NO_BUTTONS];
  this._extraButtons = [helpButton, closeButton];

  ZaXDialog.call(this, parent, null, title, w, h, null, null);

  this._currentMessage = {
    message: ZaMsg.MSG_WorkingTask,
    status: ZxWaitingDialog.STATUS_PENDING
  };
  this.initForm([], this.getMyXForm(ZaMsg.MSG_WorkingTask, ZxWaitingDialog.STATUS_PENDING));

//  this.setObject(params);
//  this.setSize(w, h);
  this._helpURL = helpUrl;
}

ZxWaitingDialog.prototype = new ZaXDialog();
ZxWaitingDialog.prototype.constructor = ZxWaitingDialog;
//ZxWaitingDialog.helpURL = ZxAdmin.helpUrl;

ZxWaitingDialog.STATUS_PENDING = 0;
ZxWaitingDialog.STATUS_SUCCESS = 1;
ZxWaitingDialog.STATUS_FAILURE = -1;

ZxWaitingDialog.ID_BTN_CLOSE = 'closeBTN';

/**
 * Set a message in the wating dialog.
 * @param {string} message
 * @param {number} status must be one of ZxWaitingDialog.STATUS_*
 */
ZxWaitingDialog.prototype.setMessage = function(message, status) {
  this._currentMessage = {
    message: message,
    status: status
  };
  this.initForm([], this.getMyXForm(message, status));
};

/**
 * Build the XForm Of the object.
 * @param {string} message
 * @param {number} status must be one of ZxWaitingDialog.STATUS_*
 * @return {{numCols: number, colSizes: Array, items: Array}}
 */
ZxWaitingDialog.prototype.getMyXForm = function(message, status) {
  var xFormObject;

  xFormObject = {
    numCols: 1,
    colSizes: ['*'],
    cssStyle: 'padding: 10px',
    items: [
      {
        type: _GROUP_,
        numCols: 2,
        colSizes: [32, '*'],
        items: [
          {
            type: _OUTPUT_,
            value: ZxWaitingDialog._getImageIconByStatus(status)
          },
          {
            type: _OUTPUT_,
            value: message
          }
        ]
      }
    ]
  };

  return xFormObject;
};

/**
 * Return the Image in HTML string tag notation.
 * @param {number} status must be one of ZxWaitingDialog.STATUS_*
 * @return {string}
 */
ZxWaitingDialog._getImageIconByStatus = function(status) {
  switch (status) {
  case ZxWaitingDialog.STATUS_PENDING:
    return '<div class="DwtWait32Icon"></div>';

  case ZxWaitingDialog.STATUS_SUCCESS:
    return AjxImg.getImageHtml(DwtMessageDialog.ICON[DwtMessageDialog.INFO_STYLE]);

  case ZxWaitingDialog.STATUS_FAILURE:
    return AjxImg.getImageHtml(DwtMessageDialog.ICON[DwtMessageDialog.WARNING_STYLE]);

  default:
    return '';
  }
};

/**
 * Handle an error displaying it in human-readable format.
 * @param {ZxError=} ex
 */
ZxWaitingDialog.prototype.catchError = function(ex) {
  this.popdown();
  ZxUtils.catchError(ex, false);
};
