/*
 * ***** BEGIN LICENSE BLOCK *****
 * Copyright (C) 2011-2016 ZeXtras
 *
 * The contents of this file are subject to the ZeXtras EULA;
 * you may not use this file except in compliance with the EULA.
 * You may obtain a copy of the EULA at
 * http://www.zextras.com/zextras-eula.html
 * ***** END LICENSE BLOCK *****
 */

function ZxFormModifier(formView, loadMetdods, modifyMethods, modifier) {
  this._tabs = {};

  if (formView) {
    formView.push(this._modifier);
  }
  if (loadMetdods) {
    loadMetdods.push(this._fireLoaders);
  }
  if (modifyMethods) {
    modifyMethods.splice(0, 0, this._fireSavers);
  }
}

/**
 * Get the correct form modifier for the specified object.
 * @param {ZaAccount|ZaCos|ZaDomain} currentItem
 * @return {ZxAccountFormModifier|ZxCOSFormModifier|ZxDomainFormModifier|undefined}
 */
ZxFormModifier.getCurrentModifier = function(currentItem) {
  if (currentItem instanceof ZaAccount || (currentItem instanceof ZaAccountXFormView)) {
    return ZeXtras.getAccountModifier();
  }
  if (currentItem instanceof ZaCos || (currentItem instanceof ZaCosXFormView)) {
    return ZeXtras.getCOSModifier();
  }
  if (currentItem instanceof ZaDomain || (currentItem instanceof ZaDomainXFormView)) {
    return ZeXtras.getDomainModifier();
  }
};

/**
 * Set a value for the ref, in the correct form instance.
 * @param {ZaItem} ctxt
 * @param {{}|boolean|string} val
 * @param {string} ref
 * @param {boolean=false} setDefault If true set the value as default, use into loadMethods.
 */
ZxFormModifier.setInstanceValue = function(ctxt, val, ref, setDefault) {
  setDefault = setDefault || false;

  var controller = ZaApp.getInstance().getCurrentController();

  if (setDefault) {
    ZxUtils.mergeObjects(ctxt, ZxUtils.createObjectFromRef(ref, val));
  }

  if (ctxt instanceof ZaAccount && !(controller instanceof ZaAccountViewController)) {
    return;
  }
  if (ctxt instanceof ZaCos && !(controller instanceof ZaCosController)) {
    return;
  }
  if (ctxt instanceof ZaDomain && !(controller instanceof ZaDomainController)) {
    return;
  }
  if (!(controller && controller._view && controller._view._localXForm)) {
    return;
  }

  if (controller._currentObject && (controller._currentObject.id !== ctxt.id)) {
    return;
  }

  controller._view._localXForm.setInstanceValue(
    val,
    ref
  );
};

/**
 * Modify the account tab.
 * @param {Object} xFormObject
 * @param {ZaAccount|ZaCos} entry
 */
ZxFormModifier.prototype._modifier = function(xFormObject, entry) {
  var cnt = xFormObject.items.length,
    modifier = ZxFormModifier.getCurrentModifier(this),
    i,
    switchGroup,
    xForm = [],
    tabBar,
    tabIx,
    tabIx2,
    foundTabbar = false,
    foundSwitchGroup = false,
    definitions,
    cTab;

  if (!modifier) {
    return;
  }
  definitions = modifier.getTabsDefinitions();
  if (ZxUtils.isObjectEmpty(definitions)) {
    return;
  }

  for (i = 0; i < cnt; i = i + 1) {
    if (xFormObject.items[i].type === 'switch') {
      switchGroup = xFormObject.items[i];
      foundSwitchGroup = true;
    }

    if (xFormObject.items[i].type === _TAB_BAR_) {
      tabBar = xFormObject.items[i];
      foundTabbar = true;
    }

    if (foundTabbar && foundSwitchGroup) {
      break;
    }
  }

  if (tabBar && switchGroup) {
    tabIx = ++this.TAB_INDEX;
    tabBar.choices.push({
      value: tabIx,
      image: 'ZeXtras',
      label: zxMsg('label')
    });
  }

  for (cTab in definitions) {
    if (!definitions.hasOwnProperty(cTab)) { continue; }
    var tabDef = definitions[cTab];

    if (ZaAccount) {
      for (i = 0; i < tabDef.model.length; i = i + 1) {
        if (tabDef.model.isZ8 && !ZxUtils.isZ8Up()) {
          // If a property is available only for z8 and not below, skip it.
          continue;
        }
        ZaAccount.myXModel.items.push(
          tabDef.model[i]
        );
      }
    }

    if (ZaCos) {
      for (i = 0; i < tabDef.model.length; i = i + 1) {
        if (tabDef.model.isZ8 && !ZxUtils.isZ8Up()) {
          // If a property is available only for z8 and not below, skip it.
          continue;
        }
        ZaCos.myXModel.items.push(
          tabDef.model[i]
        );
      }
    }

    if (ZaDomain) {
      for (i = 0; i < tabDef.model.length; i = i + 1) {
        if (tabDef.model.isZ8 && !ZxUtils.isZ8Up()) {
          // If a property is available only for z8 and not below, skip it.
          continue;
        }
        ZaDomain.myXModel.items.push(
          tabDef.model[i]
        );
      }
    }

    if (tabDef.merge) {
      xForm.push(tabDef.xform);

    } else {
      tabIx2 = ++this.TAB_INDEX;
      tabBar.choices.push({
        value: tabIx2,
        label: tabDef.label
      });

      switchGroup.items.push(
        {
          type: _ZATABCASE_,
//          id: 'zxbase_tab_account',
          numCols: 1,
          colSizes: ['auto'],
          caseKey: tabIx2,
          items: [tabDef.xform]
        }
      );
    }

  }

  switchGroup.items.push(
    {
      type: _ZATABCASE_,
//      id: 'zxbase_tab_account',
      numCols: 1,
      colSizes: ['auto'],
      caseKey: tabIx,
      items: xForm
    }
  );
};

/**
 * Return the tabs definitions for every module.
 * @return {{}}
 */
ZxFormModifier.prototype.getTabsDefinitions = function() {
  return this._tabs;
};

/**
 * Add a modification for the user tab.
 * @param {ZxModule} mod
 * @param {{}} tabDefinition
 */
ZxFormModifier.prototype.addModifier = function(mod, tabDefinition) {
  if (this._tabs[mod.getID()]) {
    return;
  }

  this._tabs[mod.getID()] = {
    xform: tabDefinition.XFORM,
    attrs: tabDefinition.ATTRS,
    rights: tabDefinition.RIGHTS,
    merge: !tabDefinition.NEW_PANEL,
    model: tabDefinition.MODEL,
    label: mod.getName(),
    icon: mod.getIcon(),
    loadfcn: tabDefinition.LOAD_ATTRS,
    savefcn: tabDefinition.SAVE_ATTRS
  };
};

/**
 * Process all informations which should be saved.
 * If a modifier model declare 'isZX' means the property are
 * managed by the modifier itself and will be removed from the 'mods'
 * object.
 * @param {{}} mods Modififactions found by Zimbra
 * @param {{}} tmpObj Original object stored by zimbra
 * @this {ZaAccount}
 */
ZxFormModifier.prototype._fireSavers = function(mods, tmpObj) {
  var currentModifier = ZxFormModifier.getCurrentModifier(this),
    definitions,
    name,
    attr,
    attrI;

  if (ZxUtils.isObjectEmpty(mods) && tmpObj) {
    mods = ZxUtils.objDiff(this.attrs, tmpObj.attrs);
  }

  if (!currentModifier) {
    return;
  }
  if (this instanceof ZaAccount && !(currentModifier instanceof ZxAccountFormModifier)) {
    return;
  }
  if (this instanceof ZaCos && !(currentModifier instanceof ZxCOSFormModifier)) {
    return;
  }
  if (this instanceof ZaDomain && !(currentModifier instanceof ZxDomainFormModifier)) {
    return;
  }

  definitions = currentModifier.getTabsDefinitions();

  for (name in definitions) {
    if (!definitions.hasOwnProperty(name)) { continue; }

    for (attrI = 0; attrI < definitions[name].model.length; attrI = attrI + 1) {
      attr = definitions[name].model[attrI];

      if (mods.hasOwnProperty(attr.id) && !!attr.isZX) {
        if (typeof definitions[name].savefcn === 'function') {
          try {
            definitions[name].savefcn.call(this, attr.id, mods[attr.id], mods);
          } catch (error) {
            ZxUtils.catchError(error);
          }
        }
        mods[attr.id] = false;
        delete mods[attr.id];
      }
    }
  }
};

/**
 * Load all informations needed by the modules
 * @param {string} by
 * @param {string} val
 * @this {ZaAccount}
 */
ZxFormModifier.prototype._fireLoaders = function(by, val) {
  var currentModifier = ZxFormModifier.getCurrentModifier(this),
    definitions,
    name;

  if (!currentModifier) {
    return;
  }
  if (this instanceof ZaAccount && !(currentModifier instanceof ZxAccountFormModifier)) {
    return;
  }
  if (this instanceof ZaCos && !(currentModifier instanceof ZxCOSFormModifier)) {
    return;
  }
  if (this instanceof ZaDomain && !(currentModifier instanceof ZxDomainFormModifier)) {
    return;
  }

  // Fix for ZX-2780
  if (currentModifier instanceof ZxAccountFormModifier && !(this.attrs && this.attrs['zimbraMailHost'])) {
    return;
  }

  definitions = currentModifier.getTabsDefinitions();
  for (name in definitions) {
    if (definitions.hasOwnProperty(name) && typeof definitions[name].loadfcn === 'function') {
      try {
        definitions[name].loadfcn.call(this);
      } catch (error) {
        ZxUtils.catchError(error);
      }
    }
  }
};
