/*
 * ***** BEGIN LICENSE BLOCK *****
 * Copyright (C) 2011-2016 ZeXtras
 *
 * The contents of this file are subject to the ZeXtras EULA;
 * you may not use this file except in compliance with the EULA.
 * You may obtain a copy of the EULA at
 * http://www.zextras.com/zextras-eula.html
 * ***** END LICENSE BLOCK *****
 */

/**
 * Create the ZxAdmin object.
 * @constructor
 */
function ZxAdmin() {
  ZxModule.call(this, ZxAdmin.MOD_ID, zxMsg('LBL_ZxAdmin'), 'ZxAdmin');

  this.registerLoadMethod(this.loadMethod, 'ZxAdminInit');
  this.registerModifyMethod(this.modifyMethod, 'ZxAdminInit');

  this._helpURL = ZxLinks.HELP_DLG_Admin;
}

ZxAdmin.prototype = new ZxModule();
ZxAdmin.prototype.constructor = ZxAdmin;
ZxAdmin.MOD_ID = 'ZxAdmin';

ZxModule.SINGLES.push(ZxAdmin.MOD_ID);
ZxModule.NE_INCOMPATIBLE.push(ZxAdmin.MOD_ID);

ZxAdmin.KEY_LIMITS = 'limits';
ZxAdmin.KEY_DOMAINS = 'domains';
ZxAdmin.KEY_QUOTA = 'quotas';
ZxAdmin.KEY_LOG_PATH = 'ZxAdmin_AdminLogPath';
ZxAdmin.KEY_QUOTA_LIMIT = 'quota_limit';
ZxAdmin.KEY_QUOTA_PAGE = 'quota_page';
ZxAdmin.KEY_QUOTA_TOTAL = 'quota_total';
ZxAdmin.KEY_NEED_MIGRATION = 'need_migration';

ZxAdmin.UNLIMITED = 'Unlimited';
ZxAdmin.NONE = 'None';
ZxAdmin.NOT_SUPPORTED = 'Not Supported';

ZxAdmin.KEY_COL_VIEW_MAIL = 'ZXViewMail';
ZxAdmin.KEY_COL_ACCOUNT = 'ZXAccount';
ZxAdmin.KEY_COL_DOMAIN = 'ZXDomain';
ZxAdmin.KEY_COL_COS_NUM = 'ZXCosNumber';
ZxAdmin.KEY_COL_GLOBAL_LIMIT = 'ZXGlobalLimit';
ZxAdmin.KEY_COL_QUOTA = 'ZXQuota';
ZxAdmin.KEY_COL_COS = 'ZXCos';
ZxAdmin.KEY_COL_COS_LIMIT = 'ZXCOSLimit';
ZxAdmin.KEY_COL_EDIT_FEATURES = 'ZXEditFeatures';

ZxAdmin.ID_LIST_DELEGATED_ADMINS = 'delegatedAdmins';
ZxAdmin.ID_LIST_DOMAIN_SETTINGS = 'domainSettings';

ZxAdmin.KEY_SEND_REPORTS_TO_DELEGATED_ENABLED = "ZxAdmin_SendReportsToDelegated";

ZxAdmin.MBOX_LIMIT_ON_VIEW = 25;

ZxAdmin.prototype.loadMethod = function(by, val) {
  this.attrs[ZxAdmin.KEY_LIMITS] = [];
  this.attrs[ZxAdmin.KEY_DOMAINS] = [];
  this.attrs[ZxAdmin.KEY_QUOTA] = [];

  this.attrs[ZxAdmin.KEY_QUOTA_PAGE] = 0;
  this.attrs[ZxAdmin.KEY_QUOTA_TOTAL] = 0;

  this.attrs.properties = {};
  this.attrs.properties[ZxAdmin.KEY_SEND_REPORTS_TO_DELEGATED_ENABLED] = false;

  this.attrs.config = {};
  this.attrs.config[ZxAdmin.KEY_NEED_MIGRATION] = true;

  var request = {};

  ZxAdminRequest.sendRequest(
    ZxAdminRequest.TARGET_LOCAL,
    'ZxAdmin',
    'getPublicProperties'
  ).then(
    AjxCallback.simpleClosure(
      function(resp) {
        ZxUtils.log(resp);
        if (!(resp instanceof ZxError)) {
          var controller = this.getCurrentController(),
            response = resp.response,
            properties = response.properties;

          this.attrs.properties[ZxAdmin.KEY_SEND_REPORTS_TO_DELEGATED_ENABLED] = (
            properties.ZxAdmin_SendReportsToDelegated === 'true'
            );

          controller.setInstanceValue('properties/' + ZxAdmin.KEY_SEND_REPORTS_TO_DELEGATED_ENABLED,
              properties.ZxAdmin_SendReportsToDelegated === 'true');
        }
      },
      this
    ),
    this._getDefaultCatchErrorFcn()
  );

  ZxAdminRequest.sendRequest(
    ZxAdminRequest.TARGET_LOCAL,
    'ZxAdmin',
    'getDelegationSettings'
  ).then(
    AjxCallback.simpleClosure(
      function(resp) {
        var controller = this.getCurrentController(),
          response = resp.response,
          i,
          entry,
          admin_quota,
          delegationData = [],
          row;

        for (i = 0; i < response.length; i = i + 1) {
          entry = response[i];

          if (entry.adminQuota < 0) {
            admin_quota = ZaMsg.L_none;
          } else if (entry.adminQuota === 0) {
            admin_quota = ZaMsg.Unlimited;
          } else {
            admin_quota = ZxUtils.bytesToSize(entry.adminQuota, 0);
          }

          row = {};
          row[ZxAdmin.KEY_COL_ACCOUNT] = entry.admin;
          row[ZxAdmin.KEY_COL_DOMAIN] = entry.domain;
          row[ZxAdmin.KEY_COL_VIEW_MAIL] = (entry.viewMail ? zxMsg('LBL_Enabled') : zxMsg('LBL_Disabled'));
          row[ZxAdmin.KEY_COL_VIEW_MAIL + '_row'] = entry.viewMail;
          row[ZxAdmin.KEY_COL_QUOTA] = admin_quota;
          row[ZxAdmin.KEY_COL_QUOTA + '_row'] = entry.adminQuota;
          row[ZxAdmin.KEY_COL_EDIT_FEATURES] = (entry.editFeatures ? zxMsg('LBL_Enabled') : zxMsg('LBL_Disabled'));
          row[ZxAdmin.KEY_COL_EDIT_FEATURES + '_row'] = entry.editFeatures;
// var row = [ entry.admin, entry.domain, (entry.viewMail ? "enabled" : "disabled"), entry.editFeatures, admin_quota ];
          delegationData.push(row);
        }
        this.attrs[ZxAdmin.KEY_LIMITS] = delegationData;
        controller.setInstanceValue(ZxAdmin.KEY_LIMITS, delegationData);
      },
      this
    ),
    this._getDefaultCatchErrorFcn()
  );

  ZxAdminRequest.sendRequest(
    ZxAdminRequest.TARGET_LOCAL,
    'ZxAdmin',
    'getDomainSettings'
  ).then(
    AjxCallback.simpleClosure(
      function(resp) {
        var controller = this.getCurrentController(),
          response = resp.response,
          z8 = ZxUtils.isZ8Up(),
          domainsInfo = [],
          i,
          j,
          entry,
          domainQuota,
          accountLimit,
          cosNumber,
          cosLimits,
          row;

        for (i = 0; i < response.length; i = i + 1) {
          entry = response[i];

          if (!z8) {

            domainQuota = (entry.domainAccountQuota <= 0) ? ZxAdmin.NOT_SUPPORTED :
                ZxUtils.bytesToSize(entry.domainAccountQuota, 0);

          } else {

            domainQuota = (entry.domainAccountQuota <= 0) ? ZxAdmin.UNLIMITED :
                ZxUtils.bytesToSize(entry.domainAccountQuota, 0);

          }

          accountLimit = (entry.accountLimit < 0) ? ZxAdmin.UNLIMITED : entry.accountLimit;
          cosNumber = entry.cosLimits.length;

          cosLimits = [];
          for (j = 0; j < entry.cosLimits.length; j = j + 1) {
            var tmpObj = {};
            tmpObj[ZxAdmin.KEY_COL_COS] = entry.cosLimits[j].cosName;
            tmpObj[ZxAdmin.KEY_COL_ACCOUNT] = entry.cosLimits[j].cosLimit;
            cosLimits.push(tmpObj);
          }

          row = {};
          row[ZxAdmin.KEY_COL_DOMAIN] = entry.domain;
          row[ZxAdmin.KEY_COL_COS_NUM] = cosNumber;
          row[ZxAdmin.KEY_COL_GLOBAL_LIMIT] = accountLimit;
          row[ZxAdmin.KEY_COL_QUOTA] = domainQuota;
          row[ZxAdmin.KEY_COL_COS_LIMIT] = cosLimits;
//        row = [entry.domain, cosNumber, accountLimit, domainQuota, cosLimits];
          domainsInfo.push(row);
        }

        this.attrs[ZxAdmin.KEY_DOMAINS] = domainsInfo;
        controller.setInstanceValue(ZxAdmin.KEY_DOMAINS, domainsInfo);
      },
      this
    ),
    this._getDefaultCatchErrorFcn()
  );

  this.getLegacyConnector().Admin.getNeedMigration(
    this.getLegacyConnector().createCallback(this, function(needMigration) {
      var controller = this.getCurrentController();
      this.attrs.config[ZxAdmin.KEY_NEED_MIGRATION] = needMigration;
      controller.setInstanceValue('config/' + ZxAdmin.KEY_NEED_MIGRATION, needMigration);
    })
  );
};

ZxAdmin.doSelectFilterForBrowseLog = function(ev) {
  var item = ZeXtras.getModuleItem(ZxAdmin, this.getForm().getInstance().id),
    opObj;

  try {
    ZaApp.getInstance().dialogs['zxBrowseAdminLogWizard'] = new ZxAdminBrowseLogWizard(item);

    opObj = new ZxAdminBrowseLogOp(item);
    opObj.init();
    ZaApp.getInstance().dialogs['zxBrowseAdminLogWizard'].setObject(opObj);
    ZaApp.getInstance().dialogs['zxBrowseAdminLogWizard'].popup();

  } catch (ex) {
    ZxUtils.log(ex, 'Exceptions?!');
  }
};

ZxAdmin.doShowMonthlyReport = function(ev) {
  var item = ZeXtras.getModuleItem(ZxAdmin, this.getForm().getInstance().id);
  item._doShowMonthlyReport();
};

ZxAdmin.prototype._doShowMonthlyReport = function() {
  var reporthDialog = new ZxAdminReportDialog(this);
  if (ZxUtils.amIDelegated()) {
    reporthDialog._showReport();
    reporthDialog.popup();
  } else {
    reporthDialog.popup();
  }
};

ZxAdmin.doShowLogPathInfo = function(ev) {
  var item = ZeXtras.getModuleItem(ZxAdmin, this.getForm().getInstance().id);
  item._doShowLogPathInfo();
};

ZxAdmin.prototype._doShowLogPathInfo = function() {
  var dialog = new ZxAdminLogPathDialog(this.getShell());
  dialog.popup();
};

ZxAdmin.doEditDomain = function(ev, selection) {
  var item = ZeXtras.getModuleItem(ZxAdmin, this.getForm().getInstance().id);

  if (selection.length === 1) {
    item._doEditDomain(selection[0]);
  }
};

ZxAdmin.needMigrationCheck = function() {
  return this.getInstance().attrs.config[ZxAdmin.KEY_NEED_MIGRATION];
};
ZxAdmin.notNeedMigrationCheck = function() {
  return !this.getInstance().attrs.config[ZxAdmin.KEY_NEED_MIGRATION];
};

ZxAdmin.doStartACLMigration = function() {
  var item = ZeXtras.getModuleItem(ZxAdmin, this.getForm().getInstance().id);
  item._doStartACLMigration();
};

ZxAdmin.prototype._doStartACLMigration = function() {
  this.getLegacyConnector().Admin.openStartMigrationConfirmDialog(ZxAdminRequest);
};

ZxAdmin.prototype._doEditDomain = function(domainData) {
  ZxUtils.log(domainData, 'Editing Domain informations');

  var dialog = new ZxEditDomainDelegationSettingsDialog(
    this,
    domainData
  );

  dialog.popup();
};

ZxAdmin.doResetDomain = function(ev, selection) {
  var item = ZeXtras.getModuleItem(ZxAdmin, this.getForm().getInstance().id);

  if (selection.length === 1) {
    item._doResetDomain(selection[0]);
  }
};

ZxAdmin.prototype._doResetDomain = function(domainData) {
  ZxUtils.log(domainData, 'Resetting Domain informations');

  var domain = domainData[ZxAdmin.KEY_COL_DOMAIN],
    helpButton = new DwtDialog_ButtonDescriptor(
      ZaXDialog.HELP_BUTTON,
      ZaMsg.TBB_Help,
      DwtDialog.ALIGN_LEFT,
      new AjxCallback(
        this,
        ZxUtils.openURL,
        [ZxLinks.HELP_DLG_Dialog_Reset_Domain_Settings]
      )
    ),
    msgDialog = new DwtMessageDialog({
      buttons: [DwtDialog.YES_BUTTON, DwtDialog.NO_BUTTON],
      title: zxMsg('LBL_DLG_Reset_domain_settings'),
      extraButtons: [helpButton],
      parent: this.getCurrentController().getShell()
    });

  msgDialog.setMessage(
    zxMsg('MSG_DLG_Are_you_sure_reset_domain', [domain]),
    DwtMessageDialog.WARNING_STYLE,
    zxMsg('LBL_DLG_Reset_domain_settings')
  );

  msgDialog.registerCallback(
    DwtDialog.YES_BUTTON,
    AjxCallback.simpleClosure(
      function(domain, dialog) {
        ZxAdminRequest.sendRequest(
          ZxAdminRequest.TARGET_LOCAL,
          'ZxAdmin',
          'resetDomainSettings',
          {
            domain: domain
          }
        ).then(
          AjxCallback.simpleClosure(
            function(module, resp) {
                if (resp instanceof ZxError) {
                  ZxUtils.catchError(resp);
                }
              module.fireLoadMethods();
              this.popdown();
              module.getCurrentController().popupMsgDialog(resp.message);
            },
            dialog,
            this
          ),
          ZxUtils.catchError
        );
      },
      this,
      domain,
      msgDialog
    )
  );

  msgDialog.popup();
};

ZxAdmin.doAddDelegatedAdmin = function(ev, selection) {
  var item = ZeXtras.getModuleItem(ZxAdmin, this.getForm().getInstance().id);
  item._doAddDelegatedAdmin();
};

ZxAdmin.doDuplicateDelegatedAdmin = function(ev, selection) {
  var item = ZeXtras.getModuleItem(ZxAdmin, this.getForm().getInstance().id);
  if (selection.length === 1) {
    item._doAddDelegatedAdmin(selection[0]);
  }
};

ZxAdmin.prototype._doAddDelegatedAdmin = function(template) {
  var mode,
    newDuplicateDialog;

  if (!template) {
    mode = ZxAddDelegatedAdminDialog.MODE_ADD;
  } else {
    mode = ZxAddDelegatedAdminDialog.MODE_DUPLICATE;
  }

  newDuplicateDialog = new ZxAddDelegatedAdminDialog(
    this,
    template || {},
    mode
  );
  newDuplicateDialog.popup();
};

ZxAdmin.doEditDelegatedAdmin = function(ev, selection) {
  var item = ZeXtras.getModuleItem(ZxAdmin, this.getForm().getInstance().id);
  if (selection.length === 1) {
    item._doEditDelegatedAdmin(selection[0]);
  }
};

ZxAdmin.prototype._doEditDelegatedAdmin = function(rowData) {
  var editDialog = new ZxAddDelegatedAdminDialog(
    this,
    rowData,
    ZxAddDelegatedAdminDialog.MODE_EDIT
  );
  editDialog.popup();
};

ZxAdmin.doDeleteDelegatedAdmin = function(ev, selection) {
  var item = ZeXtras.getModuleItem(ZxAdmin, this.getForm().getInstance().id);
  if (selection.length === 1) {
    item._doDeleteDelegatedAdmin(selection[0]);
  }
};

ZxAdmin.prototype._doDeleteDelegatedAdmin = function(rowData) {
  var helpButton, msgDialog, waitingDialog;

  helpButton = new DwtDialog_ButtonDescriptor(
    ZaXDialog.HELP_BUTTON,
    ZaMsg.TBB_Help,
    DwtDialog.ALIGN_LEFT,
    new AjxCallback(
      this,
      ZxUtils.openURL,
      [ZxLinks.HELP_DLG_Delete_delegated_admin]
    )
  );

  msgDialog = new DwtMessageDialog({
    buttons: [DwtDialog.YES_BUTTON, DwtDialog.NO_BUTTON],
    extraButtons: [helpButton],
    title: zxMsg('MSG_DLG_Delete_permissions'),
    parent: this.getCurrentController().getShell()
  });

  msgDialog.setMessage(
    zxMsg('MSG_DLG_Sure_dete_permissions_for_account', [rowData.ZXAccount, rowData.ZXDomain]),
    DwtMessageDialog.WARNING_STYLE,
    zxMsg('MSG_DLG_Delete_permissions')
  );

  waitingDialog = new ZxWaitingDialog(
    this.getCurrentController().getShell(),
    250,
    52,
    zxMsg('LBL_Saving_settings'),
    ZxLinks.HELP_DLG_Admin
  );

  msgDialog.registerCallback(
    DwtDialog.YES_BUTTON,
    new AjxCallback(
      this,
      function(dialog, waitingDialog, data) {

        ZxAdminRequest.sendRequest(
          ZxAdminRequest.TARGET_LOCAL,
          'ZxAdmin',
          'doEditDelegationSettings',
          {
            account: data[ZxAdmin.KEY_COL_ACCOUNT],
            domain: data[ZxAdmin.KEY_COL_DOMAIN],
            mode: 'ZxDoRemoveDelegationSettings'
          }
        ).then(
          AjxCallback.simpleClosure(
            function(waitingDialog, resp) {
              var hasErrors = false;
              if (resp instanceof ZxError) {
                ZxUtils.catchError(resp);
                hasErrors = true;
              }
              this.fireLoadMethods();
              if (!hasErrors) {
                waitingDialog.setMessage(zxMsg('MSG_Settings_updated_correctly'), ZxWaitingDialog.STATUS_SUCCESS);
              }
            },
            this,
            waitingDialog
          ),
          ZxUtils.catchError
        );

        dialog.popdown();
        waitingDialog.popup();

      },
      [msgDialog, waitingDialog, rowData]
    )
  );

  msgDialog.popup();
};

/**
 * Return the header list used in the list of delegated admins.
 * @return {Array}
 */
ZxAdmin.getDelegatedAdminsHeaderList = function() {
  var headerList = [];
  // function(idPrefix, text, iconInfo, width,
  // sortable, sortField, resizeable, visible)
  headerList.push(new ZaListHeaderItem(ZxAdmin.KEY_COL_ACCOUNT, zxMsg('LBL_Account'), null, 'auto',
    true, ZxAdmin.KEY_COL_ACCOUNT, false, true));
  headerList.push(new ZaListHeaderItem(ZxAdmin.KEY_COL_DOMAIN, zxMsg('LBL_Domain'), null, '350px',
    false, ZxAdmin.KEY_COL_DOMAIN, false, true));
  headerList.push(new ZaListHeaderItem(ZxAdmin.KEY_COL_VIEW_MAIL, zxMsg('LBL_DLG_delegated_auth'), null, '150px',
    false, ZxAdmin.KEY_COL_VIEW_MAIL, false, true));
  headerList.push(new ZaListHeaderItem(ZxAdmin.KEY_COL_EDIT_FEATURES, zxMsg('LBL_DLG_edit_features'), null, '150px',
    false, ZxAdmin.KEY_COL_EDIT_FEATURES, false, true));
  headerList.push(new ZaListHeaderItem(ZxAdmin.KEY_COL_QUOTA, zxMsg('LBL_DLG_grant_limit'), null, '150px',
    false, ZxAdmin.KEY_COL_QUOTA, false, true));

  return headerList;
};

/**
 * Get the list of the buttons to display below the delegated admins list.
 * @return {ZxListViewButton[]}
 */
ZxAdmin.getDelegatedAdminsListButtons = function() {
  var array = [];

  array.push(new ZxListViewButton(
    'Add',
    ZaMsg.NAD_Add,
    ZxAdmin.doAddDelegatedAdmin
  ));

  array.push(new ZxListViewButton(
    'Edit',
    ZaMsg.TBB_Edit,
    ZxAdmin.doEditDelegatedAdmin
  ));

  array.push(new ZxListViewButton(
    'Delete',
    ZaMsg.TBB_Delete,
    ZxAdmin.doDeleteDelegatedAdmin
  ));

  array.push(new ZxListViewButton(
    'Copy',
    zxMsg('LBL_Duplicate'),
    ZxAdmin.doDuplicateDelegatedAdmin
  ));

  return array;
};

/**
 * Return the header list used for the Domain List.
 * @return {ZaListHeaderItem[]}
 */
ZxAdmin.getDomainHeaderList = function() {
  var hl = [];

  hl.push(new ZaListHeaderItem(ZxAdmin.KEY_COL_DOMAIN, zxMsg('LBL_Domain'), null, 'auto',
    true, ZxAdmin.KEY_COL_DOMAIN, false, true));
  hl.push(new ZaListHeaderItem(ZxAdmin.KEY_COL_COS_NUM, zxMsg('LBL_DLG_COS_Number'), null, '100px',
    true, ZxAdmin.KEY_COL_COS_NUM, false, true));
  hl.push(new ZaListHeaderItem(ZxAdmin.KEY_COL_GLOBAL_LIMIT, zxMsg('LBL_DLG_Global_Limit'), null, '200px',
    true, ZxAdmin.KEY_COL_GLOBAL_LIMIT, false, true));
  hl.push(new ZaListHeaderItem(ZxAdmin.KEY_COL_QUOTA, zxMsg('LBL_DLG_Domain_Quota'), null, '200px',
    true, ZxAdmin.KEY_COL_QUOTA, false, true));

  return hl;
};

/**
 * Return the header list used into the Delegated admins view to show the quota usage list view.
 * @return {ZaListHeaderItem[]}
 */
ZxAdmin.getQuoteHeaderList = function() {
  var hl = [],
    sortable = 1;
  hl.push(new ZaListHeaderItem(ZaServerMBXStatsPage.XFORM_ITEM_ACCOUNT, ZaMsg.MBXStats_ACCOUNT,
    null, 250, false, ZaServerMBXStatsPage.XFORM_ITEM_ACCOUNT, true, true));
  hl.push(new ZaListHeaderItem(ZaServerMBXStatsPage.XFORM_ITEM_QUOTA, ZaMsg.MBXStats_QUOTA,
    null, 120, false, ZaServerMBXStatsPage.XFORM_ITEM_QUOTA, true, true));
  hl.push(new ZaListHeaderItem(ZaServerMBXStatsPage.XFORM_ITEM_DISKUSAGE, ZaMsg.MBXStats_DISKUSAGE,
    null, 200, false, ZaServerMBXStatsPage.XFORM_ITEM_DISKUSAGE, true, true));
  hl.push(new ZaListHeaderItem(ZaServerMBXStatsPage.XFORM_ITEM_QUOTAUSAGE, ZaMsg.MBXStats_QUOTAUSAGE,
    null, 'auto', false, ZaServerMBXStatsPage.XFORM_ITEM_QUOTAUSAGE, false, true));
  return hl;
};

/**
 * Get the list of the buttons to display below the domain list.
 * @return {ZxListViewButton[]}
 */
ZxAdmin.getDomainListButtons = function() {
  var array = [];

  array.push(new ZxListViewButton(
    'Reset',
    ZaMsg.bt_reset,
    ZxAdmin.doResetDomain
  ));

  array.push(new ZxListViewButton(
    'Edit',
    ZaMsg.TBB_Edit,
    ZxAdmin.doEditDomain
  ));

  return array;
};

/**
 * Open the window for reset the admin request.
 * @this ZxCore
 */
ZxAdmin.doResetAdminSettings = function() {
  var item = ZeXtras.getModuleItem(ZxAdmin, this.getForm().getInstance().id),
    wnd = new ZxAdminResetSettingsDialog(item);
  wnd.popup();
};

/**
 * Handle the selection on the domain.
 * @param {DwtSelectionEvent} ev
 */
ZxAdmin._onDomainSelected = function(ev) {
  if (ev.detail !== DwtListView.ITEM_DBL_CLICKED) { return; }
  ZxAdmin.doEditDomain.call(this, ev, [ev.item]);
};

/**
 * Handle the selection on delegated admins.
 * @param {DwtSelectionEvent} ev
 */
ZxAdmin._onDelegatedAdminsSelected = function(ev) {
  if (ev.detail !== DwtListView.ITEM_DBL_CLICKED) { return; }
  ZxAdmin.doEditDelegatedAdmin.call(this, ev, [ev.item]);
};
