/*
 * ***** BEGIN LICENSE BLOCK *****
 * Copyright (C) 2011-2016 ZeXtras
 *
 * The contents of this file are subject to the ZeXtras EULA;
 * you may not use this file except in compliance with the EULA.
 * You may obtain a copy of the EULA at
 * http://www.zextras.com/zextras-eula.html
 * ***** END LICENSE BLOCK *****
 */
/**
 * Account Tab Modifier for ZxAdmin module.
 * @constructor
 */
function ZxAdminAccountModifier() {}

ZxAdminAccountModifier.NEW_PANEL = false;

ZxAdminAccountModifier.KEY_DOMAINS = 'domainSettings';
ZxAdminAccountModifier.COL_KEY_ADMIN = 'admin';
ZxAdminAccountModifier.COL_KEY_DOMAIN = 'domain';
ZxAdminAccountModifier.COL_KEY_DELEGATED_ACCESS = 'viewMail';
ZxAdminAccountModifier.COL_KEY_ADMIN_QUOTA = 'adminQuota';

ZxAdminAccountModifier.MODEL = [
  {type: _LIST_, id: ZxAdminAccountModifier.KEY_DOMAINS,
    ref: 'attrs/accountSettings/' + ZxAdminAccountModifier.KEY_DOMAINS,
    isZX: true}
];

ZxAdminAccountModifier.ATTRS = [
  ZxAdminAccountModifier.KEY_DOMAINS
];
ZxAdminAccountModifier.RIGHTS = [];

ZxAdminAccountModifier.getDelegatedAdminsHeaderList = function() {
  var headerList = [];
  headerList.push(new ZaListHeaderItem(ZxAdminAccountModifier.COL_KEY_ADMIN,
    zxMsg('LBL_DLG_Admin'), null, '180px', null, null, true, true));
  headerList.push(new ZaListHeaderItem(ZxAdminAccountModifier.COL_KEY_DELEGATED_ACCESS,
    zxMsg('LBL_DLG_delegated_auth'), null, '200px', null, null, true, true));
  headerList.push(new ZaListHeaderItem(ZxAdmin.KEY_COL_EDIT_FEATURES, zxMsg('LBL_DLG_edit_features'), null, '150px',
    false, ZxAdmin.KEY_COL_EDIT_FEATURES, false, true));
  headerList.push(new ZaListHeaderItem(ZxAdminAccountModifier.COL_KEY_ADMIN_QUOTA,
    zxMsg('LBL_DLG_grant_limit'), null, '180px', null, null, true, true));
  return headerList;
};

/**
 * Function invoked by the ZxAccountModifier when the tab is
 * opened.
 * @this {ZaAccount}
 */
ZxAdminAccountModifier.LOAD_ATTRS = function() {
  if (this.attrs.zimbraIsSystemResource === 'TRUE') {
    return;
  }

  this.attrs.accountSettings = {};

  ZxAdminRequest.sendRequest(
    this.attrs[ZaAccount.A_mailHost],
    'ZxAdmin',
    'getDelegationSettings',
    {
      account: this.name
    }
  ).then(
    AjxCallback.simpleClosure(
      function(resp) {

        var respArray = resp.response,
          userDelegations = [],
          i = 0;

        for (i = 0; i < respArray.length; i = i + 1) {
          var row = respArray[i],
            obj = {};

          obj[ZxAdminAccountModifier.COL_KEY_ADMIN] = row.admin;
          obj[ZxAdminAccountModifier.COL_KEY_DOMAIN] = row.domain;
          obj[ZxAdmin.KEY_COL_EDIT_FEATURES] = (row.editFeatures ? zxMsg('LBL_Enabled') : zxMsg('LBL_Disabled'));
          obj[ZxAdminAccountModifier.COL_KEY_DELEGATED_ACCESS] =
            row.viewMail ? zxMsg('LBL_Enabled') : zxMsg('LBL_Disabled');
          obj[ZxAdminAccountModifier.COL_KEY_ADMIN_QUOTA] =
            (row.adminQuota <= 0) ? zxMsg('LBL_Unlimited') : ZxUtils.bytesToSize(row.adminQuota, 0);

          userDelegations.push(obj);
        }

        ZxFormModifier.setInstanceValue(
          this,
          userDelegations,
          'attrs/accountSettings/' + ZxAdminAccountModifier.KEY_DOMAINS
        );

      },
      this
    ),
    ZxUtils.catchError
  );
};

ZxAdminAccountModifier.SAVE_ATTRS = function(prop, val) {
  ZxUtils.log({prop: prop, val: val}, 'Saving Value:');
};

ZxAdminAccountModifier.XFORM = {
  type: _ZA_TOP_GROUPER_,
  numCols: 2,
  width: '100%',
  colSizes: ['160px', '*'],
  label: zxMsg('LBL_DLG_admin_preferences'),
  items: [
    {type: _OUTPUT_, height: 120, cssClass: 'ZxAdminLogo'},
    {
      type: _GROUP_,
      numCols: 1,
      colSizes: ['*'],
      items: [
        {
          type: _OUTPUT_,
          ref: 'attrs/accountSettings/' + ZxAdminAccountModifier.KEY_DOMAINS,
          bmolsnr: true,
          getDisplayValue: function(val) {
            if (ZxUtils.isArray(val) && val.length <= 0) {
              return zxMsg('MSG_DLG_user_is_not_delegated_admin');
            } else {
              return '';
            }
          }
        },
        {
          type: _ZX_LIST_VIEW_,
          ref: 'attrs/accountSettings/' + ZxAdminAccountModifier.KEY_DOMAINS,
          headerList: ZxAdminAccountModifier.getDelegatedAdminsHeaderList(),
          filterEnabled: false,
          navigationEnabled: false,
          hideIfEmpty: true
        }
      ]
    }
  ]
};
