/*
 * ***** BEGIN LICENSE BLOCK *****
 * Copyright (C) 2011-2016 ZeXtras
 *
 * The contents of this file are subject to the ZeXtras EULA;
 * you may not use this file except in compliance with the EULA.
 * You may obtain a copy of the EULA at
 * http://www.zextras.com/zextras-eula.html
 * ***** END LICENSE BLOCK *****
 */
/**
 * COS Tab Modifier for ZxAdmin module.
 * @constructor
 */
function ZxAdminCOSModifier() {}

ZxAdminCOSModifier.NEW_PANEL = false;

ZxAdminCOSModifier.KEY_DOMAINS = 'zxDomainsLimits';

ZxAdminCOSModifier.COL_KEY_DOMAIN = 'domain';
ZxAdminCOSModifier.COL_KEY_LIMIT = 'limit';

ZxAdminCOSModifier.MODEL = [
  {type: _LIST_, id: ZxAdminCOSModifier.KEY_DOMAINS,
    ref: ZxAdminCOSModifier.KEY_DOMAINS,
    isZX: true}
];

ZxAdminCOSModifier.ATTRS = [
//  ZxChat.KEY_USR_ZIMLET_STATUS
];
ZxAdminCOSModifier.RIGHTS = [];

ZxAdminCOSModifier.getDomainsLimitsHeaderList = function() {
  var headerList = [];
  headerList.push(new ZaListHeaderItem(ZxAdminCOSModifier.COL_KEY_DOMAIN,
    zxMsg('LBL_Domain'), null, '180px', null, null, true, true));
  headerList.push(new ZaListHeaderItem(ZxAdminCOSModifier.COL_KEY_LIMIT,
    zxMsg('LBL_Limits'), null, '200px', null, null, true, true));
  return headerList;
};

/**
 * Function invoked by the ZxAccountModifier when the tab is
 * opened.
 * @this {ZaAccount}
 */
ZxAdminCOSModifier.LOAD_ATTRS = function() {
  this[ZxAdminCOSModifier.KEY_DOMAINS] = [];

  ZxAdminRequest.sendRequest(
    ZxAdminRequest.TARGET_LOCAL,
    'ZxAdmin',
    'getDomainSettings',
    {
      showHasUsers: this.name
    }
  ).then(
    AjxCallback.simpleClosure(
      function(resp) {
        var response = resp.response,
          tmpArr = [],
          i;

        for (i = 0; i < response.length; i = i + 1) {
          if (!response[i].hasOwnProperty('cosLimits')) { continue; }
          if (!response[i].hasUsers) { continue; }
          var j;
          for (j = 0; j < response[i].cosLimits.length; j = j + 1) {
            if (response[i].cosLimits[j].cosName !== this.name) { continue; }

            var tmpLimit = response[i].cosLimits[j].cosLimit,
              tmpObj = {};

            if (tmpLimit === '') { continue; }
            tmpObj[ZxAdminCOSModifier.COL_KEY_DOMAIN] = response[i].domain;
            tmpObj[ZxAdminCOSModifier.COL_KEY_LIMIT] = tmpLimit;
            tmpArr.push(tmpObj);
          }
        }
        ZxFormModifier.setInstanceValue(
          this,
          tmpArr,
          ZxAdminCOSModifier.KEY_DOMAINS
        );
      },
      this
    ),
    ZxUtils.catchError
  );
};

ZxAdminCOSModifier.SAVE_ATTRS = function(prop, val) {
  ZxUtils.log({prop: prop, val: val}, 'Saving Value:');
};

ZxAdminCOSModifier.XFORM = {
  type: _ZA_TOP_GROUPER_,
  numCols: 2,
  width: '100%',
  colSizes: ['160px', '*'],
  label: zxMsg('LBL_DLG_admin_preferences'),
  items: [
    {type: _OUTPUT_, height: 120, cssClass: 'ZxAdminLogo'},
    {
      type: _GROUP_,
      numCols: 1,
      colSizes: ['*'],
      items: [
        {
          type: _ZX_LIST_VIEW_,
          ref: ZxAdminCOSModifier.KEY_DOMAINS,
          headerList: ZxAdminCOSModifier.getDomainsLimitsHeaderList(),
          filterEnabled: false,
          navigationEnabled: false,
          hideIfEmpty: false
        }
      ]
    }
  ]
};
