/*
 * ***** BEGIN LICENSE BLOCK *****
 * Copyright (C) 2011-2016 ZeXtras
 *
 * The contents of this file are subject to the ZeXtras EULA;
 * you may not use this file except in compliance with the EULA.
 * You may obtain a copy of the EULA at
 * http://www.zextras.com/zextras-eula.html
 * ***** END LICENSE BLOCK *****
 */

/**
 * Domain Tab Modifier for ZxAdmin module.
 * @constructor
 */
function ZxAdminDomainModifier() {}

ZxAdminDomainModifier.NEW_PANEL = false;

ZxAdminDomainModifier.KEY_DOMAIN_QUOTA = 'zxDomainQuota';
ZxAdminDomainModifier.KEY_ACCOUNT_LIMIT = 'zxAccountLimit';
ZxAdminDomainModifier.KEY_COS_LIMITS = 'zxCosLimits';
ZxAdminDomainModifier.KEY_DOMAIN_ADMIN_LIMITS = 'zxAdminCosLimit';

ZxAdminDomainModifier.COL_KEY_COS_NAME = 'cosName';
ZxAdminDomainModifier.COL_KEY_COS_LIMIT = 'cosLimit';
ZxAdminDomainModifier.COL_KEY_ADMIN = 'admin';
ZxAdminDomainModifier.COL_KEY_DELEGATED_ACCESS = 'viewMail';
ZxAdminDomainModifier.COL_KEY_ADMIN_QUOTA = 'adminQuota';

ZxAdminDomainModifier.MODEL = [
  {type: _INT_, id: ZxAdminDomainModifier.KEY_DOMAIN_QUOTA,
    ref: 'attrs/' + ZxAdminDomainModifier.KEY_DOMAIN_QUOTA,
    isZX: true},
  {type: _INT_, id: ZxAdminDomainModifier.KEY_ACCOUNT_LIMIT,
    ref: 'attrs/' + ZxAdminDomainModifier.KEY_ACCOUNT_LIMIT,
    isZX: true},
  {type: _LIST_, id: ZxAdminDomainModifier.KEY_COS_LIMITS,
    ref: 'attrs/' + ZxAdminDomainModifier.KEY_COS_LIMITS,
    isZX: true},
  {type: _LIST_, id: ZxAdminDomainModifier.KEY_DOMAIN_ADMIN_LIMITS,
    ref: 'attrs/' + ZxAdminDomainModifier.KEY_DOMAIN_ADMIN_LIMITS,
    isZX: true}
];

ZxAdminDomainModifier.ATTRS = [
//  ZxAdminDomainModifier.KEY_DOMAINS
];
ZxAdminDomainModifier.RIGHTS = [];

ZxAdminDomainModifier.getLimitsHeaderList = function() {
  var headerList = [];
  headerList.push(new ZaListHeaderItem(ZxAdminDomainModifier.COL_KEY_COS_NAME,
    zxMsg('LBL_DLG_COS_Name'), null, 'auto', null, null, true, true));
  headerList.push(new ZaListHeaderItem(ZxAdminDomainModifier.COL_KEY_COS_LIMIT,
    zxMsg('LBL_DLS_COS_limit'), null, '100px', null, null, true, true));
  return headerList;
};

ZxAdminDomainModifier.getDelegatedAdminsHeaderList = function() {
  var headerList = [];
  headerList.push(new ZaListHeaderItem(ZxAdminDomainModifier.COL_KEY_ADMIN,
    zxMsg('LBL_DLG_Admin'), null, 'auto', null, null, true, true));
  headerList.push(new ZaListHeaderItem(ZxAdminDomainModifier.COL_KEY_DELEGATED_ACCESS,
    zxMsg('LBL_DLG_delegated_auth'), null, '200px', null, null, true, true));
  headerList.push(new ZaListHeaderItem(ZxAdminDomainModifier.COL_KEY_ADMIN_QUOTA,
    zxMsg('LBL_DLG_grant_limit'), null, '180px', null, null, true, true));
  return headerList;
};

/**
 * Function invoked by the ZxAccountModifier when the tab is
 * opened.
 * @this {ZaAccount}
 */
ZxAdminDomainModifier.LOAD_ATTRS = function() {
  if (this.attrs.zimbraIsSystemResource === 'TRUE') {
    return;
  }

  this.attrs[ZxAdminDomainModifier.KEY_ACCOUNT_LIMIT] = 0;
  this.attrs[ZxAdminDomainModifier.KEY_DOMAIN_QUOTA] = 0;
  this.attrs[ZxAdminDomainModifier.KEY_COS_LIMITS] = [];
  this.attrs[ZxAdminDomainModifier.KEY_DOMAIN_ADMIN_LIMITS] = [];

  ZxAdminRequest.sendRequest(
    ZxAdminRequest.TARGET_LOCAL,
    'ZxAdmin',
    'getDomainSettings'
  ).then(
    AjxCallback.simpleClosure(
      function(resp) {
        var response = resp.response,
          tmpName,
          tmpLimit,
          tmpCosLimits = [];

        for (tmpName in response) {
          if (!response.hasOwnProperty(tmpName)) { continue; }
          if (response[tmpName].domain !== this.name) { continue; }

          ZxFormModifier.setInstanceValue(
            this,
            response[tmpName].accountLimit,
            'attrs/' + ZxAdminDomainModifier.KEY_ACCOUNT_LIMIT
          );

          ZxFormModifier.setInstanceValue(
            this,
            response[tmpName].domainAccountQuota,
            'attrs/' + ZxAdminDomainModifier.KEY_DOMAIN_QUOTA
          );

          for (tmpLimit in response[tmpName].cosLimits) {
            if (!response[tmpName].cosLimits.hasOwnProperty(tmpLimit)) { continue; }
            var tmpObj = {};
            tmpObj[ZxAdminDomainModifier.COL_KEY_COS_NAME] = response[tmpName].cosLimits[tmpLimit].cosName;
            tmpObj[ZxAdminDomainModifier.COL_KEY_COS_LIMIT] = response[tmpName].cosLimits[tmpLimit].cosLimit;
            tmpCosLimits.push(tmpObj);
          }

          ZxFormModifier.setInstanceValue(
            this,
            tmpCosLimits,
            'attrs/' + ZxAdminDomainModifier.KEY_COS_LIMITS
          );
        }
      },
      this
    ),
    ZxUtils.catchError
  );

  ZxAdminRequest.sendRequest(
    ZxAdminRequest.TARGET_LOCAL,
    'ZxAdmin',
    'getDelegationSettings',
    {
      domain: this.name
    }
  ).then(
    AjxCallback.simpleClosure(
      function(resp) {
        var response = resp.response,
          tmpLimit,
          tmpList = [];

        for (tmpLimit in response) {
          if (!response.hasOwnProperty(tmpLimit)) { continue; }

          var tmpObj = {};
          tmpObj[ZxAdminDomainModifier.COL_KEY_ADMIN] = response[tmpLimit].admin;
          tmpObj[ZxAdminDomainModifier.COL_KEY_DELEGATED_ACCESS] = response[tmpLimit].viewMail;
          tmpObj[ZxAdminDomainModifier.COL_KEY_ADMIN_QUOTA] = (response[tmpLimit].adminQuota <= 0) ?
              zxMsg('LBL_Unlimited') :
              ZxUtils.bytesToSize(response[tmpLimit].adminQuota, 0);
          tmpList.push(tmpObj);
        }

        ZxFormModifier.setInstanceValue(
          this,
          tmpList,
          'attrs/' + ZxAdminDomainModifier.KEY_DOMAIN_ADMIN_LIMITS
        );

      },
      this
    ),
    ZxUtils.catchError
  );
};

ZxAdminDomainModifier.SAVE_ATTRS = function(prop, val) {
  ZxUtils.log({prop: prop, val: val}, 'Saving Value:');
};

ZxAdminDomainModifier.XFORM = {
  type: _ZA_TOP_GROUPER_,
  numCols: 2,
  width: '100%',
  colSizes: ['160px', '*'],
  label: zxMsg('LBL_DLG_admin_preferences'),
  items: [
    {type: _OUTPUT_, height: 120, cssClass: 'ZxAdminLogo'},
    {
      type: _GROUP_,
      numCols: 2,
      colSizes: [200, '*'],
      items: [
        {
          type: _OUTPUT_,
          colSpan: 2,
          label: zxMsg('LBL_DLG_account_limit_on_domain') + ': ',
          ref: 'attrs/' + ZxAdminDomainModifier.KEY_ACCOUNT_LIMIT,
          align: _LEFT_,
          bmolsnr: true,
          getDisplayValue: function(val) {
            if (!val || val === null) { return ''; }
            return (val < 0) ? zxMsg('LBL_Unlimited') : val;
          }
        },
        {
          type: _OUTPUT_,
          colSpan: 2,
          label: zxMsg('LBL_DLG_Account_Quota') + ': ',
          ref: 'attrs/' + ZxAdminDomainModifier.KEY_DOMAIN_QUOTA,
          align: _LEFT_,
          bmolsnr: true,
          getDisplayValue: function(val) {
            if ((!val || val === null) && val !== 0) { return ''; }
            return (val <= 0) ? zxMsg('LBL_Unlimited') : ZxUtils.bytesToSize(val, 0);
          },
          visibilityChecks: [ZxUtils.isZ8Up]
        },
        {
          type: _GROUP_,
          colSpan: 2,
          numCols: 1,
          colSizes: ['100%'],
          items: [
            {
              type: _ZX_LIST_VIEW_,
              ref: 'attrs/' + ZxAdminDomainModifier.KEY_COS_LIMITS,
              headerList: ZxAdminDomainModifier.getLimitsHeaderList(),
              filterEnabled: false,
              navigationEnabled: false,
              hideIfEmpty: false
            },
            {
              type: _ZX_LIST_VIEW_,
              ref: 'attrs/' + ZxAdminDomainModifier.KEY_DOMAIN_ADMIN_LIMITS,
              headerList: ZxAdminDomainModifier.getDelegatedAdminsHeaderList(),
              filterEnabled: false,
              navigationEnabled: false,
              hideIfEmpty: false
            }
          ]
        }
      ]
    }
  ]
};

