/*
 * ***** BEGIN LICENSE BLOCK *****
 * Copyright (C) 2011-2016 ZeXtras
 *
 * The contents of this file are subject to the ZeXtras EULA;
 * you may not use this file except in compliance with the EULA.
 * You may obtain a copy of the EULA at
 * http://www.zextras.com/zextras-eula.html
 * ***** END LICENSE BLOCK *****
 */

function ZxAddDelegatedAdminDialog(itemParent, rowData, mode) {
  if (arguments.length === 0) { return; }
  this._standardButtons = [DwtDialog.OK_BUTTON, DwtDialog.CANCEL_BUTTON];
  this._editmode = mode;
  this._originalData = rowData;
  this._modItem = itemParent;

  var data = ZxUtils.cloneObj(rowData),
    title = zxMsg('LBL_DLG_Add_new_delegated_admin');

  if (this._editmode === ZxAddDelegatedAdminDialog.MODE_EDIT) {
    title = zxMsg('LBL_DLG_Edit_delegated_admin');
  }

//  if (this._editmode === ZxAddDelegatedAdminDialog.MODE_DUPLICATE) {
//    data[ZxAddDelegatedAdminDialog.KEY_ACCOUNT] = '';
//  }
  if (this._editmode === ZxAddDelegatedAdminDialog.MODE_ADD) {
    data[ZxAddDelegatedAdminDialog.KEY_QUOTA_OPTION] = -1;
    data[ZxAdmin.KEY_COL_QUOTA + '_row'] = -1;
  }

  if (data[ZxAdmin.KEY_COL_QUOTA + '_row'] < 0 ||
      this._editmode === ZxAddDelegatedAdminDialog.MODE_NEW) {

    data[ZxAddDelegatedAdminDialog.KEY_QUOTA_OPTION] = -1;

  } else if (data[ZxAdmin.KEY_COL_QUOTA + '_row'] === 0) {
    data[ZxAddDelegatedAdminDialog.KEY_QUOTA_OPTION] = 0;

  } else {
    data[ZxAddDelegatedAdminDialog.KEY_QUOTA_OPTION] = 1;
  }

  ZaXDialog.call(
    this,
    itemParent.getCurrentController().getShell(),
    null,
    ZxUtils.getTitleWithIcon('ZxAdmin', title),
    '320px',
    '180px',
    null,
    null
  );

  this.initForm([], this.getMyXForm());
  this.setObject(data);

  this.registerCallback(
    DwtDialog.OK_BUTTON,
    new AjxCallback(
      this,
      this._okBtnCallback
    )
  );

  if (mode === ZxAddDelegatedAdminDialog.MODE_ADD ||
      mode === ZxAddDelegatedAdminDialog.MODE_DUPLICATE) {

    this._helpURL = ZxLinks.HELP_DLG_Add_delegated_admin;

  } else {

    this._helpURL = ZxLinks.HELP_DLG_Modify_delegated_admin;
  }
}

ZxAddDelegatedAdminDialog.prototype = new ZaXDialog();
ZxAddDelegatedAdminDialog.prototype.constructor = ZxAddDelegatedAdminDialog;

ZxAddDelegatedAdminDialog.MODE_ADD = 1;
ZxAddDelegatedAdminDialog.MODE_EDIT = 2;
ZxAddDelegatedAdminDialog.MODE_DUPLICATE = 3;

ZxAddDelegatedAdminDialog._MODE_CALLS = {};
ZxAddDelegatedAdminDialog._MODE_CALLS[ZxAddDelegatedAdminDialog.MODE_ADD] = 'ZxDoAddDelegationSettings';
ZxAddDelegatedAdminDialog._MODE_CALLS[ZxAddDelegatedAdminDialog.MODE_EDIT] = 'ZxDoModifyDelegationSettings';
ZxAddDelegatedAdminDialog._MODE_CALLS[ZxAddDelegatedAdminDialog.MODE_DUPLICATE] = 'ZxDoAddDelegationSettings';

ZxAddDelegatedAdminDialog.KEY_QUOTA_OPTION = 'adminQuotaOption';
ZxAddDelegatedAdminDialog.KEY_VIEW_MAIL = ZxAdmin.KEY_COL_VIEW_MAIL;
ZxAddDelegatedAdminDialog.KEY_DOMAIN = ZxAdmin.KEY_COL_DOMAIN;
ZxAddDelegatedAdminDialog.KEY_ACCOUNT = ZxAdmin.KEY_COL_ACCOUNT;
ZxAddDelegatedAdminDialog.KEY_EDIT_FEATURES = ZxAdmin.KEY_COL_EDIT_FEATURES;
ZxAddDelegatedAdminDialog.KEY_QUOTA = ZxAdmin.KEY_COL_QUOTA;

ZxAddDelegatedAdminDialog.ID_ADMIN_QUOTA = 'adminQuotaId';
ZxAddDelegatedAdminDialog.ID_ADMIN_QUOTA_OPTION = 'adminQuotaOptionId';
ZxAddDelegatedAdminDialog.ID_ADMIN_QUOTA_UNIT = 'adminQuotaUnitId';

ZxAddDelegatedAdminDialog.QUOTA_OPTIONS = [
  {value: -1, label: ZaMsg.L_none},
  {value: 1, label: zxMsg('LBL_Custom')},
  {value: 0, label: zxMsg('LBL_Unlimited')}
];

ZxAddDelegatedAdminDialog.prototype._okBtnCallback = function() {
  var obj = this.getObject(),
    reqObj = {},
    waitingDialog;

  if (!obj[ZxAddDelegatedAdminDialog.KEY_ACCOUNT] ||
      obj[ZxAddDelegatedAdminDialog.KEY_ACCOUNT] === '') {

    this._modItem.getCurrentController.getShell().popupWarningDialog(
      zxMsg('LBL_DLG_No_account_specified')
    );
    return;
  }
  reqObj.account = obj[ZxAddDelegatedAdminDialog.KEY_ACCOUNT];

  if (!obj[ZxAddDelegatedAdminDialog.KEY_DOMAIN] ||
      obj[ZxAddDelegatedAdminDialog.KEY_DOMAIN] === '') {


    this._modItem.getCurrentController.getShell().popupWarningDialog(
      zxMsg('LBL_DLG_No_domain_specified')
    );
    return;
  }
  reqObj.domain = obj[ZxAddDelegatedAdminDialog.KEY_DOMAIN];

  if (obj[ZxAddDelegatedAdminDialog.KEY_QUOTA_OPTION] < 1) {
    reqObj.adminQuota = obj[ZxAddDelegatedAdminDialog.KEY_QUOTA_OPTION];
  } else {
    reqObj.adminQuota = obj[ZxAddDelegatedAdminDialog.KEY_QUOTA];
  }

  reqObj.editFeatures = obj[ZxAddDelegatedAdminDialog.KEY_EDIT_FEATURES + '_row'];
  reqObj.viewMail = obj[ZxAddDelegatedAdminDialog.KEY_VIEW_MAIL + '_row'];
  reqObj.mode = ZxAddDelegatedAdminDialog._MODE_CALLS[this._editmode];

  ZxUtils.log(reqObj, 'Saving delegation settings');

  waitingDialog = new ZxWaitingDialog(
    this._modItem.getCurrentController().getShell(),
    '250px',
    '52px',
    zxMsg('LBL_Saving_settings'),
    ZxLinks.HELP_DLG_Admin
  );

  ZxAdminRequest.sendRequest(
    ZxAdminRequest.TARGET_LOCAL,
    'ZxAdmin',
    'doEditDelegationSettings',
    reqObj
  ).then(
    AjxCallback.simpleClosure(
      function(wdialog, resp) {
        var hasErrors = false;
        if (resp instanceof ZxError) {
          ZxUtils.catchError(resp);
          hasErrors = true;
        }
        if (!hasErrors) {
          wdialog.setMessage(zxMsg('MSG_Settings_updated_correctly'), ZxWaitingDialog.STATUS_SUCCESS);
        }
        this._modItem.fireLoadMethods();
      },
      this,
      waitingDialog
    ),
    AjxCallback.simpleClosure(
      function(err) {
        this.popdown();
        ZxUtils.catchError(err);
      },
      waitingDialog
    )
  );

  waitingDialog.popup();

  this.popdown();
};

ZxAddDelegatedAdminDialog.prototype.getMyXForm = function () {
  var xFormObject,
    enableFunction = AjxCallback.simpleClosure(
      function() {
        return this._editmode !== ZxAddDelegatedAdminDialog.MODE_EDIT;
      },
      this
    );

  xFormObject = {
    numCols: 1,
    colSizes: ['100%'],
    cssStyle: 'padding: 10px',
    items: [
      {
        type: _GROUP_,
        numCols: 1,
        colSizes: ['*'],
        isTabGroup: true,
        items: [
          {
            type: _DYNSELECT_,
            ref: ZxAddDelegatedAdminDialog.KEY_ACCOUNT,
            dataFetcherClass: ZaSearch,
            dataFetcherMethod: ZaSearch.prototype.dynSelectSearch,
            dataFetcherTypes: [ZaSearch.ACCOUNTS],
            dataFetcherAttrs: [
              ZaItem.A_zimbraId,
              ZaItem.A_cn,
              ZaAccount.A_name,
              ZaAccount.A_displayname,
              ZaAccount.A_mail
            ],
            label: zxMsg('LBL_Account') + ': ',
            editable: true,
            forceUpdate: true,
            choices: new XFormChoices([], XFormChoices.OBJECT_REFERENCE_LIST, 'name', 'name'),
            onChange: function(value, event, form) {
              if (value instanceof ZaItem) {
                this.setInstanceValue(value.name);
              } else {
                this.setInstanceValue(value);
              }
            },
            keyUp: function(value, event, form) {
              if (value instanceof ZaItem) {
                this.setInstanceValue(value.name);
              } else {
                this.setInstanceValue(value);
              }
            },
            enableDisableChecks: [enableFunction]
          },
          {
            type: _DYNSELECT_,
            ref: ZxAddDelegatedAdminDialog.KEY_DOMAIN,
            label: zxMsg('LBL_Domain') + ': ',
            toolTipContent: ZaMsg.tt_StartTypingDomainName,
            dataFetcherMethod: ZaSearch.prototype.dynSelectSearchDomains,
            dataFetcherClass: ZaSearch,
            enableDisableChecks: [enableFunction],
            keyUp: function(value, event, form) {
              if (value instanceof ZaItem) {
                this.setInstanceValue(value.name);
              } else {
                this.setInstanceValue(value);
              }
            }
          },
          {
            type: _CHECKBOX_,
            labelCssClass: 'ZxCheckboxLabel',
            labelLocation: _LEFT_,
            align: _LEFT_,
            ref: ZxAddDelegatedAdminDialog.KEY_VIEW_MAIL + '_row',
            label: zxMsg('LBL_DLG_delegated_auth') + ': '
          },
          {
            type: _CHECKBOX_,
            labelCssClass: 'ZxCheckboxLabel',
            labelLocation: _LEFT_,
            align: _LEFT_,
            ref: ZxAddDelegatedAdminDialog.KEY_EDIT_FEATURES + '_row',
            label: zxMsg('LBL_DLG_edit_features') + ': '
          },
          {
            type: _OSELECT1_,
            ref: ZxAddDelegatedAdminDialog.KEY_QUOTA_OPTION,
            label: zxMsg('LBL_DLG_grant_limit') + ': ',
            id: ZxAddDelegatedAdminDialog.ID_ADMIN_QUOTA_OPTION,
            choices: ZxAddDelegatedAdminDialog.QUOTA_OPTIONS,
            onChange: function(value, event, form) {
              this.setInstanceValue(value);

              var item = form.getItemsById(ZxAddDelegatedAdminDialog.ID_ADMIN_QUOTA)[0],
                item1 = form.getItemsById(ZxAddDelegatedAdminDialog.ID_ADMIN_QUOTA_UNIT)[0];

              if (value === 1) {
                item.show();
                item1.show();
              } else {
                item.hide();
                item1.hide();
              }
            }
          },
          {
            type: _TEXTFIELD_,
            label: zxMsg('LBL_Value') + ': ',
            ref: ZxAddDelegatedAdminDialog.KEY_QUOTA,
            id: ZxAddDelegatedAdminDialog.ID_ADMIN_QUOTA,
            bmolsnr: true,
            visibilityChecks: [
              function() {
                var option = this.getForm().getItemsById(ZxAddDelegatedAdminDialog.ID_ADMIN_QUOTA_OPTION)[0];
                return option.getInstanceValue() === 1;
              }
            ]
          }
        ]
      },
      {
        type: _OUTPUT_,
        id: ZxAddDelegatedAdminDialog.ID_ADMIN_QUOTA_UNIT,
        value: zxMsg('MSG_DLG_Quota_declaration_info'),
        visibilityChecks: [
          function() {
            var option = this.getForm().getItemsById(ZxAddDelegatedAdminDialog.ID_ADMIN_QUOTA_OPTION)[0];
            return option.getInstanceValue() === 1;
          }
        ]
      }
    ]
  };
  return xFormObject;
};
