/*
 * ***** BEGIN LICENSE BLOCK *****
 * Copyright (C) 2011-2016 ZeXtras
 *
 * The contents of this file are subject to the ZeXtras EULA;
 * you may not use this file except in compliance with the EULA.
 * You may obtain a copy of the EULA at
 * http://www.zextras.com/zextras-eula.html
 * ***** END LICENSE BLOCK *****
 */

function ZxAddModifyCOSLimitDialog(parentShell, w, h, title, parentDialog, row, newMode) {
  if (arguments.length === 0) { return; }

  this._standardButtons = [DwtDialog.OK_BUTTON, DwtDialog.CANCEL_BUTTON];
  ZaXDialog.call(this, parentShell, null, ZxUtils.getTitleWithIcon('ZxAdmin', title), w, h, null, null);

  this._newMode = newMode;
  this._parentDialog = parentDialog;
  this._oldRow = row;

  this.initForm([], this.getMyXForm());
  this.setObject(ZxUtils.cloneObj(row));

  this.registerCallback(
    DwtDialog.OK_BUTTON,
    new AjxCallback(this, this._okCallback)
  );

  this._helpURL = ZxLinks.HELP_DLG_Admin_provisioning;
}

ZxAddModifyCOSLimitDialog.prototype = new ZaXDialog();
ZxAddModifyCOSLimitDialog.prototype.constructor = ZxAddModifyCOSLimitDialog;

ZxAddModifyCOSLimitDialog.KEY_COS_NAME = ZxAdmin.KEY_COL_COS;
ZxAddModifyCOSLimitDialog.KEY_ACCOUNT_LIMIT = ZxAdmin.KEY_COL_ACCOUNT;

ZxAddModifyCOSLimitDialog.prototype._okCallback = function() {
  var canBeClosed = true,
    coscheck = false,
    limit,
    errors = [],
    params = this.getObject(),
    cosLimits = AjxVector.fromArray(
      this._parentDialog.getObject()[ZxAdmin.KEY_COL_COS_LIMIT]
    );

  if (!this._newMode) {
    cosLimits.remove(this._oldRow);
  }

  coscheck = (ZaCos.getCosByName(params[ZxAddModifyCOSLimitDialog.KEY_COS_NAME]) !== null);

  if (!coscheck) {
    canBeClosed = false;
    errors.push(params[ZxAddModifyCOSLimitDialog.KEY_COS_NAME] + ' ' + zxMsg('MSG_DLG_COS_Does_not_exist') + '.');
  }

  limit = Number(params[ZxAddModifyCOSLimitDialog.KEY_ACCOUNT_LIMIT]);
  if (isNaN(limit)) {
    canBeClosed = false;
    errors.push(params[ZxAddModifyCOSLimitDialog.KEY_ACCOUNT_LIMIT] + ' ' + zxMsg('MSG_DLG_is_not_a_valid_number') + '.');
  }

  if (limit === '' || limit <= 0) {
    canBeClosed = false;
    errors.push(zxMsg('MSG_DLG_Invalid_account_limit'));
  }

  if (canBeClosed) {

    var tmpObj = {};
    tmpObj[ZxAdmin.KEY_COL_COS] = params[ZxAddModifyCOSLimitDialog.KEY_COS_NAME];
    tmpObj[ZxAdmin.KEY_COL_ACCOUNT] = limit;
    cosLimits.add(tmpObj);

    this._parentDialog._localXForm.setInstanceValue(
      cosLimits.getArray(),
      ZxEditDomainDelegationSettingsDialog.KEY_COS_LIMITS
    );

    this.popdown();

  } else {

    ZaApp.getInstance().getCurrentController().popupErrorDialog(errors.join('<br>'));

  }

};

ZxAddModifyCOSLimitDialog.prototype.getMyXForm = function () {
  var newMode = this._newMode,
    xFormObject;

  xFormObject = {
    numCols: 1,
    colSizes: ['100%'],
    items: [
      {
        type: _GROUP_,
        numCols: 1,
        colSizes: ['*'],
        cssStyle: 'margin-top: 5px; margin-left: 5px',
        isTabGroup: true,
        items: [
          {
            type: _DYNSELECT_,
            ref: ZxAddModifyCOSLimitDialog.KEY_COS_NAME,
            label: zxMsg('LBL_DLG_COS_Name') + ': ',
            toolTipContent: ZaMsg.tt_StartTypingCOSName,
            dataFetcherMethod: ZaSearch.prototype.dynSelectSearchCoses,
            dataFetcherClass: ZaSearch,
            enableDisableChecks: [
              function() {
                return newMode;
              }
            ]
          },
          {
            type: _TEXTFIELD_,
            ref: ZxAddModifyCOSLimitDialog.KEY_ACCOUNT_LIMIT,
            label: zxMsg('LBL_DLG_COS_Account_limit') + ': '
          }
        ]
      }
    ]
  };
  return xFormObject;
};
