/*
 * ***** BEGIN LICENSE BLOCK *****
 * Copyright (C) 2011-2016 ZeXtras
 *
 * The contents of this file are subject to the ZeXtras EULA;
 * you may not use this file except in compliance with the EULA.
 * You may obtain a copy of the EULA at
 * http://www.zextras.com/zextras-eula.html
 * ***** END LICENSE BLOCK *****
 */
/**
 *
 * @param {DwtShell} parent
 * @param {ZxAdminBrowseLogOp} logDetails
 * @constructor
 */
function ZxAdminLogDetailsDialog(parent, logDetails) {
  this._item = logDetails;

  var closeButton,
    title;

  title = ZxUtils.getTitleWithIcon('ZxAdmin', zxMsg('LBL_DLG_browse_logs'));

  closeButton = new DwtDialog_ButtonDescriptor(
    ZxAdminLogDetailsDialog.ID_BTN_CLOSE,
    ZaMsg.TBB_Close,
    DwtDialog.ALIGN_RIGHT,
    new AjxCallback(
      this,
      function() {
        this.popdown();
      }
    )
  );

  this._standardButtons = [DwtDialog.NO_BUTTONS];
  this._extraButtons = [closeButton];

  ZaXDialog.call(this, parent, null, title, '300px', '200px', null, null);
  this.initForm(ZxAdminLogDetailsDialog.MODEL, this.getMyXForm());
  this._setAllowSelection();
  this.setObject(this._item || {});
}

ZxAdminLogDetailsDialog.prototype = new ZaXDialog();
ZxAdminLogDetailsDialog.prototype.constructor = ZxAdminLogDetailsDialog;

ZxAdminLogDetailsDialog.ID_BTN_CLOSE = 'closeBTN';
ZxAdminLogDetailsDialog.KEY_ADMIN = 'admin';
ZxAdminLogDetailsDialog.KEY_ACTION = 'action';
ZxAdminLogDetailsDialog.KEY_TIME = 'time';
ZxAdminLogDetailsDialog.KEY_CLIENT_IP = 'clientIP';
ZxAdminLogDetailsDialog.KEY_INFO = 'info';
ZxAdminLogDetailsDialog.KEY_SUCCESS = 'success';

ZxAdminLogDetailsDialog.getInfoDetails = function(val) {
  var tmpString = [],
    obj;
  if (!val || ZxUtils.isObjectEmpty(val)) {
    return '<div>&nbsp;Data not found.</div>';
  }

  tmpString.push('<br>');

  for (obj in val) {
    if (!val.hasOwnProperty(obj)) { continue; }
    if (!val[obj] || val[obj] === '') {
      tmpString.push('&nbsp;' + obj + ': <span style="color: #FF0000">' + zxMsg('LBL_Removed') + '</span>');
    } else {
      tmpString.push('&nbsp;' + obj + ': ' + val[obj]);
    }
  }

  return tmpString.join('<br>');
};

ZxAdminLogDetailsDialog.getSuccessText = function(val) {
  if (val === true) {
    return 'Yes';
  } else {
    return 'No';
  }
};



ZxAdminLogDetailsDialog.MODEL = [
  {type: _STRING_, id: ZxAdminLogDetailsDialog.KEY_ADMIN,
    ref: ZxAdminLogDetailsDialog.KEY_ADMIN},
  {type: _STRING_, id: ZxAdminLogDetailsDialog.KEY_ACTION,
    ref: ZxAdminLogDetailsDialog.KEY_ACTION},
  {type: _STRING_, id: ZxAdminLogDetailsDialog.KEY_TIME,
    ref: ZxAdminLogDetailsDialog.KEY_TIME},
  {type: _STRING_, id: ZxAdminLogDetailsDialog.KEY_CLIENT_IP,
    ref: ZxAdminLogDetailsDialog.KEY_CLIENT_IP},
  {type: _STRING_, id: ZxAdminLogDetailsDialog.KEY_INFO,
    ref: ZxAdminLogDetailsDialog.KEY_INFO},
  {type: _STRING_, id: ZxAdminLogDetailsDialog.KEY_SUCCESS,
    ref: ZxAdminLogDetailsDialog.KEY_SUCCESS}
];

ZxAdminLogDetailsDialog.prototype.getMyXForm = function() {
  var xFormObject;

  xFormObject = {
    numCols: 1,
    colSizes: ['*'],
    cssStyle: 'padding: 10px',
    items: [
      {
        type: _GROUP_,
        numCols: 2,
        colSpan: 2,
        colSizes: [50, 250],
        items: [
          {
            type: _OUTPUT_,
            label: zxMsg('LBL_DLG_Admin') + ': ',
            ref: ZxAdminLogDetailsDialog.KEY_ADMIN,
            labelLocation: _LEFT_
          },
          {
            type: _OUTPUT_,
            label: zxMsg('LBL_DLG_Action') + ': ',
            ref: ZxAdminLogDetailsDialog.KEY_ACTION,
            labelLocation: _LEFT_
          },
          {
            type: _OUTPUT_,
            label: zxMsg('LBL_DLG_Time') + ': ',
            ref: ZxAdminLogDetailsDialog.KEY_TIME,
            labelLocation: _LEFT_
          },
          {
            type: _OUTPUT_,
            label: zxMsg('LBL_DLG_Client_IP') + ': ',
            ref: ZxAdminLogDetailsDialog.KEY_CLIENT_IP,
            labelLocation: _LEFT_
          },
          {
            type: _OUTPUT_,
            label: 'Success: ',
            ref: ZxAdminLogDetailsDialog.KEY_SUCCESS,
            labelLocation: _LEFT_,
            getDisplayValue: ZxAdminLogDetailsDialog.getSuccessText
          },
          {
            type: _OUTPUT_,
            label: 'Info: ',
            ref: ZxAdminLogDetailsDialog.KEY_INFO,
            labelLocation: _LEFT_,
            valign: _TOP_,
            getDisplayValue: ZxAdminLogDetailsDialog.getInfoDetails
          }
        ]
      }
    ]
  };

  return xFormObject;
};
