/*
 * ***** BEGIN LICENSE BLOCK *****
 * Copyright (C) 2011-2016 ZeXtras
 *
 * The contents of this file are subject to the ZeXtras EULA;
 * you may not use this file except in compliance with the EULA.
 * You may obtain a copy of the EULA at
 * http://www.zextras.com/zextras-eula.html
 * ***** END LICENSE BLOCK *****
 */

function ZxAdminReportDialog(modItem) {
  this._standardButtons = [DwtDialog.NO_BUTTONS];

  this._modItem = modItem;
  this._currentYear = new Date().getFullYear();

  var today = new Date(),
    month = today.getMonth(),
    year = today.getFullYear(),
    currentObj = {},
    cancelButton,
    showReportButton,
    helpButton;

  helpButton = new DwtDialog_ButtonDescriptor(
    ZxAdminReportDialog.ID_BTN_HELP,
    ZaMsg.TBB_Help,
    DwtDialog.ALIGN_LEFT,
    new AjxCallback(
      this,
      ZxUtils.openURL,
      [ZxLinks.HELP_DLG_Dialog_Browse_log]
    )
  );

  cancelButton = new DwtDialog_ButtonDescriptor(
    ZxAdminReportDialog.ID_BTN_DCANCEL,
    ZaMsg.TBB_Close,
    DwtDialog.ALIGN_RIGHT,
    new AjxCallback(
      this,
      this.popdown
    )
  );

  showReportButton = new DwtDialog_ButtonDescriptor(
    ZxAdminReportDialog.ID_BTN_DO_REPORT,
    zxMsg('LBL_DLG_Show_report'),
    DwtDialog.ALIGN_RIGHT,
    new AjxCallback(
      this,
      this._showReport
    )
  );

  this._extraButtons = [helpButton, cancelButton, showReportButton];

  ZaXDialog.call(
    this,
    modItem.getCurrentController().getShell(),
    null,
    ZxUtils.getTitleWithIcon('ZxAdmin', zxMsg('LBL_DLG_Monthly_Report')),
    '571px',
    '430px',
    null,
    null
  );

  this.initForm([], this.getMyXForm());

  month = month - 1;
  if (month < 0) {
    month = 11;
    year = year - 1;
  }
  currentObj[ZxAdminReportDialog.KEY_MONTH] = month;
  currentObj[ZxAdminReportDialog.KEY_YEAR] = year;
  this.setObject(currentObj);

  this._helpURL = ZxLinks.HELP_DLG_Dialog_Browse_log;
}

ZxAdminReportDialog.prototype = new ZaXDialog();
ZxAdminReportDialog.prototype.constructor = ZxAdminReportDialog;

ZxAdminReportDialog.ID_BTN_HELP = 'helpBtn';
ZxAdminReportDialog.ID_BTN_DO_REPORT = 'showReportButtonId';
ZxAdminReportDialog.ID_BTN_DCANCEL = 'filtercancelButtonId';

ZxAdminReportDialog.KEY_MONTH = 'month';
ZxAdminReportDialog.KEY_COMPLETE_MONTH = 'complete_month';
ZxAdminReportDialog.KEY_YEAR = 'year';
ZxAdminReportDialog.KEY_HIDDEN= 'hidden';

ZxAdminReportDialog.STR_REPORT_NOT_FOUND = 'report not found';
ZxAdminReportDialog.STR_MODULE_DISABLED = 'Request ignored since module is disabled';

ZxAdminReportDialog.STYLES = {
  BLACK: ['<span style="color:#000000; font-weight:bold;">', '</span>'],
  RED: ['<span style="color:#ff0000; font-weight:bold;">', '</span>'],
  ORANGE: ['<span style="color:#FF8000; font-weight:bold;">', '</span>'],
  GREEN: ['<span style="color:#008000; font-weight:bold;">', '</span>'],
  PURPLE: ['<span style="color:#800080; font-weight:bold;">', '</span>'],
  BLUE: ['<span style="color:#1919FF; font-weight:bold;">', '</span>']
};

ZxAdminReportDialog.prototype._showReport = function() {
  var obj = this.getObject();

  ZxAdminRequest.sendRequest(
    ZxAdminRequest.TARGET_LOCAL,
    'ZxAdmin',
    'getMonthlyReport',
    {
      month: [obj[ZxAdminReportDialog.KEY_MONTH] + 1, obj[ZxAdminReportDialog.KEY_YEAR]].join('/'),
      json: true
    }
  ).then(
    AjxCallback.simpleClosure(
      this._updateDialog,
      this
    ),
    AjxCallback.simpleClosure(
      this._updateDialog,
      this
    )
  );
};

ZxAdminReportDialog.prototype._updateDialog = function (response) {

  if (!(response instanceof ZxError)) {

    var respObj = response.message;
    ZxUtils.log(respObj, 'Received log');
//  this.setSize(400, 380);
    this.initForm([], this.getReportXForm(respObj));
    this.setObject(respObj);

    this.getButton(ZxAdminReportDialog.ID_BTN_DO_REPORT).setVisible(false);

  } else {

    var error = response;

    if (error.getCode() === ZxErrorCode.REPORT_NOT_FOUND) {
      var dialog;

      if (ZxUtils.amIDelegated()) {
        dialog = new ZaMsgDialog(
          this._modItem.getCurrentController().getShell(),
          null,
          [DwtDialog.DISMISS_BUTTON]
        );
        dialog.setMessage(
          zxMsg('MSG_DLG_No_report_found_for_month_delegated'),
          DwtMessageDialog.INFO_STYLE,
          zxMsg('LBL_DLG_Admins_reports')
        );
      } else {
        dialog = new ZaMsgDialog(
          this._modItem.getCurrentController().getShell(),
          null,
          [DwtDialog.YES_BUTTON, DwtDialog.NO_BUTTON]
        );
        dialog.setMessage(
          zxMsg('MSG_DLG_No_report_found_for_month',
            ['<a href="' + ZxLinks.HELP_DLG_About_Reports + '" target="_blank">', '</a>']),
          DwtMessageDialog.INFO_STYLE,
          zxMsg('LBL_DLG_Generate_on_the_fly')
        );
        dialog.registerCallback(
          DwtDialog.YES_BUTTON,
          new AjxCallback(
            this,
            function(dialog) {
              var obj = this.getObject();

              ZxAdminRequest.sendRequest(
                ZxAdminRequest.TARGET_ALL,
                'ZxAdmin',
                'doMonthlyReport',
                {
                  json: true,
                  month: [obj[ZxAdminReportDialog.KEY_MONTH] + 1, obj[ZxAdminReportDialog.KEY_YEAR]].join('/'),
                  waitandshow: false
                }
              ).then(
                AjxCallback.simpleClosure(
                  function(resp) {
                    var server, hasErrors = false;
                    for (server in resp) {
                      if (!resp.hasOwnProperty(server)) {
                        continue;
                      }
                      if (resp[server] instanceof ZxError) {
                        ZxUtils.catchError(resp[server]);
                        hasErrors = true;
                      }
                    }
                    if (!hasErrors) {
                     ZaApp.getInstance().getCurrentController().popupMsgDialog(
                       zxMsg('MSG_Operation_queued')
                     );
                    }
                  },
                  this
                ),
                ZxUtils.catchError
              );

              dialog.popdown();
            },
            [dialog]
          ),
          this,
          []
        );
      }

      this.popdown();
      dialog.popup();

    } else {
// TODO: Fixme
//    } else if (error.getCode() === ZxAdminReportDialog.STR_MODULE_DISABLED) {
      ZeXtrasItem.showMessageDialog(zxMsg('MSG_DLG_Cannot_show_report_module_disabled'));
      this.popdown();
    }
  }
};

ZxAdminReportDialog.prototype.getReportXForm = function(reportData) {
  var xFormObject, systemLogin = 0, domainItems = [], tmpDomainName, tmpDomainInfo,
    adminItems = [], tmpAdminName, tmpAdminInfo,
    returnMeFunction = function(me) { return me; };

  // Domain Reports
  for (tmpDomainName in reportData.domain_activity) {
    if (!reportData.domain_activity.hasOwnProperty(tmpDomainName)) { continue; }
    tmpDomainInfo = reportData.domain_activity[tmpDomainName];

    //for reports generated before version 1.8.1
    var maxAccounts = zxMsg('LBL_Unlimited'),
      domainDataPresent = tmpDomainInfo.tot_accounts_on_domain !== 0 ||
        tmpDomainInfo.tot_accounts_with_unlimited_space !== 0 ||
        tmpDomainInfo.tot_available_space_on_domain !== 0 ||
        tmpDomainInfo.tot_used_space_on_domain !== 0 ||
        tmpDomainInfo.tot_used_space_on_domain_by_unlimited_users !== 0;

    if (tmpDomainInfo.domain_data_computed || domainDataPresent) { //for reports generated before version 1.8.1

      maxAccounts = tmpDomainInfo.max_accounts_on_domain < 0 ?
          zxMsg('LBL_Unlimited') :
          tmpDomainInfo.max_accounts_on_domain;

//      var perc;
//      var style, accTagStyle = tagStyle;
//
//      if(tmpDomainInfo.max_accounts_on_domain < 0) {
////        style = okStyle;
//      } else if(tmpDomainInfo.max_accounts_on_domain === 0){
////        style = critStyle;
////        accTagStyle = redTagStyle;
//      } else {
//        perc = (100 * tmpDomainInfo.tot_accounts_on_domain) / tmpDomainInfo.max_accounts_on_domain;
//        if(perc > 100) {
////          style = critStyle;
////          accTagStyle = redTagStyle;
//        } else if(perc > 95) {
////          style = warnStyle;
//        } else {
////          style = okStyle;
//        }
//      }
    }

    domainItems.push({
      type: _ZAWIZ_TOP_GROUPER_,
      label: tmpDomainName,
      colSpan: 2,
      numCols: 2,
      width: 528,
      colSizes: [203, 325],
      items: [
//        {
//          type: _OUTPUT_,
//          label: zxMsg('LBL_Domain') + ': ',
//          value: tmpDomainName
//        },
        {
          type: _OUTPUT_,
          label: zxMsg('LBL_DLG_Last_login_by') + ': ',
          value: (tmpDomainInfo.last_successful_login_timestamp) ?
              zxMsg(
                'MSG_DLG_Last_login_by_at',
                [
                  tmpDomainInfo.last_successful_login_by_account,
                  ZxUtils.computeWordyDateStr(new Date(), tmpDomainInfo.last_successful_login_timestamp)
                ]
              ) : zxMsg('MSG_DLG_No_longins_in_month')
        },
        {
          type: _OUTPUT_,
          label: zxMsg('LBL_DLG_Most_active_admin') + ': ',
          value: (tmpDomainInfo.most_active_admin_action_count || tmpDomainInfo.most_active_admin_action_count === 0) ?
              zxMsg('MSG_DLG_Most_active_with_count',
                [tmpDomainInfo.most_active_admin, tmpDomainInfo.most_active_admin_action_count]) : zxMsg('LBL_NA'),
          visibilityChecks: [
            AjxCallback.simpleClosure(returnMeFunction, this, tmpDomainInfo.most_active_admin_action_count > 0)
          ]
        },
        {
          type: _OUTPUT_,
          label: zxMsg('LBL_DLG_Most_active_client') + ': ',
          value: (tmpDomainInfo.most_active_client_action_count || tmpDomainInfo.most_active_client_action_count === 0) ?
              zxMsg('MSG_DLG_Most_active_with_count',
                [tmpDomainInfo.most_active_client, tmpDomainInfo.most_active_client_action_count]) : zxMsg('LBL_NA'),
          visibilityChecks: [
            AjxCallback.simpleClosure(returnMeFunction, this, tmpDomainInfo.most_active_client_action_count > 0)
          ]
        },
        //
        {
          type: _OUTPUT_,
          label: zxMsg('LBL_DLG_Accounts_max_accounts') + ': ',
          value: tmpDomainInfo.tot_accounts_on_domain + ' / ' + maxAccounts
        },
        {
          type: _OUTPUT_,
          label: zxMsg('LBL_DLG_Current_domain_size') + ': ',
          value: ZxUtils.bytesToSize(tmpDomainInfo.tot_used_space_on_domain, 0) + ' / ' +
            ((tmpDomainInfo.tot_available_space_on_domain > 0) ?
                ZxUtils.bytesToSize(tmpDomainInfo.tot_available_space_on_domain, 0) :
                zxMsg('LBL_Unlimited'))
        },
        {
          type: _OUTPUT_,
          label: zxMsg('LBL_DLG_Accounts_with_no_quota_size') + ': ',
          value: ZxUtils.bytesToSize(tmpDomainInfo.tot_used_space_on_domain_by_unlimited_users, 0),
          visibilityChecks: [
            AjxCallback.simpleClosure(returnMeFunction, this, tmpDomainInfo.tot_used_space_on_domain_by_unlimited_users !== 0)
          ]
        },
        {
          // TODO: Add colors
          type: _OUTPUT_,
          label: zxMsg('LBL_DLG_System_resources_on_domains') + ': ',
          value: tmpDomainInfo.system_resources,
          visibilityChecks: [
            AjxCallback.simpleClosure(returnMeFunction, this, tmpDomainInfo.system_resources !== 0)
          ]
        },
        {
          type: _OUTPUT_,
          label: zxMsg('LBL_DLG_Calendar_resources_on_domains') + ': ',
          value: tmpDomainInfo.calendar_resources,
          visibilityChecks: [
            AjxCallback.simpleClosure(returnMeFunction, this, tmpDomainInfo.calendar_resources !== 0)
          ]
        },
        {
          type: _OUTPUT_,
          label: zxMsg('LBL_DLG_Successful_domain_actions') + ': ',
          value: tmpDomainInfo.successful_actions,
          visibilityChecks: [
            AjxCallback.simpleClosure(returnMeFunction, this, tmpDomainInfo.successful_actions !== 0)
          ]
        },
        {
          type: _OUTPUT_,
          label: zxMsg('LBL_DLG_Unsuccessful_domain_actions') + ': ',
          value: ZxAdminReportDialog.STYLES.RED[0] + tmpDomainInfo.unsuccessful_actions + ZxAdminReportDialog.STYLES.RED[1],
          visibilityChecks: [
            AjxCallback.simpleClosure(returnMeFunction, this, tmpDomainInfo.unsuccessful_actions !== 0)
          ]
        },
        {
          type: _OUTPUT_,
          label: zxMsg('LBL_DLG_Total_account_created') + ': ',
          value: (tmpDomainInfo.tot_created_accounts || tmpDomainInfo.tot_created_accounts === 0) ?
              tmpDomainInfo.tot_created_accounts : zxMsg('LBL_NA'),
          visibilityChecks: [
            AjxCallback.simpleClosure(returnMeFunction, this, tmpDomainInfo.tot_created_accounts > 0)
          ]
        },
        {
          type: _OUTPUT_,
          label: zxMsg('LBL_DLG_Total_account_deleted') + ': ',
          value: (tmpDomainInfo.tot_deleted_accounts || tmpDomainInfo.tot_deleted_accounts === 0) ?
              tmpDomainInfo.tot_deleted_accounts : zxMsg('LBL_NA'),
          visibilityChecks: [
            AjxCallback.simpleClosure(returnMeFunction, this, tmpDomainInfo.tot_deleted_accounts > 0)
          ]
        },
        {
          type: _OUTPUT_,
          label: zxMsg('LBL_DLG_Total_account_renamed') + ': ',
          value: (tmpDomainInfo.tot_renamed_accounts || tmpDomainInfo.tot_renamed_accounts === 0) ?
              tmpDomainInfo.tot_renamed_accounts : zxMsg('LBL_NA'),
          visibilityChecks: [
            AjxCallback.simpleClosure(returnMeFunction, this, tmpDomainInfo.tot_renamed_accounts > 0)
          ]
        },
        {
          type: _OUTPUT_,
          label: zxMsg('LBL_DLG_Total_DL_created') + ': ',
          value: (tmpDomainInfo.tot_created_distribution_lists || tmpDomainInfo.tot_created_distribution_lists === 0) ?
              tmpDomainInfo.tot_created_distribution_lists : zxMsg('LBL_NA'),
          visibilityChecks: [
            AjxCallback.simpleClosure(returnMeFunction, this, tmpDomainInfo.tot_created_distribution_lists > 0)
          ]
        },
        {
          type: _OUTPUT_,
          label: zxMsg('LBL_DLG_Total_DL_deleted') + ': ',
          value: (tmpDomainInfo.tot_deleted_distribution_lists || tmpDomainInfo.tot_deleted_distribution_lists === 0) ?
              tmpDomainInfo.tot_deleted_distribution_lists : zxMsg('LBL_NA'),
          visibilityChecks: [
            AjxCallback.simpleClosure(returnMeFunction, this, tmpDomainInfo.tot_deleted_distribution_lists > 0)
          ]
        },
        {
          type: _OUTPUT_,
          label: zxMsg('LBL_DLG_Total_DL_renamed') + ': ',
          value: (tmpDomainInfo.tot_renamed_distribution_lists || tmpDomainInfo.tot_renamed_distribution_lists === 0) ?
              tmpDomainInfo.tot_renamed_distribution_lists : zxMsg('LBL_NA'),
          visibilityChecks: [
            AjxCallback.simpleClosure(returnMeFunction, this, tmpDomainInfo.tot_renamed_distribution_lists > 0)
          ]
        }
      ]
    });

  }

  // Admins Reports
  for (tmpAdminName in reportData.admin_activity) {
    if (!reportData.admin_activity.hasOwnProperty(tmpAdminName)) { continue; }
    tmpAdminInfo = reportData.admin_activity[tmpAdminName];

    if (tmpAdminName === 'System') {
      systemLogin = tmpAdminInfo.successful_logins;
      continue;
    }

    adminItems.push(
      {
        type: _ZAWIZ_TOP_GROUPER_,
        label: tmpAdminName,
        colSpan: 2,
        numCols: 2,
        width: 528,
        colSizes: [203, 325],
        items: [
          {
            type: _OUTPUT_,
            label: zxMsg('LBL_DLG_Successful_logins') + ': ',
            value: tmpAdminInfo.successful_logins
          },
          {
            type: _OUTPUT_,
            label: zxMsg('LBL_DLG_Unsuccessful_logins') + ': ',
            value: ZxAdminReportDialog.STYLES.RED[0] + tmpAdminInfo.unsuccessful_logins +
              ZxAdminReportDialog.STYLES.RED[1],
            visibilityChecks: [
              AjxCallback.simpleClosure(returnMeFunction, this, tmpAdminInfo.unsuccessful_logins > 0)
            ]
          },
          {
            type: _OUTPUT_,
            label: zxMsg('LBL_DLG_View_mails') + ': ',
            value: tmpAdminInfo.delegated_auths
          },
          {
            type: _OUTPUT_,
            label: zxMsg('LBL_DLG_Last_login') + ': ',
            value: (tmpAdminInfo.last_login_timestamp) ?
                zxMsg('LBL_DLG_login_from', [
                  ZxUtils.computeWordyDateStr(new Date(), tmpAdminInfo.last_login_timestamp),
                  tmpAdminInfo.last_login_by_client_ip
                ]) :
                zxMsg('MSG_DLG_No_longins_in_month')
          },
          {
            type: _OUTPUT_,
            label: zxMsg('LBL_DLG_Most_used_address') + ': ',
            value: zxMsg('MSG_DLG_Most_active_with_count', [
              tmpAdminInfo.most_used_client,
              tmpAdminInfo.most_used_client_count
            ])
          },
          {
            type: _OUTPUT_,
            label: zxMsg('LBL_DLG_Successful_actions') + ': ',
            value: tmpAdminInfo.successful_actions,
            visibilityChecks: [
              AjxCallback.simpleClosure(returnMeFunction, this, tmpAdminInfo.successful_actions !== 0)
            ]
          },
          {
            type: _OUTPUT_,
            label: zxMsg('LBL_DLG_Unsuccessful_actions') + ': ',
            value: ZxAdminReportDialog.STYLES.RED[0] + tmpAdminInfo.unsuccessful_actions + ZxAdminReportDialog.STYLES.RED[1],
            visibilityChecks: [
              AjxCallback.simpleClosure(returnMeFunction, this, tmpAdminInfo.unsuccessful_actions !== 0)
            ]
          },
          {
            type: _OUTPUT_,
            label: zxMsg('LBL_DLG_Total_account_created') + ': ',
            value: (tmpAdminInfo.created_accounts || tmpAdminInfo.created_accounts === 0) ?
                tmpAdminInfo.created_accounts : zxMsg('LBL_NA'),
            visibilityChecks: [
              AjxCallback.simpleClosure(returnMeFunction, this, tmpAdminInfo.created_accounts > 0)
            ]
          },
          {
            type: _OUTPUT_,
            label: zxMsg('LBL_DLG_Total_account_deleted') + ': ',
            value: (tmpAdminInfo.deleted_accounts || tmpAdminInfo.deleted_accounts === 0) ?
                tmpAdminInfo.deleted_accounts : zxMsg('LBL_NA'),
            visibilityChecks: [
              AjxCallback.simpleClosure(returnMeFunction, this, tmpAdminInfo.deleted_accounts > 0)
            ]
          },
          {
            type: _OUTPUT_,
            label: zxMsg('LBL_DLG_Total_account_renamed') + ': ',
            value: (tmpAdminInfo.renamed_accounts || tmpAdminInfo.renamed_accounts === 0) ?
                tmpAdminInfo.renamed_accounts : zxMsg('LBL_NA'),
            visibilityChecks: [
              AjxCallback.simpleClosure(returnMeFunction, this, tmpAdminInfo.renamed_accounts > 0)
            ]
          },
          {
            type: _OUTPUT_,
            label: zxMsg('LBL_DLG_Total_Domains_created') + ': ',
            value: (tmpAdminInfo.created_domains || tmpAdminInfo.created_domains === 0) ?
                tmpAdminInfo.created_domains : zxMsg('LBL_NA'),
            visibilityChecks: [
              AjxCallback.simpleClosure(returnMeFunction, this, tmpAdminInfo.created_domains > 0)
            ]
          },
          {
            type: _OUTPUT_,
            label: zxMsg('LBL_DLG_Total_Domains_deleted') + ': ',
            value: (tmpAdminInfo.deleted_domains || tmpAdminInfo.deleted_domains === 0) ?
                tmpAdminInfo.deleted_domains : zxMsg('LBL_NA'),
            visibilityChecks: [
              AjxCallback.simpleClosure(returnMeFunction, this, tmpAdminInfo.deleted_domains > 0)
            ]
          },
          {
            type: _OUTPUT_,
            label: zxMsg('LBL_DLG_Total_DL_created') + ': ',
            value: (tmpAdminInfo.created_distribution_lists || tmpAdminInfo.created_distribution_lists === 0) ?
                tmpAdminInfo.created_distribution_lists : zxMsg('LBL_NA'),
            visibilityChecks: [
              AjxCallback.simpleClosure(returnMeFunction, this, tmpAdminInfo.created_distribution_lists > 0)
            ]
          },
          {
            type: _OUTPUT_,
            label: zxMsg('LBL_DLG_Total_DL_deleted') + ': ',
            value: (tmpAdminInfo.deleted_distribution_lists || tmpAdminInfo.deleted_distribution_lists === 0) ?
                tmpAdminInfo.deleted_distribution_lists : zxMsg('LBL_NA'),
            visibilityChecks: [
              AjxCallback.simpleClosure(returnMeFunction, this, tmpAdminInfo.deleted_distribution_lists > 0)
            ]
          },
          {
            type: _OUTPUT_,
            label: zxMsg('LBL_DLG_Total_DL_renamed') + ': ',
            value: (tmpAdminInfo.renamed_distribution_lists || tmpAdminInfo.renamed_distribution_lists === 0) ?
                tmpAdminInfo.renamed_distribution_lists : zxMsg('LBL_NA'),
            visibilityChecks: [
              AjxCallback.simpleClosure(returnMeFunction, this, tmpAdminInfo.renamed_distribution_lists > 0)
            ]
          }
        ]
      }
    );
  }

  // Build the whole form
  xFormObject = {
    numCols: 1,
    colSizes: [550],
    items: [
      {
        type: _GROUP_,
        numCols: 1,
        colSizes: [550],
        items: [
          {
            type: _TOP_GROUPER_,
            label: zxMsg('MSG_DLG_Global_report'),
            numCols: 2,
            width: 550,
            colSizes: [225, 325],
            items: [
              {
                type: _OUTPUT_,
                label: zxMsg('LBL_DLG_System_access') + ': ',
                value: systemLogin,
                visibilityChecks: [
                  AjxCallback.simpleClosure(returnMeFunction, this, ZeXtrasItem.isUserAdmin())
                ]
              },
              {
                type: _OUTPUT_,
                label: zxMsg('LBL_DLG_First_logged_action') + ': ',
                value: (reportData.first_action_timestamp) ?
                    ZxUtils.computeWordyDateStr(new Date(), reportData.first_action_timestamp) :
                    zxMsg('LBL_NA')
              },
              {
                type: _OUTPUT_,
                label: zxMsg('LBL_DLG_Last_logged_action') + ': ',
                value: (reportData.last_action_timestamp) ?
                    ZxUtils.computeWordyDateStr(new Date(), reportData.last_action_timestamp) :
                    zxMsg('LBL_NA')
              },
              {
                type: _OUTPUT_,
                label: zxMsg('LBL_DLG_Last_login_by') + ': ',
                value: (reportData.last_login_by_account) ?
                    zxMsg(
                      'MSG_DLG_Last_login_by_at',
                      [
                        reportData.last_login_by_account,
                        ZxUtils.computeWordyDateStr(new Date(), reportData.last_Login_timestamp)
                      ]
                    ) : zxMsg('LBL_NA')
              },
              {
                type: _OUTPUT_,
                label: zxMsg('LBL_DLG_Most_active_admin') + ': ',
                value: (reportData.most_active_admin && reportData.most_active_admin_action_count) ?
                    zxMsg(
                      'MSG_DLG_Most_active_with_count',
                      [
                        reportData.most_active_admin,
                        reportData.most_active_admin_action_count
                      ]
                    ) : zxMsg('LBL_NA')
              },
              {
                type: _OUTPUT_,
                label: zxMsg('LBL_DLG_Most_used_address') + ': ',
                value: (reportData.most_active_client_ip && reportData.most_active_client_ip_action_count) ?
                    zxMsg(
                      'MSG_DLG_Most_active_with_count',
                      [
                        reportData.most_active_client_ip,
                        reportData.most_active_client_ip_action_count
                      ]
                    ) : zxMsg('LBL_NA')
              },
              {
                type: _OUTPUT_,
                label: zxMsg('LBL_DLG_Total_account_number') + ': ',
                value: (reportData.total_account_number || reportData.total_account_number === 0) ?
                    reportData.total_account_number : zxMsg('LBL_NA'),
                visibilityChecks: [
                  AjxCallback.simpleClosure(returnMeFunction, this, reportData.total_account_number > 0)
                ]
              },
              {
                type: _OUTPUT_,
                label: zxMsg('LBL_DLG_Total_account_created') + ': ',
                value: (reportData.tot_created_accounts || reportData.tot_created_accounts === 0) ?
                    reportData.tot_created_accounts : zxMsg('LBL_NA'),
                visibilityChecks: [
                  AjxCallback.simpleClosure(returnMeFunction, this, reportData.tot_created_accounts > 0)
                ]
              },
              {
                type: _OUTPUT_,
                label: zxMsg('LBL_DLG_Total_account_deleted') + ': ',
                value: (reportData.tot_deleted_accounts || reportData.tot_deleted_accounts === 0) ?
                    reportData.tot_deleted_accounts : zxMsg('LBL_NA'),
                visibilityChecks: [
                  AjxCallback.simpleClosure(returnMeFunction, this, reportData.tot_deleted_accounts > 0)
                ]
              },
              {
                type: _OUTPUT_,
                label: zxMsg('LBL_DLG_Total_domains_created') + ': ',
                value: (reportData.tot_created_domains || reportData.tot_created_domains === 0) ?
                    reportData.tot_created_domains : zxMsg('LBL_NA'),
                visibilityChecks: [
                  AjxCallback.simpleClosure(returnMeFunction, this, reportData.tot_created_domains > 0)
                ]
              },
              {
                type: _OUTPUT_,
                label: zxMsg('LBL_DLG_Total_domains_deleted') + ': ',
                value: (reportData.tot_deleted_domains || reportData.tot_deleted_domains === 0) ?
                    reportData.tot_deleted_domains : zxMsg('LBL_NA'),
                visibilityChecks: [
                  AjxCallback.simpleClosure(returnMeFunction, this, reportData.tot_deleted_domains > 0)
                ]
              },
              {
                type: _OUTPUT_,
                label: zxMsg('LBL_DLG_Total_DL_created') + ': ',
                value: (reportData.tot_created_distribution_lists || reportData.tot_created_distribution_lists === 0) ?
                    reportData.tot_created_distribution_lists : zxMsg('LBL_NA'),
                visibilityChecks: [
                  AjxCallback.simpleClosure(returnMeFunction, this, reportData.tot_created_distribution_lists > 0)
                ]
              },
              {
                type: _OUTPUT_,
                label: zxMsg('LBL_DLG_Total_DL_deleted') + ': ',
                value: (reportData.tot_deleted_distribution_lists || reportData.tot_deleted_distribution_lists === 0) ?
                    reportData.tot_deleted_distribution_lists : zxMsg('LBL_NA'),
                visibilityChecks: [
                  AjxCallback.simpleClosure(returnMeFunction, this, reportData.tot_deleted_distribution_lists > 0)
                ]
              }
            ]
          },
          {
            type: _TOP_GROUPER_,
            label: zxMsg('LBL_DLG_Domains_reports'),
            numCols: 2,
            width: 550,
            colSizes: [225, 325],
            items: domainItems
          },
          {
            type: _TOP_GROUPER_,
            label: zxMsg('LBL_DLG_Admins_reports'),
            numCols: 2,
            width: 550,
            colSizes: [225, 325],
            items: adminItems
          }
        ]
      }
    ]
  };

  return xFormObject;
};

/**
 * Set month selection field
 * @param {object} current
 * @param {boolean} isNot
 * @return {boolean}
 */
ZxAdminReportDialog.isCurrentYear = function(current, isNot) {
  var today = new Date();
  if (current.__xform.parent._currentYear < today.getFullYear()) {
    return isNot;
  } else {
    return !(isNot);
  }
};

/**
 * Set month selection field
 * @param {String} currentValue
 */
ZxAdminReportDialog.setNewMonthChoices = function(currentValue) {
  var today = new Date(),
      parentItems = this.__parentItem.getItems(),
      monthItem = {};

  // Search parent item month.
  for (var i = 0; i < parentItems.length; i = i + 1)
  {
    if (parentItems[i].refPath === ZxAdminReportDialog.KEY_MONTH) {
      monthItem = parentItems[i];
      i = parentItems.length;
    }
  }

  if (!monthItem) {
    this.setInstanceValue(currentValue);
    return;
  }

  if (currentValue < today.getFullYear()) {
    monthItem.setChoices(ZxUtils.MONTHS);
  } else {
    monthItem.setChoices(ZxUtils.getMonthChoice());
  }

  if (today.getMonth() === 0) {
    monthItem.setInstanceValue(0);
  } else {
    monthItem.setInstanceValue(today.getMonth() - 1);
  }

  this.setInstanceValue(currentValue);
  this.__xform.parent._currentYear = currentValue;

  // Search parent hidden item.
  for (i = 0; i < this.__xform.items[0].items[0].items.length; i = i + 1)
  {
    if (this.__xform.items[0].items[0].items[i].refPath === ZxAdminReportDialog.KEY_HIDDEN) {
      monthItem = this.__xform.items[0].items[0].items[i];
      i = this.__xform.items[0].items[0].items.length;
    }
  }
  monthItem.getInstanceValue() === 1 ? monthItem.setInstanceValue(0) : monthItem.setInstanceValue(1);
};

ZxAdminReportDialog.prototype.getMyXForm = function() {
  var xFormObject;
  xFormObject = {
    numCols: 1,
    colSizes: [550],
    cssStyle: 'margin-top: 5px; margin-left: 5px',
    items: [
      {
        type: _GROUP_,
        numCols: 2,
        colSizes: [225, 340],
        items: [
          { type: _SPACER_, height: 10 },
          {
            colSpan: 2,
            type: _OUTPUT_,
            value: zxMsg('MSG_DLG_Select_month'),
            align: _MIDDLE_
          },
          { type: _SPACER_, height: 10 },
          {
            type: _OSELECT1_,
            ref: ZxAdminReportDialog.KEY_HIDDEN,
            label: 'hidden: ',
            visibilityChecks: [function() {return false;}],
            choices: [
              {
                value: 1,
                label: '1'
              },
              {
                value: 0,
                label: '0'
              }
            ]
          },
          {
            type: _OSELECT1_,
            ref: ZxAdminReportDialog.KEY_MONTH,
            label: zxMsg('LBL_Month') + ': ',
            visibilityChecks: [function() {return ZxAdminReportDialog.isCurrentYear(this, false);}],
            visibilityChangeEventSources: [ZxAdminReportDialog.KEY_HIDDEN],
            choices: ZxUtils.getMonthChoice()
          },
          {
            type: _OSELECT1_,
            ref: ZxAdminReportDialog.KEY_MONTH,
            label: zxMsg('LBL_Month') + ': ',
            visibilityChecks: [function() {return ZxAdminReportDialog.isCurrentYear(this, true);}],
            visibilityChangeEventSources: [ZxAdminReportDialog.KEY_HIDDEN],
            choices: ZxUtils.MONTHS
          },
          {
            type: _OSELECT1_,
            ref: ZxAdminReportDialog.KEY_YEAR,
            label: zxMsg('LBL_Year') + ': ',
            choices: ZxUtils.getYearChoiceFrom(2012), // <- Birth of Admin Module, previous log cannot exist.
            onChange: ZxAdminReportDialog.setNewMonthChoices
          }
        ]
      }

    ]
  };

  return xFormObject;
};
