/*
 * ***** BEGIN LICENSE BLOCK *****
 * Copyright (C) 2011-2016 ZeXtras
 *
 * The contents of this file are subject to the ZeXtras EULA;
 * you may not use this file except in compliance with the EULA.
 * You may obtain a copy of the EULA at
 * http://www.zextras.com/zextras-eula.html
 * ***** END LICENSE BLOCK *****
 */

/**
 * Dialog used in Admin module to modufy delegation settings for a single domain.
 * @param {ZxModule} moduleItem
 * @param {{}} info
 * @constructor
 */
function ZxEditDomainDelegationSettingsDialog(moduleItem, info) {
  if (arguments.length === 0) { return; }

  this._standardButtons = [DwtDialog.OK_BUTTON, DwtDialog.CANCEL_BUTTON];

  this.module = moduleItem;

  ZaXDialog.call(
    this,
    moduleItem.getCurrentController().getShell(),
    null,
    ZxUtils.getTitleWithIcon('ZxAdmin', zxMsg('LBL_DLG_Edit_domain_limits')),
    '600px',
    '400px',
    null,
    null
  );

  if(info.hasOwnProperty(ZxEditDomainDelegationSettingsDialog.KEY_ACCOUNT_LIMIT))
  {
    if(info[ZxEditDomainDelegationSettingsDialog.KEY_ACCOUNT_LIMIT] === 0)
    {
      info[ZxEditDomainDelegationSettingsDialog.KEY_ACCOUNT_LIMIT] = "0";
    }
  }
  this.initForm([], this.getMyXForm());
  this.setObject(info);

  this.registerCallback(
    DwtDialog.OK_BUTTON,
    new AjxCallback(
      this,
      this._okCallBack,
      [moduleItem]
    )
  );

  this._helpURL = ZxLinks.HELP_DLG_Dialog_Edit_Domain_Settings;
}

ZxEditDomainDelegationSettingsDialog.KEY_DOMAIN = ZxAdmin.KEY_COL_DOMAIN;
ZxEditDomainDelegationSettingsDialog.KEY_ACCOUNT_LIMIT = ZxAdmin.KEY_COL_GLOBAL_LIMIT;
ZxEditDomainDelegationSettingsDialog.KEY_COS_LIMITS = ZxAdmin.KEY_COL_COS_LIMIT;
ZxEditDomainDelegationSettingsDialog.KEY_DOMAIN_ACCOUNT_QUOTA = ZxAdmin.KEY_COL_QUOTA;

ZxEditDomainDelegationSettingsDialog.ID_COS_LIMITS = 'cosLimits';

ZxEditDomainDelegationSettingsDialog.prototype = new ZaXDialog();
ZxEditDomainDelegationSettingsDialog.prototype.constructor = ZxEditDomainDelegationSettingsDialog;

ZxEditDomainDelegationSettingsDialog.prototype._okCallBack = function(modItem) {
  var data = this.getObject(),
    i,
    stringLimit = [],
    waitingDialog,
    setData;

  if (data[ZxEditDomainDelegationSettingsDialog.KEY_ACCOUNT_LIMIT] === ZxAdmin.UNLIMITED ||
      data[ZxEditDomainDelegationSettingsDialog.KEY_ACCOUNT_LIMIT] === ZxAdmin.NOT_SUPPORTED) {

    data[ZxEditDomainDelegationSettingsDialog.KEY_ACCOUNT_LIMIT] = -1;
  }

  if (data[ZxEditDomainDelegationSettingsDialog.KEY_DOMAIN_ACCOUNT_QUOTA].replace) {
    // Remove spaces
    data[ZxEditDomainDelegationSettingsDialog.KEY_DOMAIN_ACCOUNT_QUOTA] =
      data[ZxEditDomainDelegationSettingsDialog.KEY_DOMAIN_ACCOUNT_QUOTA].replace(/\s/g, '');
    // Push to lowercase
    data[ZxEditDomainDelegationSettingsDialog.KEY_DOMAIN_ACCOUNT_QUOTA] =
      data[ZxEditDomainDelegationSettingsDialog.KEY_DOMAIN_ACCOUNT_QUOTA].replace(
        /[a-z]/g,
        function(letter, index) { return letter.toLowerCase(); }
      );
  }

  if (/^unlimited$/ig.test(data[ZxEditDomainDelegationSettingsDialog.KEY_DOMAIN_ACCOUNT_QUOTA]) ||
      data[ZxEditDomainDelegationSettingsDialog.KEY_DOMAIN_ACCOUNT_QUOTA] === 'notsupported') {
    // If unlimited set to 0
    data[ZxEditDomainDelegationSettingsDialog.KEY_DOMAIN_ACCOUNT_QUOTA] = 0;
  }

  for (i = 0; i < data[ZxEditDomainDelegationSettingsDialog.KEY_COS_LIMITS].length; i = i + 1) {
    stringLimit.push(data[ZxEditDomainDelegationSettingsDialog.KEY_COS_LIMITS][i][ZxAdmin.KEY_COL_COS] +
      ':' +
      data[ZxEditDomainDelegationSettingsDialog.KEY_COS_LIMITS][i][ZxAdmin.KEY_COL_ACCOUNT]
      );
  }

  ZxUtils.log(data, 'Saving Delegation settings');

  waitingDialog = new ZxWaitingDialog(
    modItem.getCurrentController().getShell(),
    '250px',
    '52px',
    zxMsg('LBL_Saving_settings'),
    ZxLinks.HELP_DLG_Admin
  );

  setData = {
    accountLimit: data[ZxEditDomainDelegationSettingsDialog.KEY_ACCOUNT_LIMIT],
    resetCosLimits: stringLimit.join(',') || 'emptyAll',
    domain: data[ZxEditDomainDelegationSettingsDialog.KEY_DOMAIN]
  };

  if (ZxUtils.isZ8Up()) {
    setData.domainAccountQuota = data[ZxEditDomainDelegationSettingsDialog.KEY_DOMAIN_ACCOUNT_QUOTA];
  }

  waitingDialog.popup();
  this.popdown();

  ZxAdminRequest.sendRequest(
    ZxAdminRequest.TARGET_LOCAL,
    'ZxAdmin',
    'setDomainSettings',
    setData
  ).then(
    AjxCallback.simpleClosure(
      function(dialog, resp) {
        if (resp instanceof ZxError) {
          ZxUtils.catchError(resp);
        }
        this.fireLoadMethods();
        dialog.popdown();
        this.getCurrentController().popupMsgDialog(resp.message);
      },
      modItem,
      waitingDialog
    ),
    AjxCallback.simpleClosure(
      function(dialog, err) {
        ZxUtils.catchError(err);
//        this.popdown();
        dialog.popdown();
      },
      this,
      waitingDialog
    )
  );
};

ZxEditDomainDelegationSettingsDialog.prototype._onSelectedDomain = function(ev) {
  if (ev.detail !== DwtListView.ITEM_DBL_CLICKED) { return; }
  this._addEditCOSRow(false, ev, [ev.item]);
};

ZxEditDomainDelegationSettingsDialog.prototype._addEditCOSRow = function(newMode, ev, selection) {
  if (!newMode && selection.length !== 1) { return; }

  var emptyObj = {};
  emptyObj[ZxAdmin.KEY_COL_COS] = '';
  emptyObj[ZxAdmin.KEY_COL_ACCOUNT] = '';

  var title = (newMode ? ZaMsg.NAD_Add : ZaMsg.TBB_Edit) + ' ' + zxMsg('LBL_DLS_COS_limit'),
    dialog = new ZxAddModifyCOSLimitDialog(
      this.module.getCurrentController().getShell(),
      '400px',
      '140px',
      title,
      this,
      (newMode) ? emptyObj : selection[0],
      newMode
    );

  dialog.popup();
};

ZxEditDomainDelegationSettingsDialog.prototype._deleteCOSRow = function(ev, selection) {
  if (selection.length !== 1) { return; }

  selection = selection[0];

  var vector = AjxVector.fromArray(
    this.getObject()[ZxAdmin.KEY_COL_COS_LIMIT]
  ),
    helpButton = new DwtDialog_ButtonDescriptor(
      ZaXDialog.HELP_BUTTON,
      ZaMsg.TBB_Help,
      DwtDialog.ALIGN_LEFT,
      new AjxCallback(
        this,
        ZxUtils.openURL,
        [ZxLinks.HELP_DLG_Admin_provisioning_remove_cos]
      )
    ),
    msgDialog = new DwtMessageDialog({
      buttons: [DwtDialog.YES_BUTTON, DwtDialog.NO_BUTTON],
      extraButtons: [helpButton],
      title: '',
      parent: this.module.getCurrentController().getShell()
    });

  msgDialog.setMessage(
    zxMsg('MSG_DLG_Sure_remove_domain', [selection[ZxAdmin.KEY_COL_COS]]),
    DwtMessageDialog.INFO_STYLE,
    zxMsg('LBL_DLG_Remove_domain')
  );

  msgDialog.registerCallback(
    DwtDialog.YES_BUTTON,
    new AjxCallback(
      this,
      function(row, dialog) {

        vector.remove(row);
        this._localXForm.setInstanceValue(
          vector.getArray(),
          ZxEditDomainDelegationSettingsDialog.KEY_COS_LIMITS
        );
        dialog.popdown();

      },
      [selection, msgDialog]
    )
  );

  msgDialog.popup();

};

ZxEditDomainDelegationSettingsDialog.prototype.getCOSListHeader = function() {
  var headerList = [];

  headerList.push(new ZaListHeaderItem(ZxAdmin.KEY_COL_COS, 'COS', null, '300px',
    false, ZxAdmin.KEY_COL_COS, false, true));
  headerList.push(new ZaListHeaderItem(ZxAdmin.KEY_COL_ACCOUNT, zxMsg('LBL_DLG_Account_limit'), null, '100px',
    false, ZxAdmin.KEY_COL_ACCOUNT, false, true));

  return headerList;
};

ZxEditDomainDelegationSettingsDialog.prototype.getListButtons = function() {
  var array = [];

  array.push(new ZxListViewButton(
    'Add',
    ZaMsg.NAD_Add,
    AjxCallback.simpleClosure(this._addEditCOSRow, this, true)
  ));

  array.push(new ZxListViewButton(
    'Edit',
    ZaMsg.TBB_Edit,
    AjxCallback.simpleClosure(this._addEditCOSRow, this, false)
  ));

  array.push(new ZxListViewButton(
    'Delete',
    ZaMsg.TBB_Delete,
    AjxCallback.simpleClosure(this._deleteCOSRow, this)
  ));

  return array;
};

ZxEditDomainDelegationSettingsDialog.checkQuotaLimit = function(value, event, form) {
  this.setInstanceValue(value);
  var valid = false,
    re = /(^\d+\s*(GB|MB|KB)$)|(unlimited)/ig;

  if (re.test(value)) {
    valid = true;
    if (this.clearError) {
      this.clearError();
    }
  } else {
    valid = false;
    if (this.setError) {
      this.setError(ZaMsg.ERROR_INVALID_VALUE);
    }
  }
  return valid;
};

ZxEditDomainDelegationSettingsDialog.prototype.getMyXForm = function() {
  return {
    numCols: 1,
    width: '100%',
    items: [
      {
        type: _GROUP_,
        numCols: 1,
        colSizes: ['*'],
        cssStyle: 'margin-top: 5px; margin-left: 5px',
        isTabGroup: true,
        items: [
          {
            type: _TEXTFIELD_,
            ref: ZxEditDomainDelegationSettingsDialog.KEY_DOMAIN,
            label: zxMsg('LBL_Domain') + ': ',
            enableDisableChecks: [
              function() {
                return false;
              }
            ]
          },
          {
            type: _TEXTFIELD_,
            ref: ZxEditDomainDelegationSettingsDialog.KEY_ACCOUNT_LIMIT,
            label: zxMsg('LBL_DLG_Global_Account_Limit') + ': '
          },
          {
            type: _TEXTFIELD_,
            ref: ZxEditDomainDelegationSettingsDialog.KEY_DOMAIN_ACCOUNT_QUOTA,
            label: zxMsg('LBL_DLG_Domain_Quota') + ': ',
            onChange: ZxEditDomainDelegationSettingsDialog.checkQuotaLimit,
            enableDisableChecks: [
              function() {
                return ZxUtils.isZ8Up();
              }
            ]
          }
        ]
      },
      {
        type: _OUTPUT_,
        value: zxMsg('MSG_DLG_Quota_declaration_info'),
        enableDisableChecks: [
          function() {
            return ZxUtils.isZ8Up();
          }
        ]
      },
      { type: _SPACER_, height: 10 },
      {
        type: _GROUPER_,
        label: zxMsg('LBL_DLG_COS_limits'),
        numCols: 1,
//        colSizes: ['*'],
        width: '100%',
//        cssStyle: 'margin: 0px 5px 0px 5px;',
        items: [
          {
            type: _ZX_LIST_VIEW_,
            id: ZxEditDomainDelegationSettingsDialog.ID_COS_LIMITS,
            ref: ZxEditDomainDelegationSettingsDialog.KEY_COS_LIMITS,
            headerList: this.getCOSListHeader(),
            optionalButtons: this.getListButtons(),
            onSelection: AjxCallback.simpleClosure(this._onSelectedDomain, this),
            filterEnabled: false,
            navigationEnabled: false
          }
        ]
      }
    ]
  };
};
