/*
 * ***** BEGIN LICENSE BLOCK *****
 * Copyright (C) 2011-2016 ZeXtras
 *
 * The contents of this file are subject to the ZeXtras EULA;
 * you may not use this file except in compliance with the EULA.
 * You may obtain a copy of the EULA at
 * http://www.zextras.com/zextras-eula.html
 * ***** END LICENSE BLOCK *****
 */

/**
 * Create the ZxBackup object.
 * @constructor
 */
function ZxBackup() {
  ZxModule.call(this, ZxBackup.MOD_ID, zxMsg('LBL_ZxBackup'), 'ZxBackup');

  this.registerLoadMethod(this.loadMethod, 'ZxBackupInit');
  this.registerModifyMethod(this.modifyMethod, 'ZxBackupInit');

  this._helpURL = ZxLinks.HELP_BCK_Backup;
}

ZxBackup.prototype = new ZxModule();
ZxBackup.prototype.constructor = ZxBackup;
ZxBackup.MOD_ID = 'ZxBackup';

ZxBackup.KEY_BACKUP_PATH = 'ZxBackup_DestPath';
ZxBackup.KEY_DO_FULLSCAN = 'ZxBackup_DoSmartScanOnStartup';
ZxBackup.KEY_FULLSCAN_START_TIME = 'ZxBackup_SmartScanStartTime';
ZxBackup.KEY_REALTIME = 'ZxBackup_RealTimeScanner';
ZxBackup.KEY_DATA_RETENTION_KEY = 'ZxBackup_DataRetentionDays';
ZxBackup.KEY_DATA_RETENTION = ZxBackup.KEY_DATA_RETENTION_KEY;
ZxBackup.KEY_FULLSCAN_SCHEDULING = 'ZxBackup_SmartScanSchedulingEnabled';
ZxBackup.KEY_SPACE_THRESHOLD = 'ZxBackup_SpaceThreshold';
ZxBackup.KEY_FULLSCAN_PURGE_WEEKDAY = 'ZxBackup_PurgeWeekDay';
ZxBackup.KEY_PURGE_CUSTOMIZATIONS = 'ZxBackup_PurgeCustomizations';
ZxBackup.KEY_BACKUP_CUSTOMIZATIONS = 'ZxBackup_BackupCustomizations';

ZxBackup.KEY_COSLIST_ENABLED = 'enabled';
ZxBackup.KEY_COSLIST_DISABLED = 'disabled';
ZxBackup.KEY_STAT_FULL_FIRST_FULLSCAN_KEY = 'fullScan_firstscan';
ZxBackup.KEY_STAT_FULL_NUM_TOAL_ACCOUNTS = 'statistics/fullScan_numTotalAccounts';
ZxBackup.KEY_STAT_FULL_NUM_BACKUPPED_ITEMS = 'statistics/fullScan_numBackuppedItems';
ZxBackup.KEY_STAT_FULL_NUM_DELETED_FILES = 'statistics/fullScan_numDeletedItems';
ZxBackup.KEY_STAT_PURGE_NUM_PURGED_ACCOUNTS = 'statistics/purge_numPurgedAccounts';
ZxBackup.KEY_STAT_PURGE_NUM_PURGED_ITEMS = 'statistics/purge_numPurgedItems';
ZxBackup.KEY_STAT_PURGE_NUM_PURGED_SERVER_CONF = 'statistics/purge_numPurgedServerConf';
ZxBackup.KEY_STAT_PURGE_NUM_PURGED_FILES = 'statistics/purge_numPurgedFiles';
ZxBackup.KEY_STAT_PURGE_PROCESSED_ITEMS_PER_SECOND = 'statistics/purge_numItemsPerSecond';
ZxBackup.KEY_STAT_SMART_ZXVERSION = 'statistics/smartScan_zextrasVersion';
ZxBackup.KEY_STAT_SMART_ZXBUILD = 'statistics/smartScan_zextrasBuild';
ZxBackup.KEY_STAT_SMART_ZVERSION = 'statistics/smartScan_zimbraVersion';
ZxBackup.KEY_STAT_SMART_FIRST_SCAN = 'statistics/firstscan';
ZxBackup.KEY_STAT_SMART_LAST_SMARTSCAN = 'statistics/smartScan_lastscan';
ZxBackup.KEY_STAT_SMART_NUM_ACTIVE_ACCOUNTS = 'statistics/smartScan_numActiveBackuppedAccounts';
ZxBackup.KEY_STAT_SMART_NUM_DELETED_ACCOUNTS = 'statistics/smartScan_numDeletedAccounts';
ZxBackup.KEY_STAT_SMART_NUM_UPDATED_ACCOUNTS = 'statistics/smartScan_numUpdatedAccounts';
ZxBackup.KEY_STAT_SMART_NUM_NEW_ACCOUNTS = 'statistics/smartScan_numNewAccounts';
ZxBackup.KEY_STAT_SMART_NUM_BACKUPPED_ITEMS = 'statistics/smartScan_numBackuppedItems';
ZxBackup.KEY_STAT_SMART_NUM_DELETED_ITEMS = 'statistics/smartScan_numDeletedItems';
ZxBackup.KEY_STAT_SMART_NUM_UPDATED_ITEMS = 'statistics/smartScan_numUpdatedItems';
ZxBackup.KEY_STAT_SMART_NUM_NEW_ITEMS = 'statistics/smartScan_numNewItems';
ZxBackup.KEY_STAT_SMART_PROCESSED_ITEMS_PER_SECOND = 'statistics/smartScan_numItemsPerSecond';
ZxBackup.KEY_STAT_SMART_NUM_SKIPPED_ACCOUNTS = 'statistics/smartScan_numSkippedAccounts';
ZxBackup.KEY_PATH_EMPTY = 'pathInfo/empty';
ZxBackup.KEY_PATH_EXISTS = 'pathInfo/exists';
ZxBackup.KEY_PATH_READABLE = 'pathInfo/readable';
ZxBackup.KEY_PATH_VALID = 'pathInfo/valid';
ZxBackup.KEY_PATH_WRITABLE = 'pathInfo/writable';
ZxBackup.KEY_PATH_CASE_SENSITIVE = 'pathInfo/caseSensitive';
ZxBackup.KEY_PATH_LOCK_OK = 'pathInfo/lockok';

ZxBackup.A_zextrasBackupEnabled = 'zextrasBackupEnabled';
ZxBackup.A_zextrasUserBackupFirstScan = 'zextrasUserBackupFirstScan';
ZxBackup.A_zextrasUserBackupLastScan = 'zextrasUserBackupLastScan';
ZxBackup.A_zextrasUserBackupNumBackupedItems = 'numBackupedItems';
ZxBackup.A_zextrasUserBackupNumDeletedItems = 'numDeletedItems';

/**
 * Method called in order to refresh item informations.
 */
ZxBackup.prototype.loadMethod = function(by, val) {
  this.attrs.properties = {};
  this.attrs.statistics = {};
  this.attrs.pathInfo = {};
  this.attrs.coslist = {};
  this.attrs.coslist.enabled = [];
  this.attrs.coslist.disabled = [];

  ZxAdminRequest.sendRequest(
    this.getCurrentServer(),
    'ZxBackup',
    'getPublicProperties'
  ).then(AjxCallback.simpleClosure(
    function(resp) {
      var server = this.getCurrentServer(),
        controller = this.getCurrentController(),
        response;

      response = resp.response.properties;

      this.attrs.properties[ZxBackup.KEY_BACKUP_PATH] = response.ZxBackup_DestPath;
      this.attrs.properties[ZxBackup.KEY_DATA_RETENTION] = response.ZxBackup_DataRetentionDays;
      this.attrs.properties[ZxBackup.KEY_DO_FULLSCAN] = (response[ZxBackup.KEY_DO_FULLSCAN] === 'true');
      this.attrs.properties[ZxBackup.KEY_FULLSCAN_SCHEDULING] =
        (response[ZxBackup.KEY_FULLSCAN_SCHEDULING] === 'true');
      this.attrs.properties[ZxBackup.KEY_FULLSCAN_START_TIME] = response[ZxBackup.KEY_FULLSCAN_START_TIME];
      this.attrs.properties[ZxBackup.KEY_FULLSCAN_PURGE_WEEKDAY] = response[ZxBackup.KEY_FULLSCAN_PURGE_WEEKDAY];
      this.attrs.properties[ZxBackup.KEY_REALTIME] = response.ZxBackup_RealTimeScanner === 'true' ?
          'Enabled' : 'Disabled';
      this.attrs.properties[ZxBackup.KEY_SPACE_THRESHOLD] = response.ZxBackup_SpaceThreshold;
      this.attrs.properties[ZxBackup.KEY_BACKUP_CUSTOMIZATIONS] = (response.ZxBackup_BackupCustomizations === 'true');
      this.attrs.properties[ZxBackup.KEY_PURGE_CUSTOMIZATIONS] = (response.ZxBackup_PurgeCustomizations === 'true');

      controller.setInstanceValue('properties/' + ZxBackup.KEY_BACKUP_PATH, response.ZxBackup_DestPath);
      controller.setInstanceValue('properties/' + ZxBackup.KEY_DATA_RETENTION, response.ZxBackup_DataRetentionDays);
      controller.setInstanceValue('properties/' + ZxBackup.KEY_DO_FULLSCAN,
        response[ZxBackup.KEY_DO_FULLSCAN] === 'true');
      controller.setInstanceValue('properties/' + ZxBackup.KEY_FULLSCAN_SCHEDULING,
        response[ZxBackup.KEY_FULLSCAN_SCHEDULING] === 'true');
      controller.setInstanceValue('properties/' + ZxBackup.KEY_FULLSCAN_START_TIME,
        response[ZxBackup.KEY_FULLSCAN_START_TIME]);
      controller.setInstanceValue('properties/' + ZxBackup.KEY_FULLSCAN_PURGE_WEEKDAY,
        response[ZxBackup.KEY_FULLSCAN_PURGE_WEEKDAY]);
      controller.setInstanceValue('properties/' + ZxBackup.KEY_REALTIME,
        response.ZxBackup_RealTimeScanner === 'true' ? 'Enabled' : 'Disabled');
      controller.setInstanceValue('properties/' + ZxBackup.KEY_SPACE_THRESHOLD, response.ZxBackup_SpaceThreshold);

      controller.setInstanceValue('properties/' + ZxBackup.KEY_BACKUP_CUSTOMIZATIONS, this.attrs.properties[ZxBackup.KEY_BACKUP_CUSTOMIZATIONS] );
      controller.setInstanceValue('properties/' + ZxBackup.KEY_PURGE_CUSTOMIZATIONS, this.attrs.properties[ZxBackup.KEY_PURGE_CUSTOMIZATIONS] );
    },
    this
  ), this._getDefaultCatchErrorFcn());

  ZxAdminRequest.sendRequest(
    this.getCurrentServer(),
    'ZxBackup',
    'getBackupInfo'
  ).then(AjxCallback.simpleClosure(
    function(resp) {
      var server = this.getCurrentServer(),
        controller = this.getCurrentController(),
        response,
        statsPurge,
        statsSmart;

      response = resp.response;
      statsPurge = response.backupStat.purge;
      statsSmart = response.backupStat.scan;

      // Purge statistics
      controller.setInstanceValue(ZxBackup.KEY_STAT_PURGE_NUM_PURGED_ACCOUNTS,
        statsPurge.numPurgedAccounts);
      controller.setInstanceValue(ZxBackup.KEY_STAT_PURGE_NUM_PURGED_FILES,
        statsPurge.numPurgedDigests);
      controller.setInstanceValue(ZxBackup.KEY_STAT_PURGE_NUM_PURGED_ITEMS,
        statsPurge.numPurgedItems);
      controller.setInstanceValue(ZxBackup.KEY_STAT_PURGE_NUM_PURGED_SERVER_CONF,
        statsPurge.numPurgedServerConf);
      controller.setInstanceValue(ZxBackup.KEY_STAT_PURGE_PROCESSED_ITEMS_PER_SECOND,
        Number(statsPurge.itemsPerSecond).toFixed(0));

      // SmartScan statistics
      this.attrs.statistics[ZxBackup.KEY_STAT_FULL_FIRST_FULLSCAN_KEY] = new Date(Number(statsSmart.firstScan));

      controller.setInstanceValue(
        ZxBackup.KEY_STAT_SMART_LAST_SMARTSCAN,
        new Date(Number(statsSmart.lastScan))
      );
      controller.setInstanceValue(
        ZxBackup.KEY_STAT_SMART_FIRST_SCAN,
        new Date(Number(statsSmart.firstScan))
      );
      controller.setInstanceValue(ZxBackup.KEY_STAT_SMART_PROCESSED_ITEMS_PER_SECOND,
        Number(statsSmart.itemsPerSecond).toFixed(0));
      controller.setInstanceValue(ZxBackup.KEY_STAT_SMART_NUM_SKIPPED_ACCOUNTS,
        statsSmart.numSkippedAccounts);
      controller.setInstanceValue(ZxBackup.KEY_STAT_SMART_NUM_ACTIVE_ACCOUNTS,
        statsSmart.numActiveBackupedAccounts);
      controller.setInstanceValue(ZxBackup.KEY_STAT_SMART_NUM_BACKUPPED_ITEMS,
        statsSmart.numBackupedItems);
      controller.setInstanceValue(ZxBackup.KEY_STAT_SMART_NUM_NEW_ITEMS,
        statsSmart.numNewItems);
      controller.setInstanceValue(ZxBackup.KEY_STAT_SMART_NUM_NEW_ACCOUNTS,
        statsSmart.numNewAccounts);
      controller.setInstanceValue(ZxBackup.KEY_STAT_SMART_NUM_UPDATED_ITEMS,
        statsSmart.numUpItems);
      controller.setInstanceValue(ZxBackup.KEY_STAT_SMART_NUM_UPDATED_ACCOUNTS,
        statsSmart.numUpAccounts);
      controller.setInstanceValue(ZxBackup.KEY_STAT_SMART_NUM_DELETED_ACCOUNTS,
        statsSmart.numDeletedAccounts);
      controller.setInstanceValue(ZxBackup.KEY_STAT_SMART_NUM_DELETED_ITEMS,
        statsSmart.numDeletedItems);
      controller.setInstanceValue(ZxBackup.KEY_STAT_SMART_ZVERSION,
        statsSmart.lastZimbraVersion);
      controller.setInstanceValue(ZxBackup.KEY_STAT_SMART_ZXVERSION,
        statsSmart.lastZeXtrasVersion);
      controller.setInstanceValue(ZxBackup.KEY_STAT_SMART_ZXBUILD,
        statsSmart.lastZeXtrasBuild);

      // Backup path informations
      controller.setInstanceValue(ZxBackup.KEY_PATH_EMPTY,
        response.empty);
      controller.setInstanceValue(ZxBackup.KEY_PATH_EXISTS,
        response.exists);
      controller.setInstanceValue(ZxBackup.KEY_PATH_READABLE,
        response.readable);
      controller.setInstanceValue(ZxBackup.KEY_PATH_VALID,
        response.valid);
      controller.setInstanceValue(ZxBackup.KEY_PATH_WRITABLE,
        response.writable);
      controller.setInstanceValue(ZxBackup.KEY_PATH_CASE_SENSITIVE,
        response.caseSensitive);
      controller.setInstanceValue(ZxBackup.KEY_PATH_LOCK_OK,
        response.lockok);
    },
    this
  ), this._getDefaultCatchErrorFcn());

  ZxAdminRequest.sendRequest(
    this.getCurrentServer(),
    'ZxBackup',
    'getCOSBackupStatus'
  ).then(AjxCallback.simpleClosure(
    function(resp) {
      var server = this.getCurrentServer(),
        controller = this.getCurrentController(),
        response,
        tmpName,
        tmpListEnabled = [],
        tmpListDisabled = [];

      response = resp.response;

      for (tmpName in response.cosList) {
        if (!response.cosList.hasOwnProperty(tmpName)) { continue; }

        if (response.cosList[tmpName] === 'Enabled') {
          tmpListEnabled.push(tmpName);
        } else {
          tmpListDisabled.push(tmpName);
        }
      }

      tmpListEnabled.sort();
      tmpListDisabled.sort();

      this.attrs.coslist[ZxBackup.KEY_COSLIST_ENABLED] = tmpListEnabled;
      this.attrs.coslist[ZxBackup.KEY_COSLIST_DISABLED] = tmpListDisabled;
      controller.setInstanceValue(
        'coslist/' + ZxBackup.KEY_COSLIST_ENABLED,
        tmpListEnabled
      );
      controller.setInstanceValue(
        'coslist/' + ZxBackup.KEY_COSLIST_DISABLED,
        tmpListDisabled
      );
    },
    this
  ), this._getDefaultCatchErrorFcn());
};

/**
 * Check if the backup path is writable.
 * Used ah visibilty check for the Waring in the GUI.
 * @return {boolean}
 * @this {ZxBackup}
 */
ZxBackup.checkPathNotWritable = function() {
  var isWritable = this.getInstanceValue('attrs/' + ZxBackup.KEY_PATH_WRITABLE),
    isValid = this.getInstanceValue('attrs/' + ZxBackup.KEY_PATH_VALID);
  if (typeof isWritable === 'undefined') {
    return false;
  }
  return isValid && !isWritable;
};

/**
 * Check if the backup path is Case Sensitive.
 * Used ah visibilty check for the Waring in the GUI.
 * @return {boolean}
 * @this {ZxBackup}
 */
ZxBackup.checkCaseInSensitive = function() {
  var isSensitive = this.getInstanceValue('attrs/' + ZxBackup.KEY_PATH_CASE_SENSITIVE),
    isValid = this.getInstanceValue('attrs/' + ZxBackup.KEY_PATH_VALID),
    isWritable = this.getInstanceValue('attrs/' + ZxBackup.KEY_PATH_WRITABLE);
  if (typeof isSensitive === 'undefined') {
    return false;
  }
  return isValid && isWritable && !isSensitive;
};

/**
 * Check if the backup path is initialized (marked as valid).
 * Used as visibility check for the buttons in gui.
 * @return {boolean}
 * @this {ZxBackup}
 */
ZxBackup.checkInitialized = function() {
  var isValid = this.getInstanceValue('attrs/' + ZxBackup.KEY_PATH_VALID);
  if (typeof isValid === 'undefined') {
    return false;
  }
  return isValid;
};

ZxBackup.isCustomizationBackupEnabled = function() {
  var enabled = this.getInstanceValue('attrs/properties/' + ZxBackup.KEY_BACKUP_CUSTOMIZATIONS);
  if (typeof enabled === 'undefined') {
    return false;
  }
  return enabled;
};


/**
 * Check if the backup path is NOT initialized (marked as valid).
 * Used as visibility check for the warning in gui.
 * @return {boolean}
 * @this {ZxBackup}
 */
ZxBackup.checkNotInitialized = function() {
  var isValid = this.getInstanceValue('attrs/' + ZxBackup.KEY_PATH_VALID);
  if (typeof isValid === 'undefined') {
    return false;
  }
  return !isValid;
};

/**
 * Check if the backup path should not be initialized again.
 * Used as visibility check for the warning in gui.
 * @return {boolean}
 * @this {Dwt_Alert_XFormItem}
 */
ZxBackup.checkDontInitialize = function() {
  var isValid = this.getInstanceValue('attrs/' + ZxBackup.KEY_PATH_VALID),
    scanRunning = ZeXtras.getOperationsManager().isBackupScanRunning(this.getInstance().name);

  return !isValid && scanRunning;
};

/**
 * Get if the backup path is really initialized.
 * @return {boolean}
 */
ZxBackup.checkInitialize = function() {
  var firstScan = this.getInstanceValue('attrs/' + ZxBackup.KEY_STAT_SMART_FIRST_SCAN),
    scanRunning = ZeXtras.getOperationsManager().isBackupScanRunning(this.getInstance().name);
  if (typeof firstScan === 'undefined') {
    return false;
  }

  return firstScan.getTime() === 0 && !scanRunning;
};

/**
 * Check if the backup path is already in use by another server.
 * Used as visibility check for the warning in gui.
 * @return {boolean}
 * @this {ZxBackup}
 */
ZxBackup.checkAlreadyInUse = function() {
  var isValid = this.getInstanceValue('attrs/' + ZxBackup.KEY_PATH_VALID),
    inUse = this.getInstanceValue('attrs/' + ZxBackup.KEY_PATH_LOCK_OK);

  return !inUse && isValid;
};

/**
 * Enable or disable a backup for a COS selected in the srcList.
 * @param {XForm} form
 * @param {DwtEvent} ev
 * @param {string} srcList
 * @param {string} destList
 */
ZxBackup.doEnableDisableBackupForCOS = function(form, ev, srcList, destList) {
  var enable = true,
    item = ZeXtras.getModuleItem(ZxBackup, form.getInstance().id),
    cosList,
    selection = [];

  if (srcList === 'enabledCOSList' && destList === 'disabledCOSList') {
    enable = false;
  } else if (srcList === 'disabledCOSList' && destList === 'enabledCOSList') {
    enable = true;
  } else {
    ZxUtils.log({src: srcList, dst: destList}, 'Invalid parameters');
    return;
  }

  cosList = form.getItemsById(srcList)[0];
  selection = cosList.getSelection();
  if (selection.length === 0) {
    return;
  }

  item._doEnableDisableBackupForCOS(selection, enable);
};

/**
 * real function to handle the enable/disable backup for a list of COS(s).
 * @param {string[]} cosList
 * @param {boolean} enable
 */
ZxBackup.prototype._doEnableDisableBackupForCOS = function(cosList, enable) {
  var i = 0,
    controller = this.getCurrentController(),
    message,
    cosUL,
    msgDialog = new DwtMessageDialog({
      buttons: [DwtDialog.YES_BUTTON, DwtDialog.NO_BUTTON],
      title: '',
      parent: controller.getShell()
    });

  if (enable) {
    message = zxMsg('MSG_BCK_Enablig_backup_for_cos_warning');
  } else {
    message = zxMsg('MSG_BCK_Disabling_backup_for_cos_warning');
  }

  cosUL = ['<ul><li>',
    cosList.join('</li><li>'),
    '</li></ul>'].join('');

  msgDialog.setMessage(
    zxMsg('MSG_BCK_Would_you_enable_or_disable_backup_for_cos',
      [enable ? zxMsg('LBL_enable') : zxMsg('LBL_disable')]) + '<br>' +
      cosUL + '<br><strong>' + message + '</strong>',
    DwtMessageDialog.WARNING_STYLE,
    ZxUtils.getTitleWithIcon('ZxBackup', 'ZxBackup')
  );

  msgDialog.registerCallback(
    DwtDialog.YES_BUTTON,
    new AjxCallback(this,
      function(cosList, enable, dialog) {
        var i;
        for (i = 0; i < cosList.length; i = i + 1) {
          ZxAdminRequest.sendRequest(
            this.getServerName(),
            'ZxBackup',
            'doEnableDisableCOS',
            {
              COSName: cosList[i],
              enableCOS: enable ? 'enable' : 'disable'
            }
          ).then(AjxCallback.simpleClosure(this.fireLoadMethods, this), ZxUtils.catchError);
        }
        dialog.popdown();
      },
      [cosList, enable, msgDialog]
      )
  );

  msgDialog.popup();
};

/**
 * Start a SmartScan
 * @param {DwtEvent} ev
 * @this Dwt_Button_XFormItem
 */
ZxBackup.doStartSmartScan = function(ev) {
  var form = this.getForm(),
    item = ZeXtras.getModuleItem(ZxBackup, form.getInstance().id);
  if (form._isDirty) {
    item.displayYouHaveUnsavedSettingsDialog();
  } else {
    item._doStartSmartScan(false, null);
  }
};

/**
 * Start a SmartScan can init backup
 * @param {DwtEvent} ev
 */
ZxBackup.doStartSmartScanInitBackup = function(ev) {
  var form = this.getForm(),
    item = ZeXtras.getModuleItem(ZxBackup, form.getInstance().id);
  if (form._isDirty) {
    item.displayYouHaveUnsavedSettingsDialog();
  } else {
    item._doStartSmartScan(true, this);
  }
};

/**
 * Start a Full Scan.
 * @param {DwtEvent} ev
 * @this Dwt_Button_XFormItem
 */
ZxBackup.doStartPurge = function(ev) {
  var form = this.getForm(),
    item = ZeXtras.getModuleItem(ZxBackup, form.getInstance().id);
  if (form._isDirty) {
    item.displayYouHaveUnsavedSettingsDialog();
  } else {
    item._doPurge();
  }
};

/**
 * Real code that start the Purge operation.
 */
ZxBackup.prototype._doPurge = function() {
    var errorFcn, msgDialog = new DwtMessageDialog({
        buttons: [DwtDialog.YES_BUTTON, DwtDialog.NO_BUTTON],
        title: '',
        parent: this.getCurrentController().getShell()
    });

    errorFcn = AjxCallback.simpleClosure(
        function(dialog, error) {
            ZxUtils.log(error, 'Error on queueing smartscan operation');
            dialog.popdown();

            var errDialog = new DwtMessageDialog({
                title: '',
                parent: this.getCurrentController().getShell()
            });
            errDialog.setMessage(
                error.getMessage(),
                DwtMessageDialog.INFO_STYLE,
                ZxUtils.getTitleWithIcon('ZxBackup', 'ZxBackup')
            );
            errDialog.popup();
        },
        this,
        msgDialog
    );

    msgDialog.setMessage(
        zxMsg('MSG_BCK_Would_you_start_purge'),
        DwtMessageDialog.INFO_STYLE,
        ZxUtils.getTitleWithIcon('ZxBackup', 'ZxBackup')
    );

    msgDialog.registerCallback(
        DwtDialog.YES_BUTTON,
        new AjxCallback(
            this,
            function(dialog) {
                var prop = {
                    notificationMails: ZaZimbraAdmin.currentUserLogin
                };

              ZxAdminRequest.sendRequest(
                this.getServerName(),
                'ZxBackup',
                'doPurge',
                prop
              ).then(
                AjxCallback.simpleClosure(
                  function(msgDialog, resp) {
                    msgDialog.popdown();
                    this.getCurrentController().popupMsgDialog(
                      zxMsg('MSG_Operation_queued')
                    );
                    ZeXtras.getOperationsManager().requestUpdate();
                    this.fireLoadMethods();
                  },
                  this,
                  dialog
                ),
                errorFcn
            );
            },
            [msgDialog]
        )
    );

    msgDialog.popup();
};

/**
 * Real code that start the SmartScan.
 * @param {boolean} init can initialize backup path?
 * @param {Dwt_Button_XFormItem=} button to disable if an initialization was requested.
 */
ZxBackup.prototype._doStartSmartScan = function(init, button) {
  var errorFcn, msgDialog = new DwtMessageDialog({
    buttons: [DwtDialog.YES_BUTTON, DwtDialog.NO_BUTTON],
    title: '',
    parent: this.getCurrentController().getShell()
  });

  errorFcn = AjxCallback.simpleClosure(
    function(dialog, error) {
      ZxUtils.log(error, 'Error on queueing smartscan operation');
      dialog.popdown();

      var errDialog = new DwtMessageDialog({
        title: '',
        parent: this.getCurrentController().getShell()
      });
      errDialog.setMessage(
        error.getMessage(),
        DwtMessageDialog.INFO_STYLE,
        ZxUtils.getTitleWithIcon('ZxBackup', 'ZxBackup')
      );
      errDialog.popup();
    },
    this,
    msgDialog
  );

  msgDialog.setMessage(
    zxMsg('MSG_BCK_Would_you_start_smartScan'),
    DwtMessageDialog.INFO_STYLE,
    ZxUtils.getTitleWithIcon('ZxBackup', 'ZxBackup')
  );

  msgDialog.registerCallback(
    DwtDialog.YES_BUTTON,
    new AjxCallback(
      this,
      function(dialog, button) {
        var prop = {
          notificationMails: ZaZimbraAdmin.currentUserLogin,
          canBeFirstScan: init
        };

        if (typeof button !== 'undefined' && button !== null) {
          button.setElementEnabled(false);
        }

        ZxAdminRequest.sendRequest(
          this.getServerName(),
          'ZxBackup',
          'doSmartScan',
          prop
        ).then(
          AjxCallback.simpleClosure(
            function(msgDialog, resp) {
              msgDialog.popdown();
              this.getCurrentController().popupMsgDialog(
                zxMsg('MSG_Operation_queued')
              );
              ZeXtras.getOperationsManager().requestUpdate().then(
                AjxCallback.simpleClosure(
                  this.fireLoadMethods,
                  this
                )
              );
            },
            this,
            dialog
          ),
          errorFcn
        );
      },
      [msgDialog, button]
    )
  );

  msgDialog.popup();
};

/**
 * Stop all the current ZxBackup operations.
 * @param {DwtEvent} ev
 */
ZxBackup.doStopAllOperations = function(ev) {
  var item = ZeXtras.getModuleItem(ZxBackup, this.getForm().getInstance().id);
  item._doStopAllOperations();
};

/**
 * real function that manage the real stop of the operation.
 */
ZxBackup.prototype._doStopAllOperations = function() {
  var msgDialog = new DwtMessageDialog({
    buttons: [DwtDialog.YES_BUTTON, DwtDialog.NO_BUTTON],
    title: '',
    parent: this.getCurrentController().getShell()
  });

  msgDialog.setMessage(
    zxMsg('MSG_BCK_would_stop_all_operation',
      ['<br>', '<ul><li>', '</li><li>', '</li></ul>']),
    DwtMessageDialog.WARNING_STYLE,
    ZxUtils.getTitleWithIcon('ZxBackup', 'ZxBackup')
  );

  msgDialog.registerCallback(
    DwtDialog.YES_BUTTON,
    new AjxCallback(
      this,
      function(dialog) {
        ZxAdminRequest.sendRequest(
          this.getServerName(),
          'ZxBackup',
          'doStopAllOperations'
        ).then(
          AjxCallback.simpleClosure(
            function(msgDialog, resp) {
              msgDialog.popdown();
              this.getCurrentController().popupMsgDialog(
                zxMsg('MSG_BCK_all_op_stop_sent')
              );
              ZeXtras.getOperationsManager().requestUpdate();
              this.fireLoadMethods();
            },
            this,
            dialog
          ),
          ZxUtils.catchError
        );
      },
      [msgDialog]
    )
  );

  msgDialog.popup();
};

/**
 * Enable the RealTime Scanner.
 * @param {DwtEvent} ev
 */
ZxBackup.doDisableRealtime = function(ev) {
  var item = ZeXtras.getModuleItem(ZxBackup, this.getForm().getInstance().id);
  item._doEnableDisableRealtime(false);
};

/**
 * Disable the RealTime Scanner.
 * @param {DwtEvent} ev
 */
ZxBackup.doEnableRealtime = function(ev) {
  var item = ZeXtras.getModuleItem(ZxBackup, this.getForm().getInstance().id);
  item._doEnableDisableRealtime(true);
};

/**
 * Really do the enabling/disabling the realtime scanner.
 * @param {boolean} enable
 */
ZxBackup.prototype._doEnableDisableRealtime = function(enable) {
  var msgDialog = new DwtMessageDialog({
      buttons: [DwtDialog.YES_BUTTON, DwtDialog.NO_BUTTON],
      title: '',
      parent: this.getCurrentController().getShell()
    }),
    doCallback = AjxCallback.simpleClosure(
      // Instrumentation code start, function used many times.
      function(isEnable, dialog) {
        ZxAdminRequest.sendRequest(
          this.getServerName(),
          'ZxBackup',
          'setPublicProperties',
          {
            properties: {
              ZxBackup_RealTimeScanner: isEnable
            }
          }
        ).then(AjxCallback.simpleClosure(
          function(enabled, diag, resp) {
            diag.popdown();
            if (enabled) {
              ZaApp.getInstance().getCurrentController().popupMsgDialog(
                zxMsg('MSG_Operation_queued')
              );
              ZeXtras.getOperationsManager().requestUpdate();
            }
            this.fireLoadMethods();
          },
          this,
          isEnable,
          dialog
        ), ZxUtils.catchError);
      },
      this,
      enable,
      msgDialog
    );

  msgDialog.setMessage(
    enable ?
        zxMsg('MSG_BCK_smarscan_needed_to_enable_realtime') :
        zxMsg('MSG_BCK_will_disable_smartscan', [this.getServerName()]),
    DwtMessageDialog.WARNING_STYLE,
    ZxUtils.getTitleWithIcon('ZxBackup', 'ZxBackup')
  );

  msgDialog.registerCallback(
    DwtDialog.YES_BUTTON,
    new AjxCallback(
      this,
      function(callback, doEnable) {
        if (doEnable && ZeXtras.getOperationsManager().isBackupScanRunning(this.getServerName())) {
          this.getCurrentController().popupMsgDialog(zxMsg('MSG_BCK_already_scanning_op_ignored'));
          callback.run();

        } else {

          if (doEnable) {
            ZxAdminRequest.sendRequest(
              this.getServerName(),
              'ZxBackup',
              'doSmartScan',
              {
                notificationMails: ZaZimbraAdmin.currentUserLogin
              }
            ).then(callback, ZxUtils.catchError);

          } else {
            callback();

          }

        }
      },
      [doCallback, enable]
    )
  );

  msgDialog.popup();
};

/**
 * Suggest a restore date between, based on date of the first scan or the retention.
 * If first fullscan is older than the retention: will be returned now - retention.
 * I the retention is older than the fullscan: will be returned the fullscan date.
 * @param {{}} account
 * @return {number}
 */
ZxBackup.prototype.suggestRestoreDate = function(account) {
  var retention = this.getRetentionDays(),
    firstFullScan = this.getFirstFullscanDate(),
    dateRetention = new Date(new Date().getTime() - ZxUtils.DaysToMillis(retention));

  if (account && account.attrs && account.attrs[ZxBackup.A_zextrasUserBackupFirstScan]) {
    firstFullScan = new Date(account.attrs[ZxBackup.A_zextrasUserBackupFirstScan]);
  }

  if (firstFullScan.getTime() > dateRetention.getTime()) {
    return firstFullScan.getTime();
  } else {
    return dateRetention.getTime();
  }
};

/**
 * Start the undelete wizard of items of an account.
 * @param {ZaAccount} account
 */
ZxBackup.prototype.doUndelete = function(account) {
  var initUndeleteFcn = function(account) {
      var opObj = new ZxRestoreOp(this);
      opObj.init();
      opObj.setAccount(account);
      ZaApp.getInstance().dialogs["zxUndeleteWizard"].setObject(opObj);
      ZaApp.getInstance().dialogs["zxUndeleteWizard"].popup();
    };

  try {
    if (account.attrs.zimbraIsExternalVirtualAccount === 'TRUE') {
      return;
    }

    ZaApp.getInstance().dialogs['zxUndeleteWizard'] = new ZxUndeleteWizard(this);

    if (this.getRetentionDays() && this.getFirstFullscanDate()) {
      initUndeleteFcn.call(this, account);
    } else {
      this.loadRetentionData().then(
        AjxCallback.simpleClosure(
          initUndeleteFcn,
          this,
          account
        )
      );
    }

  } catch (ex) {
    ZxUtils.log(ex, 'Exceptions?!');
  }
};

/**
 * Start the Export Wizard.
 * @param {DwtEvent} ev
 */
ZxBackup.doStartExportWizard = function(ev) {
  var item = ZeXtras.getModuleItem(ZxBackup, this.getForm().getInstance().id),
    opObj;

  try {
    ZaApp.getInstance().dialogs['zxExportWizard'] = new ZxExportWizard(item);

    opObj = new ZxExportOp(item);
    opObj.init();
    ZaApp.getInstance().dialogs['zxExportWizard'].setObject(opObj);
    ZaApp.getInstance().dialogs['zxExportWizard'].popup();

  } catch (ex) {
    ZxUtils.log(ex);
  }
};

/**
 * Start the Emport Wizard.
 * @param {DwtEvent} ev
 */
ZxBackup.doStartImportWizard = function(ev) {
  var item = ZeXtras.getModuleItem(ZxBackup, this.getForm().getInstance().id),
    opObj;

  try {
    ZaApp.getInstance().dialogs['zxImportWizard'] = new ZxImportWizard(item);

    opObj = new ZxImportOp(item);
    opObj.init();
    ZaApp.getInstance().dialogs['zxImportWizard'].setObject(opObj);
    ZaApp.getInstance().dialogs['zxImportWizard'].popup();

  } catch (ex) {
    ZxUtils.log(ex);
  }
};

ZxBackup.doRestoreDeletedAccount = function(ev) {
  var form = this.getForm(),
    item = ZeXtras.getModuleItem(ZxBackup, form.getInstance().id),
    opObj;
  if (form._isDirty) {
    item.displayYouHaveUnsavedSettingsDialog();
  } else {
    try {
      ZaApp.getInstance().dialogs['zxRestoreDeletedAccount'] = new ZxRestoreDeletedAccountWizard(item);

      opObj = new ZxRestoreOp(item);
      opObj.init();
      ZaApp.getInstance().dialogs['zxRestoreDeletedAccount'].setObject(opObj);
      ZaApp.getInstance().dialogs['zxRestoreDeletedAccount'].popup();

    } catch (ex) {
      ZxUtils.log(ex, 'Exceptions?!');
    }
  }
};

/**
 *
 * @param {DwtSelectionEvent} ev
 * @this {Dwt_Button_XFormItem}
 */
ZxBackup.doEnableBackupForCOS = function(ev) {
  var item = this.getInstance();

  ZxBackup._doEnableDisableBackupForCOSFromBtn(item.name, true, this, ev);
};

/**
 *
 * @param {DwtSelectionEvent} ev
 * @this {Dwt_Button_XFormItem}
 */
ZxBackup.doDisableBackupForCOS = function(ev) {
  var item = this.getInstance();

  ZxBackup._doEnableDisableBackupForCOSFromBtn(item.name, false, this, ev);
};

/**
 *
 * @param {string} cosName
 * @param {boolean} enable
 * @param {Dwt_Button_XFormItem} form
 * @param {DwtSelectionEvent} event
 */
ZxBackup._doEnableDisableBackupForCOSFromBtn = function(cosName, enable, form, event) {
  var appCtx = ZaApp.getInstance().getAppCtxt(),
    message,
    msgDialog = new DwtMessageDialog({
      buttons: [DwtDialog.YES_BUTTON, DwtDialog.NO_BUTTON],
      title: '',
      parent: appCtx.getShell()
    });

  if (enable) {
    message = zxMsg('MSG_BCK_Enablig_backup_for_cos_warning');
  } else {
    message = zxMsg('MSG_BCK_Disabling_backup_for_cos_warning');
  }

  msgDialog.setMessage(
    zxMsg('MSG_BCK_Would_you_enable_or_disable_backup_for_cos',
      [enable ? zxMsg('LBL_enable') : zxMsg('LBL_disable')]) +
      '<br><strong>' + message + '</strong>',
    DwtMessageDialog.WARNING_STYLE,
    ZxUtils.getTitleWithIcon('ZxBackup', 'ZxBackup')
  );

  msgDialog.registerCallback(
    DwtDialog.YES_BUTTON,
    new AjxCallback(
      form,
      function(cosName, enable, dialog, form, event) {
        ZxAdminRequest.sendRequest(
          ZxAdminRequest.TARGET_LOCAL,
          'ZxBackup',
          'doEnableDisableCOS',
          {
            COSName: cosName,
            enableCOS: enable ? 'enable' : 'disable'
          }
        ).then(
          AjxCallback.simpleClosure(
            function(dialog, form, event, resp) {
              dialog.popdown();
              if (resp instanceof ZxError) {
                ZaApp.getInstance().getCurrentController().popupErrorDialog(
                  resp.getMessage()
                );
//                ZaApp.getInstance().getCurrentController().popupErrorDialog(
//                  zxMsg('MSG_Error_doing_zextras_request'),
//                  {
//                    msg: resp.error
//                  }
//                );
                return;
              }
              /* Refresh the current view loading new data from the server. */
              var currentController = ZaApp.getInstance().getCurrentController(),
                objItem,
                viewId,
                currentView,
                stepValue;

              if (!AjxUtil.isEmpty(currentController)) {
                objItem = currentController._currentObject;
                viewId = currentController.getContentViewId();
                currentView = ZaApp.getInstance().getAppViewMgr().getViewContentById(viewId);
                if (AjxUtil.isEmpty(currentView)) {
                  return;
                }
                stepValue = currentView._localXForm.instance.currentTab;
                if (objItem && objItem.refresh) {
                  objItem.refresh(false, true);
                  currentView.setObject(objItem);
                }
                if (!AjxUtil.isEmpty(currentView._localXForm)) {
                  currentView._localXForm.setInstanceValue(stepValue, ZaModel.currentTab);
                }
              }
            },
            this,
            dialog,
            form,
            event
          ),
          ZxUtils.catchError
        );
      },
      [cosName, enable, msgDialog, form, event]
    )
  );

  msgDialog.popup();
};

ZxBackup.checkEnableBckForCOSBtn = function() {
  var enabled = this.getInstanceValue(ZxBackup.A_zextrasBackupEnabled);
  return enabled === 'Disabled';
};

ZxBackup.checkDisableBckForCOSBtn = function() {
  var enabled = this.getInstanceValue(ZxBackup.A_zextrasBackupEnabled);
  return enabled === 'Enabled';
};

/**
 * Get the retention days set for the Backup Module
 * @return {number|undefined}
 */
ZxBackup.prototype.getRetentionDays = function() {
  if (this.attrs && this.attrs.properties && this.attrs.properties[ZxBackup.KEY_DATA_RETENTION]) {
    return this.attrs.properties[ZxBackup.KEY_DATA_RETENTION];
  }
};

/**
 * Get the backup path set for the Backup Module
 * @return {String|undefined}
 */
ZxBackup.prototype.getBackupPath = function() {
  if (this.attrs && this.attrs[ZxBackup.KEY_BACKUP_PATH]) {
    return this.attrs[ZxBackup.KEY_BACKUP_PATH];
  }
};

/**
 * Get the date of the first fullScan.
 * @return {Date|undefined}
 */
ZxBackup.prototype.getFirstFullscanDate = function() {
  if (this.attrs && this.attrs.statistics && this.attrs.statistics[ZxBackup.KEY_STAT_FULL_FIRST_FULLSCAN_KEY]) {
    return new Date(this.attrs.statistics[ZxBackup.KEY_STAT_FULL_FIRST_FULLSCAN_KEY]);
  }
};


/**
 * Load the retention days, it return a promise, will be resolved with the attrs data of the module.
 * @return {Q.promise}
 */
ZxBackup.prototype.loadRetentionData = function() {
  var deferred = Q.defer();

  Q.all([
    ZxAdminRequest.sendRequest(
      this.getCurrentServer(),
      'ZxBackup',
      'getPublicProperties'
    ),
    ZxAdminRequest.sendRequest(
      this.getCurrentServer(),
      'ZxBackup',
      'getBackupInfo'
    )
  ]).spread(
    AjxCallback.simpleClosure(
      function(promise, propResp, statResp) {
        var response = propResp.response.properties,
          statsFull = statResp.response.backupStat.scan;

        if (!this.attrs.properties) {
          this.attrs.properties = {};
        }
        if (!this.attrs.statistics) {
          this.attrs.statistics = {};
        }

        this.attrs.properties[ZxBackup.KEY_DATA_RETENTION] = response.ZxBackup_DataRetentionDays;
        this.attrs.properties[ZxBackup.KEY_BACKUP_PATH] = response.ZxBackup_DestPath;
        this.attrs.statistics[ZxBackup.KEY_STAT_FULL_FIRST_FULLSCAN_KEY] = new Date(Number(statsFull.firstScan));

        promise.resolve(this.attrs);
      },
      this,
      deferred
    ),
    AjxCallback.simpleClosure(
      function(promise, err) {
        promise.reject(err);
      },
      this,
      deferred
    )
  );

  return deferred.promise;
};

ZxBackup.prototype.displayYouHaveUnsavedSettingsDialog = function() {
  ZeXtrasItem.showMessageDialog(
    zxMsg('MSG_BCK_You_have_unsaved_settings'),
    ZxUtils.getTitleWithIcon('ZxBackup', zxMsg('MSG_BCK_Unsaved_settings')),
    DwtMessageDialog.WARNING_STYLE
  );
};
