/*
 * ***** BEGIN LICENSE BLOCK *****
 * Copyright (C) 2011-2016 ZeXtras
 *
 * The contents of this file are subject to the ZeXtras EULA;
 * you may not use this file except in compliance with the EULA.
 * You may obtain a copy of the EULA at
 * http://www.zextras.com/zextras-eula.html
 * ***** END LICENSE BLOCK *****
 */

function ZxBackupAccountModifier() {}

ZxBackupAccountModifier.NEW_PANEL = false;

ZxBackupAccountModifier.MODEL = [
  {type: _STRING_, id: ZxBackup.A_zextrasBackupEnabled,
    ref: 'attrs/' + ZxBackup.A_zextrasBackupEnabled,
    isZX: true},
  {type: _STRING_, id: ZxBackup.A_zextrasUserBackupFirstScan,
    ref: 'attrs/' + ZxBackup.A_zextrasUserBackupFirstScan,
    isZX: true},
  {type: _STRING_, id: ZxBackup.A_zextrasUserBackupLastScan,
    ref: 'attrs/' + ZxBackup.A_zextrasUserBackupLastScan,
    isZX: true},
  {type: _STRING_, id: ZxBackup.A_zextrasUserBackupNumBackupedItems,
    ref: 'attrs/' + ZxBackup.A_zextrasUserBackupNumBackupedItems,
    isZX: true},
  {type: _STRING_, id: ZxBackup.A_zextrasUserBackupNumDeletedItems,
    ref: 'attrs/' + ZxBackup.A_zextrasUserBackupNumDeletedItems,
    isZX: true}
];

ZxBackupAccountModifier.ATTRS = [
//  ZxChat.KEY_USR_ZIMLET_STATUS
];
ZxBackupAccountModifier.RIGHTS = [];

/**
 * Function invoked by the ZxAccountModifier when the tab is
 * opened.
 * @this {ZaAccount}
 */
ZxBackupAccountModifier.LOAD_ATTRS = function() {
  this.attrs[ZxBackup.A_zextrasBackupEnabled] = '';
  this.attrs[ZxBackup.A_zextrasUserBackupFirstScan] = 0;
  this.attrs[ZxBackup.A_zextrasUserBackupLastScan] = 0;
  this.attrs[ZxBackup.A_zextrasUserBackupNumBackupedItems] = 0;
  this.attrs[ZxBackup.A_zextrasUserBackupNumDeletedItems] = 0;

  Q.delay(10).then(
    AjxCallback.simpleClosure(ZxBackupAccountModifier.ASYNC_LOAD_ATTRS, this)
  );
};

ZxBackupAccountModifier.ASYNC_LOAD_ATTRS = function() {
  var mailHost = this.attrs[ZaAccount.A_mailHost],
    cos = this.getCurrentCos();

  ZxAdminRequest.sendRequest(
    mailHost,
    'ZxBackup',
    'getCOSBackupStatus',
    {
      COSName: cos.name
    }
  ).then(
    AjxCallback.simpleClosure(
      function (server, cos, resp) {

        var coslist = resp.response.cosList;

        ZxFormModifier.setInstanceValue(
          this,
          zxMsg(
            'MSG_BCK_Account_info',
            [
              '<strong>', '</strong>', cos.name, '<br>',
                '<a target="_blank" href="' + ZxLinks.HELP_BCK_User_tab_info + '">',
              '</a>',
              (coslist[cos.name] === 'Enabled') ?
                '<span style="color: #00A000">' + zxMsg('LBL_Included') + '</span>' :
                '<span style="color: #A00000">' + zxMsg('LBL_Excluded') + '</span>'
            ]
          ),
            'attrs/' + ZxBackup.A_zextrasBackupEnabled,
          true
        );
      },
      this,
      mailHost,
      cos
    ),
    AjxCallback.simpleClosure(
      function (error) {
        if (error.getCode() !== ZxErrorCode.BACKUP_NOT_INITIALIZED &&
          error.getCode() !== ZxErrorCode.UNABLE_TO_READ_BACKUP_STATS) {
          ZxUtils.catchError(error);
        }
      },
      this
    )
  );

  ZxAdminRequest.sendRequest(
    mailHost,
    'ZxBackup',
    'getAccountBackupInfo',
    {
      account: this.id
    }
  ).then(
    AjxCallback.simpleClosure(
      function (server, cos, resp) {
        var response = resp.response.backupStats;

        ZxFormModifier.setInstanceValue(
          this,
          response.firstScan,
            'attrs/' + ZxBackup.A_zextrasUserBackupFirstScan,
          true
        );

        ZxFormModifier.setInstanceValue(
          this,
          response.lastScan,
            'attrs/' + ZxBackup.A_zextrasUserBackupLastScan,
          true
        );

        ZxFormModifier.setInstanceValue(
          this,
          response[ZxBackup.A_zextrasUserBackupNumBackupedItems],
            'attrs/' + ZxBackup.A_zextrasUserBackupNumBackupedItems,
          true
        );

        ZxFormModifier.setInstanceValue(
          this,
          response[ZxBackup.A_zextrasUserBackupNumDeletedItems],
            'attrs/' + ZxBackup.A_zextrasUserBackupNumDeletedItems,
          true
        );
      },
      this,
      mailHost,
      cos
    ),
    AjxCallback.simpleClosure(
      function (error) {
        if (error.getCode() !== ZxErrorCode.BACKUP_NOT_INITIALIZED &&
          error.getCode() !== ZxErrorCode.UNABLE_TO_READ_BACKUP_STATS) {
          ZxUtils.catchError(error);
        }
      },
      this
    )
  );
};

ZxBackupAccountModifier.SAVE_ATTRS = function(prop, val) {
  ZxUtils.log({prop: prop, val: val}, 'Saving Value:');
};

ZxBackupAccountModifier.XFORM = {
  type: _ZA_TOP_GROUPER_,
  numCols: 3,
  width: '100%',
  colSizes: ['160px', '*', '350px'],
  label: zxMsg('LBL_BCK_Backup_preferences'),
  items: [
    {type: _OUTPUT_, height: 120, cssClass: 'ZxBackupLogo'},
    {
      type: _OUTPUT_,
      ref: 'attrs/' + ZxBackup.A_zextrasBackupEnabled,
      bmolsnr: true
    },
    {
      type: _GROUPER_,
      numCols: 1,
      colSizes: ['*'],
      label: zxMsg('LBL_BCK_Last_smartscan') + ' ' + zxMsg('LBL_statistics'),
      visibilityChangeEventSources: [
        'attrs/' + ZxBackup.A_zextrasUserBackupFirstScan,
        'attrs/' + ZxBackup.A_zextrasUserBackupLastScan
      ],
      visibilityChecks: [function(val) {
        var first = this.getInstanceValue('attrs/' + ZxBackup.A_zextrasUserBackupFirstScan),
          last = this.getInstanceValue('attrs/' + ZxBackup.A_zextrasUserBackupLastScan);
        return (first !== 0 && last !== 0);
      }],
      items: [
        {
          type: _OUTPUT_,
          ref: 'attrs/' + ZxBackup.A_zextrasUserBackupFirstScan,
          bmolsnr: true,
          label: zxMsg('LBL_BCK_First_scan') + ': ',
          getDisplayValue: function(value) {
            return ZxUtils.computeWordyDateStr(
              new Date(),
              Number(value)
            );
          }
        },
        {
          type: _OUTPUT_,
          ref: 'attrs/' + ZxBackup.A_zextrasUserBackupLastScan,
          bmolsnr: true,
          label: zxMsg('LBL_BCK_Last_scan') + ': ',
          getDisplayValue: function(value) {
            return ZxUtils.computeWordyDateStr(
              new Date(),
              Number(value)
            );
          }
        },
        {
          type: _OUTPUT_,
          ref: 'attrs/' + ZxBackup.A_zextrasUserBackupNumBackupedItems,
          bmolsnr: true,
          label: zxMsg('LBL_BCK_stat_backupped_items') + ': '
        },
        {
          type: _OUTPUT_,
          ref: 'attrs/' + ZxBackup.A_zextrasUserBackupNumDeletedItems,
          bmolsnr: true,
          label: zxMsg('LBL_BCK_stat_deleted_items') + ': '
        }
        //{type:_OUTPUT_, ref:"attrs/" + ZaAccount.A_zextrasStats + "/" +ZaAccount.A_zextrasStats_zimbraItems, label:"Total Zimbra items:"}
      ]
    }
  ]
};
