/*
 * ***** BEGIN LICENSE BLOCK *****
 * Copyright (C) 2011-2016 ZeXtras
 *
 * The contents of this file are subject to the ZeXtras EULA;
 * you may not use this file except in compliance with the EULA.
 * You may obtain a copy of the EULA at
 * http://www.zextras.com/zextras-eula.html
 * ***** END LICENSE BLOCK *****
 */

function ZxBackupCOSModifier() {}

ZxBackupCOSModifier.NEW_PANEL = false;

ZxBackupCOSModifier.MODEL = [
  {type: _STRING_, id: ZxBackup.A_zextrasBackupEnabled,
    ref: ZxBackup.A_zextrasBackupEnabled,
    isZX: true}
];

ZxBackupCOSModifier.ATTRS = [
//  ZxChat.KEY_USR_ZIMLET_STATUS
];
ZxBackupCOSModifier.RIGHTS = [];

/**
 * Function invoked by the ZxAccountModifier when the tab is
 * opened.
 * @this {ZaAccount}
 */
ZxBackupCOSModifier.LOAD_ATTRS = function() {
  this[ZxBackup.A_zextrasBackupEnabled] = 'Disabled';

  ZxAdminRequest.sendRequest(
    ZxAdminRequest.TARGET_LOCAL,
    'ZxBackup',
    'getCOSBackupStatus',
    {
      COSName: this.name
    }
  ).then(
    AjxCallback.simpleClosure(
      function(resp) {
        ZxFormModifier.setInstanceValue(
          this,
          resp.response.cosList[this.name],
          ZxBackup.A_zextrasBackupEnabled
        );
      },
      this
    ),
    AjxCallback.simpleClosure(
      function(error) {
        if (error.getCode() !== ZxErrorCode.BACKUP_NOT_INITIALIZED &&
          error.getCode() !== ZxErrorCode.UNABLE_TO_READ_BACKUP_STATS) {
          ZxUtils.catchError(error);
        }
      },
      this
    )
  );
};

ZxBackupCOSModifier.SAVE_ATTRS = function(prop, val) {
  ZxUtils.log({prop: prop, val: val}, 'Saving Value:');
};

ZxBackupCOSModifier.XFORM = {
  type: _ZA_TOP_GROUPER_,
  numCols: 2,
  width: '100%',
  colSizes: ['160px', '*'],
  label: zxMsg('LBL_BCK_Backup_preferences'),
  items: [
    {type: _OUTPUT_, height: 120, cssClass: 'ZxBackupLogo'},
    {
      type: _GROUP_,
      numCols: 3,
      colSizes: [350, 100, 200],
      items: [
        {
          type: _OUTPUT_,
          label: zxMsg('LBL_BCK_Current_backup_status_for_this_COS') + ':',
          ref: ZxBackup.A_zextrasBackupEnabled,
          getDisplayValue: function(val) {
            return (val === 'Enabled') ? zxMsg('LBL_Enabled') : zxMsg('LBL_Disabled');
          },
          bmolsnr: true
        },
        {
          type: _GROUP_,
          items: [
            {
              type: _DWT_BUTTON_,
              icon: 'ToggleStatus',
              width: 200,
              label: zxMsg('LBL_Enable'),
              toolTipContent: zxMsg('LBL_Enable'),
              onActivate: ZxBackup.doEnableBackupForCOS,
              visibilityChecks: [ZxBackup.checkEnableBckForCOSBtn],
              visibilityChangeEventSources: [ZxBackup.A_zextrasBackupEnabled],
              enableDisableChecks: [ZxBackup.checkEnableBckForCOSBtn],
              enableDisableChangeEventSources: [ZxBackup.A_zextrasBackupEnabled]
            },
            {
              type: _DWT_BUTTON_,
              icon: 'ToggleStatus',
              width: 200,
              label: zxMsg('LBL_Disable'),
              toolTipContent: zxMsg('LBL_Disable'),
              onActivate: ZxBackup.doDisableBackupForCOS,
              visibilityChecks: [ZxBackup.checkDisableBckForCOSBtn],
              visibilityChangeEventSources: [ZxBackup.A_zextrasBackupEnabled],
              enableDisableChecks: [ZxBackup.checkDisableBckForCOSBtn],
              enableDisableChangeEventSources: [ZxBackup.A_zextrasBackupEnabled]
            }
          ]
        }
      ]
    }
  ]
};
