/*
 * ***** BEGIN LICENSE BLOCK *****
 * Copyright (C) 2011-2016 ZeXtras
 *
 * The contents of this file are subject to the ZeXtras EULA;
 * you may not use this file except in compliance with the EULA.
 * You may obtain a copy of the EULA at
 * http://www.zextras.com/zextras-eula.html
 * ***** END LICENSE BLOCK *****
 */

/**
 * Item containing all Export operation's informations.
 * @param {ZxModule} moduleItem
 * @constructor
 */
function ZxExportOp(moduleItem) {
  ZaItem.call(this, 'ZxExportOp');
  this._init();
  this.type = ZxExportOp.TYPE;
  this._modItem = moduleItem;
}

ZxExportOp.TYPE = 'ExportOp';
ZxExportOp.prototype = new ZaItem();
ZxExportOp.prototype.constructor = ZxExportOp;

ZxExportOp.KEY_AVAILABLE_DOMAINS = 'availableDomains';
ZxExportOp.KEY_EXPORT_DOMAINS = 'exportDomains';
ZxExportOp.KEY_EXPORT_NOTIFICATION_MAIL = 'notificationMails';
ZxExportOp.KEY_EXPORT_PATH = 'exportPath';
ZxExportOp.KEY_SPACE_THRESHOLD = 'spaceThreshold';

/**
 * Initialize the Export Operation Item.
 */
ZxExportOp.prototype.init = function() {
  this.attrs = {};

  this.attrs[ZxExportOp.KEY_EXPORT_PATH] = '/opt/zimbra/backup/export';
  this.attrs[ZxExportOp.KEY_EXPORT_DOMAINS] = [];
  this.attrs[ZxExportOp.KEY_AVAILABLE_DOMAINS] = [];
  this.attrs[ZxExportOp.KEY_EXPORT_NOTIFICATION_MAIL] = '';

  ZxAdminRequest.sendRequest(
    this._modItem.getCurrentServer(),
    'ZxBackup',
    'getCurrentDomains'
  ).then(
    AjxCallback.simpleClosure(
      function(resp) {
        var response = resp.response;

        this.attrs[ZxExportOp.KEY_AVAILABLE_DOMAINS] = response.domainList;
        this.attrs[ZxExportOp.KEY_AVAILABLE_DOMAINS].sort();
      },
      this
    ),
    ZxUtils.catchError
  );
};

/**
 * Move rows between two lists.
 * @param {DwtListView} src
 * @param {DwtListView} dest
 * @param {Button_XFormItem} btn
 */
ZxExportOp.prototype.moveLists = function(src, dest, btn) {
  var selection,
    i,
    listA,
    listB;

  if (src.getId() && ZxUtils.indexOf(src.getId(), ZxExportWizard.ID_AVAILABLE_DOMAINS) !== -1) {
    listA = this.attrs[ZxExportOp.KEY_AVAILABLE_DOMAINS];
  }
  if (src.getId() && ZxUtils.indexOf(src.getId(), ZxExportWizard.ID_SELECTED_DOMAINS) !== -1) {
    listA = this.attrs[ZxExportOp.KEY_EXPORT_DOMAINS];
  }

  if (dest.getId() && ZxUtils.indexOf(dest.getId(), ZxExportWizard.ID_AVAILABLE_DOMAINS) !== -1) {
    listB = this.attrs[ZxExportOp.KEY_AVAILABLE_DOMAINS];
  }
  if (dest.getId() && ZxUtils.indexOf(dest.getId(), ZxExportWizard.ID_SELECTED_DOMAINS) !== -1) {
    listB = this.attrs[ZxExportOp.KEY_EXPORT_DOMAINS];
  }

  if (!listA || !listB) {
    return;
  }

  if (btn.getId() && ZxUtils.indexOf(btn.getId(), ZxExportWizard.ID_BUTTON_SELECT_ALL) !== -1) {

    for (i = 0; i < listA.length; i = i + 1) {
      listB.push(
        listA[i]
      );
    }
    listB.sort();
    listA.splice(0, listA.length);

    src.updateElement(listA);
    dest.updateElement(listB);

  } else {
    selection = src.widget.getSelection();

    selection.forEach(AjxCallback.simpleClosure(
      function(src, dest, listA, listB, btn, selection) {
        listA.splice(
          ZxUtils.indexOf(listA, selection),
          1
        );
        listB.push(selection);
        listB.sort();
        src.updateElement(listA);
        dest.updateElement(listB);
      },
      this,
      src,
      dest,
      listA,
      listB,
      btn
    ));
  }
};
