/*
 * ***** BEGIN LICENSE BLOCK *****
 * Copyright (C) 2011-2016 ZeXtras
 *
 * The contents of this file are subject to the ZeXtras EULA;
 * you may not use this file except in compliance with the EULA.
 * You may obtain a copy of the EULA at
 * http://www.zextras.com/zextras-eula.html
 * ***** END LICENSE BLOCK *****
 */

/**
 * Export wizard.
 * This file is used by the standard zimlet and the migration tool.
 * @param {ZxModule} item
 * @constructor
 */
function ZxExportWizard(item) {
  this._isMig = ZeXtras.isZxMig(item.getServerName());

  ZaXWizardDialog.call(
    this,
    item.getCurrentController().getShell(),
    null,
    ZxUtils.getTitleWithIcon(
      (this._isMig ? 'ZxMig' : 'ZxBackup'),
      zxMsg('LBL_BCK_Export_Wizard')
    ),
    '720px',
    '360px',
    'ZxExportWizard'
  );

  this.stepChoices = [
    {label: zxMsg('LBL_Introduction'),
      value: ZxExportWizard.STEP_INTRO},
    {label: zxMsg('LBL_BCK_select_export_dest'),
      value: ZxExportWizard.STEP_SELECT_PATH},
    {label: zxMsg('LBL_BCK_check_export_path'),
      value: ZxExportWizard.STEP_CHECK_PATH},
    {label: zxMsg('LBL_BCK_select_domains_to_export'),
      value: ZxExportWizard.STEP_SELECT_DOMAINS},
    {label: zxMsg('LBL_BCK_op_summary'),
      value: ZxExportWizard.STEP_SUMMARY}
  ];

  if (ZeXtras.isZxMig(item.getServerName())) {
    this._helpURL = ZxLinks.HELP_MIG_Export_wizard;
  } else {
    this._helpURL = ZxLinks.HELP_BCK_Export_wizard;
  }
  this.initForm(ZxExportWizard.XModel, this.getMyXForm());

  this._localXForm.setController(ZaApp.getInstance());
  this._localXForm.addListener(DwtEvent.XFORMS_FORM_DIRTY_CHANGE,
    new AjxListener(this, ZxExportWizard.prototype.handleXFormChange));
  this._localXForm.addListener(DwtEvent.XFORMS_VALUE_ERROR,
    new AjxListener(this, ZxExportWizard.prototype.handleXFormChange));
}

ZxExportWizard.prototype = new ZaXWizardDialog();
ZxExportWizard.prototype.constructor = ZxExportWizard;

ZxExportWizard.STEP_INDEX = 1;
ZxExportWizard.STEP_INTRO = ZxExportWizard.STEP_INDEX++;
ZxExportWizard.STEP_SELECT_PATH = ZxExportWizard.STEP_INDEX++;
ZxExportWizard.STEP_CHECK_PATH = ZxExportWizard.STEP_INDEX++;
ZxExportWizard.STEP_SELECT_DOMAINS = ZxExportWizard.STEP_INDEX++;
ZxExportWizard.STEP_SUMMARY = ZxExportWizard.STEP_INDEX++;

ZxExportWizard.ID_AVAILABLE_DOMAINS = 'AvailableDomains';
ZxExportWizard.ID_SELECTED_DOMAINS = 'SelectedDomains';
ZxExportWizard.ID_BUTTON_SELECT_ALL = 'SelectAllBTN';
ZxExportWizard.ID_BUTTON_SELECT = 'SelectBTN';
ZxExportWizard.ID_BUTTON_DESELECT = 'SelectBTN';

/**
 * XModel of the Wizard.
 * @type {Array}
 */
ZxExportWizard.XModel = [
  {type: _STRING_, id: ZxExportOp.KEY_EXPORT_PATH,
    ref: 'attrs/' + ZxExportOp.KEY_EXPORT_PATH},
  {type: _LIST_, id: ZxExportOp.KEY_EXPORT_DOMAINS,
    ref: 'attrs/' + ZxExportOp.KEY_EXPORT_DOMAINS},
  {type: _LIST_, id: ZxExportOp.KEY_AVAILABLE_DOMAINS,
    ref: 'attrs/' + ZxExportOp.KEY_AVAILABLE_DOMAINS},
  {type: _STRING_, id: ZxExportOp.KEY_EXPORT_NOTIFICATION_MAIL,
    ref: 'attrs/' + ZxExportOp.KEY_EXPORT_NOTIFICATION_MAIL}
];

ZaXDialog.XFormModifiers['ZxExportWizard'] = [];

/**
 * Define the type of operation of the item.
 * Used to select in which panel show the operation when the wizard window
 * is minimized.
 * @type {number}
 */
ZxExportWizard.prototype.miniType = 2;

/**
 *
 * @param xFormObject
 */
ZxExportWizard.XFormModifier = function(xFormObject) {
  var cases = [],
    step_intro,
    step_path,
    step_checkpath,
    step_domain,
    step_summary;

  step_intro = {
    type: _CASE_,
    caseKey: ZxExportWizard.STEP_INTRO,
    items: [
      {
        type: _ZAWIZ_TOP_GROUPER_,
        label: zxMsg('LBL_Introduction'),
        numCols: 1,
        colSizes: ['*'],
        items: [
          {
            type: _OUTPUT_,
            value: (this._isMig ? zxMsg('MSG_MIG_INTRO_Export_wizard') : zxMsg('MSG_BCK_INTRO_Export_wizard'))
          }
        ]
      }
    ]
  };

  step_path = {
    type: _CASE_,
    caseKey: ZxExportWizard.STEP_SELECT_PATH,
    items: [
      {
        type: _ZAWIZ_TOP_GROUPER_,
        label: zxMsg('LBL_BCK_destination_path'),
        numCols: 2,
        colSizes: ['130px', '*'],
        items: [
          {
            type: _INPUT_,
            ref: 'attrs/' + ZxExportOp.KEY_EXPORT_PATH,
            label: zxMsg('LBL_BCK_destination_path') + ': ',
            width: 250,
            align: _LEFT_,
            labelLocation: _LEFT_
          },
          {
            type: _INPUT_,
            ref: 'attrs/' + ZxExportOp.KEY_SPACE_THRESHOLD,
            label: zxMsg('LBL_BCK_Minimum_space_threshold') + ' (MB):',
            width: 50,
            align: _LEFT_,
            labelLocation: _LEFT_
          }
        ]
      }
    ]
  };

  step_checkpath = {
    type: _CASE_,
    caseKey: ZxExportWizard.STEP_CHECK_PATH,
    items: [
      {
        type: _ZAWIZ_TOP_GROUPER_,
        label: zxMsg('LBL_BCK_check_export_path'),
        numCols: 1,
        colSizes: ['*'],
        items: [
          {
            type: _OUTPUT_,
            cssStyle: 'padding-top: 10px',
            value: '<div class="DwtWait32Icon">&nbsp;</div><b>' + zxMsg('LBL_BCK_check_export_path') + '</b>'
          }
        ]
      }
    ]
  };

  step_domain = {
    type: _CASE_,
    caseKey: ZxExportWizard.STEP_SELECT_DOMAINS,
    numCols: 3,
    colSizes: [270, 100, 270],
    items: [
      {
        type: _ZAWIZ_TOP_GROUPER_,
        label: zxMsg('LBL_Available'),
        colSizes: ['*'],
        width: 230,
        numCols: 1,
        valign: _TOP_,
        items: [
          {
            type: _DWT_BUTTON_,
            label: zxMsg('LBL_Select_All') + ' &gt;&gt;',
            id: ZxExportWizard.ID_BUTTON_SELECT_ALL,
            cssStyle: 'width: 100%',
            onActivate: function() {
              var srcList = this.getForm().getItemsById(ZxExportWizard.ID_AVAILABLE_DOMAINS)[0],
                dstList = this.getForm().getItemsById(ZxExportWizard.ID_SELECTED_DOMAINS)[0];
              this.getForm().parent._containedObject.moveLists(srcList, dstList, this);
              this.getForm().parent.handleXFormChange();
            }
          },
          {
            type: _DWT_LIST_,
            id: ZxExportWizard.ID_AVAILABLE_DOMAINS,
            ref: 'attrs/' + ZxExportOp.KEY_AVAILABLE_DOMAINS,
            bmolsnr: true,
            height: '190px',
            cssClass: 'DLSource'
          }
        ]
      },
      {
        type: _GROUP_,
        colSizes: ['*'],
        width: 100,
        numCols: 1,
        cssClass: 'ZxListCenterButtons',
        items: [
          {
            type: _DWT_BUTTON_,
            label: zxMsg('LBL_Select') + ' &gt;',
            id: ZxExportWizard.ID_BUTTON_SELECT,
            width: 100,
            onActivate: function() {
              var srcList = this.getForm().getItemsById(ZxExportWizard.ID_AVAILABLE_DOMAINS)[0],
                dstList = this.getForm().getItemsById(ZxExportWizard.ID_SELECTED_DOMAINS)[0];
              this.getForm().parent._containedObject.moveLists(srcList, dstList, this);
              this.getForm().parent.handleXFormChange();
            }
          },
          {
            type: _DWT_BUTTON_,
            label: '&lt; ' + zxMsg('LBL_Deselect'),
            id: ZxExportWizard.ID_BUTTON_DESELECT,
            width: 100,
            onActivate: function() {
              var srcList = this.getForm().getItemsById(ZxExportWizard.ID_SELECTED_DOMAINS)[0],
                dstList = this.getForm().getItemsById(ZxExportWizard.ID_AVAILABLE_DOMAINS)[0];
              this.getForm().parent._containedObject.moveLists(srcList, dstList, this);
              this.getForm().parent.handleXFormChange();
            }
          }
        ]
      },
      {
        type: _ZAWIZ_TOP_GROUPER_,
        label: zxMsg('LBL_Selected'),
        colSizes: ['*'],
        numCols: 1,
        width: 230,
        items: [
          {
            type: _DWT_BUTTON_,
            label: '&lt;&lt; ' + zxMsg('LBL_Empty_Selection'),
            id: ZxExportWizard.ID_BUTTON_SELECT_ALL,
            width: 230,
            onActivate: function() {
              var srcList = this.getForm().getItemsById(ZxExportWizard.ID_SELECTED_DOMAINS)[0],
                dstList = this.getForm().getItemsById(ZxExportWizard.ID_AVAILABLE_DOMAINS)[0];
              this.getForm().parent._containedObject.moveLists(srcList, dstList, this);
              this.getForm().parent.handleXFormChange();
            }
          },
          {
            type: _DWT_LIST_,
            id: ZxExportWizard.ID_SELECTED_DOMAINS,
            ref: 'attrs/' + ZxExportOp.KEY_EXPORT_DOMAINS,
            bmolsnr: true,
            height: 120,
            cssClass: 'DLSource'
          },
          {
            type: _OUTPUT_,
            width: 230,
            cssStyle: 'padding-top: 10px',
            value: '<em>' + zxMsg('MSG_BCK_domain_alias__no_select_no_export') + '</em>'
          }
        ]
      }
    ]
  };

  step_summary = {
    type: _CASE_,
    caseKey: ZxExportWizard.STEP_SUMMARY,
    numCols: 1,
    items: [
      {
        type: _ZAWIZ_TOP_GROUPER_,
        label: zxMsg('LBL_BCK_op_summary'),
        items: [
          {
            type: _OUTPUT_,
            label: zxMsg('LBL_BCK_destination_path') + ': ',
            ref: 'attrs/' + ZxExportOp.KEY_EXPORT_PATH,
            cssStyle: 'font-weight: bold'
          },
          {
            type: _OUTPUT_,
            label: zxMsg('LBL_BCK_select_domains_to_export') + ': ',
            ref: 'attrs/' + ZxExportOp.KEY_EXPORT_DOMAINS,
            cssStyle: 'font-weight: bold',
            getDisplayValue: function() {
              return this.getInstanceValue().join(', ');
            }
          }
        ]
      },
      {
        type: _ZAWIZ_TOP_GROUPER_,
        label: zxMsg('LBL_BCK_op_notification'),
        numCols: 3,
        colSizes: ['*', '*', '*'],
        items: [
          {
            type: _INPUT_,
            label: zxMsg('LBL_BCK_additional_notification_mail') + ': ',
            ref: 'attrs/' + ZxExportOp.KEY_EXPORT_NOTIFICATION_MAIL,
            align: _LEFT_,
            width: 180
          },
          {
            type: _OUTPUT_,
            value: zxMsg('MSG_BCK_you_will_be_notified_when_export_completed'),
            align: _MIDDLE_
          }
        ]
      }
    ]
  };

  cases.push(step_intro);
  cases.push(step_path);
  cases.push(step_checkpath);
  cases.push(step_domain);
  cases.push(step_summary);

  xFormObject.items = [
    {
      type: _OUTPUT_,
      colSpan: 2,
      align: _CENTER_,
      valign: _TOP_,
      ref: ZaModel.currentStep,
      choices: this.stepChoices,
      valueChangeEventSources: [ZaModel.currentStep],
      labelClickMethod: function() {
        var sourceValue = this.getInheritedProperty('sourceValue'),
          instanceValue = this.getInstanceValue();
        if (sourceValue < instanceValue) {}
      }
    },
    {
      type: _SEPARATOR_,
      align: _CENTER_,
      valign: _TOP_
    },
    {
      type: _SPACER_,
      align: _CENTER_,
      valign: _TOP_
    },
    {
      type: _SWITCH_,
      align: _LEFT_,
      valign: _TOP_,
      items: cases
    }
  ];
};
ZaXDialog.XFormModifiers['ZxExportWizard'].push(ZxExportWizard.XFormModifier);

/**
 * Move to next page.
 * This also trigger a check of the entire form.
 */
ZxExportWizard.prototype.goNext = function() {
  this.goPage(this._containedObject[ZaModel.currentStep] + 1);
  this.handleXFormChange();
};

/**
 * Move to previous page.
 * This also trigger a check of the entire form.
 */
ZxExportWizard.prototype.goPrev = function() {
  if (this._containedObject[ZaModel.currentStep] === ZxExportWizard.STEP_SELECT_DOMAINS) {
    this.goPage(this._containedObject[ZaModel.currentStep] - 2);

  } else {
    this.goPage(this._containedObject[ZaModel.currentStep] - 1);

  }
  this.handleXFormChange();
};

/**
 * When something is changed (also the page number),
 * validate the inputs and enable/disable wizard buttons.
 */
ZxExportWizard.prototype.handleXFormChange = function() {
  var cStep = this._containedObject[ZaModel.currentStep];
  ZxUtils.log(cStep, 'Wizard step');

  if (this._localXForm.hasErrors()) {
    this._button[DwtWizardDialog.FINISH_BUTTON].setEnabled(false);
  } else {
    this._button[DwtWizardDialog.PREV_BUTTON].setEnabled(true);

    if (cStep === ZxExportWizard.STEP_INTRO) {
      this._button[DwtDialog.CANCEL_BUTTON].setEnabled(true);
      this._button[DwtWizardDialog.NEXT_BUTTON].setEnabled(true);
      this._button[DwtWizardDialog.PREV_BUTTON].setEnabled(false);
      this._button[DwtWizardDialog.FINISH_BUTTON].setEnabled(false);
      return;
    }

    if (cStep === ZxExportWizard.STEP_SELECT_PATH) {
      if (this._containedObject.attrs[ZxExportOp.KEY_EXPORT_PATH] &&
          this._containedObject.attrs[ZxExportOp.KEY_EXPORT_PATH] !== '') {

        this._button[DwtWizardDialog.NEXT_BUTTON].setEnabled(true);
      } else {
        this._button[DwtWizardDialog.NEXT_BUTTON].setEnabled(false);
      }
      this._button[DwtDialog.CANCEL_BUTTON].setEnabled(true);
      this._button[DwtWizardDialog.PREV_BUTTON].setEnabled(true);
      this._button[DwtWizardDialog.FINISH_BUTTON].setEnabled(false);
      return;
    }

    if (cStep === ZxExportWizard.STEP_CHECK_PATH) {
      this._button[DwtDialog.CANCEL_BUTTON].setEnabled(false);
      this._button[DwtWizardDialog.PREV_BUTTON].setEnabled(false);
      this._button[DwtWizardDialog.NEXT_BUTTON].setEnabled(false);
      this._button[DwtWizardDialog.FINISH_BUTTON].setEnabled(false);
      if (this._containedObject.attrs[ZxExportOp.KEY_SPACE_THRESHOLD] && !/^\d+$/.test(this._containedObject.attrs[ZxExportOp.KEY_SPACE_THRESHOLD])) {
        ZaApp.getInstance().getCurrentController().popupWarningDialog(
          zxMsg('MSG_Value_of_is_not_valid', [zxMsg('LBL_HSM_Threshold')])
        );
        this.goPrev();
        return;
      }

      ZxAdminRequest.sendRequest(
        this._containedObject._modItem.getCurrentServer(),
        'ZxBackup',
        'getBackupInfo',
        {
          dataPath: this._containedObject.attrs[ZxExportOp.KEY_EXPORT_PATH]
        }
      ).then(
        AjxCallback.simpleClosure(
          function(resp) {
            var response = resp.response;

            if (resp instanceof ZxError) {
              ZaApp.getInstance().getCurrentController().popupErrorDialog(
                resp.getMessage()
              );
//              ZaApp.getInstance().getCurrentController().popupErrorDialog(
//                zxMsg('MSG_BCK_error_checking_export_path'),
//                resp.error || resp[this._containedObject._modItem.getServerName()].error
//              );
              this.goPrev();
              return;
            }

            if (!response.exists) {
              ZaApp.getInstance().getCurrentController().popupWarningDialog(
                zxMsg('MSG_BCK_path_does_not_exist')
              );
              this.goPrev();
              return;
            }

            if (!response.writable || !response.readable) {
              ZaApp.getInstance().getCurrentController().popupWarningDialog(
                zxMsg('MSG_BCK_path_not_readable_writable')
              );
              this.goPrev();
              return;
            }

            if (!response.caseSensitive) {
              ZaApp.getInstance().getCurrentController().popupWarningDialog(
                zxMsg('MSG_BCK_path_case_insensitive')
              );
              this.goPrev();
              return;
            }

            if (!response.empty) {
              ZaApp.getInstance().getCurrentController().popupWarningDialog(
                zxMsg('MSG_BCK_path_not_empty')
              );
              this.goPrev();
              return;
            }

            this.goNext();
          },
          this
        ),
        ZxUtils.catchError
      );

      return;
    }

    if (cStep === ZxExportWizard.STEP_SELECT_DOMAINS) {
      this._button[DwtDialog.CANCEL_BUTTON].setEnabled(true);
      this._button[DwtWizardDialog.PREV_BUTTON].setEnabled(true);
      if (this._containedObject.attrs[ZxExportOp.KEY_EXPORT_DOMAINS].length > 0) {
        this._button[DwtWizardDialog.NEXT_BUTTON].setEnabled(true);
      } else {
        this._button[DwtWizardDialog.NEXT_BUTTON].setEnabled(false);
      }
      this._button[DwtWizardDialog.FINISH_BUTTON].setEnabled(false);
      return;
    }

    if (cStep === ZxExportWizard.STEP_SUMMARY) {
      this._button[DwtDialog.CANCEL_BUTTON].setEnabled(true);
      this._button[DwtWizardDialog.PREV_BUTTON].setEnabled(true);
      this._button[DwtWizardDialog.NEXT_BUTTON].setEnabled(false);
      this._button[DwtWizardDialog.FINISH_BUTTON].setEnabled(true);
      return;
    }
  }
};

/**
 * Method invoked when the Finish button is pressed.
 */
ZxExportWizard.prototype.finishWizard = function() {
  this._button[DwtWizardDialog.PREV_BUTTON].setEnabled(false);
  this._button[DwtWizardDialog.FINISH_BUTTON].setEnabled(false);

  var item = this._containedObject._modItem,
    attrs = this._containedObject.attrs,
    reqParam = {};

  if (attrs[ZxExportOp.KEY_EXPORT_NOTIFICATION_MAIL] &&
      attrs[ZxExportOp.KEY_EXPORT_NOTIFICATION_MAIL] !== '') {

    attrs[ZxExportOp.KEY_EXPORT_NOTIFICATION_MAIL] += ',' +
      ZaZimbraAdmin.currentUserLogin;

  } else {
    attrs[ZxExportOp.KEY_EXPORT_NOTIFICATION_MAIL] = ZaZimbraAdmin.currentUserLogin;
  }

  reqParam.canBeFirstScan = true;
  reqParam.notificationMails = attrs[ZxExportOp.KEY_EXPORT_NOTIFICATION_MAIL];
  reqParam.dataPath = attrs[ZxExportOp.KEY_EXPORT_PATH];
  reqParam.domainList = attrs[ZxExportOp.KEY_EXPORT_DOMAINS];
  reqParam.spaceThreshold = attrs[ZxExportOp.KEY_SPACE_THRESHOLD];

  ZxUtils.log(reqParam, 'Sending request for Export');

  ZxAdminRequest.sendRequest(
    this._containedObject._modItem.getCurrentServer(),
    'ZxBackup',
    'doExport',
    reqParam
  ).then(
    AjxCallback.simpleClosure(function(resp) {
      ZaApp.getInstance().getCurrentController().popupMsgDialog(zxMsg('MSG_Operation_queued'));
      this.popdown();
      ZeXtras.getOperationsManager().requestUpdate();
    }, this),
    ZxUtils.catchError
  );
};

/**
 * Set the instance object.
 * When a wizard (on Z8) is moved as running tasks, when is resumed,
 * Zimbra invoke this function to restore the form with the correct informations.
 * @param {ZxExportOp} entry
 */
ZxExportWizard.prototype.setObject = function(entry) {
  this._containedObject = entry;

  this._containedObject[ZaModel.currentStep] = entry[ZaModel.currentStep] || ZxExportWizard.STEP_INTRO;
  this._localXForm.setInstance(this._containedObject);
};
