/*
 * ***** BEGIN LICENSE BLOCK *****
 * Copyright (C) 2011-2016 ZeXtras
 *
 * The contents of this file are subject to the ZeXtras EULA;
 * you may not use this file except in compliance with the EULA.
 * You may obtain a copy of the EULA at
 * http://www.zextras.com/zextras-eula.html
 * ***** END LICENSE BLOCK *****
 */

/**
 * Item containing all Import operation's informations.
 * @param {ZxModule} moduleItem
 * @constructor
 */
function ZxImportOp(moduleItem) {
  ZaItem.call(this, 'ZxImportOp');
  this._init();
  this.type = ZxImportOp.TYPE;
  this._modItem = moduleItem;
}

ZxImportOp.TYPE = 'ImportOp';
ZxImportOp.prototype = new ZaItem();
ZxImportOp.prototype.constructor = ZxImportOp;

ZxImportOp.KEY_AVAILABLE_DOMAINS = 'availableDomains';
ZxImportOp.KEY_AVAILABLE_ACCOUNTS = 'availableAccounts';
ZxImportOp.KEY_IMPORT_DOMAINS = 'importDomains';
ZxImportOp.KEY_IMPORT_ACCOUNTS = 'importAccounts';
ZxImportOp.KEY_IMPORT_PATH = 'importPath';
ZxImportOp.KEY_FILTER_DELETE_ACCOUNT = 'deleteAccount';
ZxImportOp.KEY_SKIP_SYSTEM_ACCOUNT = 'skipSystemAccount';
ZxImportOp.KEY_SKIP_ALIAS = 'skipAlias';
ZxImportOp.KEY_SKIP_DISTRIBUTION_LIST = 'skipDistributionList';
ZxImportOp.KEY_SKIP_COS = 'skipCos';
ZxImportOp.KEY_OBEY_HSM = 'obeyHSM';
ZxImportOp.KEY_IMPORT_NOTIFICATION_MAIL = 'notificationMails';

/**
 * Initialize the Import Operation Item.
 */
ZxImportOp.prototype.init = function() {
  this.attrs = {};

  this.attrs[ZxImportOp.KEY_IMPORT_PATH] = '/opt/zimbra/backup/export';
  this.attrs[ZxImportOp.KEY_AVAILABLE_DOMAINS] = [];
  this.attrs[ZxImportOp.KEY_AVAILABLE_ACCOUNTS] = [];
  this.attrs[ZxImportOp.KEY_IMPORT_DOMAINS] = [];
  this.attrs[ZxImportOp.KEY_IMPORT_ACCOUNTS] = [];
  this.attrs[ZxImportOp.KEY_EXPORT_NOTIFICATION_MAIL] = '';
  this.attrs[ZxImportOp.KEY_FILTER_DELETE_ACCOUNT] = true;
  this.attrs[ZxImportOp.KEY_SKIP_SYSTEM_ACCOUNT] = true;
  this.attrs[ZxImportOp.KEY_SKIP_ALIAS] = false;
  this.attrs[ZxImportOp.KEY_SKIP_DISTRIBUTION_LIST] = false;
  this.attrs[ZxImportOp.KEY_SKIP_COS] = false;
  this.attrs[ZxImportOp.KEY_OBEY_HSM] = false;
};

/**
 * Move rows between two lists.
 * @param {DwtListView} src
 * @param {DwtListView} dest
 * @param {Button_XFormItem} btn
 */
ZxImportOp.prototype.moveLists = function(src, dest, btn) {
  var selection,
    i,
    listA,
    listB;

  if (src.getId() && ZxUtils.indexOf(src.getId(), ZxImportWizard.ID_AVAILABLE_DOMAINS) !== -1) {
    listA = this.attrs[ZxImportOp.KEY_AVAILABLE_DOMAINS];
  }
  if (src.getId() && ZxUtils.indexOf(src.getId(), ZxImportWizard.ID_SELECTED_DOMAINS) !== -1) {
    listA = this.attrs[ZxImportOp.KEY_IMPORT_DOMAINS];
  }
  if (src.getId() && ZxUtils.indexOf(src.getId(), ZxImportWizard.ID_AVAILABLE_ACCOUNTS) !== -1) {
    listA = this.attrs[ZxImportOp.KEY_AVAILABLE_ACCOUNTS];
  }
  if (src.getId() && ZxUtils.indexOf(src.getId(), ZxImportWizard.ID_SELECTED_ACCOUNTS) !== -1) {
    listA = this.attrs[ZxImportOp.KEY_IMPORT_ACCOUNTS];
  }

  if (dest.getId() && ZxUtils.indexOf(dest.getId(), ZxImportWizard.ID_AVAILABLE_DOMAINS) !== -1) {
    listB = this.attrs[ZxImportOp.KEY_AVAILABLE_DOMAINS];
  }
  if (dest.getId() && ZxUtils.indexOf(dest.getId(), ZxImportWizard.ID_SELECTED_DOMAINS) !== -1) {
    listB = this.attrs[ZxImportOp.KEY_IMPORT_DOMAINS];
  }
  if (dest.getId() && ZxUtils.indexOf(dest.getId(), ZxImportWizard.ID_AVAILABLE_ACCOUNTS) !== -1) {
    listB = this.attrs[ZxImportOp.KEY_AVAILABLE_ACCOUNTS];
  }
  if (dest.getId() && ZxUtils.indexOf(dest.getId(), ZxImportWizard.ID_SELECTED_ACCOUNTS) !== -1) {
    listB = this.attrs[ZxImportOp.KEY_IMPORT_ACCOUNTS];
  }

  if (!listA || !listB) {
    return;
  }

  if (btn.getId() && ZxUtils.indexOf(btn.getId(), ZxImportWizard.ID_BUTTON_SELECT_ALL) !== -1) {
    if (src.isFiltered()) {
      listA = src.getFilteredList();
    }

    for (i = 0; i < listA.length; i = i + 1) {
      listB.push(
        listA[i]
      );
    }
    listB.sort();
    listA.splice(0, listA.length);

    src.updateElement(listA);
    dest.updateElement(listB);

  } else {
    selection = src.getSelection();

    selection.forEach(AjxCallback.simpleClosure(
      function(src, dest, listA, listB, btn, selection) {
        listA.splice(
          ZxUtils.indexOf(listA, selection),
          1
        );
        listB.push(selection);
        listB.sort();
        src.updateElement(listA);
        dest.updateElement(listB);
      },
      this,
      src,
      dest,
      listA,
      listB,
      btn
    ));
  }
};
