/*
 * ***** BEGIN LICENSE BLOCK *****
 * Copyright (C) 2011-2016 ZeXtras
 *
 * The contents of this file are subject to the ZeXtras EULA;
 * you may not use this file except in compliance with the EULA.
 * You may obtain a copy of the EULA at
 * http://www.zextras.com/zextras-eula.html
 * ***** END LICENSE BLOCK *****
 */

function ZxImportWizard(item) {
  if (item instanceof DwtShell) {
    ZaXWizardDialog.call(this, item,
      null, ZxUtils.getTitleWithIcon('ZxBackup', zxMsg('LBL_BCK_Import_Wizard')), '720px', '420px', 'ZxImportWizard');
  } else {
    ZaXWizardDialog.call(this, item.getCurrentController().getShell(),
      null, ZxUtils.getTitleWithIcon('ZxBackup', zxMsg('LBL_BCK_Import_Wizard')), '720px', '420px', 'ZxImportWizard');
  }

  this.stepChoices = [
    {label: zxMsg('LBL_Introduction'),
      value: ZxImportWizard.STEP_INTRO},
    {label: zxMsg('MSG_BCK_select_source_backup_path'),
      value: ZxImportWizard.STEP_SELECT_PATH},
    {label: zxMsg('LBL_BCK_check_import_path'),
      value: ZxImportWizard.STEP_CHECK_PATH},
    {label: zxMsg('LBL_BCK_select_domains_to_import'),
      value: ZxImportWizard.STEP_SELECT_DOMAINS},
    {label: zxMsg('LBL_BCK_account_infos'),
      value: ZxImportWizard.STEP_GET_ACCOUNTS},
    {label: zxMsg('MSG_BCK_select_accounts_to_import'),
      value: ZxImportWizard.STEP_SELECT_ACCOUNTS},
    {label: zxMsg('LBL_BCK_op_summary'),
      value: ZxImportWizard.STEP_SUMMARY}
  ];

  this._helpURL = ZxLinks.HELP_BCK_Import_wizard;
  this.initForm(ZxImportWizard.XModel, this.getMyXForm());

  this._localXForm.setController(ZaApp.getInstance());
  this._localXForm.addListener(DwtEvent.XFORMS_FORM_DIRTY_CHANGE,
    new AjxListener(this, ZxImportWizard.prototype.handleXFormChange));
  this._localXForm.addListener(DwtEvent.XFORMS_VALUE_ERROR,
    new AjxListener(this, ZxImportWizard.prototype.handleXFormChange));
}

ZxImportWizard.prototype = new ZaXWizardDialog();
ZxImportWizard.prototype.constructor = ZxImportWizard;

ZxImportWizard.STEP_INDEX = 1;
ZxImportWizard.STEP_INTRO = ZxImportWizard.STEP_INDEX++;
ZxImportWizard.STEP_SELECT_PATH = ZxImportWizard.STEP_INDEX++;
ZxImportWizard.STEP_CHECK_PATH = ZxImportWizard.STEP_INDEX++;
ZxImportWizard.STEP_SELECT_DOMAINS = ZxImportWizard.STEP_INDEX++;
ZxImportWizard.STEP_GET_ACCOUNTS = ZxImportWizard.STEP_INDEX++;
ZxImportWizard.STEP_SELECT_ACCOUNTS = ZxImportWizard.STEP_INDEX++;
ZxImportWizard.STEP_SUMMARY = ZxImportWizard.STEP_INDEX++;

ZxImportWizard.ID_AVAILABLE_DOMAINS = 'AvailableDomains';
ZxImportWizard.ID_SELECTED_DOMAINS = 'SelectedDomains';
ZxImportWizard.ID_AVAILABLE_ACCOUNTS = 'AvailableAccounts';
ZxImportWizard.ID_SELECTED_ACCOUNTS = 'SelectedAccounts';
ZxImportWizard.ID_BUTTON_SELECT_ALL = 'SelectAllBTN';
ZxImportWizard.ID_BUTTON_SELECT = 'SelectBTN';
ZxImportWizard.ID_BUTTON_DESELECT = 'SelectBTN';

/**
 * XModel of the Wizard.
 * @type {Array}
 */
ZxImportWizard.XModel = [
  {type: _ENUM_, id: ZxImportOp.KEY_FILTER_DELETE_ACCOUNT,
    ref: 'attrs/' + ZxImportOp.KEY_FILTER_DELETE_ACCOUNT,
    choices: ZaModel.BOOLEAN_CHOICES1},
  {type: _ENUM_, id: ZxImportOp.KEY_SKIP_SYSTEM_ACCOUNT,
    ref: 'attrs/' + ZxImportOp.KEY_SKIP_SYSTEM_ACCOUNT,
    choices: ZaModel.BOOLEAN_CHOICES1},
  {type: _ENUM_, id: ZxImportOp.KEY_SKIP_ALIAS,
    ref: 'attrs/' + ZxImportOp.KEY_SKIP_ALIAS,
    choices: ZaModel.BOOLEAN_CHOICES1},
  {type: _ENUM_, id: ZxImportOp.KEY_SKIP_DISTRIBUTION_LIST,
    ref: 'attrs/' + ZxImportOp.KEY_SKIP_DISTRIBUTION_LIST,
    choices: ZaModel.BOOLEAN_CHOICES1},
  {type: _ENUM_, id: ZxImportOp.KEY_SKIP_COS,
    ref: 'attrs/' + ZxImportOp.KEY_SKIP_COS,
    choices: ZaModel.BOOLEAN_CHOICES1},
  {type: _ENUM_, id: ZxImportOp.KEY_OBEY_HSM,
    ref: 'attrs/' + ZxImportOp.KEY_OBEY_HSM,
    choices: ZaModel.BOOLEAN_CHOICES1},
  {type: _STRING_, id: ZxImportOp.KEY_IMPORT_PATH,
    ref: 'attrs/' + ZxImportOp.KEY_IMPORT_PATH},
  {type: _LIST_, id: ZxImportOp.KEY_IMPORT_DOMAINS,
    ref: 'attrs/' + ZxImportOp.KEY_IMPORT_DOMAINS},
  {type: _LIST_, id: ZxImportOp.KEY_AVAILABLE_DOMAINS,
    ref: 'attrs/' + ZxImportOp.KEY_AVAILABLE_DOMAINS},
  {type: _LIST_, id: ZxImportOp.KEY_IMPORT_ACCOUNTS,
    ref: 'attrs/' + ZxImportOp.KEY_IMPORT_ACCOUNTS},
  {type: _LIST_, id: ZxImportOp.KEY_AVAILABLE_ACCOUNTS,
    ref: 'attrs/' + ZxImportOp.KEY_AVAILABLE_ACCOUNTS},
  {type: _STRING_, id: ZxImportOp.KEY_IMPORT_NOTIFICATION_MAIL,
    ref: 'attrs/' + ZxImportOp.KEY_IMPORT_NOTIFICATION_MAIL}
];

ZaXDialog.XFormModifiers['ZxImportWizard'] = [];

ZxImportWizard.prototype.miniType = 2; // WTF is this?

ZxImportWizard.XFormModifier = function(xFormObject) {
  var cases = [],
    step_intro,
    step_path,
    step_checkpath,
    step_domain,
    step_getAccount,
    step_accounts,
    step_summary;

  step_intro = {
    type: _CASE_,
    caseKey: ZxImportWizard.STEP_INTRO,
    items: [
      {
        type: _ZAWIZ_TOP_GROUPER_,
        label: zxMsg('LBL_Introduction'),
        numCols: 1,
        colSizes: ['*'],
        items: [
          {
            type: _OUTPUT_,
            value: zxMsg('MSG_BCK_INTRO_Import_wizard')
          }
        ]
      }
    ]
  };

  step_path = {
    type: _CASE_,
    caseKey: ZxImportWizard.STEP_SELECT_PATH,
    items: [
      {
        type: _ZAWIZ_TOP_GROUPER_,
        label: zxMsg('LBL_BCK_source_path'),
        numCols: 2,
        colSizes: [250, '*'],
        items: [
          {
            type: _INPUT_,
            ref: 'attrs/' + ZxImportOp.KEY_IMPORT_PATH,
            label: zxMsg('LBL_BCK_source_path') + ': ',
            width: '250px',
            align: _LEFT_,
            labelLocation: _LEFT_
          },
          {type: _SPACER_, align: _CENTER_, valign: _TOP_, height: '10px'},
          {
            type: _CHECKBOX_,
            labelLocation: _LEFT_,
            align: _LEFT_,
            labelCssClass: 'ZxCheckboxLabel',
            label: zxMsg('LBL_BCK_hide_deleted_account') + ': ',
            ref: 'attrs/' + ZxImportOp.KEY_FILTER_DELETE_ACCOUNT
          },
          {
            type: _CHECKBOX_,
            labelCssClass: 'ZxCheckboxLabel',
            label: zxMsg('LBL_BCK_do_not_restore_system_accounts') + ': ',
            labelLocation: _LEFT_,
            align: _LEFT_,
            ref: 'attrs/' + ZxImportOp.KEY_SKIP_SYSTEM_ACCOUNT
          },
          {
            type: _CHECKBOX_,
            labelCssClass: 'ZxCheckboxLabel',
            label: zxMsg('LBL_BCK_do_not_restore_aliases') + ': ',
            labelLocation: _LEFT_,
            align: _LEFT_,
            ref: 'attrs/' + ZxImportOp.KEY_SKIP_ALIAS
          },
          {
            type: _CHECKBOX_,
            labelCssClass: 'ZxCheckboxLabel',
            label: zxMsg('LBL_BCK_do_not_restore_distribution_lists') + ': ',
            labelLocation: _LEFT_,
            align: _LEFT_,
            ref: 'attrs/' + ZxImportOp.KEY_SKIP_DISTRIBUTION_LIST
          },
          {
            type: _CHECKBOX_,
            labelCssClass: 'ZxCheckboxLabel',
            label: zxMsg('LBL_BCK_do_not_restore_coses') + ': ',
            labelLocation: _LEFT_,
            align: _LEFT_,
            ref: ZxImportOp.KEY_SKIP_COS
          }
        ]
      }
    ]
  };

  step_checkpath = {
    type: _CASE_,
    caseKey: ZxImportWizard.STEP_CHECK_PATH,
    items: [
      {
        type: _ZAWIZ_TOP_GROUPER_,
        label: zxMsg('LBL_BCK_check_import_path'),
        numCols: 1,
        colSizes: ['*'],
        items: [
          {
            type: _OUTPUT_,
            cssStyle: 'padding-top: 10px',
            value: '<div class="DwtWait32Icon">&nbsp;</div><b>' + zxMsg('LBL_BCK_checking_import_path') + '</b>'
          }
        ]
      }
    ]
  };

  step_domain = {
    type: _CASE_,
    caseKey: ZxImportWizard.STEP_SELECT_DOMAINS,
    numCols: 3,
    colSizes: [270, 100, 270],
    items: [
      {
        type: _ZAWIZ_TOP_GROUPER_,
        label: zxMsg('LBL_Available'),
        colSizes: ['*'],
        width: 230,
        numCols: 1,
        items: [
          {
            type: _DWT_BUTTON_,
            label: zxMsg('LBL_Select_All') + ' &gt;&gt;',
            id: ZxImportWizard.ID_BUTTON_SELECT_ALL,
            cssStyle: 'width: 100%',
            onActivate: function() {
              var srcList = this.getForm().getItemsById(ZxImportWizard.ID_AVAILABLE_DOMAINS)[0],
                dstList = this.getForm().getItemsById(ZxImportWizard.ID_SELECTED_DOMAINS)[0];
              this.getForm().parent._containedObject.moveLists(srcList, dstList, this);
              this.getForm().parent.handleXFormChange();
            }
          },
          {
            type: _ZX_LIST_VIEW_,
            id: ZxImportWizard.ID_AVAILABLE_DOMAINS,
            ref: 'attrs/' + ZxImportOp.KEY_AVAILABLE_DOMAINS,
            height: 190
          }
        ]
      },
      {
        type: _GROUP_,
        colSizes: ['*'],
        width: 100,
        numCols: 1,
        cssClass: 'ZxListCenterButtons',
        items: [
          {
            type: _DWT_BUTTON_,
            label: zxMsg('LBL_Select') + ' &gt;',
            id: ZxImportWizard.ID_BUTTON_SELECT,
            width: 100,
            onActivate: function() {
              var srcList = this.getForm().getItemsById(ZxImportWizard.ID_AVAILABLE_DOMAINS)[0],
                dstList = this.getForm().getItemsById(ZxImportWizard.ID_SELECTED_DOMAINS)[0];
              this.getForm().parent._containedObject.moveLists(srcList, dstList, this);
              this.getForm().parent.handleXFormChange();
            }
          },
          {
            type: _DWT_BUTTON_,
            label: '&lt; ' + zxMsg('LBL_Deselect'),
            id: ZxImportWizard.ID_BUTTON_DESELECT,
            width: 100,
            onActivate: function() {
              var srcList = this.getForm().getItemsById(ZxImportWizard.ID_SELECTED_DOMAINS)[0],
                dstList = this.getForm().getItemsById(ZxImportWizard.ID_AVAILABLE_DOMAINS)[0];
              this.getForm().parent._containedObject.moveLists(srcList, dstList, this);
              this.getForm().parent.handleXFormChange();
            }
          }
        ]
      },
      {
        type: _ZAWIZ_TOP_GROUPER_,
        label: zxMsg('LBL_Selected'),
        colSizes: ['*'],
        numCols: 1,
        width: 230,
        items: [
          {
            type: _DWT_BUTTON_,
            label: '&lt;&lt; ' + zxMsg('LBL_Empty_Selection'),
            id: ZxImportWizard.ID_BUTTON_SELECT_ALL,
            width: 230,
            onActivate: function() {
              var srcList = this.getForm().getItemsById(ZxImportWizard.ID_SELECTED_DOMAINS)[0],
                dstList = this.getForm().getItemsById(ZxImportWizard.ID_AVAILABLE_DOMAINS)[0];
              this.getForm().parent._containedObject.moveLists(srcList, dstList, this);
              this.getForm().parent.handleXFormChange();
            }
          },
          {
            type: _ZX_LIST_VIEW_,
            id: ZxImportWizard.ID_SELECTED_DOMAINS,
            ref: 'attrs/' + ZxImportOp.KEY_IMPORT_DOMAINS,
            height: 190
          }
        ]
      }
    ]
  };

  step_getAccount = {
    type: _CASE_,
    caseKey: ZxImportWizard.STEP_GET_ACCOUNTS,
    items: [
      {
        type: _ZAWIZ_TOP_GROUPER_,
        label: zxMsg('LBL_BCK_getting_accounts_import'),
        numCols: 1,
        colSizes: ['*'],
        items: [
          {
            type: _OUTPUT_,
            cssStyle: 'padding-top: 10px',
            value: '<div class="DwtWait32Icon">&nbsp;</div><b>' + zxMsg('LBL_BCK_getting_accounts_import') + '</b>'
          }
        ]
      }
    ]
  };

  step_accounts = {
    type: _CASE_,
    caseKey: ZxImportWizard.STEP_SELECT_ACCOUNTS,
    numCols: 3,
    colSizes: [270, 100, 270],
    items: [
      {
        type: _ZAWIZ_TOP_GROUPER_,
        label: zxMsg('LBL_Available'),
        colSizes: ['*'],
        width: 230,
        numCols: 1,
        items: [
          {
            type: _DWT_BUTTON_,
            label: zxMsg('LBL_Select_All') + ' &gt;&gt;',
            id: ZxImportWizard.ID_BUTTON_SELECT_ALL,
            cssStyle: 'width: 100%',
            onActivate: function() {
              var srcList = this.getForm().getItemsById(ZxImportWizard.ID_AVAILABLE_ACCOUNTS)[0],
                dstList = this.getForm().getItemsById(ZxImportWizard.ID_SELECTED_ACCOUNTS)[0];
              this.getForm().parent._containedObject.moveLists(srcList, dstList, this);
              this.getForm().parent.handleXFormChange();
            }
          },
          {
            type: _ZX_LIST_VIEW_,
            id: ZxImportWizard.ID_AVAILABLE_ACCOUNTS,
            ref: 'attrs/' + ZxImportOp.KEY_AVAILABLE_ACCOUNTS,
            height: 190,
            filterEnabled: true
          }
        ]
      },
      {
        type: _GROUP_,
        colSizes: ['*'],
        width: 100,
        numCols: 1,
        cssClass: 'ZxListCenterButtons',
        items: [
          {
            type: _DWT_BUTTON_,
            label: zxMsg('LBL_Select') + ' &gt;',
            id: ZxImportWizard.ID_BUTTON_SELECT,
            width: 100,
            onActivate: function() {
              var srcList = this.getForm().getItemsById(ZxImportWizard.ID_AVAILABLE_ACCOUNTS)[0],
                dstList = this.getForm().getItemsById(ZxImportWizard.ID_SELECTED_ACCOUNTS)[0];
              this.getForm().parent._containedObject.moveLists(srcList, dstList, this);
              this.getForm().parent.handleXFormChange();
            }
          },
          {
            type: _DWT_BUTTON_,
            label: '&lt; ' + zxMsg('LBL_Deselect'),
            id: ZxImportWizard.ID_BUTTON_DESELECT,
            width: 100,
            onActivate: function() {
              var srcList = this.getForm().getItemsById(ZxImportWizard.ID_SELECTED_ACCOUNTS)[0],
                dstList = this.getForm().getItemsById(ZxImportWizard.ID_AVAILABLE_ACCOUNTS)[0];
              this.getForm().parent._containedObject.moveLists(srcList, dstList, this);
              this.getForm().parent.handleXFormChange();
            }
          }
        ]
      },
      {
        type: _ZAWIZ_TOP_GROUPER_,
        label: zxMsg('LBL_Selected'),
        colSizes: ['*'],
        numCols: 1,
        width: 230,
        items: [
          {
            type: _DWT_BUTTON_,
            label: '&lt;&lt; ' + zxMsg('LBL_Empty_Selection'),
            id: ZxImportWizard.ID_BUTTON_SELECT_ALL,
            width: 230,
            onActivate: function() {
              var srcList = this.getForm().getItemsById(ZxImportWizard.ID_SELECTED_ACCOUNTS)[0],
                dstList = this.getForm().getItemsById(ZxImportWizard.ID_AVAILABLE_ACCOUNTS)[0];
              this.getForm().parent._containedObject.moveLists(srcList, dstList, this);
              this.getForm().parent.handleXFormChange();
            }
          },
          {
            type: _ZX_LIST_VIEW_,
            id: ZxImportWizard.ID_SELECTED_ACCOUNTS,
            ref: 'attrs/' + ZxImportOp.KEY_IMPORT_ACCOUNTS,
            height: 190,
            filterEnabled: true
          }
        ]
      }
    ]
  };

  step_summary = {
    type: _CASE_,
    caseKey: ZxImportWizard.STEP_SUMMARY,
    numCols: 1,
    items: [
      {
        type: _ZAWIZ_TOP_GROUPER_,
        label: zxMsg('LBL_BCK_op_summary'),
        items: [
          {
            type: _OUTPUT_,
            label: zxMsg('LBL_BCK_source_path') + ': ',
            ref: 'attrs/' + ZxImportOp.KEY_IMPORT_PATH,
            cssStyle: 'font-weight: bold'
          },
          {
            type: _OUTPUT_,
            label: zxMsg('LBL_BCK_select_domains_to_import') + ': ',
            ref: 'attrs/' + ZxImportOp.KEY_IMPORT_DOMAINS,
            cssStyle: 'font-weight: bold',
            getDisplayValue: function() {
              return this.getInstanceValue().join(', ');
            }
          },
          {
            type: _OUTPUT_,
            label: zxMsg('LBL_BCK_num_accounts_to_import') + ': ',
            ref: 'attrs/' + ZxImportOp.KEY_IMPORT_ACCOUNTS,
            cssStyle: 'font-weight: bold',
            getDisplayValue: function() {
              return this.getInstanceValue().length;
            }
          }
        ]
      },
      {
        type: _ZAWIZ_TOP_GROUPER_,
        label: zxMsg('LBL_BCK_operation_options'),
        numCols: 2,
        colSizes: ['*', '*'],
        visibilityChecks: [function() {
          return true; //ZXBackup.hasSecondaryVolumeActive();
        }],
        items: [
          {
            type: _GROUP_,
            numCols: 2,
            colSizes: ['*', '*'],
            items: [
              {
                type: _CHECKBOX_,
                labelCssClass: 'ZxCheckboxLabel',
                label: zxMsg('LBL_HSM_Apply_policy') + ': ',
                ref: 'attrs/' + ZxImportOp.KEY_OBEY_HSM,
                labelLocation: _LEFT_,
                align: _LEFT_,
                enableDisableChecks: [function() {
                  return !ZaSettings.isNetworkVersion();
                }]
              },
              {
                type: _OUTPUT_,
                value: zxMsg('LBL_option_available_on_ose'),
                visibilityChecks: [function() {
                  return ZaSettings.isNetworkVersion();
                }],
                align: _MIDDLE_
              }
            ]
          },
          {
            type: _OUTPUT_,
            value: zxMsg('LBL_BCK_import_according_to_hsm'),
            align: _MIDDLE_
          }
        ]
      },
      {
        type: _ZAWIZ_TOP_GROUPER_,
        label: zxMsg('LBL_BCK_op_notification'),
        numCols: 3,
        colSizes: ['*', '*', '*'],
        items: [
          {
            type: _INPUT_,
            label: zxMsg('LBL_BCK_additional_notification_mail') + ': ',
            ref: 'attrs/' + ZxImportOp.KEY_IMPORT_NOTIFICATION_MAIL,
            align: _LEFT_,
            width: 180
          },
          {
            type: _OUTPUT_,
            value: zxMsg('MSG_BCK_you_will_be_notified_when_import_completed'),
            align: _MIDDLE_
          },
          {
            type: _SPACER_,
            align: _CENTER_,
            valign: _TOP_,
            height: '50px'
          },
          {
            type: _GROUP_,
            colSizes: ['100%'],
            numCols: 1,
            colSpan: 3,
            items: [
              {
                type: _OUTPUT_,
                value: '<strong>' + zxMsg('MSG_BCK_wait_before_connect_zibra_desktop') + '</strong>',
                align: _MIDDLE_
              }
            ]
          }
        ]
      }
    ]
  };

  cases.push(step_intro);
  cases.push(step_path);
  cases.push(step_checkpath);
  cases.push(step_domain);
  cases.push(step_getAccount);
  cases.push(step_accounts);
  cases.push(step_summary);

  xFormObject.items = [
    {
      type: _OUTPUT_,
      colSpan: 2,
      align: _CENTER_,
      valign: _TOP_,
      ref: ZaModel.currentStep,
      choices: this.stepChoices,
      valueChangeEventSources: [ZaModel.currentStep]
    },
    {
      type: _SEPARATOR_,
      align: _CENTER_,
      valign: _TOP_
    },
    {
      type: _SPACER_,
      align: _CENTER_,
      valign: _TOP_
    },
    {
      type: _SWITCH_,
      align: _LEFT_,
      valign: _TOP_,
      items: cases
    }
  ];
};
ZaXDialog.XFormModifiers['ZxImportWizard'].push(ZxImportWizard.XFormModifier);

/**
 * Move to next page.
 * This also trigger a check of the entire form.
 */
ZxImportWizard.prototype.goNext = function() {
  this.goPage(this._containedObject[ZaModel.currentStep] + 1);
  this.handleXFormChange();
};

/**
 * Move to previous page.
 * This also trigger a check of the entire form.
 */
ZxImportWizard.prototype.goPrev = function() {
  if (this._containedObject[ZaModel.currentStep] === ZxImportWizard.STEP_SELECT_DOMAINS ||
      this._containedObject[ZaModel.currentStep] === ZxImportWizard.STEP_SELECT_ACCOUNTS) {
    this.goPage(this._containedObject[ZaModel.currentStep] - 2);

  } else {
    this.goPage(this._containedObject[ZaModel.currentStep] - 1);

  }
  this.handleXFormChange();
};

/**
 * When something is changed (also the page number),
 * validate the inputs and enable/disable wizard buttons.
 */
ZxImportWizard.prototype.handleXFormChange = function() {
  var cStep = this._containedObject[ZaModel.currentStep];
  ZxUtils.log(cStep, 'Wizard step');

  if (this._localXForm.hasErrors()) {
    this._button[DwtWizardDialog.FINISH_BUTTON].setEnabled(false);
  } else {
    this._button[DwtWizardDialog.PREV_BUTTON].setEnabled(true);

    if (cStep === ZxImportWizard.STEP_INTRO) {
      this._button[DwtDialog.CANCEL_BUTTON].setEnabled(true);
      this._button[DwtWizardDialog.NEXT_BUTTON].setEnabled(true);
      this._button[DwtWizardDialog.PREV_BUTTON].setEnabled(false);
      this._button[DwtWizardDialog.FINISH_BUTTON].setEnabled(false);
      return;
    }

    if (cStep === ZxImportWizard.STEP_SELECT_PATH) {
      if (this._containedObject.attrs[ZxImportOp.KEY_IMPORT_PATH] &&
          this._containedObject.attrs[ZxImportOp.KEY_IMPORT_PATH] !== '') {

        this._button[DwtWizardDialog.NEXT_BUTTON].setEnabled(true);
      } else {
        this._button[DwtWizardDialog.NEXT_BUTTON].setEnabled(false);
      }
      this._button[DwtDialog.CANCEL_BUTTON].setEnabled(true);
      this._button[DwtWizardDialog.PREV_BUTTON].setEnabled(true);
      this._button[DwtWizardDialog.FINISH_BUTTON].setEnabled(false);
      return;
    }

    if (cStep === ZxImportWizard.STEP_CHECK_PATH) {
      this._button[DwtDialog.CANCEL_BUTTON].setEnabled(false);
      this._button[DwtWizardDialog.PREV_BUTTON].setEnabled(false);
      this._button[DwtWizardDialog.NEXT_BUTTON].setEnabled(false);
      this._button[DwtWizardDialog.FINISH_BUTTON].setEnabled(false);

      ZxAdminRequest.sendRequest(
        this._containedObject._modItem.getCurrentServer(),
        'ZxBackup',
        'getBackupInfo',
        {
          dataPath: this._containedObject.attrs[ZxImportOp.KEY_IMPORT_PATH]
        }
      ).then(
        AjxCallback.simpleClosure(
          function(resp) {
            if (resp instanceof ZxError) {
              ZaApp.getInstance().getCurrentController().popupErrorDialog(
                resp.getMessage()
              );
//              ZaApp.getInstance().getCurrentController().popupErrorDialog(
//                zxMsg('MSG_BCK_error_checking_import_path'),
//                resp[this._containedObject._modItem.getServerName()].error
//              );
              this.goPrev();
              return;
            }

            var response = resp.response;

            if (!response.valid) {
              ZaApp.getInstance().getCurrentController().popupWarningDialog(
                zxMsg('MSG_BCK_please_select_valid_import_path')
              );
              this.goPrev();
              return;
            }

            ZxAdminRequest.sendRequest(
              this._containedObject._modItem.getCurrentServer(),
              'ZxBackup',
              'getAvailableDomains',
              {
                dataPath: this._containedObject.attrs[ZxImportOp.KEY_IMPORT_PATH]
              }
            ).then(
              AjxCallback.simpleClosure(function(resp) {
                if (!(resp instanceof ZxError)) {
                  var response = resp.response;

                  this._containedObject.attrs[ZxImportOp.KEY_AVAILABLE_DOMAINS] = response.domainList;
                  this._containedObject.attrs[ZxImportOp.KEY_IMPORT_DOMAINS] = [];
                  this.goNext();
                } else {
                  this.goPrev();
                }
              }, this),
              AjxCallback.simpleClosure(function(err) {
                ZxUtils.catchError(err);
                this.goPrev();
              }, this)
            );
          },
          this
        ),
        AjxCallback.simpleClosure(function(err) {
          ZxUtils.catchError(err);
          this.goPrev();
        }, this)
      );

      return;
    }

    if (cStep === ZxImportWizard.STEP_SELECT_DOMAINS) {
      this._button[DwtDialog.CANCEL_BUTTON].setEnabled(true);
      this._button[DwtWizardDialog.PREV_BUTTON].setEnabled(true);
      if (this._containedObject.attrs[ZxImportOp.KEY_IMPORT_DOMAINS].length > 0) {
        this._button[DwtWizardDialog.NEXT_BUTTON].setEnabled(true);
      } else {
        this._button[DwtWizardDialog.NEXT_BUTTON].setEnabled(false);
      }
      this._button[DwtWizardDialog.FINISH_BUTTON].setEnabled(false);
      return;
    }

    if (cStep === ZxImportWizard.STEP_GET_ACCOUNTS) {
      this._button[DwtDialog.CANCEL_BUTTON].setEnabled(false);
      this._button[DwtWizardDialog.PREV_BUTTON].setEnabled(false);
      this._button[DwtWizardDialog.NEXT_BUTTON].setEnabled(false);
      this._button[DwtWizardDialog.FINISH_BUTTON].setEnabled(false);

      ZxAdminRequest.sendRequest(
        this._containedObject._modItem.getCurrentServer(),
        'ZxBackup',
        'getAvailableAccounts',
        {
          dataPath: this._containedObject.attrs[ZxImportOp.KEY_IMPORT_PATH],
          domainList: this._containedObject.attrs[ZxImportOp.KEY_IMPORT_DOMAINS],
          filterDeleteAccount: this._containedObject.attrs[ZxImportOp.KEY_FILTER_DELETE_ACCOUNT],
          skipSystemAccount: this._containedObject.attrs[ZxImportOp.KEY_SKIP_SYSTEM_ACCOUNT]
        }
      ).then(
        AjxCallback.simpleClosure(function(resp) {

          if (!(resp instanceof ZxError)) {
            var response = resp.response;

            this._containedObject.attrs[ZxImportOp.KEY_AVAILABLE_ACCOUNTS] = response.accountList;
            this._containedObject.attrs[ZxImportOp.KEY_IMPORT_ACCOUNTS] = [];

            this.goNext();
          } else {
            this.goPrev();
          }

        }, this),
        AjxCallback.simpleClosure(function(err) {
          ZxUtils.catchError(err);
          this.goPrev();
        }, this)
      );
    }

    if (cStep === ZxImportWizard.STEP_SELECT_ACCOUNTS) {
      this._button[DwtDialog.CANCEL_BUTTON].setEnabled(true);
      this._button[DwtWizardDialog.PREV_BUTTON].setEnabled(true);
      if (this._containedObject.attrs[ZxImportOp.KEY_IMPORT_ACCOUNTS].length > 0) {
        this._button[DwtWizardDialog.NEXT_BUTTON].setEnabled(true);
      } else {
        this._button[DwtWizardDialog.NEXT_BUTTON].setEnabled(false);
      }
      this._button[DwtWizardDialog.FINISH_BUTTON].setEnabled(false);
      return;
    }

    if (cStep === ZxImportWizard.STEP_SUMMARY) {
      this._button[DwtDialog.CANCEL_BUTTON].setEnabled(true);
      this._button[DwtWizardDialog.PREV_BUTTON].setEnabled(true);
      this._button[DwtWizardDialog.NEXT_BUTTON].setEnabled(false);
      this._button[DwtWizardDialog.FINISH_BUTTON].setEnabled(true);
      return;
    }
  }
};

ZxImportWizard.prototype.finishWizard = function() {
  this._button[DwtWizardDialog.PREV_BUTTON].setEnabled(false);
  this._button[DwtWizardDialog.FINISH_BUTTON].setEnabled(false);

  var item = this._containedObject._modItem,
    attrs = this._containedObject.attrs,
    reqParam = {};

  if (attrs[ZxImportWizard.KEY_IMPORT_NOTIFICATION_MAIL] &&
      attrs[ZxImportWizard.KEY_IMPORT_NOTIFICATION_MAIL] !== '') {

    attrs[ZxImportWizard.KEY_IMPORT_NOTIFICATION_MAIL] += ',' +
      ZaZimbraAdmin.currentUserLogin;

  } else {
    attrs[ZxImportWizard.KEY_IMPORT_NOTIFICATION_MAIL] = ZaZimbraAdmin.currentUserLogin;
  }

  reqParam.notificationMails = attrs[ZxImportOp.KEY_IMPORT_NOTIFICATION_MAIL];
  reqParam.dataPath = attrs[ZxImportOp.KEY_IMPORT_PATH];
  reqParam.domainList = attrs[ZxImportOp.KEY_IMPORT_DOMAINS];
  reqParam.accountList = attrs[ZxImportOp.KEY_IMPORT_ACCOUNTS];
  reqParam.filterDeletedAccount = attrs[ZxImportOp.KEY_FILTER_DELETE_ACCOUNT];
  reqParam.obeyHSM = attrs[ZxImportOp.KEY_OBEY_HSM];
  reqParam.skipDistList = attrs[ZxImportOp.KEY_SKIP_DISTRIBUTION_LIST];
  reqParam.skipCos = attrs[ZxImportOp.KEY_SKIP_COS];
  reqParam.skipSystemAccount = attrs[ZxImportOp.KEY_SKIP_SYSTEM_ACCOUNT];
  reqParam.skipAliases = attrs[ZxImportOp.KEY_SKIP_ALIAS];

  ZxUtils.log(reqParam, 'Sending request for import');

  ZxAdminRequest.sendRequest(
    this._containedObject._modItem.getCurrentServer(),
    'ZxBackup',
    'doExternalRestore',
    reqParam
  ).then(
    AjxCallback.simpleClosure(function(resp) {
      ZaApp.getInstance().getCurrentController().popupMsgDialog(zxMsg('MSG_Operation_queued'));
      this.popdown();
      ZeXtras.getOperationsManager().requestUpdate();
    }, this),
    AjxCallback.simpleClosure(function(err) {
      ZxUtils.catchError(err);
      this.popdown();
    }, this)
  );
};

/**
 * Set the instance object.
 * When a wizard (on Z8) is moved as running tasks, when is resumed,
 * Zimbra invoke this function to restore the form with the correct informations.
 * @param {ZxImportOp} entry
 */
ZxImportWizard.prototype.setObject = function(entry) {
  this._containedObject = entry;

  this._containedObject[ZaModel.currentStep] = entry[ZaModel.currentStep] || ZxImportWizard.STEP_INTRO;
  this._localXForm.setInstance(this._containedObject);
};
