/*
 * ***** BEGIN LICENSE BLOCK *****
 * Copyright (C) 2011-2016 ZeXtras
 *
 * The contents of this file are subject to the ZeXtras EULA;
 * you may not use this file except in compliance with the EULA.
 * You may obtain a copy of the EULA at
 * http://www.zextras.com/zextras-eula.html
 * ***** END LICENSE BLOCK *****
 */

function ZxRestoreDeletedAccountWizard(item) {
  if (item instanceof DwtShell) {
    ZaXWizardDialog.call(this, item,
      null, ZxUtils.getTitleWithIcon('ZxBackup', zxMsg('LBL_BCK_Undelete_account_Wizard')),
      '720px', '380px', 'ZxRestoreDeletedAccountWizard');
  } else {
    ZaXWizardDialog.call(this, item.getCurrentController().getShell(),
      null, ZxUtils.getTitleWithIcon('ZxBackup', zxMsg('LBL_BCK_Undelete_account_Wizard')),
      '720px', '380px', 'ZxRestoreDeletedAccountWizard');
  }

  this._denyModifications = false;

  this.stepChoices = [
    {label: zxMsg('LBL_Introduction'),
      value: ZxRestoreDeletedAccountWizard.STEP_INTRO},
    {label: zxMsg('LBL_BCK_Select_date'),
      value: ZxRestoreDeletedAccountWizard.STEP_SELECT_DATE},
    {label: zxMsg('LBL_BCK_account_infos'),
      value: ZxRestoreDeletedAccountWizard.STEP_LOAD_INFO},
    {label: zxMsg('LBL_BCK_Select_account'),
      value: ZxRestoreDeletedAccountWizard.STEP_SELECT_ACCOUNT},
    {label: zxMsg('LBL_BCK_Checking_limits'),
      value: ZxRestoreDeletedAccountWizard.STEP_CHECK_LIMITS},
    {label: zxMsg('LBL_BCK_Destination_account'),
      value: ZxRestoreDeletedAccountWizard.STEP_SELECT_DESTINATION_ACCOUNT},
    {label: zxMsg('LBL_BCK_op_summary'),
      value: ZxRestoreDeletedAccountWizard.STEP_SUMMARY}
  ];

  this._helpURL = ZxLinks.HELP_BCK_Restore_wizard;
  this.initForm(ZxRestoreDeletedAccountWizard.XModel, this.getMyXForm());

  this._localXForm.setController(ZaApp.getInstance());
  this._localXForm.addListener(DwtEvent.XFORMS_FORM_DIRTY_CHANGE,
    new AjxListener(this, this.handleXFormChange));
  this._localXForm.addListener(DwtEvent.XFORMS_VALUE_ERROR,
    new AjxListener(this, this.handleXFormChange));
}

ZxRestoreDeletedAccountWizard.prototype = new ZaXWizardDialog();
ZxRestoreDeletedAccountWizard.prototype.constructor = ZxRestoreDeletedAccountWizard;

ZxRestoreDeletedAccountWizard.STEP_INDEX = 1;
ZxRestoreDeletedAccountWizard.STEP_INTRO = ZxRestoreDeletedAccountWizard.STEP_INDEX++;
ZxRestoreDeletedAccountWizard.STEP_SELECT_DATE = ZxRestoreDeletedAccountWizard.STEP_INDEX++;
ZxRestoreDeletedAccountWizard.STEP_LOAD_INFO = ZxRestoreDeletedAccountWizard.STEP_INDEX++;
ZxRestoreDeletedAccountWizard.STEP_SELECT_ACCOUNT = ZxRestoreDeletedAccountWizard.STEP_INDEX++;
ZxRestoreDeletedAccountWizard.STEP_CHECK_LIMITS = ZxRestoreDeletedAccountWizard.STEP_INDEX++;
ZxRestoreDeletedAccountWizard.STEP_SELECT_DESTINATION_ACCOUNT = ZxRestoreDeletedAccountWizard.STEP_INDEX++;
ZxRestoreDeletedAccountWizard.STEP_SUMMARY = ZxRestoreDeletedAccountWizard.STEP_INDEX++;

ZxRestoreDeletedAccountWizard.ID_AVAILABLE_ACCOUNTS = 'AvailableAccounts';
ZxRestoreDeletedAccountWizard.ID_SELECTION_DATE = 'DateSelection';
ZxRestoreDeletedAccountWizard.ID_SELECTED_ACCOUNTS = 'SelectedAccounts';
ZxRestoreDeletedAccountWizard.ID_BUTTON_SELECT_ALL = 'SelectAllBTN';
ZxRestoreDeletedAccountWizard.ID_BUTTON_SELECT = 'SelectBTN';
ZxRestoreDeletedAccountWizard.ID_BUTTON_DESELECT = 'SelectBTN';

/**
 * XModel of the Wizard.
 * @type {Array}
 */
ZxRestoreDeletedAccountWizard.XModel = [
  {type: _DATETIME_, id: ZxRestoreOp.KEY_DATE,
    ref: 'attrs/' + ZxRestoreOp.KEY_DATE},
  {type: _STRING_, id: ZxRestoreOp.KEY_ACCOUNT,
    ref: 'attrs/' + ZxRestoreOp.KEY_ACCOUNT},
  {type: _STRING_, id: ZxRestoreOp.KEY_DESTINATION_ACCOUNT,
    ref: 'attrs/' + ZxRestoreOp.KEY_DESTINATION_ACCOUNT},
  {type: _LIST_, id: ZxRestoreOp.KEY_AVAILABLE_ACCOUNTS,
    ref: 'attrs/' + ZxRestoreOp.KEY_AVAILABLE_ACCOUNTS},
  {type: _STRING_, id: ZxRestoreOp.KEY_EXPORT_NOTIFICATION_MAIL,
    ref: 'attrs/' + ZxRestoreOp.KEY_EXPORT_NOTIFICATION_MAIL},
  {type: _NUMBER_, id: ZxRestoreOp.KEY_ACCOUNT_LIMIT,
    ref: 'attrs/' + ZxRestoreOp.KEY_ACCOUNT_LIMIT},
  {type: _NUMBER_, id: ZxRestoreOp.KEY_COS_LIMIT,
    ref: 'attrs/' + ZxRestoreOp.KEY_COS_LIMIT},
  {type: _NUMBER_, id: ZxRestoreOp.KEY_ACCOUNT_LIMIT_COUNT,
    ref: 'attrs/' + ZxRestoreOp.KEY_ACCOUNT_LIMIT_COUNT},
  {type: _NUMBER_, id: ZxRestoreOp.KEY_COS_LIMIT_COUNT,
    ref: 'attrs/' + ZxRestoreOp.KEY_COS_LIMIT_COUNT},
  {type: _ENUM_, id: ZxRestoreOp.KEY_OBEY_HSM,
    ref: 'attrs/' + ZxRestoreOp.KEY_OBEY_HSM,
    choices: ZaModel.BOOLEAN_CHOICES1}
];

ZaXDialog.XFormModifiers['ZxRestoreDeletedAccountWizard'] = [];

ZxRestoreDeletedAccountWizard.prototype.miniType = 2; // WTF is this?

ZxRestoreDeletedAccountWizard.XFormModifier = function(xFormObject) {
  var cases = [],
    step_intro,
    step_date,
    step_load,
    step_account,
    step_load_limit,
    step_destination_account,
    step_summary;

  step_intro = {
    type: _CASE_,
    caseKey: ZxRestoreDeletedAccountWizard.STEP_INTRO,
    items: [
      {
        type: _ZAWIZ_TOP_GROUPER_,
        label: zxMsg('LBL_Introduction'),
        numCols: 1,
        colSizes: ['*'],
        items: [
          {
            type: _OUTPUT_,
            value: zxMsg('MSG_BCK_INTRO_Restore_deleted_account_wizard')
          }
        ]
      }
    ]
  };

  step_load = {
    type: _CASE_,
    caseKey: ZxRestoreDeletedAccountWizard.STEP_LOAD_INFO,
    items: [
      {
        type: _ZAWIZ_TOP_GROUPER_,
        label: zxMsg('LBL_BCK_account_infos'),
        numCols: 1,
        colSizes: ['*'],
        items: [
          {
            type: _OUTPUT_,
            cssStyle: 'padding-top: 10px',
            value: '<div class="DwtWait32Icon">&nbsp;</div><b>' + zxMsg('LBL_BCK_getting_accounts_import') + '</b>'
          }
        ]
      }
    ]
  };

  step_date = {
    type: _CASE_,
    caseKey: ZxRestoreDeletedAccountWizard.STEP_SELECT_DATE,
    items: [
      {
        type: _ZAWIZ_TOP_GROUPER_,
        label: zxMsg('LBL_BCK_Date_selection'),
        numCols: 2,
        colSizes: ['50%', '50%'],
        items: [
          {
            type: _OUTPUT_,
            value: zxMsg('MSG_BCK_Select_date_infos'),
            colSpan: 2
          },
          { type: _SPACER_, colSpan: 2 },
          {
            type: _GROUP_,
            numCols: 1,
            valign: _TOP_,
            items: [
              {
                type: _ZX_DATETIME_,
                id: ZxRestoreDeletedAccountWizard.ID_SELECTION_DATE,
                ref: 'attrs/' + ZxRestoreOp.KEY_DATE,
                labelLocation: _NONE_,
                visibilityChangeEventSources: ['attrs/' + ZxRestoreOp.KEY_USE_LAST],
                visibilityChecks: [function() {
                  var op = this.getInstanceValue('attrs/' + ZxRestoreOp.KEY_USE_LAST);
                  return op === false;
                }]
              },
              {
                type: _GROUP_,
                items: [
                  {
                    type: _CHECKBOX_,
                    label: zxMsg('LBL_BCK_Use_last_available'),
                    labelLocation: _LEFT_,
                    align: _LEFT_,
                    ref: 'attrs/' + ZxRestoreOp.KEY_USE_LAST,
                    onChange: function(value) {
                      this.setInstanceValue(value);
                      var view = this.getForm().getItemsById(ZxRestoreDeletedAccountWizard.ID_SELECTION_DATE)[0];
                      if (value) {
                        view.setInstanceValue(new Date());
                        view.hide();
                      } else {
                        view.show();
                      }
                    }
                  }
                ]
              }
            ]
          },
          {
            type: _GROUPER_,
            label: zxMsg('LBL_BCK_Backup_info'),
            valign: _TOP_,
            items: [
              {
                type: _OUTPUT_,
                label: zxMsg('LBL_BCK_First_fullscan') + ': ',
                ref: 'attrs/' + ZxRestoreOp.KEY_FIRST_SCAN
              },
              {
                type: _OUTPUT_,
                label: zxMsg('LBL_BCK_Current_retention') + ': ',
                ref: 'attrs/' + ZxRestoreOp.KEY_RETENTION
              }
            ]
          }
        ]
      }
    ]
  };

  step_account = {
    type: _CASE_,
    caseKey: ZxRestoreDeletedAccountWizard.STEP_SELECT_ACCOUNT,
    items: [
      {
        type: _ZAWIZ_TOP_GROUPER_,
        label: zxMsg('LBL_BCK_Select_account'),
        numCols: 1,
        colSizes: ['*'],
        items: [
          {
            type: _OUTPUT_,
            value: zxMsg('MSG_BCK_Select_account_infos')
          },
          { type: _SPACER_ },
          {
            type: _ZX_LIST_VIEW_,
            id: ZxRestoreDeletedAccountWizard.ID_AVAILABLE_ACCOUNTS,
            ref: 'attrs/' + ZxRestoreOp.KEY_AVAILABLE_ACCOUNTS,
            filterEnabled: true,
            navigationEnabled: true,
            pageSize: 10,
            multiselect: false,
            onSelection: function(ev) {
              this.setInstanceValue(ev.item, 'attrs/' + ZxRestoreOp.KEY_ACCOUNT);
              this.getForm().parent.handleXFormChange();
            }
          }
        ]
      }
    ]
  };

  step_load_limit = {
    type: _CASE_,
    caseKey: ZxRestoreDeletedAccountWizard.STEP_CHECK_LIMITS,
    numCols: 1,
    colSizes: ['*'],
    items: [
      {
        type: _ZAWIZ_TOP_GROUPER_,
        label: zxMsg('LBL_BCK_Checking_limits'),
        numCols: 1,
        colSizes: ['*'],
        items: [
          {
            type: _OUTPUT_,
            cssStyle: 'padding-top: 10px',
            value: '<div class="DwtWait32Icon">&nbsp;</div><b>' + zxMsg('LBL_BCK_Checking_limits') + '</b>'
          }
        ]
      }
    ]
  };

  step_destination_account = {
    type: _CASE_,
    caseKey: ZxRestoreDeletedAccountWizard.STEP_SELECT_DESTINATION_ACCOUNT,
    numCols: 1,
    colSizes: ['*'],
    items: [
      {
        type: _ZAWIZ_TOP_GROUPER_,
        label: zxMsg('LBL_BCK_Select_destination_account'),
        numCols: 1,
        colSizes: ['*'],
        items: [
          {
            type: _OUTPUT_,
            value: zxMsg('MSG_BCK_Destination_account_infos')
          },
          { type: _SPACER_ },
          {
            type: _GROUP_,
            numCols: 3,
            colSizes: ['*', '*', '*'],
            items: [
              {
                type: _INPUT_,
                label: zxMsg('LBL_BCK_Destination_account_name'),
                ref: 'attrs/' + ZxRestoreOp.KEY_DESTINATION_ACCOUNT,
                getDisplayValue: function(value) {
                  if (value !== '') {
                    return value;
                  }
                  var tmpString = this.getInstanceValue('attrs/' + ZxRestoreOp.KEY_ACCOUNT),
                    hintValue = tmpString.substring(0,
                      ZxUtils.indexOf(tmpString, '@')) + '_' + zxMsg('HNT_BCK_restored');
                  this.setInstanceValue(hintValue);
                  return hintValue;
                }
              },
              {
                type: _OUTPUT_,
                ref: 'attrs/' + ZxRestoreOp.KEY_ACCOUNT,
                align: _LEFT_,
                getDisplayValue: function(value) {
                  if (value) {
                    return value.substring(ZxUtils.indexOf(value, '@'));
                  }
                  return '';
                }
              }
            ]
          },
          {
            type: _GROUP_,
            numCols: 1,
            colSizes: ['*'],
            items: [
              {
                type: _CHECKBOX_,
                label: zxMsg('LBL_BCK_Hide_new_account_in_gal'),
                ref: 'attrs/' + ZxRestoreOp.KEY_HIDE_IN_GAL,
                labelLocation: _LEFT_,
                align: _LEFT_
              }
            ]
          }
        ]
      }
    ]
  };

  step_summary = {
    type: _CASE_,
    caseKey: ZxRestoreDeletedAccountWizard.STEP_SUMMARY,
    numCols: 1,
    items: [
      {
        type: _ZAWIZ_TOP_GROUPER_,
        label: zxMsg('LBL_BCK_op_summary'),
        items: [
          {
            type: _OUTPUT_,
            label: zxMsg('LBL_BCK_Restore_account') + ': ',
            ref: 'attrs/' + ZxRestoreOp.KEY_ACCOUNT,
            cssStyle: 'font-weight: bold'
          },
          {
            type: _OUTPUT_,
            label: zxMsg('LBL_BCK_Destination_account') + ': ',
            ref: 'attrs/' + ZxRestoreOp.KEY_DESTINATION_ACCOUNT,
            cssStyle: 'font-weight: bold',
            getDisplayValue: function(value) {
              var originalAccount = this.getInstanceValue('attrs/' + ZxRestoreOp.KEY_ACCOUNT),
                prefix = this.getInstanceValue('attrs/' + ZxRestoreOp.KEY_DESTINATION_ACCOUNT),
                suffix;

              suffix = originalAccount.substring(ZxUtils.indexOf(originalAccount, '@'));
              return [prefix, suffix].join('');
            }
          },
          {
            type: _OUTPUT_,
            label: zxMsg('LBL_BCK_Hide_new_account_in_gal') + ': ',
            ref: 'attrs/' + ZxRestoreOp.KEY_HIDE_IN_GAL,
            cssStyle: 'font-weight: bold',
            getDisplayValue: function(value) {
              return value ? ZaMsg.Yes : ZaMsg.No;
            }
          },
          {
            type: _OUTPUT_,
            label: zxMsg('LBL_Date') + ': ',
            ref: 'attrs/' + ZxRestoreOp.KEY_DATE,
            cssStyle: 'font-weight: bold',
            getDisplayValue: function(val) {
              return ZxUtils.computeWordyDateStr(new Date(), val);
            }
          },
          {
            type: _OUTPUT_,
            label: zxMsg('LBL_BCK_Domain_Accounts_Limit') + ': ',
            cssStyle: 'font-weight: bold',
            visibilityChangeEventSources: [
              'attrs/' + ZxRestoreOp.KEY_ACCOUNT_LIMIT
            ],
            visibilityChecks: [function() {
              var limit = this.getForm().parent._containedObject.attrs[ZxRestoreOp.KEY_ACCOUNT_LIMIT];
              return limit !== -1;
            }],
            valueChangeEventSources: [
              'attrs/' + ZxRestoreOp.KEY_ACCOUNT_LIMIT_COUNT,
              'attrs/' + ZxRestoreOp.KEY_ACCOUNT_LIMIT
            ],
            getDisplayValue: function(val) {
              var count = this.getForm().parent._containedObject.attrs[ZxRestoreOp.KEY_ACCOUNT_LIMIT_COUNT],
                limit = this.getForm().parent._containedObject.attrs[ZxRestoreOp.KEY_ACCOUNT_LIMIT];
              return count + ' / ' + ((limit === -1) ? zxMsg('LBL_Unlimited') : limit);
            }
          },
          {
            type: _OUTPUT_,
            label: zxMsg('LBL_BCK_COS_Accounts_Limit') + ': ',
            cssStyle: 'font-weight: bold',
            visibilityChangeEventSources: [
              'attrs/' + ZxRestoreOp.KEY_COS_LIMIT
            ],
            visibilityChecks: [
              function(val) {
                var limit = this.getForm().parent._containedObject.attrs[ZxRestoreOp.KEY_COS_LIMIT];
                return limit !== -1;
              }
            ],
            valueChangeEventSources: [
              'attrs/' + ZxRestoreOp.KEY_COS_LIMIT_COUNT,
              'attrs/' + ZxRestoreOp.KEY_COS_LIMIT
            ],
            getDisplayValue: function(val) {
              var count = this.getForm().parent._containedObject.attrs[ZxRestoreOp.KEY_COS_LIMIT_COUNT],
                limit = this.getForm().parent._containedObject.attrs[ZxRestoreOp.KEY_COS_LIMIT];
              return count + ' / ' + ((limit === -1) ? zxMsg('LBL_Unlimited') : limit);
            }
          }
        ]
      },
      {
        type: _ZAWIZ_TOP_GROUPER_,
        label: zxMsg('LBL_BCK_operation_options'),
        numCols: 2,
        colSizes: ['*', '*'],
        visibilityChecks: [function() {
          return true; //ZXBackup.hasSecondaryVolumeActive();
        }],
        items: [
          {
            type: _GROUP_,
            numCols: 2,
            colSizes: ['*', '*'],
            items: [
              {
                type: _CHECKBOX_,
                labelCssClass: 'ZxCheckboxLabel',
                label: zxMsg('LBL_HSM_Apply_policy') + ': ',
                ref: 'attrs/' + ZxRestoreOp.KEY_OBEY_HSM,
                labelLocation: _LEFT_,
                align: _LEFT_,
                enableDisableChecks: [function() {
                  return !ZaSettings.isNetworkVersion();
                }]
              },
              {
                type: _OUTPUT_,
                value: zxMsg('LBL_option_available_on_ose'),
                visibilityChecks: [function() {
                  return ZaSettings.isNetworkVersion();
                }],
                align: _MIDDLE_
              }
            ]
          },
          {
            type: _OUTPUT_,
            value: zxMsg('LBL_BCK_import_according_to_hsm'),
            align: _MIDDLE_
          }
        ]
      },
      {
        type: _ZAWIZ_TOP_GROUPER_,
        label: zxMsg('LBL_BCK_op_notification'),
        numCols: 3,
        colSizes: ['*', '*', '*'],
        items: [
          {
            type: _INPUT_,
            label: zxMsg('LBL_BCK_additional_notification_mail') + ': ',
            ref: 'attrs/' + ZxRestoreOp.KEY_NOTIFICATION_MAIL,
            align: _LEFT_,
            width: 180
          },
          {
            type: _OUTPUT_,
            value: zxMsg('MSG_BCK_you_will_be_notified_chen_account_undelete_completed1'),
            align: _MIDDLE_
          },
          { type: _SPACER_, colSpan: 3 },
          {
            type: _OUTPUT_,
            value: zxMsg('MSG_BCK_you_will_be_notified_chen_account_undelete_completed2'),
            colSpan: 3
          }
        ]
      }
    ]
  };

  cases.push(step_load);
  cases.push(step_intro);
  cases.push(step_date);
  cases.push(step_account);
  cases.push(step_load_limit);
  cases.push(step_destination_account);
  cases.push(step_summary);

  xFormObject.items = [
    {
      type: _OUTPUT_,
      colSpan: 2,
      align: _CENTER_,
      valign: _TOP_,
      ref: ZaModel.currentStep,
      choices: this.stepChoices,
      enableDisableChecks: [function() { return false; }],
      valueChangeEventSources: [ZaModel.currentStep],
      labelClickMethod: function() {
        var sourceValue = this.getInheritedProperty('sourceValue'),
          instanceValue = this.getInstanceValue();
        if (sourceValue < instanceValue) {}
      }
    },
    {
      type: _SEPARATOR_,
      align: _CENTER_,
      valign: _TOP_
    },
    {
      type: _SPACER_,
      align: _CENTER_,
      valign: _TOP_
    },
    {
      type: _SWITCH_,
      align: _LEFT_,
      valign: _TOP_,
      items: cases
    }
  ];
};
ZaXDialog.XFormModifiers['ZxRestoreDeletedAccountWizard'].push(ZxRestoreDeletedAccountWizard.XFormModifier);

/**
 * Move to next page.
 * This also trigger a check of the entire form.
 */
ZxRestoreDeletedAccountWizard.prototype.goNext = function() {
  this.goPage(this._containedObject[ZaModel.currentStep] + 1);
  this.handleXFormChange();
};

/**
 * Move to previous page.
 * This also trigger a check of the entire form.
 */
ZxRestoreDeletedAccountWizard.prototype.goPrev = function() {
  if (this._containedObject[ZaModel.currentStep] === ZxRestoreDeletedAccountWizard.STEP_SELECT_ACCOUNT) {
    this.goPage(this._containedObject[ZaModel.currentStep] - 2);
  } else {
    if(this._containedObject[ZaModel.currentStep] === ZxRestoreDeletedAccountWizard.STEP_SELECT_DESTINATION_ACCOUNT) {
      this.goPage(this._containedObject[ZaModel.currentStep] - 2);
    }
    else {
      this.goPage(this._containedObject[ZaModel.currentStep] - 1);
    }
  }
  this.handleXFormChange();
};

/**
 * When something is changed (also the page number),
 * validate the inputs and enable/disable wizard buttons.
 */
ZxRestoreDeletedAccountWizard.prototype.handleXFormChange = function() {
  var cStep = this._containedObject[ZaModel.currentStep];
  ZxUtils.log(cStep, 'XForm changes');

  if (this._denyModifications) {
    return;
  }

  if (this._localXForm.hasErrors()) {
    this._button[DwtWizardDialog.FINISH_BUTTON].setEnabled(false);
  } else {
    this._button[DwtWizardDialog.PREV_BUTTON].setEnabled(true);

    if (cStep === ZxRestoreDeletedAccountWizard.STEP_INTRO) {
      this._button[DwtDialog.CANCEL_BUTTON].setEnabled(true);
      this._button[DwtWizardDialog.NEXT_BUTTON].setEnabled(true);
      this._button[DwtWizardDialog.PREV_BUTTON].setEnabled(false);
      this._button[DwtWizardDialog.FINISH_BUTTON].setEnabled(false);
      return;
    }

    if (cStep === ZxRestoreDeletedAccountWizard.STEP_LOAD_INFO) {
      this._button[DwtWizardDialog.NEXT_BUTTON].setEnabled(true);
      this._button[DwtDialog.CANCEL_BUTTON].setEnabled(true);
      this._button[DwtWizardDialog.PREV_BUTTON].setEnabled(false);
      this._button[DwtWizardDialog.FINISH_BUTTON].setEnabled(false);

      ZxAdminRequest.sendRequest(
          this._containedObject._modItem.getCurrentServer(),
          'ZxBackup',
          'getAvailableAccounts',
          {
            dataPath: this._containedObject.attrs[ZxRestoreOp.KEY_DATA_PATH],
            filterExistingAccount: true,
            filterDeleteAccount: false,
            skipSystemAccount: true,
            date: this._containedObject.attrs[ZxRestoreOp.KEY_DATE].getTime()
          }
      ).then(AjxCallback.simpleClosure(function(resp) {
            if (!(resp instanceof ZxError)) {
              this._containedObject.attrs[ZxRestoreOp.KEY_AVAILABLE_ACCOUNTS] =
                  resp.response.accountList;
              this._containedObject.attrs[ZxRestoreOp.KEY_ACCOUNTS_ID_MAP] =
                  resp.response.accountMap;
              this.goNext();
            } else {
              ZaApp.getInstance().getCurrentController().popupErrorDialog(
                  zxMsg('LBL_BCK_error_getting_accounts')
              );
              this.popdown();
            }
          }, this),
          ZxUtils.catchError);
      return;
    }

    if (cStep === ZxRestoreDeletedAccountWizard.STEP_SELECT_DATE) {
      this._button[DwtWizardDialog.NEXT_BUTTON].setEnabled(true);
      this._button[DwtDialog.CANCEL_BUTTON].setEnabled(true);
      this._button[DwtWizardDialog.PREV_BUTTON].setEnabled(true);
      this._button[DwtWizardDialog.FINISH_BUTTON].setEnabled(false);
      return;
    }

    if (cStep === ZxRestoreDeletedAccountWizard.STEP_SELECT_ACCOUNT) {
      this._button[DwtDialog.CANCEL_BUTTON].setEnabled(true);
      this._button[DwtWizardDialog.PREV_BUTTON].setEnabled(true);
      if (this._containedObject.attrs[ZxRestoreOp.KEY_ACCOUNT] !== '') {
        this._button[DwtWizardDialog.NEXT_BUTTON].setEnabled(true);
      } else {
        this._button[DwtWizardDialog.NEXT_BUTTON].setEnabled(false);
      }
      this._button[DwtWizardDialog.FINISH_BUTTON].setEnabled(false);
      return;
    }

    if (cStep === ZxRestoreDeletedAccountWizard.STEP_CHECK_LIMITS) {
      this._button[DwtDialog.CANCEL_BUTTON].setEnabled(true);
      this._button[DwtWizardDialog.PREV_BUTTON].setEnabled(false);
      this._button[DwtWizardDialog.NEXT_BUTTON].setEnabled(false);
      this._button[DwtWizardDialog.FINISH_BUTTON].setEnabled(false);

      ZxAdminRequest.sendRequest(
        this._containedObject._modItem.getCurrentServer(),
        'ZxBackup',
        'getDomainAccountLimitInfo',
        {
          account: this._containedObject.attrs[ZxRestoreOp.KEY_ACCOUNT]
        }
      ).then(
        AjxCallback.simpleClosure(
          function(resp) {
            var response = resp.response;
            this._containedObject.attrs[ZxRestoreOp.KEY_ACCOUNT_LIMIT] = (response.maxAccounts === 'unlimited') ?
                -1 : response.maxAccounts;
            this._containedObject.attrs[ZxRestoreOp.KEY_ACCOUNT_LIMIT_COUNT] = response.numAccounts;

            ZxAdminRequest.sendRequest(
              this._containedObject._modItem.getCurrentServer(),
              'ZxBackup',
              'getCosAccountLimitInfo',
              {
                account: this._containedObject.attrs[ZxRestoreOp.KEY_ACCOUNT]
              }
            ).then(
              AjxCallback.simpleClosure(
                function(resp) {
                  var response = resp.response;
                  this._containedObject.attrs[ZxRestoreOp.KEY_COS_LIMIT] = (response.maxAccounts === 'unlimited') ?
                      -1 : response.maxAccounts;
                  this._containedObject.attrs[ZxRestoreOp.KEY_COS_LIMIT_COUNT] = response.numAccounts;

                  if ((this._containedObject.attrs[ZxRestoreOp.KEY_ACCOUNT_LIMIT] === -1 ||
                      this._containedObject.attrs[ZxRestoreOp.KEY_ACCOUNT_LIMIT_COUNT] <
                      this._containedObject.attrs[ZxRestoreOp.KEY_ACCOUNT_LIMIT]) &&
                      (this._containedObject.attrs[ZxRestoreOp.KEY_COS_LIMIT] === -1 ||
                      this._containedObject.attrs[ZxRestoreOp.KEY_COS_LIMIT_COUNT] <
                      this._containedObject.attrs[ZxRestoreOp.KEY_COS_LIMIT])) {

                    this.goNext();
                  } else {
                    ZaApp.getInstance().getCurrentController().popupWarningDialog(
                      zxMsg('LBL_BCK_Restore_deleted_account_exceed_cos_limit')
                    );
                    this.goPrev();
                  }
                },
                this
              ),
              AjxCallback.simpleClosure(function(error) {
                if (error.getCode() === ZxErrorCode.UNABLE_TO_FIND_COS) {
                  this.goNext();
                } else {
                  ZxUtils.catchError(error);
                  this.goPrev();
                }
              }, this)
            );
          },
          this
        ),
        ZxUtils.catchError
      );
      return;
    }

    if (cStep === ZxRestoreDeletedAccountWizard.STEP_SELECT_DESTINATION_ACCOUNT) {
      if (this._containedObject.attrs[ZxRestoreOp.KEY_DESTINATION_ACCOUNT] !==
          this._containedObject.attrs[ZxRestoreOp.KEY_ACCOUNT]) {
      }

      this._button[DwtDialog.CANCEL_BUTTON].setEnabled(true);
      this._button[DwtWizardDialog.PREV_BUTTON].setEnabled(true);
      this._button[DwtWizardDialog.NEXT_BUTTON].setEnabled(true);
      this._button[DwtWizardDialog.FINISH_BUTTON].setEnabled(false);
      return;
    }

    if (cStep === ZxRestoreDeletedAccountWizard.STEP_SUMMARY) {
      this._button[DwtDialog.CANCEL_BUTTON].setEnabled(true);
      this._button[DwtWizardDialog.PREV_BUTTON].setEnabled(true);
      this._button[DwtWizardDialog.NEXT_BUTTON].setEnabled(false);
      this._button[DwtWizardDialog.FINISH_BUTTON].setEnabled(true);
      return;
    }
  }
};

ZxRestoreDeletedAccountWizard.prototype.finishWizard = function() {
  this._denyModifications = true;
  this._button[DwtWizardDialog.PREV_BUTTON].setEnabled(false);
  this._button[DwtWizardDialog.FINISH_BUTTON].setEnabled(false);

  var item = this._containedObject._modItem,
    attrs = this._containedObject.attrs,
    reqParam = {};

  if (attrs[ZxRestoreOp.KEY_EXPORT_NOTIFICATION_MAIL] &&
      attrs[ZxRestoreOp.KEY_EXPORT_NOTIFICATION_MAIL] !== '') {

    attrs[ZxRestoreOp.KEY_EXPORT_NOTIFICATION_MAIL] += ',' +
      ZaZimbraAdmin.currentUserLogin;

  } else {
    attrs[ZxRestoreOp.KEY_EXPORT_NOTIFICATION_MAIL] = ZaZimbraAdmin.currentUserLogin;
  }

  reqParam.srcAccountName = attrs[ZxRestoreOp.KEY_ACCOUNT];

  reqParam.dstAccountName = attrs[ZxRestoreOp.KEY_DESTINATION_ACCOUNT] +
    reqParam.srcAccountName.substring(ZxUtils.indexOf(reqParam.srcAccountName, '@'));

  reqParam.srcAccountId = this.getAccountId(reqParam.srcAccountName);

  if (!attrs[ZxRestoreOp.KEY_USE_LAST]) {
    reqParam.date = attrs[ZxRestoreOp.KEY_DATE].getTime();
  }
  reqParam.hideInGAL = attrs[ZxRestoreOp.KEY_HIDE_IN_GAL];
  reqParam.obeyHSM = attrs[ZxRestoreOp.KEY_OBEY_HSM];
  reqParam.restoreChatBuddies = false;
  reqParam.notificationMails = attrs[ZxRestoreOp.KEY_EXPORT_NOTIFICATION_MAIL];

  ZxUtils.log(reqParam, 'Sending request for Export');

  ZxAdminRequest.sendRequest(
    this._containedObject._modItem.getCurrentServer(),
    'ZxBackup',
    'doRestoreOnNewAccount',
    reqParam
  ).then(
    AjxCallback.simpleClosure(function(resp) {
      if (!(resp instanceof ZxError)) {
        ZaApp.getInstance().getCurrentController().popupMsgDialog(zxMsg('MSG_Operation_queued'));
        ZeXtras.getOperationsManager().requestUpdate();
        this.popdown();
      } else {
        this.goPage(ZxRestoreDeletedAccountWizard.STEP_SELECT_DATE);
        this.handleXFormChange();
        ZaApp.getInstance().getCurrentController().popupErrorDialog(resp.getMessage());
      }
    }, this),
    AjxCallback.simpleClosure(
      function(err) {
        this._denyModifications = false;
        this._button[DwtWizardDialog.PREV_BUTTON].setEnabled(true);
        this._button[DwtWizardDialog.FINISH_BUTTON].setEnabled(false);
        ZaApp.getInstance().getCurrentController().popupErrorDialog(
          err.getMessage(),
          null,
          false,
          false
        );
      },
      this
    )
  );

};

/**
 * Set the instance object.
 * When a wizard (on Z8) is moved as running tasks, when is resumed,
 * Zimbra invoke this function to restore the form with the correct informations.
 * @param {ZxRestoreOp} entry
 */
ZxRestoreDeletedAccountWizard.prototype.setObject = function(entry) {
  this._containedObject = entry;

  this._containedObject[ZaModel.currentStep] = entry[ZaModel.currentStep] ||
      ZxRestoreDeletedAccountWizard.STEP_INTRO;
  this._localXForm.setInstance(this._containedObject);
};


/**
 * Returns the accountId of a given accountName
 * @param accountName
 * @returns accountId
 */
ZxRestoreDeletedAccountWizard.prototype.getAccountId = function(accountName) {
  var accountId, accountMap;
  accountMap = this._containedObject.attrs[ZxRestoreOp.KEY_ACCOUNTS_ID_MAP];
  accountId = accountMap[accountName];
  return accountId;
};
