/*
 * ***** BEGIN LICENSE BLOCK *****
 * Copyright (C) 2011-2016 ZeXtras
 *
 * The contents of this file are subject to the ZeXtras EULA;
 * you may not use this file except in compliance with the EULA.
 * You may obtain a copy of the EULA at
 * http://www.zextras.com/zextras-eula.html
 * ***** END LICENSE BLOCK *****
 */

/**
 * Item containing all Restore operation's informations.
 * @param {ZxModule|ZxBackup} moduleItem
 * @constructor
 */
function ZxRestoreOp(moduleItem) {
  ZaItem.call(this, 'ZxRestoreOp');
  this._init();
  this.type = ZxRestoreOp.TYPE;
  this._modItem = moduleItem;
  this._account = null;
  this.attrs = {};
}

ZxRestoreOp.TYPE = 'RestoreOp';
ZxRestoreOp.prototype = new ZaItem();
ZxRestoreOp.prototype.constructor = ZxRestoreOp;

ZxRestoreOp.KEY_DATA_PATH = 'dataPath';
ZxRestoreOp.KEY_DATE = 'date';
ZxRestoreOp.KEY_DATE_END = 'dateEnd';
ZxRestoreOp.KEY_ACCOUNT = 'account';
ZxRestoreOp.KEY_HIDE_IN_GAL = 'hideInGAL';
ZxRestoreOp.KEY_OBEY_HSM = 'obeyHSM';
ZxRestoreOp.KEY_USE_LAST = 'useLast';
ZxRestoreOp.KEY_USE_FIRST = 'useFirst';
ZxRestoreOp.KEY_AVAILABLE_ACCOUNTS = 'availableAccounts';
ZxRestoreOp.KEY_ACCOUNTS_ID_MAP = 'accountsIdMap';
ZxRestoreOp.KEY_DESTINATION_ACCOUNT = 'destinationAccount';
ZxRestoreOp.KEY_NOTIFICATION_MAIL = 'notificationMails';
ZxRestoreOp.KEY_FIRST_SCAN = 'firstScan';
ZxRestoreOp.KEY_RETENTION = 'retention';
ZxRestoreOp.KEY_ACCOUNT_LIMIT = 'accountLimit';
ZxRestoreOp.KEY_COS_LIMIT = 'cosLimit';
ZxRestoreOp.KEY_ACCOUNT_LIMIT_COUNT = 'accountLimitUsed';
ZxRestoreOp.KEY_COS_LIMIT_COUNT = 'cosLimitUsed';
ZxRestoreOp.KEY_OPERATION = 'operation';
ZxRestoreOp.KEY_QUOTA = 'quota';
ZxRestoreOp.KEY_QUOTA_LIMIT = 'quotaLimit';
ZxRestoreOp.KEY_EXPORT_NOTIFICATION_MAIL = 'notificationMails';

ZxRestoreOp.OP_UNDELETE = 'doUndelete';
ZxRestoreOp.OP_RESTORE_ON_NEW = 'doRestoreOnNewAccount';

/**
 * Initialize the Restore Operation Item.
 */
ZxRestoreOp.prototype.init = function() {
  this.attrs[ZxRestoreOp.KEY_DATA_PATH] = this._modItem.getBackupPath();
  this.attrs[ZxRestoreOp.KEY_DATE] = new Date(this._modItem.suggestRestoreDate(this._account));
  this.attrs[ZxRestoreOp.KEY_DATE_END] = new Date();
  this.attrs[ZxRestoreOp.KEY_DESTINATION_ACCOUNT] = this.attrs[ZxRestoreOp.KEY_DESTINATION_ACCOUNT] || '';
  this.attrs[ZxRestoreOp.KEY_ACCOUNT] = this.attrs[ZxRestoreOp.KEY_ACCOUNT] || '';
  this.attrs[ZxRestoreOp.KEY_USE_FIRST] = false;
  this.attrs[ZxRestoreOp.KEY_USE_LAST] = false;
  this.attrs[ZxRestoreOp.KEY_HIDE_IN_GAL] = true;
  this.attrs[ZxRestoreOp.KEY_AVAILABLE_ACCOUNTS] = [];
  this.attrs[ZxRestoreOp.KEY_ACCOUNTS_ID_MAP] = {};
  this.attrs[ZxRestoreOp.KEY_EXPORT_NOTIFICATION_MAIL] = '';
  this.attrs[ZxRestoreOp.KEY_ACCOUNT_LIMIT] = -1;
  this.attrs[ZxRestoreOp.KEY_COS_LIMIT] = -1;
  this.attrs[ZxRestoreOp.KEY_ACCOUNT_LIMIT_COUNT] = 0;
  this.attrs[ZxRestoreOp.KEY_COS_LIMIT_COUNT] = 0;
  this.attrs[ZxRestoreOp.KEY_OPERATION] = '';
  this.attrs[ZxRestoreOp.KEY_QUOTA] = 0;
  this.attrs[ZxRestoreOp.KEY_QUOTA_LIMIT] = -1;
  this.attrs[ZxRestoreOp.KEY_OBEY_HSM] = false;

  this.attrs[ZxRestoreOp.KEY_RETENTION] = this._modItem.getRetentionDays();
  this.attrs[ZxRestoreOp.KEY_FIRST_SCAN] = this._modItem.getFirstFullscanDate();
};

/**
 * Specify an account associated to the operation.
 * @param {ZaAccount} account
 */
ZxRestoreOp.prototype.setAccount = function(account) {
  account.refresh(false, true);
  var quota;
  if (typeof account.attrs.zimbraMailQuota === "undefined") {
    // The mail quota is not set on the user, but it can be set in the COS
    var cos = account.getCurrentCos();
    quota = cos.attrs.zimbraMailQuota;
  } else {
    quota = account.attrs.zimbraMailQuota;
  }

  this._account = account;

  this.attrs[ZxRestoreOp.KEY_ACCOUNT] = account.name;
  this.attrs[ZxRestoreOp.KEY_DESTINATION_ACCOUNT] = zxMsg('LBL_BCK_restored_hint') + '_' +
    account.name.substring(0, ZxUtils.indexOf(account.name, '@'));

  this.attrs[ZxRestoreOp.KEY_QUOTA] = account.attrs.mbxSize;
  this.attrs[ZxRestoreOp.KEY_QUOTA_LIMIT] = parseInt(quota);

  ZxAdminRequest.sendRequest(
      this._account.attrs.zimbraMailHost,
      'ZxBackup',
      'getAccountBackupInfo',
      {
        account: account.id
      }
  ).then(
      AjxCallback.simpleClosure(
          function (resp) {
            var response = resp.response.backupStats;

            this._account.attrs.zextrasUserBackupFirstScan = response.firstScan;
            this.attrs[ZxRestoreOp.KEY_DATE] = new Date(this._modItem.suggestRestoreDate(this._account));
          },
          this
      ),
      ZxUtils.catchError
  );
};
