/*
 * ***** BEGIN LICENSE BLOCK *****
 * Copyright (C) 2011-2016 ZeXtras
 *
 * The contents of this file are subject to the ZeXtras EULA;
 * you may not use this file except in compliance with the EULA.
 * You may obtain a copy of the EULA at
 * http://www.zextras.com/zextras-eula.html
 * ***** END LICENSE BLOCK *****
 */

function ZxUndeleteWizard(item) {
  if (item instanceof DwtShell) {
    ZaXWizardDialog.call(this, item,
      null, ZxUtils.getTitleWithIcon('ZxBackup', zxMsg('LBL_BCK_Restore_Wizard')), '720px', '360px', 'ZxUndeleteWizard');
  } else {
    ZaXWizardDialog.call(this, item.getCurrentController().getShell(),
      null, ZxUtils.getTitleWithIcon('ZxBackup', zxMsg('LBL_BCK_Restore_Wizard')), '720px', '360px', 'ZxUndeleteWizard');
  }

  this.stepChoices = [
    {label: zxMsg('LBL_BCK_account_infos'),
      value: ZxUndeleteWizard.STEP_LOAD_INFO},
    {label: zxMsg('LBL_Introduction'),
      value: ZxUndeleteWizard.STEP_INTRO},
    {label: zxMsg('LBL_BCK_Select_restore_operation'),
      value: ZxUndeleteWizard.STEP_SELECT_OPERATION},
    {label: zxMsg('LBL_BCK_Select_date'),
      value: ZxUndeleteWizard.STEP_SELECT_DATE},
    {label: zxMsg('LBL_BCK_op_summary'),
      value: ZxUndeleteWizard.STEP_SUMMARY}
  ];

  this._helpURL = ZxLinks.HELP_BCK_Restore_wizard;
  this.initForm(ZxUndeleteWizard.XModel, this.getMyXForm());

  this._localXForm.setController(ZaApp.getInstance());
  this._localXForm.addListener(DwtEvent.XFORMS_FORM_DIRTY_CHANGE,
    new AjxListener(this, this.handleXFormChange));
  this._localXForm.addListener(DwtEvent.XFORMS_VALUE_ERROR,
    new AjxListener(this, this.handleXFormChange));
}

ZxUndeleteWizard.prototype = new ZaXWizardDialog();
ZxUndeleteWizard.prototype.constructor = ZxUndeleteWizard;

ZxUndeleteWizard.STEP_INDEX = 1;
ZxUndeleteWizard.STEP_LOAD_INFO = ZxUndeleteWizard.STEP_INDEX++;
ZxUndeleteWizard.STEP_INTRO = ZxUndeleteWizard.STEP_INDEX++;
ZxUndeleteWizard.STEP_SELECT_OPERATION = ZxUndeleteWizard.STEP_INDEX++;
ZxUndeleteWizard.STEP_SELECT_DATE = ZxUndeleteWizard.STEP_INDEX++;
ZxUndeleteWizard.STEP_SUMMARY = ZxUndeleteWizard.STEP_INDEX++;

ZxUndeleteWizard.ID_SELECTION_DATE = 'DateSelection';
ZxUndeleteWizard.ID_SELECTION_DATE_END = 'DateSelectionEnd';
ZxUndeleteWizard.ID_SELECTION_DATE_START = 'DateSelectionStart';
ZxUndeleteWizard.ID_RADIO_OP_TYPE = 'operationType';

/**
 * XModel of the Wizard.
 * @type {Array}
 */
ZxUndeleteWizard.XModel = [
  {type: _DATETIME_, id: ZxRestoreOp.KEY_DATE,
    ref: 'attrs/' + ZxRestoreOp.KEY_DATE},
  {type: _DATETIME_, id: ZxRestoreOp.KEY_DATE_END,
    ref: 'attrs/' + ZxRestoreOp.KEY_DATE_END},
  {type: _STRING_, id: ZxRestoreOp.KEY_OPERATION,
    ref: 'attrs/' + ZxRestoreOp.KEY_OPERATION},
  {type: _STRING_, id: ZxRestoreOp.KEY_ACCOUNT,
    ref: 'attrs/' + ZxRestoreOp.KEY_ACCOUNT},
  {type: _STRING_, id: ZxRestoreOp.KEY_DESTINATION_ACCOUNT,
    ref: 'attrs/' + ZxRestoreOp.KEY_DESTINATION_ACCOUNT},
  {type: _STRING_, id: ZxRestoreOp.KEY_EXPORT_NOTIFICATION_MAIL,
    ref: 'attrs/' + ZxRestoreOp.KEY_EXPORT_NOTIFICATION_MAIL}
];

ZaXDialog.XFormModifiers['ZxUndeleteWizard'] = [];

ZxUndeleteWizard.prototype.miniType = 2; // WTF is this?

ZxUndeleteWizard.XFormModifier = function(xFormObject) {
  var cases = [],
    step_intro,
    step_load,
    step_date,
    step_selectOp,
    step_summary;

  step_intro = {
    type: _CASE_,
    caseKey: ZxUndeleteWizard.STEP_INTRO,
    items: [
      {
        type: _ZAWIZ_TOP_GROUPER_,
        label: zxMsg('LBL_Introduction'),
        numCols: 1,
        colSizes: ['*'],
        items: [
          {
            type: _OUTPUT_,
            value: zxMsg('MSG_BCK_INTRO_Restore_wizard_generic')
          },
          {
            type: _ZAWIZ_TOP_GROUPER_,
            label: zxMsg('LBL_BCK_Undelete'),
            numCols: 1,
            colSizes: ['*'],
            items: [
              {
                type: _OUTPUT_,
                value: zxMsg('MSG_BCK_INTRO_Restore_wizard_undelete')
              }
            ]
          },
          {
            type: _ZAWIZ_TOP_GROUPER_,
            label: zxMsg('LBL_BCK_Restore_on_new_account'),
            numCols: 1,
            colSizes: ['*'],
            items: [
              {
                type: _OUTPUT_,
                value: zxMsg('MSG_BCK_INTRO_Restore_wizard_restore_on_new')
              }
            ]
          }
        ]
      }
    ]
  };

  step_load = {
    type: _CASE_,
    caseKey: ZxUndeleteWizard.STEP_LOAD_INFO,
    items: [
      {
        type: _ZAWIZ_TOP_GROUPER_,
        label: zxMsg('LBL_BCK_account_infos'),
        numCols: 1,
        colSizes: ['*'],
        items: [
          {
            type: _OUTPUT_,
            cssStyle: 'padding-top: 10px',
            value: '<div class="DwtWait32Icon">&nbsp;</div><b>' + zxMsg('LBL_BCK_getting_accounts_import') + '</b>'
          }
        ]
      }
    ]
  };

  step_selectOp = {
    type: _CASE_,
    caseKey: ZxUndeleteWizard.STEP_SELECT_OPERATION,
    items: [
      {
        type: _ZAWIZ_TOP_GROUPER_,
        label: zxMsg('LBL_BCK_Select_restore_operation'),
        numCols: 2,
        colSizes: [100, '*'],
        items: [
          {
            type: _OUTPUT_,
            label: zxMsg('LBL_Account') + ': ',
            ref: 'attrs/' + ZxRestoreOp.KEY_ACCOUNT,
            colSpan: 2
          },
          {
            type: _OUTPUT_,
            label: zxMsg('LBL_Quota_Usage') + ': ',
            colSpan: 2,
            valueChangeEventSources: ['attrs/' + ZxRestoreOp.KEY_QUOTA, 'attrs/' + ZxRestoreOp.KEY_QUOTA_LIMIT],
            getDisplayValue: function() {
              var quota = this.getInstanceValue('attrs/' + ZxRestoreOp.KEY_QUOTA) || 0,
                quotaLimit = this.getInstanceValue('attrs/' + ZxRestoreOp.KEY_QUOTA_LIMIT) || -1;

              if (quotaLimit < 0 || isNaN(quotaLimit)) {
                quotaLimit = zxMsg('LBL_Unlimited');
              }

              if (typeof quota === 'number') {
                quota = ZxUtils.bytesToSize(quota, 2);
              }
              if (typeof quotaLimit === 'number') {
                quotaLimit = ZxUtils.bytesToSize(quotaLimit, 2);
              }

              return [quota, quotaLimit].join(' / ');
            }
          },
          { type: _SPACER_, colSpan: 2 },
          {
            type: _OUTPUT_,
            value: zxMsg('MSG_BCK_Select_restore_operation'),
            colSpan: 2
          },
          {
            type: _ZAWIZ_TOP_GROUPER_,
            label: zxMsg('LBL_BCK_Undelete'),
            colSpan: 3,
            numCols: 2,
            colSizes: [25, 'auto'],
            items: [
              {
                type: _RADIO_,
                groupName: ZxUndeleteWizard.ID_RADIO_OP_TYPE,
                value: ZxRestoreOp.OP_UNDELETE,
                ref: 'attrs/' + ZxRestoreOp.KEY_OPERATION,
                label: zxMsg('LBL_BCK_WIZ_radio_Undelete_Op'),
                bmolsnr: true
              }
            ]
          },
          {
            type: _ZAWIZ_TOP_GROUPER_,
            label: zxMsg('LBL_BCK_Restore_on_new_account'),
            colSpan: 3,
            numCols: 2,
            colSizes: [25, 'auto'],
            items: [
              {
                type: _RADIO_,
                groupName: ZxUndeleteWizard.ID_RADIO_OP_TYPE,
                value: ZxRestoreOp.OP_RESTORE_ON_NEW,
                ref: 'attrs/' + ZxRestoreOp.KEY_OPERATION,
                label: zxMsg('LBL_BCK_WIZ_radio_Restore_Op'),
                bmolsnr: true
              },
              {
                type: _GROUP_,
                visibilityChangeEventSources: ['attrs/' + ZxRestoreOp.KEY_OPERATION],
                visibilityChecks: [function() {
                  var op = this.getInstanceValue('attrs/' + ZxRestoreOp.KEY_OPERATION);
                  return op === ZxRestoreOp.OP_RESTORE_ON_NEW;
                }],
                colSpan: 2,
                numCols: 1,
                items: [
                  {
                    type: _GROUP_,
                    numCols: 3,
                    colSize: ['auto', 150, 'auto'],
                    items: [
                      {
                        type: _INPUT_,
                        ref: 'attrs/' + ZxRestoreOp.KEY_DESTINATION_ACCOUNT,
                        label: zxMsg('LBL_BCK_Destination_account_name')
                      },
                      {
                        type: _OUTPUT_,
                        ref: 'attrs/' + ZxRestoreOp.KEY_ACCOUNT,
                        getDisplayValue: function(value) {
                          if (value) {
                            return value.substring(ZxUtils.indexOf(value, '@'));
                          }
                          return '';
                        }
                      }
                    ]
                  },
                  {
                    type: _GROUP_,
                    colSpan: 3,
                    items: [
                      {
                        type: _CHECKBOX_,
                        labelCssClass: 'ZxCheckboxLabel',
                        label: zxMsg('LBL_BCK_Hide_new_account_in_gal') + ': ',
                        colSpan: 3,
                        labelLocation: _LEFT_,
                        align: _LEFT_,
                        ref: 'attrs/' + ZxRestoreOp.KEY_HIDE_IN_GAL
                      }
                    ]
                  }
                ]
              }
            ]
          }
        ]
      }
    ]
  };

  step_date = {
    type: _CASE_,
    caseKey: ZxUndeleteWizard.STEP_SELECT_DATE,
    items: [
      {
        type: _ZAWIZ_TOP_GROUPER_,
        label: zxMsg('LBL_BCK_Date_selection'),
        numCols: 1,
        items: [
          {
            type: _OUTPUT_,
            value: zxMsg('MSG_BCK_WIZ_op_undelete'),
            colSpan: 3,
            visibilityChangeEventSources: ['attrs/' + ZxRestoreOp.KEY_OPERATION],
            visibilityChecks: [function() {
              var op = this.getInstanceValue('attrs/' + ZxRestoreOp.KEY_OPERATION);
              return op === ZxRestoreOp.OP_UNDELETE;
            }]
          },
          {
            type: _OUTPUT_,
            value: zxMsg('MSG_BCK_WIZ_op_restore'),
            colSpan: 3,
            visibilityChangeEventSources: ['attrs/' + ZxRestoreOp.KEY_OPERATION],
            visibilityChecks: [function() {
              var op = this.getInstanceValue('attrs/' + ZxRestoreOp.KEY_OPERATION);
              return op === ZxRestoreOp.OP_RESTORE_ON_NEW;
            }]
          },
          { type: _SPACER_ },
          {   // Visibile in case of Undelete
            type: _GROUP_,
            colSpan: 3,
            numCols: 2,
            colSizes: ['50%', '50%'],
            visibilityChangeEventSources: ['attrs/' + ZxRestoreOp.KEY_OPERATION],
            visibilityChecks: [function() {
              var op = this.getInstanceValue('attrs/' + ZxRestoreOp.KEY_OPERATION);
              return op === ZxRestoreOp.OP_UNDELETE;
            }],
            items: [
              {
                type: _GROUPER_,
                label: zxMsg('LBL_BCK_Date_to_restore_from'),
                numCols: 1,
                valign: _TOP_,
                items: [
                  {
                    type: _ZX_DATETIME_,
                    id: ZxUndeleteWizard.ID_SELECTION_DATE_START,
                    ref: 'attrs/' + ZxRestoreOp.KEY_DATE,
                    labelLocation: _NONE_,
                    visibilityChangeEventSources: ['attrs/' + ZxRestoreOp.KEY_USE_FIRST],
                    visibilityChecks: [function() {
                      return !this.getInstanceValue('attrs/' + ZxRestoreOp.KEY_USE_FIRST);
                    }]
                  },
                  {
                    type: _GROUP_,
                    items: [
                      {
                        type: _CHECKBOX_,
                        label: zxMsg('LBL_BCK_Use_first_available'),
                        labelLocation: _LEFT_,
                        align: _LEFT_,
                        ref: 'attrs/' + ZxRestoreOp.KEY_USE_FIRST
                      }
                    ]
                  }
                ]
              },
              {
                type: _GROUPER_,
                label: zxMsg('LBL_BCK_Backup_info'),
                valign: _TOP_,
                items: [
                  {
                    type: _OUTPUT_,
                    label: zxMsg('LBL_BCK_First_fullscan') + ': ',
                    ref: 'attrs/' + ZxRestoreOp.KEY_FIRST_SCAN
                  },
                  {
                    type: _OUTPUT_,
                    label: zxMsg('LBL_BCK_Current_retention') + ': ',
                    ref: 'attrs/' + ZxRestoreOp.KEY_RETENTION
                  }
                ]
              },
              {
                type: _GROUPER_,
                label: zxMsg('LBL_BCK_Date_to_restore_to'),
                numCols: 1,
                valign: _TOP_,
                items: [
                  {
                    type: _ZX_DATETIME_,
                    id: ZxUndeleteWizard.ID_SELECTION_DATE_END,
                    ref: 'attrs/' + ZxRestoreOp.KEY_DATE_END,
                    labelLocation: _NONE_,
                    visibilityChangeEventSources: ['attrs/' + ZxRestoreOp.KEY_USE_LAST],
                    visibilityChecks: [function() {
                      return !this.getInstanceValue('attrs/' + ZxRestoreOp.KEY_USE_LAST);
                    }]
                  },
                  {
                    type: _GROUP_,
                    items: [
                      {
                        type: _CHECKBOX_,
                        label: zxMsg('LBL_BCK_Use_last_available'),
                        labelLocation: _LEFT_,
                        align: _LEFT_,
                        ref: 'attrs/' + ZxRestoreOp.KEY_USE_LAST
                      }
                    ]
                  }
                ]
              }
            ]
          },
          {  // Visibile in case of Restore on new
            type: _GROUP_,
            colSpan: 3,
            numCols: 2,
            colSizes: ['50%', '50%'],
            visibilityChangeEventSources: ['attrs/' + ZxRestoreOp.KEY_OPERATION],
            visibilityChecks: [function() {
              var op = this.getInstanceValue('attrs/' + ZxRestoreOp.KEY_OPERATION);
              return op === ZxRestoreOp.OP_RESTORE_ON_NEW;
            }],
            items: [
              {
                type: _GROUPER_,
                label: zxMsg('LBL_BCK_Date_to_restore_from'),
                numCols: 1,
                valign: _TOP_,
                items: [
                  {
                    type: _ZX_DATETIME_,
                    id: ZxUndeleteWizard.ID_SELECTION_DATE,
                    ref: 'attrs/' + ZxRestoreOp.KEY_DATE,
                    labelLocation: _NONE_,
                    visibilityChangeEventSources: ['attrs/' + ZxRestoreOp.KEY_USE_LAST],
                    visibilityChecks: [function() {
                      return !this.getInstanceValue('attrs/' + ZxRestoreOp.KEY_USE_LAST);
                    }]
                  },
                  {
                    type: _GROUP_,
                    items: [
                      {
                        type: _CHECKBOX_,
                        label: zxMsg('LBL_BCK_Use_last_available'),
                        labelLocation: _LEFT_,
                        align: _LEFT_,
                        ref: 'attrs/' + ZxRestoreOp.KEY_USE_LAST
                      }
                    ]
                  }
                ]
              },
              {
                type: _GROUPER_,
                label: zxMsg('LBL_BCK_Backup_info'),
                valign: _TOP_,
                items: [
                  {
                    type: _OUTPUT_,
                    label: zxMsg('LBL_BCK_First_fullscan') + ': ',
                    ref: 'attrs/' + ZxRestoreOp.KEY_FIRST_SCAN
                  },
                  {
                    type: _OUTPUT_,
                    label: zxMsg('LBL_BCK_Current_retention') + ': ',
                    ref: 'attrs/' + ZxRestoreOp.KEY_RETENTION
                  }
                ]
              }
            ]
          }
        ]
      }
    ]
  };

  step_summary = {
    type: _CASE_,
    caseKey: ZxUndeleteWizard.STEP_SUMMARY,
    numCols: 1,
    items: [
      {
        type: _ZAWIZ_TOP_GROUPER_,
        label: zxMsg('LBL_BCK_op_summary'),
        items: [
          {
            type: _OUTPUT_,
            label: zxMsg('LBL_BCK_Select_restore_operation') + ': ',
            ref: 'attrs/' + ZxRestoreOp.KEY_OPERATION,
            cssStyle: 'font-weight: bold',
            getDisplayValue: function(val) {
              if (val === ZxRestoreOp.OP_RESTORE_ON_NEW) {
                return zxMsg('LBL_BCK_Restore_on_new_account');
              }
              if (val === ZxRestoreOp.OP_UNDELETE) {
                return zxMsg('LBL_BCK_Undelete');
              }
            }
          },
          {
            type: _OUTPUT_,
            label: zxMsg('LBL_BCK_Restore_account') + ': ',
            ref: 'attrs/' + ZxRestoreOp.KEY_ACCOUNT,
            cssStyle: 'font-weight: bold'
          },
          {
            type: _OUTPUT_,
            label: zxMsg('LBL_BCK_Destination_account') + ': ',
            ref: 'attrs/' + ZxRestoreOp.KEY_DESTINATION_ACCOUNT,
            cssStyle: 'font-weight: bold',
            visibilityChangeEventSources: ['attrs/' + ZxRestoreOp.KEY_OPERATION],
            visibilityChecks: [function() {
              var op = this.getInstanceValue('attrs/' + ZxRestoreOp.KEY_OPERATION);
              return op === ZxRestoreOp.OP_RESTORE_ON_NEW;
            }],
            getDisplayValue: function(value) {
              var tmpString = this.getForm().parent._containedObject.attrs[ZxRestoreOp.KEY_ACCOUNT];
              return value + tmpString.substring(ZxUtils.indexOf(tmpString, '@'));
            }
          },
          {
            type: _OUTPUT_,
            label: zxMsg('LBL_BCK_Hide_new_account_in_gal') + ': ',
            ref: 'attrs/' + ZxRestoreOp.KEY_HIDE_IN_GAL,
            cssStyle: 'font-weight: bold',
            visibilityChangeEventSources: ['attrs/' + ZxRestoreOp.KEY_OPERATION],
            visibilityChecks: [function() {
              var op = this.getInstanceValue('attrs/' + ZxRestoreOp.KEY_OPERATION);
              return op === ZxRestoreOp.OP_RESTORE_ON_NEW;
            }],
            getDisplayValue: function(value) {
              return value ? ZaMsg.Yes : ZaMsg.No;
            }
          },
          {
            type: _OUTPUT_,
            label: zxMsg('LBL_BCK_Date_to_restore_from') + ': ',
            ref: 'attrs/' + ZxRestoreOp.KEY_DATE,
            cssStyle: 'font-weight: bold',
            valueChangeEventSources: ['attrs/' + ZxRestoreOp.KEY_DATE, 'attrs/' + ZxRestoreOp.KEY_USE_FIRST, ZxRestoreOp.KEY_USE_LAST],
            getDisplayValue: function(val) {
              var useFirst;
              if (this.getInstanceValue('attrs/' + ZxRestoreOp.KEY_OPERATION) === ZxRestoreOp.OP_UNDELETE) {
                useFirst = this.getInstanceValue('attrs/' + ZxRestoreOp.KEY_USE_FIRST);
              } else {
                useFirst = this.getInstanceValue('attrs/' + ZxRestoreOp.KEY_USE_LAST);
              }
              return useFirst ? zxMsg('LBL_BCK_Use_first_available') :
                ZxUtils.computeWordyDateStr(new Date(), val);

            }
          },
          {
            type: _OUTPUT_,
            label: zxMsg('LBL_BCK_Date_to_restore_to') + ': ',
            ref: 'attrs/' + ZxRestoreOp.KEY_DATE_END,
            cssStyle: 'font-weight: bold',
            valueChangeEventSources: ['attrs/' + ZxRestoreOp.KEY_DATE_END, 'attrs/' + ZxRestoreOp.KEY_USE_LAST],
            visibilityChangeEventSources: ['attrs/' + ZxRestoreOp.KEY_OPERATION],
            visibilityChecks: [function() {
              var op = this.getInstanceValue('attrs/' + ZxRestoreOp.KEY_OPERATION);
              return op === ZxRestoreOp.OP_UNDELETE;
            }],
            getDisplayValue: function(val) {
              var useFirst = this.getInstanceValue('attrs/' + ZxRestoreOp.KEY_USE_LAST);

              return useFirst ? zxMsg('LBL_BCK_Use_last_available') :
                  ZxUtils.computeWordyDateStr(new Date(), val);
            }
          },
          {
            type: _OUTPUT_,
            label: zxMsg('LBL_Interval') + ': ',
            cssStyle: 'font-weight: bold',
            visibilityChangeEventSources: ['attrs/' + ZxRestoreOp.KEY_OPERATION],
            visibilityChecks: [function() {
              var op = this.getInstanceValue('attrs/' + ZxRestoreOp.KEY_OPERATION),
                useFirst = this.getInstanceValue('attrs/' + ZxRestoreOp.KEY_USE_FIRST),
                useLast = this.getInstanceValue('attrs/' + ZxRestoreOp.KEY_USE_LAST);
              return op === ZxRestoreOp.OP_UNDELETE && !useLast && !useFirst;
            }],
            valueChangeEventSources: ['attrs/' + ZxRestoreOp.KEY_DATE,
              'attrs/' + ZxRestoreOp.KEY_DATE_END],
            getDisplayValue: function(value) {
              var start = this.getForm().parent._containedObject.attrs[ZxRestoreOp.KEY_DATE],
                end = this.getForm().parent._containedObject.attrs[ZxRestoreOp.KEY_DATE_END];
              return AjxDateUtil.computeDuration(end.getTime() - start.getTime(), false, false);
            }
          },
          {
            type: _OUTPUT_,
            label: zxMsg('LBL_BCK_Domain_Accounts_Limit') + ': ',
            cssStyle: 'font-weight: bold',
            visibilityChangeEventSources: [
              'attrs/' + ZxRestoreOp.KEY_OPERATION,
              'attrs/' + ZxRestoreOp.KEY_ACCOUNT_LIMIT
            ],
            visibilityChecks: [function() {
              var op = this.getInstanceValue('attrs/' + ZxRestoreOp.KEY_OPERATION),
                limit = this.getForm().parent._containedObject.attrs[ZxRestoreOp.KEY_ACCOUNT_LIMIT];
              return (op === ZxRestoreOp.OP_RESTORE_ON_NEW && limit !== -1);
            }],
            valueChangeEventSources: [
              'attrs/' + ZxRestoreOp.KEY_ACCOUNT_LIMIT_COUNT,
              'attrs/' + ZxRestoreOp.KEY_ACCOUNT_LIMIT
            ],
            getDisplayValue: function(val) {
              var count = this.getForm().parent._containedObject.attrs[ZxRestoreOp.KEY_ACCOUNT_LIMIT_COUNT],
                limit = this.getForm().parent._containedObject.attrs[ZxRestoreOp.KEY_ACCOUNT_LIMIT];
              return count + ' / ' + ((limit === -1) ? zxMsg('LBL_Unlimited') : limit);
            }
          },
          {
            type: _OUTPUT_,
            label: zxMsg('LBL_BCK_COS_Accounts_Limit') + ': ',
            cssStyle: 'font-weight: bold',
            visibilityChangeEventSources: [
              'attrs/' + ZxRestoreOp.KEY_OPERATION,
              'attrs/' + ZxRestoreOp.KEY_COS_LIMIT
            ],
            visibilityChecks: [function() {
              var op = this.getInstanceValue('attrs/' + ZxRestoreOp.KEY_OPERATION),
                limit = this.getForm().parent._containedObject.attrs[ZxRestoreOp.KEY_COS_LIMIT];
              return (op === ZxRestoreOp.OP_RESTORE_ON_NEW && limit !== -1);
            }],
            valueChangeEventSources: [
              'attrs/' + ZxRestoreOp.KEY_COS_LIMIT_COUNT,
              'attrs/' + ZxRestoreOp.KEY_COS_LIMIT
            ],
            getDisplayValue: function(val) {
              var count = this.getForm().parent._containedObject.attrs[ZxRestoreOp.KEY_COS_LIMIT_COUNT],
                limit = this.getForm().parent._containedObject.attrs[ZxRestoreOp.KEY_COS_LIMIT];
              return count + ' / ' + ((limit === -1) ? zxMsg('LBL_Unlimited') : limit);
            }
          }
        ]
      },
      {
        type: _ZAWIZ_TOP_GROUPER_,
        label: zxMsg('LBL_BCK_op_notification'),
        numCols: 3,
        colSizes: ['33%', '33%', '33%'],
        items: [
          {
            type: _INPUT_,
            label: zxMsg('LBL_BCK_additional_notification_mail') + ': ',
            ref: 'attrs/' + ZxRestoreOp.KEY_NOTIFICATION_MAIL,
            align: _LEFT_,
            width: 180
          },
          {
            type: _OUTPUT_,
            value: zxMsg('MSG_BCK_you_will_be_notified_chen_account_undelete_completed1'),
            align: _MIDDLE_
          },
          { type: _SPACER_, colSpan: 3 },
          {
            type: _OUTPUT_,
            value: zxMsg('MSG_BCK_you_will_be_notified_chen_account_undelete_completed2'),
            colSpan: 3
          },
          { type: _SPACER_, colSpan: 3 },
          {
            type: _OUTPUT_,
            value: zxMsg('MSG_BCK_WIZ_Wait_before_sync_zimbra_desktop'),
            colSpan: 3,
            visibilityChangeEventSources: ['attrs/' + ZxRestoreOp.KEY_OPERATION],
            visibilityChecks: [function() {
              var op = this.getInstanceValue('attrs/' + ZxRestoreOp.KEY_OPERATION);
              return op === ZxRestoreOp.OP_RESTORE_ON_NEW;
            }]
          }
        ]
      }
    ]
  };

  cases.push(step_intro);
  cases.push(step_selectOp);
  cases.push(step_date);
  cases.push(step_summary);

  xFormObject.items = [
    {
      type: _OUTPUT_,
      colSpan: 2,
      align: _CENTER_,
      valign: _TOP_,
      ref: ZaModel.currentStep,
      choices: this.stepChoices,
      valueChangeEventSources: [ZaModel.currentStep],
      labelClickMethod: function() {
        var sourceValue = this.getInheritedProperty('sourceValue'),
          instanceValue = this.getInstanceValue();
        if (sourceValue < instanceValue) {}
      }
    },
    {
      type: _SEPARATOR_,
      align: _CENTER_,
      valign: _TOP_
    },
    {
      type: _SPACER_,
      align: _CENTER_,
      valign: _TOP_
    },
    {
      type: _SWITCH_,
      align: _LEFT_,
      valign: _TOP_,
      items: cases
    }
  ];
};
ZaXDialog.XFormModifiers['ZxUndeleteWizard'].push(ZxUndeleteWizard.XFormModifier);

/**
 * Move to next page.
 * This also trigger a check of the entire form.
 */
ZxUndeleteWizard.prototype.goNext = function() {
  this.goPage(this._containedObject[ZaModel.currentStep] + 1);
  this.handleXFormChange();
};

/**
 * Move to previous page.
 * This also trigger a check of the entire form.
 */
ZxUndeleteWizard.prototype.goPrev = function() {
  this.goPage(this._containedObject[ZaModel.currentStep] - 1);
  this.handleXFormChange();
};

/**
 * When something is changed (also the page number),
 * validate the inputs and enable/disable wizard buttons.
 */
ZxUndeleteWizard.prototype.handleXFormChange = function() {
  var cStep = this._containedObject[ZaModel.currentStep];
  ZxUtils.log(cStep, 'Wizard step');

  if (this._localXForm.hasErrors()) {
    this._button[DwtWizardDialog.FINISH_BUTTON].setEnabled(false);
  } else {
    this._button[DwtWizardDialog.PREV_BUTTON].setEnabled(true);

    if (cStep === ZxUndeleteWizard.STEP_INTRO) {
      this._button[DwtDialog.CANCEL_BUTTON].setEnabled(true);
      this._button[DwtWizardDialog.NEXT_BUTTON].setEnabled(true);
      this._button[DwtWizardDialog.PREV_BUTTON].setEnabled(false);
      this._button[DwtWizardDialog.FINISH_BUTTON].setEnabled(false);
      return;
    }

    if (cStep === ZxUndeleteWizard.STEP_LOAD_INFO) {
      this._button[DwtWizardDialog.NEXT_BUTTON].setEnabled(true);

      ZxAdminRequest.sendRequest(
        this._containedObject._modItem.getCurrentServer(),
        'ZxBackup',
        'getDomainAccountLimitInfo',
        {
          account: this._containedObject.attrs[ZxRestoreOp.KEY_ACCOUNT]
        }
      ).then(
        AjxCallback.simpleClosure(
          function(resp) {
            var response = resp.response;
            this._containedObject.attrs[ZxRestoreOp.KEY_ACCOUNT_LIMIT] = (response.maxAccounts === 'unlimited') ?
                -1 : response.maxAccounts;
            this._containedObject.attrs[ZxRestoreOp.KEY_ACCOUNT_LIMIT_COUNT] = response.numAccounts;

            ZxAdminRequest.sendRequest(
              this._containedObject._modItem.getCurrentServer(),
              'ZxBackup',
              'getCosAccountLimitInfo',
              {
                account: this._containedObject.attrs[ZxRestoreOp.KEY_ACCOUNT]
              }
            ).then(
              AjxCallback.simpleClosure(
                function(resp) {
                  if (!(resp instanceof ZxError)) {
                    var response = resp.response;
                    this._containedObject.attrs[ZxRestoreOp.KEY_COS_LIMIT] =
                        (response.maxAccounts === 'unlimited') ? -1 : response.maxAccounts;
                    this._containedObject.attrs[ZxRestoreOp.KEY_COS_LIMIT_COUNT] = response.numAccounts;
                    this.goNext();

                  } else {
                    ZaApp.getInstance().getCurrentController().popupErrorDialog(resp.getMessage());
                    this.popdown();
                  }
                },
                this
              ),
              AjxCallback.simpleClosure(function(error) {
                if (error.getCode() === ZxErrorCode.UNABLE_TO_FIND_COS) {
                  this.goNext();
                } else {
                  ZxUtils.catchError(error);
                  this.goPrev();
                }
              }, this)
            );
          }, this),
          ZxUtils.catchError
      );

      this._button[DwtDialog.CANCEL_BUTTON].setEnabled(true);
      this._button[DwtWizardDialog.PREV_BUTTON].setEnabled(false);
      this._button[DwtWizardDialog.FINISH_BUTTON].setEnabled(false);
      return;
    }

    if (cStep === ZxUndeleteWizard.STEP_SELECT_OPERATION) {
      this._button[DwtDialog.CANCEL_BUTTON].setEnabled(true);
      this._button[DwtWizardDialog.PREV_BUTTON].setEnabled(true);

      this._button[DwtWizardDialog.NEXT_BUTTON].setEnabled(false);

      if (this._containedObject.attrs[ZxRestoreOp.KEY_OPERATION] === ZxRestoreOp.OP_UNDELETE) {
        this._button[DwtWizardDialog.NEXT_BUTTON].setEnabled(true);
      }

      if (this._containedObject.attrs[ZxRestoreOp.KEY_OPERATION] === ZxRestoreOp.OP_RESTORE_ON_NEW) {
        var tmpAcc = this._containedObject.attrs[ZxRestoreOp.KEY_ACCOUNT],
          tmpDest = this._containedObject.attrs[ZxRestoreOp.KEY_DESTINATION_ACCOUNT],
          dest = tmpDest + tmpAcc.substring(ZxUtils.indexOf(tmpAcc, '@'));

        if (tmpDest !== '' && tmpAcc !== dest) {
          this._button[DwtWizardDialog.NEXT_BUTTON].setEnabled(true);
        }
      }

      this._button[DwtWizardDialog.FINISH_BUTTON].setEnabled(false);
      return;
    }

    if (cStep === ZxUndeleteWizard.STEP_SELECT_DATE) {
      this._button[DwtWizardDialog.NEXT_BUTTON].setEnabled(true);
      this._button[DwtDialog.CANCEL_BUTTON].setEnabled(true);
      this._button[DwtWizardDialog.PREV_BUTTON].setEnabled(true);
      this._button[DwtWizardDialog.FINISH_BUTTON].setEnabled(false);
      return;
    }

    if (cStep === ZxUndeleteWizard.STEP_SUMMARY) {
      this._button[DwtDialog.CANCEL_BUTTON].setEnabled(true);
      this._button[DwtWizardDialog.PREV_BUTTON].setEnabled(true);
      this._button[DwtWizardDialog.NEXT_BUTTON].setEnabled(false);
      this._button[DwtWizardDialog.FINISH_BUTTON].setEnabled(true);

      if (this._containedObject.attrs[ZxRestoreOp.KEY_OPERATION] === ZxRestoreOp.OP_RESTORE_ON_NEW) {
        if ((this._containedObject.attrs[ZxRestoreOp.KEY_ACCOUNT_LIMIT] === -1 ||
            this._containedObject.attrs[ZxRestoreOp.KEY_ACCOUNT_LIMIT_COUNT] <
              this._containedObject.attrs[ZxRestoreOp.KEY_ACCOUNT_LIMIT]) &&
            (this._containedObject.attrs[ZxRestoreOp.KEY_COS_LIMIT] === -1 ||
            this._containedObject.attrs[ZxRestoreOp.KEY_COS_LIMIT_COUNT] <
              this._containedObject.attrs[ZxRestoreOp.KEY_COS_LIMIT])) {
        } else {
          ZaApp.getInstance().getCurrentController().popupWarningDialog(
            zxMsg('LBL_BCK_Restore_deleted_account_exceed_cos_limit')
          );
          this._button[DwtWizardDialog.FINISH_BUTTON].setEnabled(false);
        }
      }
      return;
    }
  }
};

ZxUndeleteWizard.prototype.finishWizard = function() {
  this._button[DwtWizardDialog.PREV_BUTTON].setEnabled(false);
  this._button[DwtWizardDialog.FINISH_BUTTON].setEnabled(false);

  var item = this._containedObject._modItem,
    attrs = this._containedObject.attrs,
    reqParam = {},
    action = attrs[ZxRestoreOp.KEY_OPERATION];

  if (action !== ZxRestoreOp.OP_RESTORE_ON_NEW && action !== ZxRestoreOp.OP_UNDELETE) {
    throw 'Trying to do an illegal command!';
  }

  if (attrs[ZxRestoreOp.KEY_EXPORT_NOTIFICATION_MAIL] &&
      attrs[ZxRestoreOp.KEY_EXPORT_NOTIFICATION_MAIL] !== '') {

    attrs[ZxRestoreOp.KEY_EXPORT_NOTIFICATION_MAIL] += ',' +
      ZaZimbraAdmin.currentUserLogin;

  } else {
    attrs[ZxRestoreOp.KEY_EXPORT_NOTIFICATION_MAIL] = ZaZimbraAdmin.currentUserLogin;
  }

  if (action === ZxRestoreOp.OP_RESTORE_ON_NEW) {
    reqParam.srcAccountName = attrs[ZxRestoreOp.KEY_ACCOUNT];
    reqParam.hideInGAL = attrs[ZxRestoreOp.KEY_HIDE_IN_GAL];
    reqParam.restoreChatBuddies = false;
    reqParam.dstAccountName = attrs[ZxRestoreOp.KEY_DESTINATION_ACCOUNT] +
      reqParam.srcAccountName.substring(ZxUtils.indexOf(reqParam.srcAccountName, '@'));

    if (!attrs[ZxRestoreOp.KEY_USE_LAST]) {
      reqParam.date = attrs[ZxRestoreOp.KEY_DATE].getTime();
    }
  }

  if (action === ZxRestoreOp.OP_UNDELETE) {
    reqParam.accountName = attrs[ZxRestoreOp.KEY_ACCOUNT];
    if (!attrs[ZxRestoreOp.KEY_USE_FIRST]) {
      reqParam.startDate = attrs[ZxRestoreOp.KEY_DATE].getTime();
    }
    if (!attrs[ZxRestoreOp.KEY_USE_LAST]) {
      reqParam.endDate = attrs[ZxRestoreOp.KEY_DATE_END].getTime();
    }
  }

  reqParam.obeyHSM = false;
  reqParam.notificationMails = attrs[ZxRestoreOp.KEY_EXPORT_NOTIFICATION_MAIL];

  ZxUtils.log(reqParam, 'Sending request for Restore process');

  ZxAdminRequest.sendRequest(
    this._containedObject._modItem.getCurrentServer(),
    'ZxBackup',
    action,
    reqParam
  ).then(
    AjxCallback.simpleClosure(function(resp) {
      var server = this._containedObject._modItem.getCurrentServer();
      if (!(resp instanceof ZxError)) {
        ZaApp.getInstance().getCurrentController().popupMsgDialog(zxMsg('MSG_Operation_queued'));
        ZeXtras.getOperationsManager().requestUpdate();
      } else {
        // TODO: Is this really necessary?
        var dia = new ZxErrorDialog(resp, server);
      }
      this.popdown();
    }, this),
    AjxCallback.simpleClosure(function(err) {
      ZxUtils.catchError(err);
      this.popdown();
    }, this)
  );

};

/**
 * Set the instance object.
 * When a wizard (on Z8) is moved as running tasks, when is resumed,
 * Zimbra invoke this function to restore the form with the correct information.
 * @param {ZxRestoreOp} entry
 */
ZxUndeleteWizard.prototype.setObject = function(entry) {
  this._containedObject = entry;

  this._containedObject[ZaModel.currentStep] = entry[ZaModel.currentStep] || ZxUndeleteWizard.STEP_LOAD_INFO;
  this._localXForm.setInstance(this._containedObject);
};
