/*
 * ***** BEGIN LICENSE BLOCK *****
 * Copyright (C) 2011-2016 ZeXtras
 *
 * The contents of this file are subject to the ZeXtras EULA;
 * you may not use this file except in compliance with the EULA.
 * You may obtain a copy of the EULA at
 * http://www.zextras.com/zextras-eula.html
 * ***** END LICENSE BLOCK *****
 */
/**
 * Create the ZxChat object.
 * @constructor
 */
function ZxChat() {
  ZxModule.call(this, ZxChat.MOD_ID, zxMsg('LBL_ZxChat'), 'ZxChat');

  this.registerLoadMethod(this.loadMethod, 'ZxChatInit');
  this.registerModifyMethod(this.modifyMethod, 'ZxChatInit');

  this._helpURL = ZxLinks.HELP_CHT_Chat;
}

ZxChat.prototype = new ZxModule();
ZxChat.prototype.constructor = ZxChat;
ZxChat.MOD_ID = 'ZxChat';

ZxChat.ZIMLET_NAME = 'com_zextras_chat';
ZxChat.ZIMLET_MANDATORY = "!";
ZxChat.ZIMLET_ENABLED = "+";
ZxChat.ZIMLET_DISABLED = "-";


ZxChat.KEY_USE_LEGACY_SSL_PORT = 'ZxChat_UseLegacySSLPort';
ZxChat.KEY_ALLOW_UNENCRYPTED_PASSWORD = 'ZxChat_AllowUnencryptedPassword';
ZxChat.KEY_DISABLE_HISTORY = 'ZxChat_DisableChatHistory';
ZxChat.KEY_REMOVE_ZXEXTRAS_BRAND = 'ZxChat_RemoveZextrasBrand';
ZxChat.KEY_STATUS = 'chat_status';
ZxChat.KEY_IS_RUNNING = 'isRunning';
ZxChat.KEY_STATS_ACTIVE_USER_COUNT = 'active_users_count';
ZxChat.KEY_STATS_ACTIVE_SESSION_COUNT = 'active_sessions_count';
ZxChat.KEY_STATS_DB_VERSION = 'DB_version';
ZxChat.KEY_ZIMLET_INSTALLED = 'zimlet_installed';
ZxChat.KEY_ZIMLET_ENABLED = 'zimlet_enabled';
ZxChat.KEY_INFO_ZIMLET_FOUND = 'zimlet_found';
ZxChat.KEY_INFO_ZIMLET_VERSION = 'zimlet_version';
ZxChat.KEY_INFO_ZIMLET_AVAILABLE_VERSION = 'zimlet_available';
ZxChat.KEY_CORE_VERSION_CURRENT = 'currentCoreVersion';
ZxChat.KEY_CORE_VERSION_MINIMUM = 'minCoreVersion';
ZxChat.KEY_REQUIRED_ZIMLET = 'required_zimlet';
ZxChat.KEY_TMP_DISTRIBUTION_LIST_NAME = 'tmp_distributionListName';
ZxChat.KEY_TMP_FIRST_USER = 'tmp_firstAccount';
ZxChat.KEY_TMP_SECOND_USER = 'tmp_secondAccount';

ZxChat.KEY_USR_ZIMLET_STATUS = 'zxchat_zimletStatus';

ZxChat.Zimlet = function() {};
ZxChat.Zimlet.PRESENCE = false;
ZxChat.Zimlet.CACHE_TIME = 5; // Validity of the cache in minutes.
ZxChat.Zimlet.LAST_UPDATE = 0;
ZxChat.Zimlet.isPresent = function() {
  var expired = (ZxChat.Zimlet.LAST_UPDATE + (ZxChat.Zimlet.CACHE_TIME * 60 * 1000)) < new Date(),
    zimletList,
    i;
  if (expired) {
    zimletList = ZaZimlet.getAll().getArray();
    for (i = 0; i < zimletList.length; i = i + 1) {
      if (zimletList[i].name === ZxChat.ZIMLET_NAME) {
        ZxChat.Zimlet.PRESENCE = true;
        break;
      }
    }
    ZxChat.Zimlet.PRESENCE = false;
    ZxChat.Zimlet.LAST_UPDATE = new Date();
  }
  return ZxChat.Zimlet.PRESENCE;
};

/**
 * Load informations for the view.
 */
ZxChat.prototype.loadMethod = function(by, val) {
// ZxUtils.log(arguments, 'Called load method');
  this.attrs.stats = {};
  this.attrs.stats[ZxChat.KEY_STATS_ACTIVE_USER_COUNT] = 0;
  this.attrs.stats[ZxChat.KEY_STATS_ACTIVE_SESSION_COUNT] = 0;
  this.attrs.stats[ZxChat.KEY_STATS_DB_VERSION] = 0;

  this.attrs.properties = {};

  this.attrs.info = {};
  this.attrs.info[ZxChat.KEY_ZIMLET_INSTALLED] = false;
  this.attrs.info[ZxChat.KEY_ZIMLET_ENABLED] = false;
  this.attrs.info[ZxChat.KEY_INFO_ZIMLET_FOUND] = false;
  this.attrs.info[ZxChat.KEY_INFO_ZIMLET_VERSION] = ZaMsg.splashScreenLoading;
  this.attrs.info[ZxChat.KEY_INFO_ZIMLET_AVAILABLE_VERSION] = ZaMsg.splashScreenLoading;

  var chatZimlet = ZaZimlet.getAll().getItemById(ZxChat.ZIMLET_NAME);
  if (!!chatZimlet) {
    this.attrs.info[ZxChat.KEY_ZIMLET_INSTALLED] = true;
    this.attrs.info[ZxChat.KEY_ZIMLET_ENABLED] = chatZimlet.isEnabled();
  }

  ZxAdminRequest.sendRequest(
    this.getCurrentServer(),
    'ZxChat',
    'getPublicProperties',
    {}
  ).then(
    AjxCallback.simpleClosure(
      function(resp) {
        var response = resp.response,
          controller = this.getCurrentController();

        this.attrs.properties[ZxChat.KEY_USE_LEGACY_SSL_PORT] =
          response.properties[ZxChat.KEY_USE_LEGACY_SSL_PORT] === 'true';
        this.attrs.properties[ZxChat.KEY_ALLOW_UNENCRYPTED_PASSWORD] =
          response.properties[ZxChat.KEY_ALLOW_UNENCRYPTED_PASSWORD] === 'true';
        this.attrs.properties[ZxChat.KEY_DISABLE_HISTORY] =
          response.properties[ZxChat.KEY_DISABLE_HISTORY] === 'true';
        this.attrs.properties[ZxChat.KEY_REMOVE_ZXEXTRAS_BRAND] =
          response.properties[ZxChat.KEY_REMOVE_ZXEXTRAS_BRAND] === 'true';

        controller.setInstanceValue(
          'properties/' + ZxChat.KEY_USE_LEGACY_SSL_PORT,
          response.properties[ZxChat.KEY_USE_LEGACY_SSL_PORT] === 'true'
        );
        controller.setInstanceValue(
          'properties/' + ZxChat.KEY_ALLOW_UNENCRYPTED_PASSWORD,
          response.properties[ZxChat.KEY_ALLOW_UNENCRYPTED_PASSWORD] === 'true'
        );
        controller.setInstanceValue(
          'properties/' + ZxChat.KEY_DISABLE_HISTORY,
          response.properties[ZxChat.KEY_DISABLE_HISTORY] === 'true'
        );
        controller.setInstanceValue(
            'properties/' + ZxChat.KEY_REMOVE_ZXEXTRAS_BRAND,
            response.properties[ZxChat.KEY_REMOVE_ZXEXTRAS_BRAND] === 'true'
        );
      },
      this
    ),
    this._getDefaultCatchErrorFcn()
  );

  ZxAdminRequest.sendRequest(
    this.getCurrentServer(),
    'ZxChat',
    'getServerStatus'
  ).then(
    AjxCallback.simpleClosure(
      function(resp) {
        // ZxUtils.log(resp, 'Response for ZxCore.getPublicProperties()');
        var response = resp.response,
          controller = this.getCurrentController();

        this.attrs[ZxChat.KEY_IS_RUNNING] = response.server_status;
        this.attrs[ZxChat.KEY_STATUS] = ((response.server_status === 'running') ?
            zxMsg('LBL_Enabled') : zxMsg('LBL_Disabled'));
        this.attrs.stats[ZxChat.KEY_STATS_ACTIVE_USER_COUNT] = response[ZxChat.KEY_STATS_ACTIVE_USER_COUNT];
        this.attrs.stats[ZxChat.KEY_STATS_ACTIVE_SESSION_COUNT] =
          response[ZxChat.KEY_STATS_ACTIVE_SESSION_COUNT];
        this.attrs.stats[ZxChat.KEY_STATS_DB_VERSION] = response[ZxChat.KEY_STATS_DB_VERSION];
        this.attrs[ZxChat.KEY_REQUIRED_ZIMLET] = response.required_zimlet;

        controller.setInstanceValue(
          ZxChat.KEY_IS_RUNNING,
          response.server_status
        );

        controller.setInstanceValue(
          ZxChat.KEY_STATUS,
          ((response.server_status === 'running') ?
              zxMsg('LBL_Enabled') : zxMsg('LBL_Disabled'))
        );

        controller.setInstanceValue(
          'stats/' + ZxChat.KEY_STATS_ACTIVE_USER_COUNT,
          response[ZxChat.KEY_STATS_ACTIVE_USER_COUNT]
        );

        controller.setInstanceValue(
          'stats/' + ZxChat.KEY_STATS_ACTIVE_SESSION_COUNT,
          response[ZxChat.KEY_STATS_ACTIVE_SESSION_COUNT]
        );

        controller.setInstanceValue(
          'stats/' + ZxChat.KEY_STATS_DB_VERSION,
          response[ZxChat.KEY_STATS_DB_VERSION]
        );
      },
      this
    ),
    this._getDefaultCatchErrorFcn()
  );

  ZxAdminRequest.sendRequest(
    this.getCurrentServer(),
    'ZxCore',
    'getUpdateInfo'
  ).then(
    AjxCallback.simpleClosure(
      function(resp) {
        var info = resp.response,
          zimletVersion = info.zxchatZimletCurrentVersion;
        if (zimletVersion === '0.0' || zimletVersion === '') {
          zimletVersion = 'None';
        }

        this.attrs[ZxChat.KEY_CORE_VERSION_MINIMUM] = info.minCoreVersion;
        this.attrs[ZxChat.KEY_CORE_VERSION_CURRENT] = info.currentVersion;
        this.attrs.info[ZxChat.KEY_INFO_ZIMLET_VERSION] = zimletVersion;
        this.attrs.info[ZxChat.KEY_INFO_ZIMLET_FOUND] = (zimletVersion !== 'None');
        this.attrs.info[ZxChat.KEY_INFO_ZIMLET_AVAILABLE_VERSION] = info.zxchatZimletAvailableVersion;
        this._applyUpdateInfo(info);
      },
      this
    ),
    this._getDefaultCatchErrorFcn()
  );

};

/**
 * Apply update information to the instance object.
 * @param {{}} info
 */
ZxChat.prototype._applyUpdateInfo = function(info) {
  var controller = this.getCurrentController(),
    zimletVersion;

  zimletVersion = info.zxchatZimletCurrentVersion;
  if (zimletVersion === '0.0' || zimletVersion === '') {
    zimletVersion = 'None';
  }

  controller.setInstanceValue(
    ZxChat.KEY_CORE_VERSION_MINIMUM,
    info.minCoreVersion
  );

  controller.setInstanceValue(
    ZxChat.KEY_CORE_VERSION_CURRENT,
    info.currentVersion
  );

  controller.setInstanceValue(
    'info/' + ZxChat.KEY_INFO_ZIMLET_VERSION,
    zimletVersion
  );

  controller.setInstanceValue(
    'info/' + ZxChat.KEY_INFO_ZIMLET_FOUND,
    (zimletVersion !== 'None')
  );

  controller.setInstanceValue(
    'info/' + ZxChat.KEY_INFO_ZIMLET_AVAILABLE_VERSION,
    info.zxchatZimletAvailableVersion
  );
};

/**
 * Trigger a check update for the chat zimlet.
 * @param {DwtSelectionEvent} ev
 * @this {Dwt_Button_XFormItem}
 */
ZxChat.doChekUpdates = function(ev) {
  var server = this.getInstance().id;
  ZeXtras.getModuleItem('ZxChat', server)._doCheckUpdates();
};

/**
 * Chech the updates for the ZxChat system.
 */
ZxChat.prototype._doCheckUpdates = function() {
  var controller = this.getCurrentController(),
    loadingDialog = new ZxCheckUpdateDialog(
      controller.getShell(),
      this,
      new AjxCallback(this, function(resp) {
        this._applyUpdateInfo(resp);
      })
    );
  loadingDialog.popup();
};

/**
 * Deploy the latest chat zimlet.
 * @param {DwtSelectionEvent} ev
 * @this {Dwt_Button_XFormItem}
 */
ZxChat.doDeployLatestZimlet = function(ev) {
  var server = this.getInstance().id;
  ZeXtras.getModuleItem('ZxChat', server)._doDeployLatestZimlet();
};

/**
 * Deploy the latest zimlet version for ZxChat
 */
ZxChat.prototype._doDeployLatestZimlet = function() {
  var controller = this.getCurrentController(),
    currentVersion = this.attrs.info[ZxChat.KEY_INFO_ZIMLET_VERSION],
    deployingVersion = this.attrs.info[ZxChat.KEY_INFO_ZIMLET_AVAILABLE_VERSION],
    requiredZimletVersion = this.attrs[ZxChat.KEY_REQUIRED_ZIMLET],
    msgDialog,
    isCurrentCoreVersion_Oldest_MinimumCoreVersion,
    depDialog = new ZxDeployChatZimletDialog(
      controller.getShell(),
      null,
      '100px',
      zxMsg('LBL_CHT_deploy_latest_zimlet_version'),
      {}
    );

  /**
   * Compare version in format major.minor.micro[.patch].
   * Can return:
   *  * false if a[ZxChat.KEY_CORE_VERSION_MINIMUM] > b[ZxChat.KEY_CORE_VERSION_CURRENT]
   *  * true if a[ZxChat.KEY_CORE_VERSION_MINIMUM] <= b[ZxChat.KEY_CORE_VERSION_CURRENT]
   *
   * @param {string} a
   * @return {boolean}
   */
  isCurrentCoreVersion_Oldest_MinimumCoreVersion = function(a) {
    return ZxUtils.compareVersions(a[ZxChat.KEY_CORE_VERSION_MINIMUM], a[ZxChat.KEY_CORE_VERSION_CURRENT]) < 0;
  };

  currentVersion = (currentVersion === 'None') ? requiredZimletVersion : currentVersion;

  if (ZxUtils.compareVersions(requiredZimletVersion, deployingVersion) < 0) {
    msgDialog = new DwtMessageDialog({
      buttons: [DwtDialog.OK_BUTTON],
      title: '',
      parent: controller.getShell()
    });
    msgDialog.setMessage(
      zxMsg('MSG_CHT_trying_to_deploy_not_compatible', [requiredZimletVersion]),
      DwtMessageDialog.WARNING_STYLE,
      ZxUtils.getTitleWithIcon('ZxChat', 'ZxChat')
    );
    msgDialog.popup();

  } else if (ZxUtils.compareVersions(deployingVersion, currentVersion) > 0) {
    msgDialog = new DwtMessageDialog({
      buttons: [DwtDialog.YES_BUTTON, DwtDialog.NO_BUTTON],
      title: '',
      parent: controller.getShell()
    });
    msgDialog.setMessage(
      zxMsg('MSG_CHT_trying_to_deploy_older_zimlet'),
      DwtMessageDialog.INFO_STYLE,
      ZxUtils.getTitleWithIcon('ZxChat', 'ZxChat')
    );
    msgDialog.registerCallback(DwtDialog.YES_BUTTON, new AjxCallback(
      this,
      function() {
        depDialog.deployZimlet(this.getCurrentServer(), this);
        msgDialog.popdown();
      }
    ));
    msgDialog.popup();

  } else if (isCurrentCoreVersion_Oldest_MinimumCoreVersion(this.attrs)) {

    msgDialog = new DwtMessageDialog({
      buttons: [DwtDialog.DISMISS_BUTTON],
      title: '',
      parent: controller.getShell()
    });
    msgDialog.setMessage(zxMsg('MSG_CHT_zimlet_update_require_core_update'));
    msgDialog.popup();

  } else {
    depDialog.deployZimlet(this.getCurrentServer(), this);
  }
};

/**
 * Connect every user of the group to each other.
 * @param {DwtSelectionEvent} ev
 * @this {Dwt_Button_XFormItem}
 */
ZxChat.doConnectGroup = function(ev) {
  var server = this.getInstance().id,
    groupName = this.getInstanceValue('attr/' + ZxChat.KEY_TMP_DISTRIBUTION_LIST_NAME);
  if (!groupName) {
    return;
  }
  ZeXtras.getModuleItem('ZxChat', server)._doConnectGroup(groupName);
  this.setInstanceValue('', 'attr/' + ZxChat.KEY_TMP_DISTRIBUTION_LIST_NAME);
};

/**
 * Real function of connecting user's group.
 * @param {string} groupName
 */
ZxChat.prototype._doConnectGroup = function(groupName) {
  var server = this.getCurrentServer(),
    waitingDialog = new ZxWaitingDialog(
      this.getCurrentController().getShell(),
      '250px',
      '52px',
      zxMsg('LBL_Saving_settings'),
      ZxLinks.HELP_CHT_Chat
    );

  ZxAdminRequest.sendRequest(
    server,
    'ZxChat',
    'doMakeBuddies',
    {
      list: groupName
    }
  ).then(
    AjxCallback.simpleClosure(
      function(server, waitingDialog, resp) {
        var message = [resp.response.message],
          skipped = resp.response.skipped,
          i;

        if (skipped.length > 0 && false) {
          // TODO: WTF i have received?
          message.push(zxMsg('LBL_CHT_account_skipped'));
          message.push('<br>');
          message.push('<ul>');
          for (i = 0; i < skipped.length; i = i + 1) {
            message.push('<li>');
            message.push(skipped[i]);
            message.push('</li>');
          }
          message.push('</ul>');
        }
        waitingDialog.setMessage(message, ZxWaitingDialog.STATUS_SUCCESS);
      },
      this,
      server,
      waitingDialog
    ),
    AjxCallback.simpleClosure(
      waitingDialog.catchError,
      waitingDialog
    )
  );
  waitingDialog.popup();
};

/**
 * Connect two users to each other.
 * @param {DwtSelectionEvent} ev
 * @this {Dwt_Button_XFormItem}
 */
ZxChat.doConnectBuddies = function(ev) {
  var server = this.getInstance().id,
    user1 = this.getInstanceValue('attr/' + ZxChat.KEY_TMP_FIRST_USER),
    user2 = this.getInstanceValue('attr/' + ZxChat.KEY_TMP_SECOND_USER);
  if (!user1 || !user2) {
    return;
  }
  ZeXtras.getModuleItem('ZxChat', server)._doConnectBuddies(user1, user2);
  this.setInstanceValue('', 'attr/' + ZxChat.KEY_TMP_FIRST_USER);
  this.setInstanceValue('', 'attr/' + ZxChat.KEY_TMP_SECOND_USER);
};

/**
 * Real function of connecting two users to each other.
 * @param {string} user1
 * @param {string} user2
 */
ZxChat.prototype._doConnectBuddies = function(user1, user2) {
  var server = this.getCurrentServer(),
    waitingDialog = new ZxWaitingDialog(
      this.getCurrentController().getShell(),
      '400px',
      '80px',
      zxMsg('LBL_Saving_settings'),
      ZxLinks.HELP_CHT_Chat
    );

  ZxAdminRequest.sendRequest(
    server,
    'ZxChat',
    'doAddBuddies',
    {
      user1: user1,
      user2: user2
    }
  ).then(
    AjxCallback.simpleClosure(
      function(waitingDialog, resp) {
        waitingDialog.setMessage(resp.message, ZxWaitingDialog.STATUS_SUCCESS);
      },
      this,
      waitingDialog
    ),
    AjxCallback.simpleClosure(
      waitingDialog.catchError,
      waitingDialog
    )
  );
  waitingDialog.popup();
};

/**
 * Check if the zimlet is enabled in the server.
 * @return {boolean}
 */
ZxChat.checkZimletEnabled = function() {
  return this.getInstanceValue('attrs/info/' + ZxChat.KEY_ZIMLET_ENABLED);
};

/**
 * Check if the zimlet is not enabled in the server.
 * @return {boolean}
 */
ZxChat.checkZimletNotEnabled = function() {
  return !this.getInstanceValue('attrs/info/' + ZxChat.KEY_ZIMLET_ENABLED);
};


/**
 * Return if the zimlet chat is installed on the server.
 * @return {boolean}
 */
ZxChat.checkZimletInstalled = function() {
  return this.getInstanceValue('attrs/info/' + ZxChat.KEY_ZIMLET_INSTALLED);
};

/**
 * Return if the zimlet chat is not installed on the server.
 * @return {boolean}
 */
ZxChat.checkZimletNotInstalled = function() {
  return !this.getInstanceValue('attrs/info/' + ZxChat.KEY_ZIMLET_INSTALLED);
};

ZxChat._getBuddyOrderPrefChoices = function() {
  return [
    {value: 'name', label: zxMsg('add_friends_username')},
    {value: 'presence', label: zxMsg('presence')}
  ];
};

ZxChat._getIdlePrefChoices = function() {
  return [
    {value: 1,  label: zxMsg('LBL_1_minute')},
    {value: 5,  label: zxMsg('LBL_5_minutes')},
    {value: 10, label: zxMsg('LBL_10_minutes')},
    {value: 20, label: zxMsg('LBL_20_minutes')},
    {value: 30, label: zxMsg('LBL_30_minutes')},
    {value: 60, label: zxMsg('LBL_60_minutes')}
  ];
};

/**
 * Get if a module meet the requirements to allow the user to remove the brand.
 * @return {boolean}
 */
ZxChat.canRemoveBrand = function() {
  var license = ZeXtras.getLicense();
  if (license.isTrial()) {
    return false;
  }
  return license.canRemoveChatBrand();
};
