/*
 * ***** BEGIN LICENSE BLOCK *****
 * Copyright (C) 2011-2016 ZeXtras
 *
 * The contents of this file are subject to the ZeXtras EULA;
 * you may not use this file except in compliance with the EULA.
 * You may obtain a copy of the EULA at
 * http://www.zextras.com/zextras-eula.html
 * ***** END LICENSE BLOCK *****
 */

function ZxChatAccountModifier() {}

if (ZaAccount) {
  ZaAccount.A_zimbraPrefIMBuddyListSort = 'zimbraPrefIMBuddyListSort';
  ZaAccount.A_zimbraPrefIMSoundsEnabled = 'zimbraPrefIMSoundsEnabled';
  ZaAccount.A_zimbraPrefIMFlashTitle = 'zimbraPrefIMFlashTitle';
  ZaAccount.A_zimbraPrefIMToasterEnabled = 'zimbraPrefIMToasterEnabled';
  ZaAccount.A_zimbraPrefIMReportIdle = 'zimbraPrefIMReportIdle';
  ZaAccount.A_zimbraPrefIMIdleTimeout = 'zimbraPrefIMIdleTimeout';
}

ZxChatAccountModifier.NEW_PANEL = false;

ZxChatAccountModifier.MODEL = [
  {type: _STRING_, id: ZxChat.KEY_USR_ZIMLET_STATUS,
    ref: 'attrs/' + ZxChat.KEY_USR_ZIMLET_STATUS,
    isZX: true},
  {id: ZaAccount.A_zimbraPrefIMBuddyListSort,
    type: _COS_ENUM_, ref: 'attrs/' + ZaAccount.A_zimbraPrefIMBuddyListSort,
    choices: ZxChat._getBuddyOrderPrefChoices()},
  {id: ZaAccount.A_zimbraPrefIMSoundsEnabled,
    type: _COS_ENUM_, ref: 'attrs/' + ZaAccount.A_zimbraPrefIMSoundsEnabled,
    choices: ZaModel.BOOLEAN_CHOICES},
  {id: ZaAccount.A_zimbraPrefIMFlashTitle,
    type: _COS_ENUM_, ref: 'attrs/' + ZaAccount.A_zimbraPrefIMFlashTitle,
    choices: ZaModel.BOOLEAN_CHOICES},
  {id: ZaAccount.A_zimbraPrefIMToasterEnabled,
    type: _COS_ENUM_, ref: 'attrs/' + ZaAccount.A_zimbraPrefIMToasterEnabled,
    choices: ZaModel.BOOLEAN_CHOICES},
  {id: ZaAccount.A_zimbraPrefIMReportIdle,
    type: _COS_ENUM_, ref: 'attrs/' + ZaAccount.A_zimbraPrefIMReportIdle,
    choices: ZaModel.BOOLEAN_CHOICES},
  {id: ZaAccount.A_zimbraPrefIMIdleTimeout,
    type: _COS_ENUM_, ref: 'attrs/' + ZaAccount.A_zimbraPrefIMIdleTimeout,
    choices: ZxChat._getIdlePrefChoices()}
];

ZxChatAccountModifier.ATTRS = [
  ZxChat.KEY_USR_ZIMLET_STATUS
];
ZxChatAccountModifier.RIGHTS = [];

/**
 * Function invoked by the ZxAccountModifier when the tab is
 * opened.
 * @this {ZaAccount}
 */
ZxChatAccountModifier.LOAD_ATTRS = function() {
  var mailHost;

  if (this.attrs[ZaAccount.A_zimbraIsSystemAccount] === 'TRUE' ||
      this.attrs[ZaAccount.A_zimbraIsSystemResource] === 'TRUE') {

    this.attrs[ZxChat.KEY_USR_ZIMLET_STATUS] =
      zxMsg('LBL_CHT_service_not_available_for_resources');
  }

  this.attrs[ZxChat.KEY_USR_ZIMLET_STATUS] = ZaMsg.splashScreenLoading;

  mailHost = this.attrs[ZaAccount.A_mailHost];
  ZxAdminRequest.sendRequest(
    mailHost,
    'ZxChat',
    'getServerStatus'
  ).then(AjxCallback.simpleClosure(
    function(server, resp) {
      var response = resp.response,
        serverRunning = false,
        message = [];

      if (response.server_status === 'running') {
        serverRunning = true;
      }

      if (serverRunning) {
        message.push(
          zxMsg('MSG_CHT_server_running')
        );
      } else {
        message.push(
          zxMsg('MSG_CHT_server_not_running')
        );
      }

      ZxFormModifier.setInstanceValue(
        this,
        message.join(''),
        'attrs/' + ZxChat.KEY_USR_ZIMLET_STATUS
      );
    },
    this,
    mailHost
  ), ZxUtils.catchError);
};

ZxChatAccountModifier.SAVE_ATTRS = function(prop, val) {
  ZxUtils.log({prop: prop, val: val}, 'Saving Value:');
};

ZxChatAccountModifier.XFORM = {
  type: _ZA_TOP_GROUPER_,
  numCols: 2,
  width: '100%',
  colSizes: ['160px', '*'],
  label: zxMsg('LBL_CHT_chat_preferences'),
  items: [
    {type: _OUTPUT_, height: 120, cssClass: 'ZxChatLogo'},
    {
      type: _GROUP_,
      colSpan: 2,
      numCols: 1,
      colSizes: ['*'],
      items: [
        {
          type: _ZAWIZ_TOP_GROUPER_,
          label: zxMsg('LBL_Status'),
          colSpan: 2,
          numCols: 2,
          width: '100%',
          colSizes: [275, '*'],
          items: [
            {
              type: _OUTPUT_,
              label: zxMsg('LBL_CHT_System_status') + ':',
              ref: 'attrs/' + ZxChat.KEY_USR_ZIMLET_STATUS,
              bmolsnr: true
            },
            {
              type: _OUTPUT_,
              dataType: _STRING_,
              label: zxMsg('LBL_CHT_Zimlet_status') + ':<br>&nbsp;',
              ref: ZaAccount.A_zimbraZimletAvailableZimlets,
              getDisplayValue: function(list) {
                return ZxChatAccountModifier.getZimletStatusMessage(
                  list,
                  this.getInstanceValue('attrs/info/' + ZxChat.KEY_ZIMLET_INSTALLED),
                  this.getInstanceValue('attrs/info/' + ZxChat.KEY_ZIMLET_ENABLED)
                );
              }
            }

          ]
        },
        {
          type: _ZAWIZ_TOP_GROUPER_,
          label: zxMsg('pref_title_buddy_list'),
          colSpan: 2,
          numCols: 2,
          width: '100%',
          colSizes: [275, '*'],
          items: [
            {
              type: _SUPER_SELECT1_,
              label: zxMsg('pref_title_display_order'),
              resetToSuperLabel: ZaMsg.NAD_ResetToCOS,
              ref: ZaAccount.A_zimbraPrefIMBuddyListSort
            }
          ]
        },
        {
          type: _ZAWIZ_TOP_GROUPER_,
          label: zxMsg('pref_title_notifications'),
          colSpan: 2,
          numCols: 2,
          width: '100%',
          colSizes: [275, '*'],
          items: [
            {
              type: _SUPER_CHECKBOX_,
              checkBoxLabel: zxMsg('pref_title_play_sound'),
              resetToSuperLabel: ZaMsg.NAD_ResetToCOS,
              ref: ZaAccount.A_zimbraPrefIMSoundsEnabled,
              trueValue: 'TRUE',
              falseValue: 'FALSE'
            },
            {
              type: _SUPER_CHECKBOX_,
              checkBoxLabel: zxMsg('pref_title_flash_browser_title'),
              resetToSuperLabel: ZaMsg.NAD_ResetToCOS,
              ref: ZaAccount.A_zimbraPrefIMFlashTitle,
              trueValue: 'TRUE',
              falseValue: 'FALSE'
            },
            {
              type: _SUPER_CHECKBOX_,
              checkBoxLabel: zxMsg('pref_title_desktop_notification_text'),
              resetToSuperLabel: ZaMsg.NAD_ResetToCOS,
              ref: ZaAccount.A_zimbraPrefIMToasterEnabled,
              trueValue: 'TRUE',
              falseValue: 'FALSE'
            }
          ]
        },
        {
          type: _ZAWIZ_TOP_GROUPER_,
          label: zxMsg('pref_title_idle'),
          colSpan: 2,
          numCols: 2,
          width: '100%',
          colSizes: [275, '*'],
          items: [
            {
              type: _SUPER_CHECKBOX_,
              checkBoxLabel: zxMsg('pref_title_idle_enabled'),
              ref: ZaAccount.A_zimbraPrefIMReportIdle,
              resetToSuperLabel: ZaMsg.NAD_ResetToCOS,
              trueValue: 'TRUE',
              falseValue: 'FALSE'
            },
            {
              type: _SUPER_SELECT1_,
              label: zxMsg('pref_title_idle_timeout'),
              colSpan: 2,
              resetToSuperLabel: ZaMsg.NAD_ResetToCOS,
              ref: ZaAccount.A_zimbraPrefIMIdleTimeout
            }
          ]
        }
      ]
    }
  ]
};

/**
 * Get the message to notify the status of the ZxChat zimlet for the user.
 * @param {?string[]} list of the user's zimlets.
 * @param {boolean} isZimletInstalled
 * @param {boolean} isZimletEnabled
 * @return {string}
 */
ZxChatAccountModifier.getZimletStatusMessage = function(list, isZimletInstalled, isZimletEnabled) {
  if (!list) {
    return ZaMsg.splashScreenLoading;
  }

  var i,
    name,
    message = zxMsg('MSG_CHT_zimlet_not_available_for_user'),
    found = false;

  if (isZimletInstalled) {
    if (isZimletEnabled) {
      message = zxMsg('MSG_CHT_zimlet_installed') + '<br>';
    } else {
      message = zxMsg('MSG_CHT_zimlet_installed_not_enabled') + '<br>';
    }
  } else {
    message = zxMsg('MSG_CHT_zimlet_not_installed') + '<br>';
  }

  // Check if zimlet is enabled for the user
  for (i = 0; i < list.length; i = i + 1) {
    name = list[i];
    if (name.substring(1) === ZxChat.ZIMLET_NAME) {
      found = true;
      switch (name.substring(0, 1)) {

      case ZxChat.ZIMLET_MANDATORY:
      case ZxChat.ZIMLET_ENABLED:
          message += zxMsg('MSG_CHT_zimlet_enabled_for_user');
          break;
      case ZxChat.ZIMLET_DISABLED:
          message += zxMsg('MSG_CHT_zimlet_not_enabled_for_user');
          break;
      }
    }
  }

  if (!found) {
    message += '&nbsp;';
  }

  return message;
};
