/*
 * ***** BEGIN LICENSE BLOCK *****
 * Copyright (C) 2011-2016 ZeXtras
 *
 * The contents of this file are subject to the ZeXtras EULA;
 * you may not use this file except in compliance with the EULA.
 * You may obtain a copy of the EULA at
 * http://www.zextras.com/zextras-eula.html
 * ***** END LICENSE BLOCK *****
 */

function ZxChatCOSModifier() {}

if (ZaCos) {
  ZaCos.A_zimbraPrefIMBuddyListSort = 'zimbraPrefIMBuddyListSort';
  ZaCos.A_zimbraPrefIMSoundsEnabled = 'zimbraPrefIMSoundsEnabled';
  ZaCos.A_zimbraPrefIMFlashTitle = 'zimbraPrefIMFlashTitle';
  ZaCos.A_zimbraPrefIMToasterEnabled = 'zimbraPrefIMToasterEnabled';
  ZaCos.A_zimbraPrefIMReportIdle = 'zimbraPrefIMReportIdle';
  ZaCos.A_zimbraPrefIMIdleTimeout = 'zimbraPrefIMIdleTimeout';
}

ZxChatCOSModifier.NEW_PANEL = false;

ZxChatCOSModifier.MODEL = [
  {id: ZaCos.A_zimbraPrefIMBuddyListSort,
    type: _ENUM_, ref: 'attrs/' + ZaCos.A_zimbraPrefIMBuddyListSort,
    choices: ZxChat._getBuddyOrderPrefChoices()},
  {id: ZaCos.A_zimbraPrefIMSoundsEnabled,
    type: _ENUM_, ref: 'attrs/' + ZaCos.A_zimbraPrefIMSoundsEnabled,
    choices: ZaModel.BOOLEAN_CHOICES},
  {id: ZaCos.A_zimbraPrefIMFlashTitle,
    type: _ENUM_, ref: 'attrs/' + ZaCos.A_zimbraPrefIMFlashTitle,
    choices: ZaModel.BOOLEAN_CHOICES},
  {id: ZaCos.A_zimbraPrefIMToasterEnabled,
    type: _ENUM_, ref: 'attrs/' + ZaCos.A_zimbraPrefIMToasterEnabled,
    choices: ZaModel.BOOLEAN_CHOICES},
  {id: ZaCos.A_zimbraPrefIMReportIdle,
    type: _ENUM_, ref: 'attrs/' + ZaCos.A_zimbraPrefIMReportIdle,
    choices: ZaModel.BOOLEAN_CHOICES},
  {id: ZaCos.A_zimbraPrefIMIdleTimeout,
    type: _ENUM_, ref: 'attrs/' + ZaCos.A_zimbraPrefIMIdleTimeout,
    choices: ZxChat._getIdlePrefChoices()}
];

ZxChatCOSModifier.ATTRS = [];
ZxChatCOSModifier.RIGHTS = [];

/**
 * Function invoked by the ZxAccountModifier when the tab is
 * opened.
 * @this {ZaCOS}
 */
ZxChatCOSModifier.LOAD_ATTRS = function() {
};

ZxChatCOSModifier.SAVE_ATTRS = function(prop, val) {
  ZxUtils.log({prop: prop, val: val}, 'Saving Value:');
};

ZxChatCOSModifier.XFORM = {
  type: _ZA_TOP_GROUPER_,
  numCols: 2,
  width: '100%',
  colSizes: ['160px', '*'],
  label: zxMsg('LBL_CHT_chat_preferences'),
  items: [
    {type: _OUTPUT_, height: 120, cssClass: 'ZxChatLogo'},
    {
      type: _GROUP_,
      colSpan: 2,
      numCols: 1,
      colSizes: ['*'],
      items: [
        {
          type: _ZAWIZ_TOP_GROUPER_,
          label: zxMsg('LBL_Status'),
          colSpan: 2,
          numCols: 2,
          width: '100%',
          colSizes: [275, '*'],
          items: [
            {
              type: _OUTPUT_,
              dataType: _STRING_,
              label: zxMsg('LBL_CHT_Zimlet_status') + ':<br>&nbsp;',
              ref: ZaCos.A_zimbraZimletAvailableZimlets,
              getDisplayValue: function(list) {
                return ZxChatCOSModifier.getZimletStatusMessage(
                  list,
                  this.getInstanceValue('attrs/info/' + ZxChat.KEY_ZIMLET_INSTALLED),
                  this.getInstanceValue('attrs/info/' + ZxChat.KEY_ZIMLET_ENABLED)
                );
              }
            }
          ]
        },
        {
          type: _ZAWIZ_TOP_GROUPER_,
          label: zxMsg('pref_title_buddy_list'),
          colSpan: 2,
          numCols: 2,
          width: '100%',
          colSizes: [275, '*'],
          items: [
            {
              type: _SELECT1_,
              label: zxMsg('pref_title_display_order'),
              ref: ZaCos.A_zimbraPrefIMBuddyListSort
            }
          ]
        },
        {
          type: _ZAWIZ_TOP_GROUPER_,
          label: zxMsg('pref_title_notifications'),
          colSpan: 2,
          numCols: 2,
          width: '100%',
          colSizes: [275, '*'],
          items: [
            {
              type: _CHECKBOX_,
              label: zxMsg('pref_title_play_sound'),
              ref: ZaCos.A_zimbraPrefIMSoundsEnabled,
              trueValue: 'TRUE',
              falseValue: 'FALSE'
            },
            {
              type: _CHECKBOX_,
              label: zxMsg('pref_title_flash_browser_title'),
              ref: ZaCos.A_zimbraPrefIMFlashTitle,
              trueValue: 'TRUE',
              falseValue: 'FALSE'
            },
            {
              type: _CHECKBOX_,
              label: zxMsg('pref_title_desktop_notification_text'),
              ref: ZaCos.A_zimbraPrefIMToasterEnabled,
              trueValue: 'TRUE',
              falseValue: 'FALSE'
            }
          ]
        },
        {
          type: _ZAWIZ_TOP_GROUPER_,
          label: zxMsg('pref_title_idle'),
          colSpan: 2,
          numCols: 2,
          width: '100%',
          colSizes: [275, '*'],
          items: [
            {
              type: _CHECKBOX_,
              label: zxMsg('pref_title_idle_enabled'),
              ref: ZaCos.A_zimbraPrefIMReportIdle,
              trueValue: 'TRUE',
              falseValue: 'FALSE'
            },
            {
              type: _SELECT1_,
              label: zxMsg('pref_title_idle_timeout'),
              ref: ZaCos.A_zimbraPrefIMIdleTimeout
            }
          ]
        }
      ]
    }
  ]
};

/**
 * Get the message to notify the status of the ZxChat zimlet for the COS.
 * @param {?string[]} list of the COS's zimlets.
 * @param {boolean} isZimletInstalled
 * @param {boolean} isZimletEnabled
 * @return {string}
 */
ZxChatCOSModifier.getZimletStatusMessage = function(list, isZimletInstalled, isZimletEnabled) {
  if (!list) {
    return ZaMsg.splashScreenLoading;
  }

  var i,
    name,
    message = zxMsg('MSG_CHT_zimlet_not_available_for_cos'),
    found = false;

  if (isZimletInstalled) {
    if (isZimletEnabled) {
      message = zxMsg('MSG_CHT_zimlet_installed') + '<br>';
    } else {
      message = zxMsg('MSG_CHT_zimlet_installed_not_enabled') + '<br>';
    }
  } else {
    message = zxMsg('MSG_CHT_zimlet_not_installed') + '<br>';
  }

  // Check if zimlet is enabled for the user
  for (i = 0; i < list.length; i = i + 1) {
    name = list[i];
    if (name.substring(1) === ZxChat.ZIMLET_NAME) {
      found = true;
      switch (name.substring(0, 1)) {

      case ZxChat.ZIMLET_MANDATORY:
      case ZxChat.ZIMLET_ENABLED:
          message += zxMsg('MSG_CHT_zimlet_enabled_for_cos');
          break;
      case ZxChat.ZIMLET_DISABLED:
          message += zxMsg('MSG_CHT_zimlet_not_enabled_for_cos');
          break;
      }
    }
  }

  if (!found) {
    message += '&nbsp;';
  }

  return message;
};
