/*
 * ***** BEGIN LICENSE BLOCK *****
 * Copyright (C) 2011-2016 ZeXtras
 *
 * The contents of this file are subject to the ZeXtras EULA;
 * you may not use this file except in compliance with the EULA.
 * You may obtain a copy of the EULA at
 * http://www.zextras.com/zextras-eula.html
 * ***** END LICENSE BLOCK *****
 */
/**
 * Dialog that show deploy status of the ZxChat window during the deploy process.
 * @param {DwtShell} parent
 * @param {number} w
 * @param {number} h
 * @param {String} title
 * @param {{}} params
 * @constructor
 */
function ZxDeployChatZimletDialog(parent, w, h, title, params) {
  var closeButton = new DwtDialog_ButtonDescriptor(
    'closeButtonId',
    'Close',
    DwtDialog.ALIGN_RIGHT,
    new AjxCallback(
      this,
      function() {
        this.popdown();
      }
    )
  );

  this._standardButtons = [DwtDialog.NO_BUTTONS];
  this._extraButtons = [closeButton];

  ZaXDialog.call(this, parent, null, title, w, h, null, null);
  this.initForm([], this.getMyXForm('pending'));
  this.setObject(params);
  this.setSize(null, w);

  this._helpURL = ZxLinks.HELP_CHT_Dialog_Deploy_Zimlet;
}

ZxDeployChatZimletDialog.prototype = new ZaXDialog();
ZxDeployChatZimletDialog.prototype.constructor = ZxDeployChatZimletDialog;

/**
 * Send the request to start the deploy of the zimlet.
 * @param {ZxChat} module to refresh once the zimlet is deployed
 * @param {string} server
 */
ZxDeployChatZimletDialog.prototype.deployZimlet = function(server, module) {
  this.popup();
  ZxAdminRequest.sendRequest(
    server,
    'ZxChat',
    'doDeployZimlet'
  ).then(AjxCallback.simpleClosure(
    function(module, resp) {
      module.fireLoadMethods();
      this.initForm([], this.getMyXForm('success'));
    },
    this,
    module
  ), AjxCallback.simpleClosure(
    function(error) {
      this.catchError(error);
    },
    this
  ));
};

/**
 * Return the XFormItem for the dialog.
 * @param {string} state 'pending|success|error'.
 * @param {string} message
 * @return {{numCols: number, colSizes: Array, items: Array}}
 */
ZxDeployChatZimletDialog.prototype.getMyXForm = function(state, message) {
  var html, xFormObject;

  if (state === 'pending') {
    html = {
      type: _OUTPUT_,
      id: 'deployStatusMessageId',
      value: ['<table cellspacing="0" cellpadding="0" border="0" width="100%" height="100%"><tr><td valign="top">',
        AjxImg.getImageHtml(
          DwtMessageDialog.ICON[DwtMessageDialog.INFO_STYLE],
          null,
          'id="deployStatusMessageIconId"'
        ),
        '</td><td class="DwtMsgArea" id="', this._msgCellId, '_Msg">',
        zxMsg('MSG_CHT_deploying_zimlet'),
        '</td></tr></table>'].join('')
    };

  } else if (state === 'success') {
    html = {
      type: _OUTPUT_,
      id: 'deployStatusMessageId',
      value: ['<table cellspacing="0" cellpadding="0" border="0" width="100%" height="100%"><tr><td valign="top">',
        AjxImg.getImageHtml(
          DwtMessageDialog.ICON[DwtMessageDialog.INFO_STYLE],
          null,
          'id="deployStatusMessageIconId"'
        ),
        '</td><td class="DwtMsgArea" id="', this._msgCellId, '_Msg">',
        zxMsg('MSG_CHT_deploy_zimlet_successfull'),
        '</td></tr></table>'].join('')
    };

  } else { //failure
    html = {
      type: _OUTPUT_,
      id: 'deployStatusMessageId',
      value: ['<table cellspacing="0" cellpadding="0" border="0" width="100%" height="100%"><tr><td valign="top">',
        AjxImg.getImageHtml(
          DwtMessageDialog.ICON[DwtMessageDialog.WARNING_STYLE],
          null,
          'id="deployStatusMessageIconId"'
        ),
        '</td><td class="DwtMsgArea" id="', this._msgCellId, '_Msg">',
        zxMsg('MSG_CHT_deploy_zimlet_error', ['<br>', message]),
        '</td></tr></table>'].join('')
    };
  }

  xFormObject = {
    numCols: 1,
    colSizes: ['100%'],
    items: [
      {type: _SPACER_, height: 10},
      {type: _GROUP_,
        numCols: 1,
        colSizes: ['*'],
        items: [html]}
    ]
  };

  return xFormObject;
};

ZxDeployChatZimletDialog.prototype.catchError = function(error) {
  this.popdown();
  ZxUtils.catchError(error);
};
