/*
 * ***** BEGIN LICENSE BLOCK *****
 * Copyright (C) 2011-2016 ZeXtras
 *
 * The contents of this file are subject to the ZeXtras EULA;
 * you may not use this file except in compliance with the EULA.
 * You may obtain a copy of the EULA at
 * http://www.zextras.com/zextras-eula.html
 * ***** END LICENSE BLOCK *****
 */

/**
 * XForm For ZxCore Module
 * @type {[]}
 */
var ZxCoreXFormItem = [
  {
    type: _GROUP_,
    cssClass: 'ZmSelectedHeaderBg',
    colSpan: '*',
    id: 'xform_header',
    items: [
      {
        type: _GROUP_,
        cssClass: 'ZxCoreSpacer',
        numCols: 3,
        width: '100%',
        colSizes: ['160px', '*', '200px'],
        items: [
          { type: _CELL_SPACER_ },
          {
            type: _OUTPUT_,
            ref: 'name',
            valign: _TOP_,
            cssClass: 'AdminTitle'
          },
          {
            type: _GROUP_,
            width: 200,
            numCols: 1,
            items: [
              {
                type: _DWT_BUTTON_,
                icon: 'RightObject',
                width: 200,
                align: _LEFT_,
                label: zxMsg('LBL_COR_client_errors'),
                toolTipContent: zxMsg('LBL_COR_client_errors'),
                onActivate: ZxCore.displayClientErrors,
                visibilityChecks: [function() { return false; } /*, ZxError.hasCaughtErrors*/]
              },
              { type: _CELL_SPACER_ }
            ]
          }
        ]
      }
    ]
  },
  {
    type: _GROUP_,
    numCols: 1,
    colSpan: 2,
    width: '100%',
    colSizes: ['100%'],
    items: [
      {
        type: _DWT_ALERT_,
        cssClass: 'DwtTabTable',
        visibilityChangeEventSources: ['attrs/license/' + ZxCore.KEY_LICENSE_STATUS],
        valueChangeEventSources: ['attrs/license/' + ZxCore.KEY_LICENSE_STATUS],
        visibilityChecks: [function() {
          var server = this.getInstance().name,
            lic = ZeXtras.getLicense();
          return (lic.isNotYetValid(server) || lic.isExpired(server) ||
                  lic.isAboutToExpire(server) || lic.isTrial(server)) &&
                  !lic.isEmptyLicense(server) && !lic.isPerpetualV2(server);
        }],
        containerCssStyle: 'font-weight: bolder;color:red;padding-bottom:0px',
        style: DwtAlert.WARNING,
        iconVisible: true,
        getDisplayValue: function(newValue) {
          var server = this.getInstance().name;
          return ZeXtras.getLicense().getWarningText(server);
        },
        colSpan: '*'
      },
      {
        type: _DWT_ALERT_,
        cssClass: 'DwtTabTable',
        visibilityChangeEventSources: ['attrs/license/' + ZxCore.KEY_LICENSE_STATUS],
        valueChangeEventSources: ['attrs/license/' + ZxCore.KEY_LICENSE_STATUS],
        visibilityChecks: [function() {
          var server = this.getInstance().name;
          return ZeXtras.getLicense().isUserExceeded(server);
        }],
        containerCssStyle: 'font-weight: bolder;color:red;padding-bottom:0px',
        style: DwtAlert.CRITICAL,
        iconVisible: true,
        getDisplayValue: function(newValue) {
          var server = this.getInstance().name;
          return ZeXtras.getLicense().getWarningText(server);
        },
        colSpan: '*'
      },

      {
        type: _DWT_ALERT_,
        cssClass: 'DwtTabTable',
        ref: 'attrs/updateinfo/' + ZxCore.KEY_UPDATE_AVAILABLE,
        visibilityChangeEventSources: ['attrs/updateinfo/' + ZxCore.KEY_UPDATE_AVAILABLE],
        bmolsnr: true,
        getDisplayValue: function(newVal) {
          return zxMsg(
            'MSG_COR_warning_new_suite_available',
            ['<a href="' + ZxLinks.COR_Download_ZeXtras_for_Zimbra + '" target="_blank">',
              '</a>']
          );
        },
        visibilityChecks: [ZxCore.checkNewZextrasVersionAvailableWarning],
        containerCssStyle: 'padding-bottom:0px',
        style: DwtAlert.WARNING,
        iconVisible: true,
        colSpan: '*'
      }
    ]
  }
];

ZxCoreXFormItem.push(
  {
    type: _GROUP_,
    numCols: 1,
    colSizes: ['*'],
    width: '100%',
    tableCssStyle: 'width:100%;overflow:auto;padding-right:15px;padding-left:15px;',
    items: [
      {
        type: _TOP_GROUPER_,
        label: zxMsg('LBL_current_status'),
        colSizes: ['100%'],
        colSpan: 2,
        numCols: 1,
        items: [
          {
            type: _ZAWIZ_TOP_GROUPER_,
            label: zxMsg('LBL_Modules'),
            numCols: 8,
            width: '100%',
            colSizes: ['15%', 100, 100, 20, 150, 20, 150, '*'],
            items: [
              {
                type: _OUTPUT_,
                label: zxMsg('LBL_ZxAdmin') + ': ',
                valueChangeEventSources: [
                  'attrs/' + ZxCore.KEY_RUNNING_MODS + '/' + ZxCore.KEY_LICENSE_ZXADMIN
                ],
                getDisplayValue: function() {
                  return ZxCore.getStatusDisplayValue.call(this, ZxCore.KEY_LICENSE_ZXADMIN);
                },
                cssStyle: 'font-weight:bold;',
                align: _LEFT_,
                labelLocation: _LEFT_
              },
              { type: _CELL_SPACER_, colSpan: 6 },
              {
                type: _OUTPUT_,
                label: zxMsg('LBL_ZxBackup') + ': ',
                valueChangeEventSources: [
                  'attrs/' + ZxCore.KEY_RUNNING_MODS + '/' + ZxCore.KEY_LICENSE_ZXBACKUP
                ],
                getDisplayValue: function() {
                  return ZxCore.getStatusDisplayValue.call(this, ZxCore.KEY_LICENSE_ZXBACKUP);
                },
                cssStyle: 'font-weight:bold;',
                align: _LEFT_,
                labelLocation: _LEFT_
              },
              {
                type: _GROUP_,
                colSizes: [100],
                numCols: 1,
                width: 100,
                items: [
                  {
                    type: _DWT_BUTTON_,
                    label: zxMsg('LBL_Start'),
                    width: 100,
                    icon: 'ToggleStatus',
                    onActivate: ZxCore.doStartBackup,
                    visibilityChangeEventSources: [
                      'attrs/' + ZxCore.KEY_RUNNING_MODS + '/' + ZxCore.KEY_LICENSE_ZXBACKUP
                    ],
                    visibilityChecks: [
                      function() {
                        return !ZxCore.checkRunningModule.call(this, ZxCore.KEY_LICENSE_ZXBACKUP);
                      }
                    ]
                  },
                  {
                    type: _DWT_BUTTON_,
                    label: zxMsg('LBL_Stop'),
                    width: 100,
                    icon: 'ToggleStatus',
                    onActivate: ZxCore.doStopBackup,
                    visibilityChangeEventSources: [
                      'attrs/' + ZxCore.KEY_RUNNING_MODS + '/' + ZxCore.KEY_LICENSE_ZXBACKUP
                    ],
                    visibilityChecks: [
                      function() {
                        return ZxCore.checkRunningModule.call(this, ZxCore.KEY_LICENSE_ZXBACKUP);
                      }
                    ]
                  }
                ]
              },
              {
                type: _CHECKBOX_,
                ref: 'attrs/' + ZxCore.KEY_ENABLED_ON_STARTUP_MODS + '/' + ZxCore.KEY_LICENSE_ZXBACKUP,
                bmolsnr: true,
                labelLocation: _RIGHT_,
                valueChangeEventSources: [
                  'attrs/' + ZxCore.KEY_ENABLED_ON_STARTUP_MODS + '/' + ZxCore.KEY_LICENSE_ZXBACKUP
                ],
                label: zxMsg('LBL_Enabled_on_startup'),
                getDisplayValue: function(val) {
                  return ZxCore.checkEnableModuleOnStartup.call(this, ZxCore.KEY_LICENSE_ZXBACKUP);
                },
                onChange: ZxCore.doEnableDisableBackup
              },
              { type: _CELL_SPACER_, colSpan: 3 },
              {
                type: _OUTPUT_,
                label: zxMsg('LBL_ZxChat') + ': ',
                valueChangeEventSources: [
                  'attrs/' + ZxCore.KEY_RUNNING_MODS + '/' + ZxCore.KEY_LICENSE_ZXCHAT
                ],
                getDisplayValue: function() {
                  return ZxCore.getStatusDisplayValue.call(this, ZxCore.KEY_LICENSE_ZXCHAT);
                },
                cssStyle: 'font-weight:bold;',
                align: _LEFT_,
                labelLocation: _LEFT_,
                visibilityChecks: [ ZeXtrasItem.isZeXtrasSuiteFcn ]
              },
              {
                type: _GROUP_,
                colSizes: [100],
                numCols: 1,
                width: 100,
                visibilityChecks:[ ZeXtrasItem.isZeXtrasSuiteFcn ],
                items: [
                  {
                    type: _DWT_BUTTON_,
                    label: zxMsg('LBL_Start'),
                    width: 100,
                    icon: 'ToggleStatus',
                    onActivate: ZxCore.doStartChat,
                    visibilityChangeEventSources: [
                      'attrs/' + ZxCore.KEY_RUNNING_MODS + '/' + ZxCore.KEY_LICENSE_ZXCHAT
                    ],
                    visibilityChecks: [
                      function() {
                        return !ZxCore.checkRunningModule.call(this, ZxCore.KEY_LICENSE_ZXCHAT);
                      }
                    ]
                  },
                  {
                    type: _DWT_BUTTON_,
                    label: zxMsg('LBL_Stop'),
                    width: 100,
                    icon: 'ToggleStatus',
                    onActivate: ZxCore.doStopChat,
                    visibilityChangeEventSources: [
                      'attrs/' + ZxCore.KEY_RUNNING_MODS + '/' + ZxCore.KEY_LICENSE_ZXCHAT
                    ],
                    visibilityChecks: [
                      function() {
                        return ZxCore.checkRunningModule.call(this, ZxCore.KEY_LICENSE_ZXCHAT);
                      }
                    ]
                  }
                ]
              },
              {
                type: _CHECKBOX_,
                ref: 'attrs/' + ZxCore.KEY_ENABLED_ON_STARTUP_MODS + '/' + ZxCore.KEY_LICENSE_ZXCHAT,
                bmolsnr: true,
                labelLocation: _RIGHT_,
                valueChangeEventSources: [
                  'attrs/' + ZxCore.KEY_ENABLED_ON_STARTUP_MODS + '/' + ZxCore.KEY_LICENSE_ZXCHAT
                ],
                label: zxMsg('LBL_Enabled_on_startup'),
                visibilityChecks: [ ZeXtrasItem.isZeXtrasSuiteFcn ],
                getDisplayValue: function(val) {
                  return ZxCore.checkEnableModuleOnStartup.call(this, ZxCore.KEY_LICENSE_ZXCHAT);
                },
                onChange: ZxCore.doEnableDisableChat
              },
              { type: _CELL_SPACER_, colSpan: 3, visibilityChecks: [ ZeXtrasItem.isZeXtrasSuiteFcn ] },
              {
                type: _OUTPUT_,
                label: zxMsg('LBL_ZxMobile') + ': ',
                valueChangeEventSources: [
                  'attrs/' + ZxCore.KEY_RUNNING_MODS + '/' + ZxCore.KEY_LICENSE_ZXMOBILE
                ],
                getDisplayValue: function() {
                  return ZxCore.getStatusDisplayValue.call(this, ZxCore.KEY_LICENSE_ZXMOBILE);
                },
                cssStyle: 'font-weight:bold;',
                align: _LEFT_,
                labelLocation: _LEFT_
              },
              {
                type: _GROUP_,
                colSizes: [100],
                numCols: 1,
                width: 100,
                items: [
                  {
                    type: _DWT_BUTTON_,
                    label: zxMsg('LBL_Start'),
                    width: 100,
                    icon: 'ToggleStatus',
                    onActivate: ZxCore.doStartMobile,
                    visibilityChangeEventSources: [
                      'attrs/' + ZxCore.KEY_RUNNING_MODS + '/' + ZxCore.KEY_LICENSE_ZXMOBILE
                    ],
                    visibilityChecks: [
                      function() {
                        return !ZxCore.checkRunningModule.call(this, ZxCore.KEY_LICENSE_ZXMOBILE);
                      }
                    ],
                    enableDisableChangeEventSources: ['attrs/properties/' + ZxCore.KEY_OVERRIDE_NE_MOBILE],
                    enableDisableChecks: [
                      function() {
                        if (!ZaSettings.isNetworkVersion()) { return true; }
                        return !this.getInstance().attrs.properties[ZxCore.KEY_OVERRIDE_NE_MOBILE];
                      }
                    ]
                  },
                  {
                    type: _DWT_BUTTON_,
                    label: zxMsg('LBL_Stop'),
                    width: 100,
                    icon: 'ToggleStatus',
                    onActivate: ZxCore.doStopMobile,
                    visibilityChangeEventSources: [
                      'attrs/' + ZxCore.KEY_RUNNING_MODS + '/' + ZxCore.KEY_LICENSE_ZXMOBILE
                    ],
                    visibilityChecks: [
                      function() {
                        return ZxCore.checkRunningModule.call(this, ZxCore.KEY_LICENSE_ZXMOBILE);
                      }
                    ],
                    enableDisableChangeEventSources: ['attrs/properties/' + ZxCore.KEY_OVERRIDE_NE_MOBILE],
                    enableDisableChecks: [
                      function() {
                        if (!ZaSettings.isNetworkVersion()) { return true; }
                        return !this.getInstance().attrs.properties[ZxCore.KEY_OVERRIDE_NE_MOBILE];
                      }
                    ]
                  }
                ]
              },
              {
                type: _CHECKBOX_,
                ref: 'attrs/' + ZxCore.KEY_ENABLED_ON_STARTUP_MODS + '/' + ZxCore.KEY_LICENSE_ZXMOBILE,
                bmolsnr: true,
                labelLocation: _RIGHT_,
                valueChangeEventSources: [
                  'attrs/' + ZxCore.KEY_ENABLED_ON_STARTUP_MODS + '/' + ZxCore.KEY_LICENSE_ZXMOBILE
                ],
                label: zxMsg('LBL_Enabled_on_startup'),
                getDisplayValue: function(val) {
                  return ZxCore.checkEnableModuleOnStartup.call(this, ZxCore.KEY_LICENSE_ZXMOBILE);
                },
                onChange: ZxCore.doEnableDisableMobile,
                enableDisableChangeEventSources: ['attrs/properties/' + ZxCore.KEY_OVERRIDE_NE_MOBILE],
                enableDisableChecks: [
                  function() {
                    if (!ZaSettings.isNetworkVersion()) { return true; }
                    return !this.getInstance().attrs.properties[ZxCore.KEY_OVERRIDE_NE_MOBILE];
                  }
                ]
              },
              {
                type: _CHECKBOX_,
                ref: 'attrs/properties/' + ZxCore.KEY_OVERRIDE_NE_MOBILE,
                labelLocation: _RIGHT_,
                valueChangeEventSources: [
                  'attrs/properties/' + ZxCore.KEY_OVERRIDE_NE_MOBILE
                ],
                label: zxMsg('LBL_Override_mobile_NE'),
                onChange: ZxCore.doOverrideNEMobile,
                visibilityChecks: [
                  ZaSettings.isNetworkVersion
                ]
              },
              {
                type: _OUTPUT_,
                value: zxMsg('LBL_COR_license_restart_required'),
                cssStyle: 'margin-left:10px;font-weight:bold;',
                visibilityChecks: [
                  ZxCore.checkMobileRestartRequired,
                  ZaSettings.isNetworkVersion
                ],
                visibilityChangeEventSources: [
                  'attrs/' + ZxCore.KEY_MOBILE_HANDLER_REGISTERED
                ]
              },
              {
                type: _OUTPUT_,
                label: zxMsg('LBL_ZxPowerstore') + ': ',
                valueChangeEventSources: [
                  'attrs/' + ZxCore.KEY_RUNNING_MODS + '/' + ZxCore.KEY_LICENSE_ZXPOWERSTORE
                ],
                getDisplayValue: function() {
                  return ZxCore.getStatusDisplayValue.call(this, ZxCore.KEY_LICENSE_ZXPOWERSTORE);
                },
                cssStyle: 'font-weight:bold;',
                align: _LEFT_,
                labelLocation: _LEFT_
              },
              {
                type: _GROUP_,
                colSizes: [100],
                numCols: 1,
                width: 100,
                items: [
                  {
                    type: _DWT_BUTTON_,
                    label: zxMsg('LBL_Start'),
                    width: 100,
                    icon: 'ToggleStatus',
                    onActivate: ZxCore.doStartPowerstore,
                    visibilityChangeEventSources: [
                      'attrs/' + ZxCore.KEY_RUNNING_MODS + '/' + ZxCore.KEY_LICENSE_ZXPOWERSTORE
                    ],
                    visibilityChecks: [
                      function() {
                        return !ZxCore.checkRunningModule.call(this, ZxCore.KEY_LICENSE_ZXPOWERSTORE);
                      }
                    ]
                  },
                  {
                    type: _DWT_BUTTON_,
                    label: zxMsg('LBL_Stop'),
                    width: 100,
                    icon: 'ToggleStatus',
                    onActivate: ZxCore.doStopPowerstore,
                    visibilityChangeEventSources: [
                      'attrs/' + ZxCore.KEY_RUNNING_MODS + '/' + ZxCore.KEY_LICENSE_ZXPOWERSTORE
                    ],
                    visibilityChecks: [
                      function() {
                        return ZxCore.checkRunningModule.call(this, ZxCore.KEY_LICENSE_ZXPOWERSTORE);
                      }
                    ]
                  }
                ]
              },
              {
                type: _CHECKBOX_,
                ref: 'attrs/' + ZxCore.KEY_ENABLED_ON_STARTUP_MODS + '/' + ZxCore.KEY_LICENSE_ZXPOWERSTORE,
                bmolsnr: true,
                labelLocation: _RIGHT_,
                valueChangeEventSources: [
                  'attrs/' + ZxCore.KEY_ENABLED_ON_STARTUP_MODS + '/' + ZxCore.KEY_LICENSE_ZXPOWERSTORE
                ],
                label: zxMsg('LBL_Enabled_on_startup'),
                getDisplayValue: function(val) {
                  return ZxCore.checkEnableModuleOnStartup.call(this, ZxCore.KEY_LICENSE_ZXPOWERSTORE);
                },
                onChange: ZxCore.doEnableDisablePowerstore
              },
              { type: _CELL_SPACER_, colSpan: 3 }
            ]
          },
          {
            type: _ZAWIZ_TOP_GROUPER_,
            label: zxMsg('LBL_License'),
            width: '100%',
            numCols: 2,
            colSizes: [150, '*'],
            items: [
              {
                type: _OUTPUT_,
                ref: 'attrs/license/' + ZxCore.KEY_LICENSE_STATUS,
                label: zxMsg('LBL_COR_License_Status') + ': ',
                bmolsnr: true,
                cssClass: 'AdminTitle',
                value: ZaMsg.splashScreenLoading,
                align: _LEFT_,
                labelLocation: _LEFT_,
                getDisplayValue: function() {
                  var val = this.getInstanceValue('attrs/license/' + ZxCore.KEY_LICENSE_STATUS),
                    type = this.getInstanceValue('attrs/license/' + ZxCore.KEY_LICENSE_TYPE),
                    ispLegacy = this.getInstanceValue('attrs/license/' + ZxCore.KEY_LICENSE_TYPE_ISP_V2),
                    hide = this.getInstanceValue('attrs/license/' + ZxCore.KEY_LICENSE_HIDE_RENEW_TIME_LEFT),
                    renew = this.getInstanceValue('attrs/license/' + ZxCore.KEY_LICENSE_RENEW_TIME_LEFT);

                  if (type === ZxLicense.LICENSE_ISP || ispLegacy === true) {
                    if (renew <= 0) {
                      return '<span style="color: #FF0000">' + val +
                        ' - ' + zxMsg('MSG_LIC_Failed_ISP_audit_short') + '</span>';
                    } else {
                      return val;
                    }
                  } else {
                    return val;
                  }
                }
              },
              {type: _SPACER_, height: '5px'},
              {
                type: _OUTPUT_,
                ref: 'attrs/license/' + ZxCore.KEY_LICENSE_COUNT_TOTAL,
                label: zxMsg('MSG_COR_license_total_mailboxes') + ': ',
                bmolsnr: true,
                value: ZaMsg.splashScreenLoading,
                align: _LEFT_,
                labelLocation: _LEFT_
              },
              {
                type: _OUTPUT_,
                ref: 'attrs/license/' + ZxCore.KEY_LICENSE_COUNT_RESOURCE,
                label: zxMsg('MSG_COR_license_resource_mailboxes') + ': ',
                bmolsnr: true,
                value: ZaMsg.splashScreenLoading,
                align: _LEFT_,
                labelLocation: _LEFT_
              },
              {type: _SPACER_, height: '5px'},
              {
                type: _OUTPUT_,
                ref: 'attrs/license/' + ZxCore.KEY_LICENSE_COUNT_ACCOUNT_TEXT,
                label: '<b>' + zxMsg('MSG_COR_license_your_mailboxes') + ':</b> ',
                bmolsnr: true,
                value: ZaMsg.splashScreenLoading,
                cssStyle: 'font-weight:bold;',
                align: _LEFT_,
                labelLocation: _LEFT_
              }
            ]
          },
          {
            type: _ZAWIZ_TOP_GROUPER_,
            label: zxMsg('LBL_ISP_Validation'),
            colSizes: ['60%', '40%'],
            colSpan: 2,
            numCols: 2,
            visibilityChangeEventSources: ['attrs/license/' + ZxCore.KEY_LICENSE_TYPE],
            visibilityChecks: [function() {
              var type = this.getInstanceValue('attrs/license/' + ZxCore.KEY_LICENSE_TYPE);
              return type === ZxLicense.LICENSE_ISP;
            }, function() {
                var ispLegacy = this.getInstanceValue('attrs/license/' + ZxCore.KEY_LICENSE_TYPE_ISP_V2);
                return ispLegacy === true;
            }],
            items: [
              {
                type: _GROUP_,
                valign: _TOP_,
                items: [
                  {
                    type: _OUTPUT_,
                    ref: 'attrs/license/' + ZxCore.KEY_LICENSE_STATUS,
                    label: zxMsg('LBL_COR_Validation_Status') + ': ',
                    bmolsnr: true,
                    value: ZaMsg.splashScreenLoading,
                    align: _LEFT_,
                    labelLocation: _LEFT_,
                    getDisplayValue: function() {
                      var renew = this.getInstanceValue('attrs/license/' + ZxCore.KEY_LICENSE_RENEW_TIME_LEFT);

                      if (renew <= 0) {
                        return '<span style="color: #FF0000">' + zxMsg('MSG_LIC_Failed_ISP_audit_short') + '</span>';
                      } else {
                        return zxMsg('LBL_Ok');
                      }
                    }
                  },
                  {
                    type: _OUTPUT_,
                    ref: 'attrs/license/' + ZxCore.KEY_LICENSE_LAST_VALIDATION,
                    label: zxMsg('LBL_ISP_Last_license_validation') + ': ',
                    getDisplayValue: function(val) {
                      if (val instanceof Date) {
                        return ZxUtils.computeWordyDateStr(new Date(), val);
                      } else {
                        return ZaMsg.splashScreenLoading;
                      }
                    },
                    bmolsnr: true,
                    value: ZaMsg.splashScreenLoading,
                    align: _LEFT_,
                    labelLocation: _LEFT_
                  },
                  {type: _SPACER_, height: '5px'},
                  {
                    type: _OUTPUT_,
                    ref: 'attrs/license/' + ZxCore.KEY_LICENSE_SERVER_ID,
                    label: zxMsg('LBL_ISP_Server_ID') + ': ',
                    bmolsnr: true,
                    value: ZaMsg.splashScreenLoading,
                    align: _LEFT_,
                    labelLocation: _LEFT_
                  },
                  {type: _SPACER_, height: '5px'},
                  {
                    type: _OUTPUT_,
                    ref: 'attrs/license/' + ZxCore.KEY_LICENSE_RENEW_TIME_LEFT,
                    label: zxMsg('LBL_Grace_period') + ': ',
                    cssClass: 'AdminTitle',
                    bmolsnr: true,
                    value: ZaMsg.splashScreenLoading,
                    visibilityChangeEventSources: ['attrs/license/' + ZxCore.KEY_LICENSE_HIDE_RENEW_TIME_LEFT],
                    visibilityChecks: [function() {
                      var hide = this.getInstanceValue('attrs/license/' + ZxCore.KEY_LICENSE_HIDE_RENEW_TIME_LEFT);
                      return hide === false;
                    }],
                    getDisplayValue: function(val) {
                      if (typeof val === 'number') {
                        var tmpDate = new Date().getTime() + val;
                        if (val > 0) {
                          return '<span style="color: #FFFF00">' + zxMsg('MSG_LIC_License_about_to_expire_isp',
                            [ZxUtils.computeWordyDateStr(new Date(), tmpDate)]
                          ) + '</span>';
                        } else {
                          return '<span style="color: #FF0000">' + zxMsg('LBL_Expired') + '</span>';
                        }
                      } else {
                        return ZaMsg.splashScreenLoading;
                      }
                    },
                    align: _LEFT_,
                    labelLocation: _LEFT_
                  }
                ]
              },
              {
                type: _DWT_BUTTON_,
                label: zxMsg('LBL_ISP_Check_now'),
                onActivate: ZxCore.doCheckLicenseValidation
              }
            ]
          }
        ]
      },
      {
        type: _TOP_GROUPER_,
        label: zxMsg('LBL_COR_License_file_info'),
        numCols: 1,
        width: '100%',
        colSizes: ['100%'],
        colSpan: 2,
        items: [
          {
            type: _GROUP_,
            numCols: 2,
            colSizes: ['50%', '50%'],
            cssStyle: 'margin:5px;',
            width: '*',
            items: [
              {type: _GROUP_, width: '100%', colSizes: [150, '*'], numCols: 2, items: [
                {type: _OUTPUT_, ref: 'attrs/license/' + ZxCore.KEY_LICENSE_LICENSED_USER_TEXT,
                  label: zxMsg('LBL_COR_license_licensed_mailboxes') + ': ',
                  bmolsnr: true,
                  value: ZaMsg.splashScreenLoading,
                  cssStyle: 'font-weight:bold;', align: _LEFT_, labelLocation: _LEFT_},
                {type: _OUTPUT_, ref: 'attrs/license/' + ZxCore.KEY_LICENSE_TYPE,
                  label: zxMsg('LBL_COR_license_Type') + ': ',
                  bmolsnr: true,
                  cssStyle: 'font-weight:bold;', value: ZaMsg.splashScreenLoading,
                  align: _LEFT_, labelLocation: _LEFT_},
                {type: _OUTPUT_, ref: 'attrs/license/' + ZxCore.KEY_LICENSE_COMPANY,
                  label: zxMsg('LBL_COR_license_Company') + ': ',
                  bmolsnr: true,
                  cssStyle: 'font-weight:bold;', value: ZaMsg.splashScreenLoading,
                  align: _LEFT_, labelLocation: _LEFT_},
                {type: _OUTPUT_, ref: 'attrs/license/' + ZxCore.KEY_LICENSE_CUSTOMER,
                  label: zxMsg('LBL_COR_license_Reseller') + ': ',
                  bmolsnr: true,
                  visibilityChecks: [function(){
                    return this.getInstanceValue('attrs/license/' + ZxCore.KEY_LICENSE_RESELLER) === true
                  }],
                  cssStyle: 'font-weight:bold;', value: ZaMsg.splashScreenLoading,
                  align: _LEFT_, labelLocation: _LEFT_},
                {type: _OUTPUT_, ref: 'attrs/license/' + ZxCore.KEY_LICENSE_ZIMBRA_VERSION,
                  label: zxMsg('LBL_COR_license_limited_to_zimbra_version') + ': ',
                  bmolsnr: true,
                  cssStyle: 'font-weight:bold;',
                  visibilityChecks: [ZxCore.checkHideVersionLimit],
                  getDisplayValue: ZxCore.getZimbraVersionLimitedLicenceDispalyValue,
                  value: ZaMsg.splashScreenLoading,
                  align: _LEFT_, labelLocation: _LEFT_},
                {type: _OUTPUT_, ref: 'attrs/license/' + ZxCore.KEY_LICENSE_DATE_START_TEXT,
                  label: zxMsg('LBL_COR_license_valid_since') + ': ',
                  bmolsnr: true,
                  cssStyle: 'font-weight:bold;', value: ZaMsg.splashScreenLoading,
                  align: _LEFT_, labelLocation: _LEFT_},
                {type: _OUTPUT_, ref: 'attrs/license/' + ZxCore.KEY_LICENSE_DATE_END_TEXT,
                  label: zxMsg('LBL_COR_license_valid_until') + ': ',
                  bmolsnr: true,
                  visibilityChecks: [ZxCore.checkHideLicenseDateEnd],
                  cssStyle: 'font-weight:bold;', value: ZaMsg.splashScreenLoading,
                  align: _LEFT_, labelLocation: _LEFT_},
                {type: _OUTPUT_, ref: 'attrs/license/' + ZxCore.KEY_LICENSE_MAINTENANCE_END_TEXT,
                  label: zxMsg('LBL_COR_maintenance_end') + ': ',
                  bmolsnr: true,
                  visibilityChecks: [ZxCore.checkIsLicensePerpetualV2],
                  visibilityChangeEventSources: ['attrs/license/' + ZxCore.KEY_LICENSE_TYPE],
                  cssStyle: 'font-weight:bold;', value: ZaMsg.splashScreenLoading,
                  align: _LEFT_, labelLocation: _LEFT_},
                {type: _OUTPUT_, ref: 'attrs/license/' + ZxCore.KEY_LICENSE_ORDER_ID,
                  label: zxMsg('LBL_COR_license_order_id') + ': ',
                  bmolsnr: true,
                  value: ZaMsg.splashScreenLoading,
                  cssStyle: 'font-weight:bold;', align: _LEFT_, labelLocation: _LEFT_}
              ]},
              {  // TODO: Edit Informations
                type: _GROUPER_,
                label: zxMsg('LBL_Modules'),
                cssStyle: 'margin:5px;',
                width: '*',
                numCols: 3,
                colSizes: ['*', '*', 100],
                items: [
                  {
                    type: _OUTPUT_,
                    ref: 'attrs/license/modules/' + ZxCore.KEY_LICENSE_ZXADMIN,
                    label: zxMsg('LBL_ZxAdmin') + ': ',
                    bmolsnr: true,
                    cssStyle: 'font-weight:bold;',
                    align: _LEFT_,
                    labelLocation: _LEFT_,
                    getDisplayValue: ZxCore.getLicensedDispalyValue
                  },
                  { type: _CELL_SPACER_ },
                  {
                    type: _OUTPUT_,
                    ref: 'attrs/license/modules/' + ZxCore.KEY_LICENSE_ZXBACKUP,
                    label: zxMsg('LBL_ZxBackup') + ': ',
                    bmolsnr: true,
                    cssStyle: 'font-weight:bold;',
                    align: _LEFT_,
                    labelLocation: _LEFT_,
                    getDisplayValue: ZxCore.getLicensedDispalyValue
                  },
                  {
                    type: _CELL_SPACER_,
                    visibilityChecks:[ ZeXtrasItem.isZeXtrasSuiteFcn ]
                  },
                  {
                    type: _OUTPUT_,
                    ref: 'attrs/license/modules/' + ZxCore.KEY_LICENSE_ZXCHAT,
                    label: zxMsg('LBL_ZxChat') + ': ',
                    bmolsnr: true,
                    cssStyle: 'font-weight:bold;',
                    align: _LEFT_,
                    labelLocation: _LEFT_,
                    getDisplayValue: function() { return zxMsg('LBL_Free'); },
                    visibilityChecks:[ ZeXtrasItem.isZeXtrasSuiteFcn ]
                  },
                  {
                    type: _CELL_SPACER_, // Associated to the chat row
                    visibilityChecks:[ ZeXtrasItem.isZeXtrasSuiteFcn ]
                  },
                  {
                    type: _OUTPUT_,
                    ref: 'attrs/license/modules/' + ZxCore.KEY_LICENSE_ZXMOBILE,
                    label: zxMsg('LBL_ZxMobile') + ': ',
                    bmolsnr: true,
                    cssStyle: 'font-weight:bold;',
                    align: _LEFT_,
                    labelLocation: _LEFT_,
                    getDisplayValue: ZxCore.getLicensedDispalyValue
                  },
                  { type: _CELL_SPACER_ },
                  {
                    type: _OUTPUT_,
                    ref: 'attrs/license/modules/' + ZxCore.KEY_LICENSE_ZXPOWERSTORE,
                    label: zxMsg('LBL_ZxPowerstore') + ': ',
                    bmolsnr: true,
                    cssStyle: 'font-weight:bold;',
                    align: _LEFT_,
                    labelLocation: _LEFT_,
                    getDisplayValue: ZxCore.getLicensedDispalyValue
                  },
                  { type: _CELL_SPACER_ }
                ]
              },
              {type: _GROUP_, width: '100%', numCols: 1, valign: _TOP_, items: [
                {
                  type: _OUTPUT_,
                  label: zxMsg('LBL_COR_license_upload') + ': ',
                  cssStyle: 'margin:5px;',
                  value: ZaMsg.splashScreenLoading,
                  getDisplayValue: function(newValue) {
                    if (!this.getInstance()) {
                      return ZaMsg.splashScreenLoading;
                    }
                    var serverID = this.getInstance().id;
                    return ['<form enctype="multipart/form-data"',
                      'method="POST"',
                      'id="' + ZxCore.ClientUploadFormId + '_' + serverID + '"',
                      'action="/service/extension/zextras?module=ZxCore&action=doWebUploadLicense">',
                      '<input id="license" type="file" name="uplicense"',
                      'onChange="ZxCore.doUploadLicense(this.parentElement, \'' + serverID + '\');" /></form>'].join(' ');
                  }
                }
              ]},
              { type: _SPACER_ },
              {
                type: _DWT_BUTTON_,
                width: 100,
                cssStyle: 'margin-left:150px;',
                label: zxMsg('LBL_COR_license_remove'),
                onActivate: ZxCore.doRemoveLicense
              }
            ]
          }
        ]
      },
      { // TODO: Move this into a separate TAB
        type: _TOP_GROUPER_,
        label: zxMsg('LBL_COR_client_zimlet') + ' ' + ZxUtils.getInfoIcon(ZxLinks.HELP_CLN_about_client_zimlet),
        /* id: "UpdatesGroup",*/
        colSpan: 2,
        numCols: 1,
        colSizes: ['*'],
        align: _LEFT_,
        items: [
          {
            type: _GROUP_,
            colSizes: ['*'],
            numCols: 1,
            cssStyle: 'margin: 5px',
            items: [
              {
                type: _DWT_ALERT_,
                cssClass: 'DwtTabTable',
                visibilityChangeEventSources: ['attrs/' + ZxCore.KEY_CLIENT_ZIMLET_DEPLOYED],
                visibilityChecks: [ZxCore.checkClientZimletNotDeployed],
                containerCssStyle: 'font-weight: bolder;color:red;padding-bottom:0px',
                style: DwtAlert.WARNING,
                iconVisible: true,
                value: zxMsg('MSG_COR_Client_zimlet_desc1')
              },
              {
                type: _OUTPUT_,
                cssStyle: 'margin:5px;',
                value: zxMsg('MSG_COR_Client_zimlet_desc1'),
                visibilityChangeEventSources: ['attrs/' + ZxCore.KEY_CLIENT_ZIMLET_DEPLOYED],
                visibilityChecks: [ZxCore.checkClientZimletDeployed]
              }
            ]
          },
          {
            type: _GROUP_,
            colSizes: ['*'],
            numCols: 1,
            cssStyle: 'margin: 5px',
            items: [
              {
                type: _DWT_BUTTON_,
                label: zxMsg('LBL_deploy'),
                icon: 'Deploy',
                align: _CENTER_,
                onActivate: ZxCore.doDeployLatestZimlet
              }
            ]
          }
        ]
      },
      {
        type: _TOP_GROUPER_,
        label: zxMsg('LBL_COR_updates'),
        /* id: "UpdatesGroup",*/
        colSpan: 2,
        numCols: 1,
        colSizes: ['*'],
        align: _LEFT_,
        items: [
          {
            type: _GROUP_,
            numCols: 3,
            colSizes: [150, '*', 150],
            cssStyle: 'margin:5px;',
            items: [
              {
                type: _OUTPUT_,
                ref: 'attrs/updateinfo/' + ZxCore.KEY_UPDATE_CURRENT_VERSION,
                bmolsnr: true,
                label: zxMsg('LBL_COR_update_version') + ': ',
                value: ZaMsg.splashScreenLoading,
                cssStyle: 'font-weight:bold;'
              },
              { type: _CELL_SPACER_ },
              {
                type: _OUTPUT_,
                label: zxMsg('LBL_COR_build_id') + ': ',
                value: ZeXtrasItem.COMMIT
              },
              {type: _CELL_SPACER_ },
              {
                type: _OUTPUT_,
                ref: 'attrs/updateinfo/' + ZxCore.KEY_UPDATE_AVAILABLE,
                bmolsnr: true,
                label: zxMsg('LBL_COR_update_available_version') + ': ',
                value: ZaMsg.splashScreenLoading,
                cssStyle: 'font-weight:bold;'
              },
              {
                type: _DWT_BUTTON_,
                label: zxMsg('LBL_COR_update_check'),
                icon: 'Refresh',
                onActivate: ZxCore.doCheckUpdate
              },
              {
                type: _OUTPUT_,
                ref: 'attrs/updateinfo/' + ZxCore.KEY_UPDATE_LAST_UPDATE,
                bmolsnr: true,
                label: zxMsg('LBL_COR_update_last_check') + ': ',
                value: ZaMsg.splashScreenLoading,
                cssStyle: 'font-weight:bold;'
              },
              { type: _CELL_SPACER_ },
              {
                type: _CHECKBOX_,
                labelCssClass: 'ZxCheckboxLabel',
                ref: 'attrs/properties/' + ZxCore.KEY_NOTIFY_UPDATES,
                bmolsnr: true,
                label: zxMsg('LBL_COR_update_notify') + ': ',
                align: _LEFT_,
                labelLocation: _LEFT_
              },
              { type: _CELL_SPACER_ }
            ]
          }
        ]
      },
      {
        type: _TOP_GROUPER_,
        label: zxMsg('LBL_NOT_Notifications'),
        numCols: 1,
        colSizes: ['*'],
        colSpan: 2,
        align: _LEFT_,
        items: [
          {
            type: _GROUP_,
            numCols: 2,
            colSizes: ['*', '*'],
            cssStyle: 'margin:5px;',
            items: [
              {
                type: _TEXTFIELD_,
                ref: 'attrs/properties/' + ZxCore.KEY_NOTIFICATION_FROM,
                bmolsnr: true,
                label: zxMsg('LBL_COR_notification_email_sender') + ': ',
                labelLocation: _LEFT_,
                align: _LEFT_,
                width: 150
              },
              {
                type: _TEXTFIELD_,
                ref: 'attrs/properties/' + ZxCore.KEY_NOTIFICATION_TO,
                bmolsnr: true,
                label: zxMsg('LBL_COR_notification_email_recipient') + ': ',
                labelLocation: _LEFT_,
                align: _LEFT_,
                width: 150
              },
              {
                type: _TEXTFIELD_,
                ref: 'attrs/properties/' + ZxCore.KEY_NOTIFICATION_LICENSE_ADDITIONAL,
                bmolsnr: true,
                label: zxMsg('LBL_COR_additional_notification_email_recipient') + ': ',
                labelLocation: _LEFT_,
                align: _LEFT_,
                width: 150
              },
              {
                type: _OSELECT1_,
                ref: 'attrs/properties/' + ZxCore.KEY_NOTIFICATION_LEVEL,
                bmolsnr: true,
                label: zxMsg('LBL_COR_notification_level') + ': ',
                labelLocation: _LEFT_,
                align: _LEFT_,
                choices: ZxCore.LogLevels
              },
              {
                type: _OSELECT1_,
                ref: 'attrs/properties/' + ZxCore.KEY_LOG_LEVEL,
                bmolsnr: true,
                label: zxMsg('LBL_COR_log_level') + ': ',
                labelLocation: _LEFT_,
                align: _LEFT_,
                choices: ZxCore.LogLevels
              }
            ]
          }
        ]
      },
      {
        type: _TOP_GROUPER_,
        label: zxMsg('LBL_COR_Privacy'),
        numCols: 1,
        colSizes: ['*'],
        colSpan: 2,
        align: _LEFT_,
        items: [
          {
            type: _GROUP_,
            numCols: 2,
            colSizes: [20, '*'],
            cssStyle: 'margin:5px;',
            items: [
              {
                type: _CHECKBOX_,
                ref: 'attrs/properties/' + ZxCore.KEY_SEND_FULL_ERROR_REPORTS,
                bmolsnr: true,
                label: zxMsg('LBL_COR_Send_full_error_report') + ' ' + ZxUtils.getInfoIcon(ZxLinks.HELP_Privacy_Policy),
                labelLocation: _RIGHT_
              }
            ]
          }
        ]
      }
    ]
  }
);
