/*
 * ***** BEGIN LICENSE BLOCK *****
 * Copyright (C) 2011-2016 ZeXtras
 *
 * The contents of this file are subject to the ZeXtras EULA;
 * you may not use this file except in compliance with the EULA.
 * You may obtain a copy of the EULA at
 * http://www.zextras.com/zextras-eula.html
 * ***** END LICENSE BLOCK *****
 */

function ZxCheckUpdateDialog(parent, item, callback) {
  this._standardButtons = [DwtDialog.CANCEL_BUTTON];
  this._item = item;
  this._controller = item.getCurrentController();
  this._callbackExternal = callback;
  ZaXDialog.call(this, parent, null, ZxUtils.getTitleWithIcon('ZeXtras', zxMsg('LBL_check_update')),
    '350px', '150px', null, null);
  this._msgCellId = Dwt.getNextId();
  this._dwtIdIcon = Dwt.getNextId();

  this.initForm([], this.getMyXForm());
  this.setObject({});

  this.registerCallback(
    DwtDialog.CANCEL_BUTTON,
    new AjxCallback(
      this,
      function() {
        this.popdown();
      }
    )
  );

  var updateDialogCbk = AjxCallback.simpleClosure(
      this.setMessage,
      this
    );

  ZxAdminRequest.sendRequest(
    ZxAdminRequest.TARGET_ALL,
    'ZxUpdate',
    'doCheckUpdate'
  ).then(AjxCallback.simpleClosure(
    function(updateDialogCbk, resp) {
      var controller = this._controller,
        server = this._item.getCurrentServer(),
        mitem = this._item,
        update_result,
        installedZextrasVersionString,
        requiredZextrasVersionString,
        installedZextrasVersion,
        requiredZextrasVersion;

      if (resp[server] instanceof ZxError) {
        this._helpURL = ZxLinks.HELP_COR_Dialog_CheckUpdate_Troubleshooting;
        updateDialogCbk(
          ZxCheckUpdateDialog.FAILURE,
          zxMsg('MSG_COR_checkupdate_unable_to_communicate_with_server')
        );
      } else {
        installedZextrasVersionString = resp[server].response.currentCoreVersion;
        if (resp[server].response.updateChat) {
          requiredZextrasVersionString = resp[server].response.updateChat.min_core_version;
        } else {
          requiredZextrasVersionString = '0.0';
        }

        installedZextrasVersion = Number(installedZextrasVersionString.split('.').join(''));
        requiredZextrasVersion = Number(requiredZextrasVersionString.split('.').join(''));

        if (!(resp[server].ok)) {
          updateDialogCbk(
            ZxCheckUpdateDialog.FAILURE,
            zxMsg('MSG_COR_checkupdate_unable_to_communicate_with_server')
          );

        } else {
          ZxAdminRequest.sendRequest(
            server,
            'ZxCore',
            'getUpdateInfo'
          ).then(AjxCallback.simpleClosure(
            function(updateDialogCbk, respUpInfo) {
              var server = this._item.getCurrentServer(),
                installedChatVersionString,
                availableChatVersionString,
                installedChatVersion,
                availableChatVersion;

              installedChatVersionString = respUpInfo.response.zxchatZimletCurrentVersion;
              availableChatVersionString = respUpInfo.response.zxchatZimletAvailableVersion;

              installedChatVersion = Number(installedChatVersionString);
              availableChatVersion = Number(availableChatVersionString);

              if (installedChatVersionString === '0.0') {
                installedChatVersionString = zxMsg('MSG_COR_checkupdate_none');
              }

              if (respUpInfo.response.availableVersion !== 'none') { //ZeXtras update available
                if (ZxUtils.isZ7Up() && !ZeXtras.isMigZimlet()) {
                  if (availableChatVersion === installedChatVersion) { //ZeXtras update available, ZxChat up to date
                    update_result = zxMsg('MSG_COR_checkupdate_new_version_available',
                      ['<b>(' + respUpInfo.response.availableVersion + ')</b>',
                        '<a href="' + ZxLinks.COR_Download_ZeXtras_for_Zimbra + '" target="_blank">', '</a>',
                        '<br>']);

                  } else { //Both ZeXtras and ZxChat updates available
                    if (installedZextrasVersion < requiredZextrasVersion) { //ZeXtras update necessary to update ZxChat
                      update_result = zxMsg('MSG_COR_checkupdate_new_version_core_and_chat',
                        ['<b>(' + respUpInfo.response.availableVersion + ')</b>',
                          '<a href="' + ZxLinks.COR_Download_ZeXtras_for_Zimbra + '" target="_blank">', '</a>',
                          '<br>']);
                    } else { //ZeXtras update not necessary to update ZxChat
                      update_result = zxMsg('MSG_COR_checkupdate_new_version_core_and_chat',
                        ['<b>(' + respUpInfo.response.availableVersion + ')</b>',
                          '<a href="' + ZxLinks.COR_Download_ZeXtras_for_Zimbra + '" target="_blank">', '</a>',
                          '<br>', installedChatVersionString, availableChatVersion]);
                    }
                  }

                } else { // Hide zimlet chat update for Zimbra 6 and ZxMig
                  update_result = zxMsg('MSG_COR_checkupdate_new_version_available',
                    ['<b>(' + respUpInfo + ')</b>',
                      '<a href="' + ZxLinks.COR_Download_ZeXtras_for_Zimbra + '" target="_blank">', '</a>',
                      '<br >']);
                }
              } else if (respUpInfo.response.availableCommit !== 'none' &&
                  respUpInfo.response.currentCommit !== respUpInfo.response.availableCommit) {

                update_result = zxMsg('MSG_COR_Commit_differ', [
                  respUpInfo.response.availableCommit,
                  '<a href="' + ZxLinks.CHANGELOG + '" target="_blank">',
                  '<a href="' + ZxLinks.Download + '" target="_blank">',
                  '</a>'
                ]);

              } else { //ZeXtras up to date
                if (!ZeXtras.isMigZimlet()) {
                  if (availableChatVersion === installedChatVersion) { //Both ZeXtras and ZxChat up to date
                    update_result = zxMsg('MSG_COR_checkupdate_core_up_to_date');
                  } else { //ZeXtras up to date, ZxChat update available
                    update_result = zxMsg('MSG_COR_checkupdate_new_zimlet_chat_version_available',
                      [installedChatVersionString, availableChatVersion, '<br>']);
                  }
                } else {
                  update_result = zxMsg('MSG_COR_checkupdate_core_up_to_date');
                }
              }

              updateDialogCbk(ZxCheckUpdateDialog.SUCCESS, update_result);
              if (this._callbackExternal) {
                this._callbackExternal.run(respUpInfo.response);
              }
            },
            this,
            updateDialogCbk
          ), ZxUtils.catchError);
        }
      }
    },
    this,
    updateDialogCbk
  ), ZxUtils.catchError);

  this._helpURL = ZxLinks.HELP_COR_Dialog_CheckUpdate;
}

ZxCheckUpdateDialog.prototype = new ZaXDialog();
ZxCheckUpdateDialog.prototype.constructor = ZxCheckUpdateDialog;

ZxCheckUpdateDialog.FAILURE = 'failure';
ZxCheckUpdateDialog.PENDING = 'pending';
ZxCheckUpdateDialog.SUCCESS = 'success';

ZxCheckUpdateDialog.prototype.setMessage = function(status, message) {
  var msgCell = document.getElementById(this._msgCellId),
    iconCell = document.getElementById(this._dwtIdIcon),
    btn = this.getButton(DwtDialog.CANCEL_BUTTON);

  if (status === ZxCheckUpdateDialog.PENDING) {
    iconCell.innerHTML = '<div class="DwtWait32Icon">&nbsp;</div>';
  } else if (status === ZxCheckUpdateDialog.SUCCESS) {
    iconCell.innerHTML =
      AjxImg.getImageHtml(DwtMessageDialog.ICON[DwtMessageDialog.INFO_STYLE], null, null, false, false);
    btn.setText(ZaMsg.TBB_Close);
  } else {
    iconCell.innerHTML =
      AjxImg.getImageHtml(DwtMessageDialog.ICON[DwtMessageDialog.WARNING_STYLE], null, null, false, false);
    btn.setText(ZaMsg.TBB_Close);
  }
  msgCell.innerHTML = message;
};

/**
 * Get the XForm Object of the checkupdate window.
 * @return {{numCols: number, colSizes: Array, items: Array}}
 */
ZxCheckUpdateDialog.prototype.getMyXForm = function() {
  var xFormObject = {
    numCols: 1,
    colSizes: ['100%'],
    items: [
      {type: _SPACER_, height: 10},
      {
        type: _GROUP_,
        numCols: 2,
        colSizes: ['32', '*'],
        items: [
          {
            type: _OUTPUT_,
            value: '<div id="' + this._dwtIdIcon + '"><div class="DwtWait32Icon">&nbsp</div></div>'
          },
          {
            type: _OUTPUT_,
            value: '<div id="' + this._msgCellId + '">' +
              zxMsg('MSG_COR_checkupdate_pending') +
              '</div>'
          }
        ]
      }
    ]
  };
  return xFormObject;
};
