/*
 * ***** BEGIN LICENSE BLOCK *****
 * Copyright (C) 2011-2016 ZeXtras
 *
 * The contents of this file are subject to the ZeXtras EULA;
 * you may not use this file except in compliance with the EULA.
 * You may obtain a copy of the EULA at
 * http://www.zextras.com/zextras-eula.html
 * ***** END LICENSE BLOCK *****
 */

function ZxRemoveLicenseDialog(parent, item, callback) {
  this._standardButtons = [DwtDialog.OK_BUTTON, DwtDialog.CANCEL_BUTTON];
  this._item = item;
  this._controller = item.getCurrentController();
  this._callbackExternal = callback;
  ZaXDialog.call(this, parent, null, ZxUtils.getTitleWithIcon('ZeXtras', zxMsg('LBL_COR_license_remove')),
    '400px', '130px', null, null);
  this._msgCellId = Dwt.getNextId();
  this._dwtIdIcon = Dwt.getNextId();

  this.initForm([], this.getMyXForm());
  this.setObject({});

  this.registerCallback(
    DwtDialog.CANCEL_BUTTON,
    new AjxCallback(
      this,
      function() {
        this.popdown();
      }
    )
  );

  this.getButton(DwtDialog.OK_BUTTON).setEnabled(false);

  var updateDialogCbk = AjxCallback.simpleClosure(
      this.setMessage,
      this
    );

  this.registerCallback(
    DwtDialog.OK_BUTTON,
    new AjxCallback(
      this,
      function() {
        ZxAdminRequest.sendRequest(
          item.getServerName(),
          item.getID(),
          'doRemoveLicense',
          {
            iamsure: true
          }
        ).then(
          AjxCallback.simpleClosure(
            function(item, resp) {
              this.popdown();
              item.loadMethod();
              ZeXtras.getLicense().invalidate();
            },
            this,
            item
          ),
          ZxUtils.catchError
        );
      }
    )
  );

  ZxAdminRequest.sendRequest(
    item.getServerName(),
    item.getID(),
    'getTrialInfo'
  ).then(
    AjxCallback.simpleClosure(
      function(item, resp) {
        if (!(resp instanceof ZxError)) {
          var response = resp.response,
            expiration = new Date(response.expirationDate),
            now = new Date(),
            diff = expiration.getMilliseconds() - now.getMilliseconds(),
            message;

          if (diff <= 0) {
            message = zxMsg('MSG_COR_reverting_to_trial_are_you_sure',
              [zxMsg('MSG_COR_reverting_to_trial_expired')]);
          } else {
            message = zxMsg('MSG_COR_reverting_to_trial_are_you_sure',
              [zxMsg('MSG_COR_reverting_to_trial_remaining_n_days', [
                ZxUtils.computeWordyDateStr(now, expiration.getTime())
              ])]);
          }

          this.setMessage(ZxRemoveLicenseDialog.SUCCESS, message);
          this.getButton(DwtDialog.OK_BUTTON).setEnabled(true);
        } else {
          this.popdown();
        }
      },
      this,
      item
    ),
    ZxUtils.catchError
  );

  this._helpURL = ZxLinks.HELP_COR_Core;
}

ZxRemoveLicenseDialog.prototype = new ZaXDialog();
ZxRemoveLicenseDialog.prototype.constructor = ZxRemoveLicenseDialog;

ZxRemoveLicenseDialog.FAILURE = 'failure';
ZxRemoveLicenseDialog.PENDING = 'pending';
ZxRemoveLicenseDialog.SUCCESS = 'success';

ZxRemoveLicenseDialog.prototype.setMessage = function(status, message) {
  var msgCell = document.getElementById(this._msgCellId),
    iconCell = document.getElementById(this._dwtIdIcon),
    btn = this.getButton(DwtDialog.CANCEL_BUTTON);

  if (status === ZxRemoveLicenseDialog.PENDING) {
    iconCell.innerHTML = '<div class="DwtWait32Icon">&nbsp;</div>';
  } else if (status === ZxRemoveLicenseDialog.SUCCESS) {
    iconCell.innerHTML =
      AjxImg.getImageHtml(DwtMessageDialog.ICON[DwtMessageDialog.WARNING_STYLE], null, null, false, false);
    btn.setText(ZaMsg.TBB_Close);
  } else {
    iconCell.innerHTML =
      AjxImg.getImageHtml(DwtMessageDialog.ICON[DwtMessageDialog.WARNING_STYLE], null, null, false, false);
    btn.setText(ZaMsg.TBB_Close);
  }
  msgCell.innerHTML = message;
};

/**
 * Get the XForm Object of the checkupdate window.
 * @return {{numCols: number, colSizes: Array, items: Array}}
 */
ZxRemoveLicenseDialog.prototype.getMyXForm = function() {
  var xFormObject = {
    numCols: 1,
    colSizes: ['100%'],
    items: [
      {type: _SPACER_, height: 10},
      {
        type: _GROUP_,
        numCols: 2,
        colSizes: ['32', '*'],
        items: [
          {
            type: _OUTPUT_,
            value: '<div id="' + this._dwtIdIcon + '"><div class="DwtWait32Icon">&nbsp</div></div>'
          },
          {
            type: _OUTPUT_,
            value: '<div id="' + this._msgCellId + '">' +
              zxMsg('MSG_COR_retrieving_trial_info') +
              '</div>'
          }
        ]
      }
    ]
  };
  return xFormObject;
};
