/*
 * ***** BEGIN LICENSE BLOCK *****
 * Copyright (C) 2011-2016 ZeXtras
 *
 * The contents of this file are subject to the ZeXtras EULA;
 * you may not use this file except in compliance with the EULA.
 * You may obtain a copy of the EULA at
 * http://www.zextras.com/zextras-eula.html
 * ***** END LICENSE BLOCK *****
 */

/**
 * Create the ZxMobile object.
 * @constructor
 */
function ZxMobile() {
  ZxModule.call(this, ZxMobile.MOD_ID, zxMsg('LBL_ZxMobile'), 'ZxMobile');

  this.registerLoadMethod(this.loadMethod, 'ZxMobileInit');
  this.registerModifyMethod(this.modifyMethod, 'ZxMobileInit');

  this._helpURL = ZxLinks.HELP_MOB_Mobile;
}

ZxMobile.prototype = new ZxModule();
ZxMobile.prototype.constructor = ZxMobile;
ZxMobile.MOD_ID = 'ZxMobile';

ZxMobile.KEY_SESSIONS = 'sessions';
ZxMobile.KEY_REGISTERED_DEVICES = 'devices';
ZxMobile.KEY_COS_ENABLED = 'CosEnabled';
ZxMobile.KEY_ACCOUNT_ENABLED = 'accountEnabled';
ZxMobile.KEY_NOTIFICATION_LATENCY = 'ZxMobile_NotificationsLatency';
ZxMobile.KEY_USE_INSTANT_NOTIFICATIONS = 'ZxMobile_UseInstantNotifications';
ZxMobile.KEY_MAX_PING_HEARTBEAT = 'ZxMobile_MaxPingHeartbeat';
ZxMobile.KEY_MAX_EAS_SAMSUNG_VERSION = 'ZxMobile_MaxSamsungVersion';
ZxMobile.KEY_MAX_EAS_GLOBAL_VERSION = 'ZxMobile_MaxGlobalVersion';
ZxMobile.KEY_PROVISIONING_ENABLED = 'ZxMobile_DeviceManagementEnabled';

ZxMobile.KEY_SHOW_ONLY_ONLINE_DEVICES = '_showOnlyOnlineDevices';

ZxMobile.ID_SESSION_LIST = 'ZXSessionList';
ZxMobile.ID_REGISTERED_DEVICES = 'ZXDeviceList';
ZxMobile.ID_WIPE_CONFIRM_BUTTON = 'ZXWipeConfirmButton';
ZxMobile.ID_WIPE_CONFIRM_CHECKBOX = 'ZXWipeConfirmCheckbox';

ZxMobile.COL_ID_HAS_MOBILE_PASSWORD = 'hasMobilePassword';
ZxMobile.COL_ID_IS_ONLINE = 'isOnline';
ZxMobile.COL_ID_IS_ONLINE_HINT = 'isOnlineHint';
ZxMobile.COL_ID_REMOTE_FOLDERS_COUNT = 'numRemoteFolders';
ZxMobile.COL_ID_REGISTERED = 'firstSeen';
ZxMobile.COL_ID_LAST_SEEN_RAW = 'lastSeenRaw';
ZxMobile.COL_ID_LAST_SEEN = 'lastSeen';
ZxMobile.COL_ID_LAST_SEEN_HINT = 'lastSeenHint';
ZxMobile.COL_ID_LAST_PING_TIMEOUT = 'lastPingTimeout';
ZxMobile.COL_ID_DEVICE = 'deviceType';
ZxMobile.COL_ID_DEVICE_ID = 'deviceId';
ZxMobile.COL_ID_SESSION_ID = 'sessionId';
ZxMobile.COL_ID_PROTOCOL = 'protocolVersion';
ZxMobile.COL_ID_ACCOUNT = 'accountEmail';
ZxMobile.COL_ID_ACCOUNT_NAME = 'accountName';
ZxMobile.COL_ID_EMAIL_ITEM = 'numEmails';
ZxMobile.COL_ID_CALENDAR_ITEM = 'numEvents';
ZxMobile.COL_ID_CONTACTS_ITEM = 'numContacts';
ZxMobile.COL_ID_TASK_ITEM = 'numTasks';
ZxMobile.COL_ID_BAD_ITEM = 'numBadItems';
ZxMobile.COL_ID_SERVER = 'server';
ZxMobile.COL_ID_STATUS = 'status';
ZxMobile.COL_ID_USER_AGENT = 'userAgent';
ZxMobile.COL_ID_DEVICE_TYPE = 'deviceType';

ZxMobile.A_enableDevicePassword = 'zextrasMobileDevicePasswordEnabled';
ZxMobile.A_devicePassword = 'zextrasMobileDevicePassword';
ZxMobile.A_devicePasswordConfirm = 'zextrasMobileDevicePasswordConfirm';
ZxMobile.A_devicesList = 'zextrasMobileDevicesList';

ZxMobile.CALENDAR_AGE_FILTER_CHOICES = [
  {label: zxMsg('LBL_MOB_Past_all'), value: 0},
  {label: zxMsg('LBL_MOB_Two_weeks'), value: 4},
  {label: zxMsg('LBL_MOB_One_month'), value: 5},
  {label: zxMsg('LBL_MOB_Three_months'), value: 6},
  {label: zxMsg('LBL_MOB_Six_months'), value: 7}
];

ZxMobile.EMAIL_AGE_FILTER_CHOICES = [
  {label: zxMsg('LBL_MOB_Past_all'), value: 0},
  {label: zxMsg('LBL_MOB_One_day'), value: 1},
  {label: zxMsg('LBL_MOB_Three_days'), value: 2},
  {label: zxMsg('LBL_MOB_One_week'), value: 3},
  {label: zxMsg('LBL_MOB_Two_weeks'), value: 4},
  {label: zxMsg('LBL_MOB_One_month'), value: 5}
];

ZxMobile.SMIME_ALGORITHM_NEGOTIATION_CHOICES = [
  {label: zxMsg('LBL_MOB_Block_negotiation'), value: 0},
  {label: zxMsg('LBL_MOB_Strong_algorithm_only'), value: 1},
  {label: zxMsg('LBL_MOB_Allow_any_algorithm'), value: 2}
];

ZxMobile.BLUETOOTH_CHOICES = [
  {label: zxMsg('LBL_Disable'), value: 0},
  {label: zxMsg('LBL_MOB_Hands_free_only'), value: 1},
  {label: zxMsg('LBL_MOB_Allow'), value: 2}
];

ZxMobile.EAS_VERSIONS = [
  {value: '14.1',
    label: '14.1'},
  {value: '14.0',
    label: '14.0'},
  {value: '12.1',
    label: '12.1'},
  {value: '12.0',
    label: '12.0'},
  {value: '2.5',
    label: '2.5'}
];

/**
 * Load or reload module data.
 */
ZxMobile.prototype.loadMethod = function(by, val) {
  this.attrs.stats = {};
  this.attrs.properties = {};

  this.attrs[ZxMobile.KEY_SESSIONS] = [];
  this.attrs[ZxMobile.KEY_REGISTERED_DEVICES] = [];

  this.attrs.properties[ZxMobile.KEY_NOTIFICATION_LATENCY] = 0;
  this.attrs.properties[ZxMobile.KEY_USE_INSTANT_NOTIFICATIONS] = false;
  this.attrs.properties[ZxMobile.KEY_MAX_PING_HEARTBEAT] = 0;
  this.attrs.properties[ZxMobile.KEY_MAX_EAS_SAMSUNG_VERSION] = '12.1';
  this.attrs.properties[ZxMobile.KEY_MAX_EAS_GLOBAL_VERSION] = '14.1';
  this.attrs.properties[ZxMobile.KEY_PROVISIONING_ENABLED] = false;

  this.attrs.stats[ZxMobile.KEY_COS_ENABLED] = zxMsg('LBL_NA');
  this.attrs.stats[ZxMobile.KEY_ACCOUNT_ENABLED] = zxMsg('LBL_NA');

  if (ZxUtils.amIAdmin()) {
    try {
      this.attrs.stats[ZxMobile.KEY_COS_ENABLED] = ZaSearch.search(
        '(zimbraFeatureMobileSyncEnabled=TRUE)',
        [ZaSearch.COSES],
        1,
        ZaCos.A_uid,
        true,
        'zimbraCOSId',
        0
      ).list.size();
    } catch (ex) {
      this._getDefaultCatchErrorFcn()(ex);
    }

    try {
      this.attrs.stats[ZxMobile.KEY_ACCOUNT_ENABLED] = ZaSearch.search(
        '(zimbraFeatureMobileSyncEnabled=TRUE)',
        [ZaSearch.ACCOUNTS],
        1,
        ZaAccount.A_uid,
        true,
        'zimbraId',
        0
      ).list.size();

    } catch (ex) {
      if (ex.code !== ZmCsfeException.TOO_MANY_SEARCH_RESULTS) {
        this._getDefaultCatchErrorFcn()(ex);
      }
    }
  }

  ZxAdminRequest.sendRequest(
    this.getServerName(),
    'ZxMobile',
    'getPublicProperties'
  ).then(
    AjxCallback.simpleClosure(
      function(resp) {
        ZxUtils.log(resp);
        if (!(resp instanceof ZxError)) {
          var controller = this.getCurrentController(),
            response = resp.response,
            properties = response.properties;

          this.attrs.properties[ZxMobile.KEY_MAX_PING_HEARTBEAT] = properties.ZxMobile_MaxPingHeartbeat;
          this.attrs.properties[ZxMobile.KEY_NOTIFICATION_LATENCY] = properties.ZxMobile_NotificationsLatency;
          this.attrs.properties[ZxMobile.KEY_USE_INSTANT_NOTIFICATIONS] = (
            properties.ZxMobile_UseInstantNotifications === 'true'
          );
          this.attrs.properties[ZxMobile.KEY_MAX_EAS_SAMSUNG_VERSION] = properties[ZxMobile.KEY_MAX_EAS_SAMSUNG_VERSION];
          this.attrs.properties[ZxMobile.KEY_MAX_EAS_GLOBAL_VERSION] = properties[ZxMobile.KEY_MAX_EAS_GLOBAL_VERSION];
          this.attrs.properties[ZxMobile.KEY_PROVISIONING_ENABLED] = properties[ZxMobile.KEY_PROVISIONING_ENABLED];

          controller.setInstanceValue('properties/' + ZxMobile.KEY_MAX_PING_HEARTBEAT,
            properties.ZxMobile_MaxPingHeartbeat);
          controller.setInstanceValue('properties/' + ZxMobile.KEY_NOTIFICATION_LATENCY,
            properties.ZxMobile_NotificationsLatency);
          controller.setInstanceValue('properties/' + ZxMobile.KEY_USE_INSTANT_NOTIFICATIONS,
            properties.ZxMobile_UseInstantNotifications === 'true');
          controller.setInstanceValue('properties/' + ZxMobile.KEY_MAX_EAS_SAMSUNG_VERSION,
            properties[ZxMobile.KEY_MAX_EAS_SAMSUNG_VERSION]);
          controller.setInstanceValue('properties/' + ZxMobile.KEY_MAX_EAS_GLOBAL_VERSION,
            properties[ZxMobile.KEY_MAX_EAS_GLOBAL_VERSION]);
          controller.setInstanceValue('properties/' + ZxMobile.KEY_PROVISIONING_ENABLED,
              properties[ZxMobile.KEY_PROVISIONING_ENABLED] === 'true');
        }
      },
      this
    ),
    this._getDefaultCatchErrorFcn()
  );

  ZxAdminRequest.sendRequest(
    this.getServerName(),
    'ZxMobile',
    'getAllDevices'
  ).then(
    AjxCallback.simpleClosure(
      function(resp) {
        ZxUtils.log(resp);
        var controller = this.getCurrentController(),
          devices = resp.response.devices,
          i,
          e,
          sessionList = [],
          tic,
          toc;

        for (i = 0; i < devices.length; i = i + 1) {
          e = devices[i];

          var row = {},
            registeredDate = zxMsg('LBL_NA'),
            lastSeenHint = zxMsg('LBL_NA'),
            lastSeen = zxMsg('LBL_NA');

          if (e.firstSeen > 0) {
            // ZX-2848 Override the 'N/A' text if the date was set.
            registeredDate = ZxUtils.formatDateShort(new Date(e.firstSeen));
          }
          if (e.lastSeen > 0) {
            // ZX-2848 Override the 'N/A' text if the date was set.
            lastSeenHint = ZxUtils.formatDateShort(new Date(e.lastSeen));
            lastSeen = ZxMobile.agoTime(e.lastSeen);
          }

          row[ZxMobile.COL_ID_ACCOUNT] = e.accountEmail;
          row[ZxMobile.COL_ID_ACCOUNT_NAME] = e.accountName;
          row[ZxMobile.COL_ID_SERVER] = e.accountServer;
          row[ZxMobile.COL_ID_DEVICE_ID] = e.deviceId;
          row[ZxMobile.COL_ID_DEVICE] = e.deviceType;
          row[ZxMobile.COL_ID_REGISTERED] = registeredDate;
          row[ZxMobile.COL_ID_LAST_SEEN_RAW] = e.lastSeen;
          row[ZxMobile.COL_ID_LAST_SEEN] = lastSeen;
          row[ZxMobile.COL_ID_LAST_SEEN_HINT] = lastSeenHint;
          row[ZxMobile.COL_ID_BAD_ITEM] = e.numBadItems;
          row[ZxMobile.COL_ID_CONTACTS_ITEM] = e.numContacts;
          row[ZxMobile.COL_ID_EMAIL_ITEM] = e.numEmails;
          row[ZxMobile.COL_ID_CALENDAR_ITEM] = e.numEvents;
          row[ZxMobile.COL_ID_TASK_ITEM] = e.numTasks;
          row[ZxMobile.COL_ID_PROTOCOL] = e.protocolVersion;
          row[ZxMobile.COL_ID_STATUS] = ZxMobile.getDeviceStatusString(e.status);
          row[ZxMobile.COL_ID_USER_AGENT] = e.userAgent;
          row[ZxMobile.COL_ID_HAS_MOBILE_PASSWORD] = e.hasMobilePassword || false;
          row[ZxMobile.COL_ID_IS_ONLINE] = e.isOnline || false;
          row[ZxMobile.COL_ID_IS_ONLINE_HINT] = e.isOnline ? zxMsg('LBL_MOB_Connected') : zxMsg('LBL_MOB_Disconnected');
          row[ZxMobile.COL_ID_REMOTE_FOLDERS_COUNT] = e.numRemoteFolders || 0;
          row[ZxMobile.COL_ID_LAST_PING_TIMEOUT] = e.lastPingTimeout || 0;

          sessionList.push(row);
        }

        // Order the array:
        tic = Date.now();
        sessionList.sort(function(a, b) {
          if (typeof a[ZxMobile.COL_ID_LAST_SEEN_RAW] === 'undefined') {
            return -1;
          }
          if (typeof b[ZxMobile.COL_ID_LAST_SEEN_RAW] === 'undefined') {
            return 1;
          }
          return b[ZxMobile.COL_ID_LAST_SEEN_RAW] - a[ZxMobile.COL_ID_LAST_SEEN_RAW];
        });
        toc = Date.now();
        ZxUtils.log((toc - tic) + 'ms', 'Time spent on sorting device list');

        this.attrs[ZxMobile.KEY_SESSIONS] = sessionList;
        controller.setInstanceValue(ZxMobile.KEY_SESSIONS, sessionList);
      },
      this
    ),
    this._getDefaultCatchErrorFcn()
  );
};

/**
 * Enable or disable to override the Zimbra Network mobile module.
 * @param {boolean} useZx
 * @param {Function} callback
 */
ZxMobile.prototype.__overrideNE = function(useZx, callback) {
  var controller = this.getCurrentController(),
    msgDialog = new DwtMessageDialog({
      buttons: [DwtDialog.YES_BUTTON, DwtDialog.NO_BUTTON],
      title: '',
      parent: controller.getShell()
    }),
    setOverrideCallback;

  setOverrideCallback = new AjxCallback(
    this,
    function(controller, dialog, useZx, callback) {
      ZxAdminRequest.sendRequest(
        this.getCurrentServer(),
        'ZxCore',
        'setPublicProperties',
        ZxUtils.createObjectFromRef('properties/' + ZxCore.KEY_OVERRIDE_NE_MOBILE, useZx)
      ).then(
        AjxCallback.simpleClosure(
          function(controller, dialog, useZx, callback, resp) {
            if (!useZx) {
              ZxModule.prototype.__doStartStopModule.call(this, false, callback);
              ZxModule.prototype.__doEnableDisableModuleOnStartup.call(this, false, callback);
            }

            var mod = ZeXtras.getModuleItem('ZxCore', this.getServerName());
            mod.fireLoadMethods();

            if (useZx) {
              ZxAdminRequest.sendRequest(
                this.getCurrentServer(),
                'ZxMobile',
                'isHandleRegistered'
              ).then(AjxCallback.simpleClosure(
                function(controller, callback, resp) {
                  var response = resp.response;
                  controller.setInstanceValue(
                    ZxCore.KEY_MOBILE_HANDLER_REGISTERED,
                    response.isRegistered
                  );
                  if (!response.isRegistered) {
                    controller.popupWarningDialog(
                      zxMsg('MSG_COR_question_override_network_mobile_feature_changed_please_restart_mailboxd')
                    );
                  }
                  if (typeof callback === 'function') {
                    callback();
                  }
                },
                this,
                controller,
                callback
              ), ZxUtils.catchError);
            } else {
              if (typeof callback === 'function') {
                callback();
              }
            }
            dialog.popdown();
          },
          this,
          controller,
          dialog,
          useZx,
          callback
        ),
        ZxUtils.catchError
      );
    },
    [
      controller, msgDialog, useZx, callback
    ]
  );

  msgDialog.setMessage(
    zxMsg('MSG_COR_question_override_network_mobile_feature_enabling'),
    DwtMessageDialog.WARNING_STYLE,
    ZxUtils.getTitleWithIcon('ZxMobile', 'ZxMobile - ' + zxMsg('LBL_Override_mobile_NE'))
  );

  msgDialog.registerCallback(DwtDialog.YES_BUTTON, setOverrideCallback);
  msgDialog.registerCallback(DwtDialog.NO_BUTTON,
    new AjxCallback(
      this,
      function(dialog, callback) {
        if (typeof callback === 'function') {
          callback();
        }
        dialog.popdown();
      },
      [
        msgDialog,
        callback
      ]
    ));

  if (!useZx) {
    setOverrideCallback.run();
  } else {
    msgDialog.popup();
  }
};

/**
 * Method invoked when Mobile Provisioning is enabled/disabled.
 * @param {boolean} enable
 * @this ZxMobile
 */
ZxMobile.doEnableDisableProvisioning = function(enable) {
  var controller = ZaApp.getInstance().getCurrentController();
  if (enable) {
    controller.popupWarningDialog(zxMsg('MSG_MOB_Enable_provisioning_feature', ['<a target="_blank" href="'+ZxLinks.MSG_Mobile_Device_Management+'">', '</a>']));
  }
};

/**
 * Method invoked when is asked to save some modified values.
 * @param {{}} mods
 * @this ZxMobile
 */
ZxMobile.prototype.modifyMethod = function(mods) {
  if (ZxUtils.isObjectEmpty(mods)) {
    return;
  }

  if (typeof mods.properties !== 'undefined') {

    if (mods.properties[ZxMobile.KEY_NOTIFICATION_LATENCY] < 0 ||
        mods.properties[ZxMobile.KEY_NOTIFICATION_LATENCY] > 60) {
      this.getCurrentController().popupWarningDialog(zxMsg('MSG_MOB_Invalid_latency_value'));
      return;
    }

    ZxModule.prototype.modifyMethod.call(this, mods);
  }
};

/**
 * Invoke the function to delete a device history.
 * @param {DwtSelectionEvent} ev
 * @param {[]} selection
 */
ZxMobile.doDeleteDeviceHistory = function(ev, selection) {
  var item = ZeXtras.getModuleItem(ZxMobile, this.getForm().getInstance().id);

  if (selection.length !== 1) {
    return;
  }

  selection = selection[0];
  item._doDeleteDeviceHistory(
    selection[ZxMobile.COL_ID_ACCOUNT],
    selection[ZxMobile.COL_ID_DEVICE_ID],
    selection[ZxMobile.COL_ID_SERVER]
  );
};

/**
 * Delete a device history.
 * @param {string} accountEmail
 * @param {string} deviceID
 * @param {string} server
 */
ZxMobile.prototype._doDeleteDeviceHistory = function (accountEmail, deviceID, server) {
  var controller = this.getCurrentController(),
    helpButton = new DwtDialog_ButtonDescriptor(
      ZaXDialog.HELP_BUTTON,
      ZaMsg.TBB_Help,
      DwtDialog.ALIGN_LEFT,
      new AjxCallback(
        this,
        ZxUtils.openURL,
        [ZxLinks.HELP_MOB_Remove_device]
      )
    ),
    msgDialog = new DwtMessageDialog({
      buttons: [DwtDialog.YES_BUTTON, DwtDialog.NO_BUTTON],
      extraButtons: [helpButton],
      title: ZaMsg.ACL_D,
      parent: controller.getShell()
    }),
    message = zxMsg('MSG_MOB_Sure_remove_device') + '<br>';

  message += '<span style="margin-left: 10px;">' + zxMsg('LBL_Account') + ': ' + accountEmail + '</span><br>';
  message += '<span style="margin-left: 10px;">' + zxMsg('LBL_MOB_Device_id') + ': ' + deviceID + '</span>';

  msgDialog.setMessage(message);
  msgDialog.registerCallback(
    DwtDialog.YES_BUTTON,
    new AjxCallback(
      this,
      function(account, devId, server, dialog) {
        ZxAdminRequest.sendRequest(
          server,
          'ZxMobile',
          'doRemoveDevice',
          {
            accountName: account,
            deviceId: devId
          }
        ).then(
          AjxCallback.simpleClosure(
            function(dialog) {
              this.fireLoadMethods();
              dialog.popdown();
            },
            this,
            dialog
          ),
          ZxUtils.catchError
        );
      },
      [accountEmail, deviceID, server, msgDialog]
    )
  );
  msgDialog.popup();
};

/**
 * Invoke the function to reset the server's device cache.
 * @param {DwtSelectionEvent} ev
 * @param {[]} selection
 */
ZxMobile.doResetDevice = function(ev, selection) {
  var item = ZeXtras.getModuleItem(ZxMobile, this.getForm().getInstance().id);
  if (selection.length !== 1) {
    return;
  }
  selection = selection[0];
  item._doResetDevice(
    selection[ZxMobile.COL_ID_ACCOUNT],
    selection[ZxMobile.COL_ID_DEVICE_ID],
    selection[ZxMobile.COL_ID_SERVER]
  );
};

/**
 * Reset server's device cache.
 * @param {string} accountEmail
 * @param {string} deviceId
 * @param {string} userServer
 */
ZxMobile.prototype._doResetDevice = function(accountEmail, deviceId, userServer) {
  var helpButton = new DwtDialog_ButtonDescriptor(
    ZaXDialog.HELP_BUTTON,
    ZaMsg.TBB_Help,
    DwtDialog.ALIGN_LEFT,
    new AjxCallback(
      this,
      ZxUtils.openURL,
      ZxLinks.HELP_MOB_Reset_device
    )
  ),
    msgDialog = new DwtMessageDialog({
      buttons: [DwtDialog.YES_BUTTON, DwtDialog.NO_BUTTON],
      extraButtons: [helpButton],
      title: zxMsg('LBL_MOB_ResetDevice'),
      parent: this.getCurrentController().getShell()
    }),
    message = [zxMsg('MSG_MOB_Sure_reset_session')];

  message.push('<span style="margin-left:10px;">' + zxMsg('LBL_Account') + ': ' + accountEmail + '</span>');
  message.push('<span style="margin-left:10px;">' + zxMsg('LBL_MOB_Device_id') + ': ' + deviceId + '</span>');

  msgDialog.setMessage(message.join('<br>'), null, zxMsg('LBL_MOB_ResetDevice'));
  msgDialog.registerCallback(
    DwtDialog.YES_BUTTON,
    new AjxCallback(
      this,
      function(account, devId, server, dialog) {
        ZxAdminRequest.sendRequest(
          server,
          'ZxMobile',
          'doResetDevice',
          {
            accountName: account,
            deviceId: devId
          }
        ).then(
          AjxCallback.simpleClosure(
            function(dialog, resp) {
              this.fireLoadMethods();
              dialog.popdown();
            },
            this,
            dialog
          ),
          ZxUtils.catchError
        );
      },
      [accountEmail, deviceId, userServer, msgDialog]
    )
  );

  msgDialog.popup();
};

/**
 * Handle the selection on a device on device list.
 * @param {DwtSelectionEvent} ev
 */
ZxMobile._onDeviceSelected = function(ev) {
  if (ev.detail !== DwtListView.ITEM_DBL_CLICKED) { return; }
  ZxMobile.doShowDeviceDetails.call(this, ev, [ev.item]);
};

/**
 * Filter the list of the mobile devices to show only the connected devices.
 * @param {boolean} value
 * @param {DwtKeyEvent} event
 * @param {XForm} xform
 */
ZxMobile._showOnlyOnlineDevices = function(value, event, xform) {
  //console.log(arguments);
};

/**
 * Invoke the function to show the device details.
 * @param {DwtSelectionEvent} ev
 * @param {[]} selection
 */
ZxMobile.doShowDeviceDetails = function(ev, selection) {
  var item = ZeXtras.getModuleItem(ZxMobile, this.getForm().getInstance().id),
    selectedItem;

  if (!item) {
    return;
  }

  if (selection.length !== 1) {
    return;
  }

  selectedItem = selection[0];

  item._doShowDeviceDetails(selectedItem);
};

/**
 * Show device details.
 * @param {{}} deviceData
 * @param {{}} resp
 */
ZxMobile.prototype._doShowDeviceDetails = function(deviceData) {
  var controller = this.getCurrentController(),
    dialog;

  dialog = new ZxDeviceDetailsDialog(
    controller.getShell(),
    deviceData[ZxMobile.COL_ID_SERVER],
    deviceData[ZxMobile.COL_ID_ACCOUNT],
    deviceData[ZxMobile.COL_ID_DEVICE_ID]
  );
  dialog.popup();
};

/**
 * Invoke the function to suspend the device synchronization.
 * @param {DwtSelectionEvent} ev
 * @param {[]} selection
 */
ZxMobile.doSuspendDeviceSync = function(ev, selection) {
  var item = ZeXtras.getModuleItem(ZxMobile, this.getForm().getInstance().id);

  if (selection.length !== 1) {
    return;
  }

  selection = selection[0];
  item._doSuspendResumeDeviceSync(
    false,
    selection[ZxMobile.COL_ID_ACCOUNT],
    selection[ZxMobile.COL_ID_DEVICE_ID],
    selection[ZxMobile.COL_ID_SERVER]
  );
};

/**
 * Invoke the function to resume the device synchronization.
 * @param {DwtSelectionEvent} ev
 * @param {[]} selection
 */
ZxMobile.doResumeDeviceSync = function(ev, selection) {
  var item = ZeXtras.getModuleItem(ZxMobile, this.getForm().getInstance().id);

  if (selection.length !== 1) {
    return;
  }

  selection = selection[0];
  item._doSuspendResumeDeviceSync(
    true,
    selection[ZxMobile.COL_ID_ACCOUNT],
    selection[ZxMobile.COL_ID_DEVICE_ID],
    selection[ZxMobile.COL_ID_SERVER]
  );
};

/**
 * Suspend or resume the device synchronization..
 * @param {boolean} sync if <code>true</code> resume, if <code>false</code> suspend.
 * @param {string} accountEmail
 * @param {string} deviceId
 * @param {string} userServer
 */
ZxMobile.prototype._doSuspendResumeDeviceSync = function(sync, accountEmail, deviceId, userServer) {
  ZxAdminRequest.sendRequest(
    userServer,
    'ZxMobile',
    sync ? 'doResumeDeviceSync' : 'doSuspendDeviceSync',
    {
      deviceId: deviceId,
      accountName: accountEmail
    }
  ).then(
    AjxCallback.simpleClosure(
      function(sync, result) {
        var controller = this.getCurrentController();
        controller.popupMsgDialog(
          zxMsg('MSG_MOB_Device_sync_response',
            [sync ? zxMsg('MSG_MOB_Resumed') : zxMsg('MSG_MOB_Suspended')])
        );
        this.fireLoadMethods();
      },
      this,
      sync
    ),
    ZxUtils.catchError
  );
};

/**
 * Invoke the function to wipe the device data.
 * User should be scared to use this function.
 * @param {DwtSelectionEvent} ev
 * @param {[]} selection
 */
ZxMobile.doWipeDevice = function(ev, selection) {
  var item = ZeXtras.getModuleItem(ZxMobile, this.getForm().getInstance().id);

  if (selection.length !== 1) {
    return;
  }

  selection = selection[0];
  item._doWipeDevice(
    selection[ZxMobile.COL_ID_ACCOUNT],
    selection[ZxMobile.COL_ID_DEVICE_ID],
    selection[ZxMobile.COL_ID_SERVER]
  );
};

/**
 * Wipe the device data, showing a dialog in order to confirm the operation.
 * User should be scared to use this function.
 * @param {string} accountEmail
 * @param {string} deviceId
 * @param {string} userServer
 */
ZxMobile.prototype._doWipeDevice = function(accountEmail, deviceId, userServer) {
  var controller = this.getCurrentController(),
    wipeButton,
    checkbox,
    dialog = {},
    message = [],
    wipeCallback;

  message.push(zxMsg('MSG_MOB_Sure_to_use_remote_wipe1'));
  message.push('</br><ul><li>' + zxMsg('LBL_Account') + ': ' + accountEmail + '</li>' +
    '<li>' + zxMsg('LBL_MOB_Device_id') + ': ' + deviceId + '</li></ul></br>');
  message.push('<input id="' + ZxMobile.ID_WIPE_CONFIRM_CHECKBOX + '" ' +
    'name="' + ZxMobile.ID_WIPE_CONFIRM_CHECKBOX + '" type="checkbox" value="doWipe" />');
  message.push(zxMsg('MSG_MOB_Sure_to_use_remote_wipe2'));

  wipeButton = new DwtDialog_ButtonDescriptor(
    ZxMobile.ID_WIPE_CONFIRM_BUTTON,
    '',//zxMsg('LBL_MOB_Wipe_device'),
    DwtDialog.ALIGN_RIGHT,
    function() {}
  );

  dialog = new DwtMessageDialog({
    buttons: [DwtDialog.CANCEL_BUTTON],
    extraButtons: [wipeButton],
    title: zxMsg('LBL_MOB_Wipe_device'),
    parent: controller.getShell()
  });

  wipeButton.callback = new AjxCallback(
    this,
    function(dialog, account, devId, server) {
      ZxAdminRequest.sendRequest(
        server,
        'ZxMobile',
        'doWipeDevice',
        {
          deviceId: devId,
          accountName: account,
          confirm: true
        }
      ).then(
        AjxCallback.simpleClosure(
          function(dialog, resp) {
            dialog.popdown();
            var controller = this.getCurrentController();
            this.fireLoadMethods();
            controller.popupMsgDialog(zxMsg('MSG_MOB_Wipe_command_requested'));
          },
          this,
          dialog
        ),
        ZxUtils.catchError
      );
    },
    [dialog, accountEmail, deviceId, userServer]
  );

  dialog.__timerCounter = 10;
  dialog.__checkValue = false;

  dialog.setMessage(
    message.join(''),
    DwtMessageDialog.WARNING_STYLE,
    zxMsg('LBL_MOB_Wipe_device')
  );

  dialog._timedAction = new AjxTimedAction(
    dialog,
    function() {
      if (this.__timerCounter > 0) {
        this.__timerCounter = this.__timerCounter - 1;
        this.getButton(ZxMobile.ID_WIPE_CONFIRM_BUTTON)
          .setText(zxMsg('LBL_MOB_Wipe_device') + ' (' + this.__timerCounter + ')');
        AjxTimedAction.scheduleAction(this._timedAction, 1000);
      } else {
        this.getButton(ZxMobile.ID_WIPE_CONFIRM_BUTTON)
          .setText(zxMsg('LBL_MOB_Wipe_device'));
        this.getButton(ZxMobile.ID_WIPE_CONFIRM_BUTTON)
          .setEnabled(this.__checkValue);
      }
    }
  );

  dialog.registerCallback(
    DwtDialog.CANCEL_BUTTON,
    new AjxCallback(
      dialog,
      function() {
        this.popdown();
        AjxTimedAction.cancelAction(this._timedAction);
        this.__timerCounter = 10;
        this.__checkValue = false;
      }
    )
  );

  dialog.getButton(ZxMobile.ID_WIPE_CONFIRM_BUTTON)
    .setText(zxMsg('LBL_MOB_Wipe_device') + ' (' + dialog.__timerCounter + ')');
  dialog.getButton(ZxMobile.ID_WIPE_CONFIRM_BUTTON)
    .setEnabled(false);

  dialog.popup();

  checkbox = document.getElementById(ZxMobile.ID_WIPE_CONFIRM_CHECKBOX);
  wipeCallback = AjxCallback.simpleClosure(
    function(ev) {
      var checked = ev.target.checked;
      this.__checkValue = checked;
      if (this.__timerCounter <= 0) {
        this.getButton(ZxMobile.ID_WIPE_CONFIRM_BUTTON)
          .setEnabled(this.__checkValue);
      }
    },
    dialog
  );

  if (!!checkbox.addEventListener) {
    checkbox.addEventListener(
      'change',
      wipeCallback
    );
  } else {
    checkbox.attachEvent(
      'onchange',
      wipeCallback
    );
  }

  AjxTimedAction.scheduleAction(dialog._timedAction, 1000);
};


/**
 * Perform a search when a button is pressed.
 * @param {{}} searchType
 */
ZxMobile.doButtonsSearch = function(searchType) {
  var controller = ZaApp.getInstance().getSearchListController(),
    busyId = Dwt.getNextId(),
    callback,
    searchTypes = searchType,
    searchParams;

  if ((searchType[0] === ZaSearch.COSES) && (ZxUtils.isZ6())) {
    controller.popupWarningDialog(zxMsg('MSG_MOB_Z6_not_supported'));
    return;
  }

  callback = new AjxCallback(
    controller,
    controller.searchCallback,
    {
      limit: controller.RESULTSPERPAGE,
      show: true,
      busyId: busyId
    }
  );
  controller._currentQuery = '(zimbraFeatureMobileSyncEnabled=TRUE)';

  if (controller.setSearchTypes) {
    controller.setSearchTypes(searchTypes);
  }
  // search attributes
  controller.fetchAttrs = AjxBuffer.concat(
    ZaAlias.searchAttributes,
    ',',
    ZaDistributionList.searchAttributes,
    ',',
    ZaResource.searchAttributes,
    ',',
    ZaSearch.standardAttributes
  );
  // set current pagenum
  controller._currentPageNum = 1;

  searchParams = {
    query: controller._currentQuery,
    domain: controller._currentDomain,
    types: searchTypes,
    attrs: controller.fetchAttrs,
    callback: callback,
    controller: controller,
    showBusy: true,
    busyId: busyId,
    busyMsg: ZaMsg.BUSY_SEARCHING,
    skipCallbackIfCancelled: false
  };
  ZaSearch.searchDirectory(searchParams);
};

/**
 * Get the header list used into the active sessions list.
 * (Basically it unlock the server visualization view)
 * @return {ZaListHeaderItem[]}
 */
ZxMobile.getSessionsListHeader = function() {
  var headerList = ZxMobile.getSessionsListHeaderDelegated(),
    header;

  header = new ZaListHeaderItem(ZxMobile.COL_ID_SERVER, zxMsg('LBL_MOB_Server'), null, '200px',
    true, ZxMobile.COL_ID_SERVER, false, true);
  header.defaultPosition = headerList.length + 1;
  headerList.push(header);
  return headerList;
};

/**
 * Get the header list used into the active sessions list in the delegated admin view.
 * @return {ZaListHeaderItem[]}
 */
ZxMobile.getSessionsListHeaderDelegated = function() {
  var headerList = [],
    position = 0,
    header;

  header = new ZaListHeaderItem(ZxMobile.COL_ID_IS_ONLINE, '', 'Mobile', '16px',
    false, ZxMobile.COL_ID_IS_ONLINE, false, true);
  header.defaultPosition = position++;
  header.icons = {
    trueIcon: 'Mobile',
    trueHint: zxMsg('LBL_MOB_Connected'),
    falseHint: zxMsg('LBL_MOB_Disconnected')
  };
  header.hintField = ZxMobile.COL_ID_IS_ONLINE_HINT;
  headerList.push(header);

  header = new ZaListHeaderItem(ZxMobile.COL_ID_LAST_SEEN, zxMsg('LBL_MOB_Last_seen'), null, '100px',
    true, ZxMobile.COL_ID_LAST_SEEN_RAW, false, true);
  header.defaultPosition = position++;
  headerList.push(header);

  header = new ZaListHeaderItem(ZxMobile.COL_ID_HAS_MOBILE_PASSWORD, '', 'PadLock', '16px',
    true, ZxMobile.COL_ID_HAS_MOBILE_PASSWORD, false, true);
  header.defaultPosition = position++;
  header.icons = {
    trueIcon: 'Padlock',
    trueHint: zxMsg('LBL_MOB_Mobile_pwd_set'),
    falseHint: zxMsg('LBL_MOB_Mobile_pwd_not_set')
  };
  header.hintField = ZxMobile.COL_ID_LAST_SEEN_HINT;
  headerList.push(header);

  header = new ZaListHeaderItem(ZxMobile.COL_ID_ACCOUNT, zxMsg('LBL_Account'), null, '*',
    true, ZxMobile.COL_ID_ACCOUNT, false, true);
  header.defaultPosition = position++;
  headerList.push(header);

  header = new ZaListHeaderItem(ZxMobile.COL_ID_DEVICE, zxMsg('LBL_MOB_Device'), null, '150px',
    true, ZxMobile.COL_ID_DEVICE, false, true);
  header.defaultPosition = position++;
  headerList.push(header);

  header = new ZaListHeaderItem(ZxMobile.COL_ID_DEVICE_ID, zxMsg('LBL_MOB_Device_id'), null, '200px',
    true, ZxMobile.COL_ID_DEVICE_ID, false, true);
  header.defaultPosition = position++;
  headerList.push(header);

  header = new ZaListHeaderItem(ZxMobile.COL_ID_STATUS, zxMsg('LBL_Status'), null, '100px',
    true, ZxMobile.COL_ID_STATUS, false, true);
  header.defaultPosition = position++;
  headerList.push(header);

  header = new ZaListHeaderItem(ZxMobile.COL_ID_PROTOCOL, zxMsg('LBL_MOB_EAS'), null, '50px',
    true, ZxMobile.COL_ID_PROTOCOL, false, true);
  header.defaultPosition = position++;
  headerList.push(header);

  return headerList;
};

/**
 * Return the array of the descriptors of optionals buttons used in the ZxListView
 * associated to the session list.
 * @return {ZxListViewButton[]}
 */
ZxMobile.getSessionListButtons = function() {
  var array = [];

  array.push(new ZxListViewButton(
    'Delete',
    ZaMsg.ACL_D,
    ZxMobile.doDeleteDeviceHistory
  ));

  array.push(new ZxListViewButton(
    'Refresh',
    zxMsg('LBL_MOB_ResetDevice'),
    ZxMobile.doResetDevice
  ));

  array.push(new ZxListViewButton(
    'Information',
    zxMsg('LBL_details'),
    ZxMobile.doShowDeviceDetails
  ));

  array.push(new ZxListViewButton(
    'Pause',
    zxMsg('LBL_MOB_Suspend_sync'),
    ZxMobile.doSuspendDeviceSync
  ));

  array.push(new ZxListViewButton(
    'Play',
    zxMsg('LBL_MOB_Resume_sync'),
    ZxMobile.doResumeDeviceSync
  ));

  array.push(new ZxListViewButton(
    'MobileWipe',
    zxMsg('LBL_MOB_Wipe_device'),
    ZxMobile.doWipeDevice
  ));

   return array;
};

/**
 * Return true if the account or COS count is configured with ZxMobile.
 * @return {boolean}
 * @this Dwt_Alert_XFormItem
 */
ZxMobile.checkNotHasAccountOrCos = function() {
  var item = this.getForm().getInstance(),
    cosCount = 0,
    accountCount = 0;

  if (!isNaN(Number(item.attrs.stats[ZxMobile.KEY_ACCOUNT_ENABLED]))) {
    accountCount = Number(item.attrs.stats[ZxMobile.KEY_ACCOUNT_ENABLED]);
  }

  if (!isNaN(Number(item.attrs.stats[ZxMobile.KEY_COS_ENABLED]))) {
    cosCount = Number(item.attrs.stats[ZxMobile.KEY_COS_ENABLED]);
  }

  return (cosCount + accountCount) <= 0;
};

ZxMobile.getEnabledCosNumber = function() {
  var item = ZeXtras.getModuleItem(ZxMobile, this.getForm().getInstance().id);
  return item.attrs.stats[ZxMobile.KEY_COS_ENABLED];
};

ZxMobile.getEnabledAccountNumber = function() {
  var item = ZeXtras.getModuleItem(ZxMobile, this.getForm().getInstance().id);
  return item.attrs.stats[ZxMobile.KEY_ACCOUNT_ENABLED];
};

/**
 * Get the status message for a device, converting it from the associated
 * integer tag.
 * @param {number} statusByte
 * @return {string}
 */
ZxMobile.getDeviceStatusString = function(statusByte) {
  switch (statusByte) {
  case 0:
    return zxMsg('LBL_MOB_Need_provision');

  case 1:
    return zxMsg('LBL_MOB_Normal');

  case 2:
    return zxMsg('LBL_MOB_Suspended');

  case 3:
    return zxMsg('LBL_MOB_Wipe_required');

  case 4:
    return zxMsg('LBL_MOB_Wipe_completed');

  default:
    return ZaMsg.LBL_HOmeStatusUnknown;
  }
};

/**
 * Check if ZxMobile is not using instant notification.
 * @return {boolean}
 * @this {ZxMobile}
 */
ZxMobile.checkNotUsingInstant = function() {
  var usingInstant = this.getInstanceValue('attrs/properties/' + ZxMobile.KEY_USE_INSTANT_NOTIFICATIONS);
  return !usingInstant;
};

ZxMobile.checkMobilePwdEnabled = function() {
  var mobilePwdEnabled = this.getInstanceValue(ZxMobile.A_enableDevicePassword);
  return (mobilePwdEnabled === 'TRUE');
};

ZxMobile.checkMobileSyncEnabled = function() {
  var mobileSyncEnabled = this.getInstanceValue(ZaAccount.A_zimbraFeatureMobileSyncEnabled ||
    ZaCos.A_zimbraFeatureMobileSyncEnabled);
  return (mobileSyncEnabled === 'TRUE');
};

ZxMobile.checkMobilePolicyEnabled = function() {
  if (!ZxMobile.checkMobileSyncEnabled.call(this)) { return false; }
  var mobilePolicyEnabled = this.getInstanceValue(ZaAccount.A_zimbraFeatureMobilePolicyEnabled ||
    ZaCos.A_zimbraFeatureMobilePolicyEnabled);
  return (mobilePolicyEnabled === 'TRUE');
};

ZxMobile.isPwdRequired = function() {
  var policyPwdEnabled = this.getInstanceValue(ZaAccount.A_zimbraMobilePolicyDevicePasswordEnabled ||
    ZaCos.A_zimbraMobilePolicyDevicePasswordEnabled);
  return (policyPwdEnabled === 'TRUE');
};

/**
 * Generate a random password and show a dialog displaying it.
 * If user accept to use the generated password, will be setted into the right place.
 * @param {DwtSelectionEvent} ev
 * @this {Dwt_Button_XFormItem}
 */
ZxMobile.doGenerateDevicePassword = function(ev) {
  var passwordGenerated = ZxUtils.generateRandomPassword(12, 2, 2),
    msgDialog = new DwtMessageDialog({
      buttons: [DwtDialog.YES_BUTTON, DwtDialog.DISMISS_BUTTON],
      title: zxMsg('LBL_MOB_Random_mobile_password'),
      parent: ZaApp.getInstance().getCurrentController()._shell
    });

  msgDialog.setMessage(
    [
      zxMsg('LBL_Password') + ': ',
      '<h1>' + passwordGenerated + '</h1>',
      zxMsg('MSG_MOB_Set_generated_password')
    ].join('')
  );

  msgDialog.registerCallback(
    DwtDialog.YES_BUTTON,
    new AjxCallback(
      this,
      function(dialog) {
        this.setInstanceValue(passwordGenerated, ZxMobile.A_devicePassword);
        this.setInstanceValue(passwordGenerated, ZxMobile.A_devicePasswordConfirm);
        this.getForm().parent.setDirty(true);
        dialog.popdown();
      },
      [msgDialog]
    )
  );

  msgDialog.popup();
};

/**
 *
 * @param {number} dateMSec
 * @return {string}
 */
ZxMobile.agoTime = function(dateMSec) {
  var deltaMSec = (new Date()).getTime() - dateMSec;
  var durationStr = AjxDateUtil.computeDuration(deltaMSec, false, true);
  return durationStr ? (durationStr + " " + AjxMsg.ago) : null;
};
