
 /*
 * ***** BEGIN LICENSE BLOCK *****
 * Copyright (C) 2011-2016 ZeXtras
 *
 * The contents of this file are subject to the ZeXtras EULA;
 * you may not use this file except in compliance with the EULA.
 * You may obtain a copy of the EULA at
 * http://www.zextras.com/zextras-eula.html
 * ***** END LICENSE BLOCK *****
*/

function ZxDeviceDetailsDialog(parent, server, accountName, deviceId) {
  if (arguments.length === 0) { return; }
  this._standardButtons = [DwtDialog.OK_BUTTON];

  this._extraButtons = [
    new DwtDialog_ButtonDescriptor(
      ZxDeviceDetailsDialog.REFRESH_BUTTON,
      ZaMsg.TBB_Refresh,
      DwtDialog.ALIGN_LEFT,
      new AjxCallback(this, this._refreshButtonListener)
    )
  ];

  ZaXDialog.call(
    this,
    parent,
    null,
    ZxUtils.getTitleWithIcon('ZxMobile', zxMsg('LBL_MOB_Device_details')),
    '420px',
    '*',
    null,
    ZxAddBlockedAppDialog.DLG_ID
  );
  this.attrs = {};
  this.initForm(this.getMyXModel(), this.getMyXForm());
  this._helpURL = ZxLinks.HELP_MOB_Mobile;

  this.server = server;
  this.accountName = accountName;
  this.deviceId = deviceId;
}

ZxDeviceDetailsDialog.prototype = new ZaXDialog;
ZxDeviceDetailsDialog.prototype.constructor = ZxDeviceDetailsDialog;

ZxDeviceDetailsDialog.REFRESH_BUTTON = ++DwtDialog.LAST_BUTTON;
ZxDeviceDetailsDialog.DLG_ID = 'ZxDeviceDetailsDialog';

ZxDeviceDetailsDialog.prototype._refreshButtonListener = function() {
  this.setObject({ attrs: {} });
  this._getDeviceDetails();
};

/**
 *
 */
ZxDeviceDetailsDialog.prototype._getDeviceDetails = function() {
  ZxAdminRequest.sendRequest(
    this.server,
    'ZxMobile',
    'getDeviceStatistics',
    {
      accountName: this.accountName,
      deviceId: this.deviceId
    }
  ).then(
    AjxCallback.simpleClosure(
      function(resp) {
        this._processGetDeviceDetailsResponse(resp);
      },
      this
    )
  );
};

/**
*
*/
ZxDeviceDetailsDialog.prototype.popup = function() {
 ZaXDialog.prototype.popup.call(this);
 this._getDeviceDetails();
};

/**
 * @param {{}} resp
 */
ZxDeviceDetailsDialog.prototype._processGetDeviceDetailsResponse = function(resp) {
  if (resp.response) {
    this.setObject({ attrs: resp.response });
  }
};

/**
 * @return {{}}
 */
ZxDeviceDetailsDialog.prototype.getMyXModel = function() {
 var xModelObject = {
   items: []
 };
 return xModelObject;
};

/**
 *
 * @return {{numCols: number, items: Array}}
 */
ZxDeviceDetailsDialog.prototype.getMyXForm = function() {
 var xFormObject = {
   numCols: 1,
   items: [
     {
       type: _GROUP_,
       width: 410,
       cssStyle: 'padding:5px;',
       numCols: 2,
       colSizes: [135, '*'],
       items: [
         {
           type: _OUTPUT_,
           ref: 'attrs/accountEmail',
           label: zxMsg('LBL_Account') + ':',
           cssStyle: 'font-weight:bold;'
         },
         {
           type: _OUTPUT_,
           ref: 'attrs/accountServer',
           label: zxMsg('LBL_MOB_Server') + ':',
           visibilityChecks: [function() { return ZxUtils.amINotDelegated(); }]
         },
         {
           type: _OUTPUT_,
           ref: 'attrs/status',
           label: zxMsg('LBL_Status') + ':',
           getDisplayValue: function(val) {
             return ZxMobile.getDeviceStatusString(val);
           }
         },
         {
           type: _OUTPUT_,
           ref: 'attrs/hasMobilePassword',
           label: zxMsg('LBL_MOB_Has_mobile_password') + ':',
           getDisplayValue: function(val) {
             return val ? zxMsg('LBL_Set') : zxMsg('LBL_Not_set');
           }
         },
         {type: _SPACER_, colSpan: 2},
         {
           type: _OUTPUT_,
           ref: 'attrs/firstSeen',
           label: zxMsg('LBL_MOB_Registerd_on') + ':',
           getDisplayValue: function(val) {
             if (val > 0) {
               return ZxUtils.formatDateShort(new Date(val));
             } else {
               return zxMsg('LBL_NA');
             }
           }
         },
         {
           type: _OUTPUT_,
           ref: 'attrs/lastSeen',
           label: zxMsg('LBL_MOB_Last_seen') + ':',
           getDisplayValue: function(val) {
             if (val > 0) {
               return ZxUtils.formatDateShort(new Date(val));
             } else {
               return zxMsg('LBL_NA');
             }
           }
         },
         {
           type: _OUTPUT_,
           ref: 'attrs/lastPingTimeout',
           label: zxMsg('LBL_MOB_Last_ping_timeout') + ':',
           getDisplayValue: function(val) {
             if (val > 0) {
               return Math.ceil(val / 1000) + 's';
             } else {
               return zxMsg('LBL_NA');
             }
           }
         },
         {type: _SPACER_, colSpan: 2},
         {
           type: _OUTPUT_,
           ref: 'attrs/deviceType',
           label: zxMsg('LBL_MOB_Device') + ':'
         },
         {
           type: _OUTPUT_,
           ref: 'attrs/deviceId',
           label: zxMsg('LBL_MOB_Device_id') + ':'
         },
         {
           type: _OUTPUT_,
           ref: 'attrs/userAgent',
           label: zxMsg('LBL_MOB_User_agent') + ':'
         },
         {
           type: _OUTPUT_,
           ref: 'attrs/protocolVersion',
           label: zxMsg('LBL_MOB_EAS') + ':'
         }
       ]
     },
     {
       type: _TOP_GROUPER_,
       label: zxMsg('LBL_MOB_Synced_items'),
       numCols: 2,
       colSizes: [185, 185],
       items: [
         {
           type: _GROUP_,
           numCols: 2,
           items: [
             {
               type: _OUTPUT_,
               ref: 'attrs/numEmails',
               label: zxMsg('LBL_MOB_Emails') + ':'
             },
             {
               type: _OUTPUT_,
               ref: 'attrs/numEvents',
               label: zxMsg('LBL_MOB_Events') + ':'
             }
           ]
         },
         {
           type: _GROUP_,
           numCols: 2,
           items: [
             {
               type: _OUTPUT_,
               ref: 'attrs/numContacts',
               label: zxMsg('LBL_MOB_Contacts') + ':'
             },
             {
               type: _OUTPUT_,
               ref: 'attrs/numTasks',
               label: zxMsg('LBL_MOB_Tasks') + ':'
             }
           ]
         },
         {
           type: _GROUP_,
           numCols: 2,
           colSpan: 2,
           items: [
             {
               type: _OUTPUT_,
               ref: 'attrs/numBadItems',
               label: zxMsg('LBL_MOB_Skipped') + ':'
             }
           ]
         },
         {
           type: _GROUP_,
           numCols: 2,
           colSpan: 2,
           items: [
             {
               type: _OUTPUT_,
               ref: 'attrs/numRemoteFolders',
               label: zxMsg('LBL_MOB_Remote_folders') + ':'
             }
           ]
         }
       ]
     }
   ]
 };
 return xFormObject;
};
