/*
 * ***** BEGIN LICENSE BLOCK *****
 * Copyright (C) 2011-2016 ZeXtras
 *
 * The contents of this file are subject to the ZeXtras EULA;
 * you may not use this file except in compliance with the EULA.
 * You may obtain a copy of the EULA at
 * http://www.zextras.com/zextras-eula.html
 * ***** END LICENSE BLOCK *****
 */

/**
 * Create the ZxNotifications object.
 * @constructor
 */
function ZxNotifications() {
  ZxModule.call(this, ZxNotifications.MOD_ID, zxMsg('LBL_ZxNotifications'), 'ZxNotifications');

  this.registerLoadMethod(this.loadMethod, 'ZxNotificationsInit');
//  this.registerExtraToolbar(ZxNotifications.initExtraToolbar);
//  this.registerPopupMenu(ZxNotifications.initPopupMenu);

  this._helpURL = ZxLinks.HELP_NOT_Notifications;

  this.attrs[ZxModule.KEY_ENABLED_ON_STARTUP] = true;
  this.attrs[ZxModule.KEY_RUNNING] = true;
}

ZxNotifications.prototype = new ZxModule();
ZxNotifications.prototype.constructor = ZxNotifications;
ZxNotifications.MOD_ID = 'ZxNotifications';

ZxNotifications.KEY_DATE = 'date';
ZxNotifications.KEY_LEVEL = 'level';
ZxNotifications.KEY_SUBJECT = 'subject';
ZxNotifications.KEY_TEXT = 'text';
ZxNotifications.KEY_ID = 'id';
ZxNotifications.KEY_READ = 'ack';

ZxNotifications.KEY_NOTIFICATIONS = 'notifications';

ZxNotifications.SEVERITY_TO_CODE = {
  Debug: 0,
  Information: 1,
  Warning: 2,
  Error: 3,
  Critical: 4
};
ZxNotifications.CODE_TO_SEVERITY = [
  'Debug',
  'Information',
  'Warning',
  'Error',
  'Critical'
];
ZxNotifications.CODE_TO_LEVEL = [
  zxMsg('LBL_LVL_debug'),
  zxMsg('LBL_LVL_information'),
  zxMsg('LBL_LVL_warning'),
  zxMsg('LBL_LVL_error'),
  zxMsg('LBL_LVL_critical')
];

ZaOperation.ZE_MARK_ALL_READ = ++ZA_OP_INDEX;
ZaOperation.ZE_SHOW_OP_QUEUE = ++ZA_OP_INDEX;

ZxNotifications.prototype.markAllAsRead = function(ev) {
  var server = this.getCurrentServer(),
    controller = this.getCurrentController(),
    data = controller._view._containedObject.attrs.notifications,
    row,
    ack,
    id,
    n;

  // TODO: Optimize the callback generation, do not put into a loop!
  for (n = 0; n < data.length; n = n + 1) {
    row = data[n];
    ack = row[ZxNotifications.KEY_READ];
    if (!ack) {
      id = row[ZxNotifications.KEY_ID];
      ZxAdminRequest.sendRequest(
        this.getCurrentServer(),
        'ZxCore',
        'setNotificationAttr',
        {
          notificationId: id,
          key: 'ack',
          value: true
        }
      ).then(AjxCallback.simpleClosure(
        function(n, resp) {
          var controller = this.getCurrentController(),
            childrens = controller._view._localXForm.getChildren(),
            listView,
            currentDiv,
            row,
            i;

          for (i = 0; i < childrens.length; i = i + 1) {
            if (childrens[i] instanceof ZxNotificationsListView) {
              listView = childrens[i];
              break;
            }
          }
          if (!listView) {
            return;
          }

          currentDiv = listView._listDiv.childNodes[n];
          row = currentDiv.getElementsByTagName('tr')[0];

          row.className = row.className.replace('ZXNotificationUnread', '');
          this.attrs.notifications[n][ZxNotifications.KEY_READ] = true;
          listView._list.get(n)[ZxNotifications.KEY_READ] = true;
        },
        this,
        n
      ),
      function(error) {
        if (error.getCode != null) {
          if (error.getCode() === ZxErrorCode.UNABLE_TO_FIND_NOTIFICATIONS) {
            // Ignore the error
            return;
          }
        }
        ZxUtils.catchError(error, true);
      });
    }
  }
};

ZxNotifications.doMarkAllAsRead = function(ev) {
  var item = ZeXtras.getModuleItem(ZxNotifications, this.getForm().getInstance().id);
  item.markAllAsRead(ev);
};

/**
 * Show the current operation queue.
 * @param {DwtEvent} ev
 */
ZxNotifications.prototype.showOpQueue = function(ev) {
  var controller = this.getCurrentController(),
    opManager = ZeXtras.getOperationsManager();

  opManager.requestUpdate().then(
    AjxCallback.simpleClosure(
      function(controller, operations) {
        var counter = function(listObj) {
          var count = 0,
            tmpName;
          for (tmpName in listObj) {
            if (!listObj.hasOwnProperty(tmpName)) { continue; }
            count += 1;
          }
          return count;
        };
        if (counter(operations) <= 0) {
          controller.popupMsgDialog(zxMsg('MSG_NOT_no_op_queued'));
          return;
        }

        var dialog = new ZxOpQueueDialog(
          controller.getShell(),
          '550px',
          '200px',
          zxMsg('LBL_NOT_ops_queue'),
          operations
        );

        dialog.registerCallback(
          DwtDialog.OK_BUTTON,
          new AjxCallback(
            dialog,
            dialog.popdown
          )
        );
        dialog.popup();
      },
      this,
      controller
    )
  );
};

ZxNotifications.doShowOpQueue = function(ev) {
  var item = ZeXtras.getModuleItem(ZxNotifications, this.getForm().getInstance().id);
  item.showOpQueue(ev);
};

//ZxNotifications.initExtraToolbar = function() {
//  var op1 = new ZaOperation(ZaOperation.ZE_MARK_ALL_READ,
//      zxMsg('LBL_NOT_mark_all_as_read'),
//      zxMsg('LBL_NOT_mark_all_as_read'),
//      'RightObject', 'RightObject',
//      new AjxListener(this._item, this._item.markAllAsRead)),
//    op2 = new ZaOperation(ZaOperation.ZE_SHOW_OP_QUEUE,
//      zxMsg('LBL_NOT_op_queue'),
//      zxMsg('LBL_NOT_op_queue'),
//      'ZxOpQueue', 'ZxOpQueue',
//      new AjxListener(this._item, this._item.showOpQueue));
//
//  this._toolbarOperations[ZaOperation.ZE_MARK_ALL_READ] = op1;
//  this._toolbarOrder.push(ZaOperation.ZE_MARK_ALL_READ);
//
//  this._toolbarOperations[ZaOperation.ZE_SHOW_OP_QUEUE] = op2;
//  this._toolbarOrder.push(ZaOperation.ZE_SHOW_OP_QUEUE);
//};
//
//ZxNotifications.initPopupMenu = function() {
//  var op1 = new ZaOperation(ZaOperation.ZE_MARK_ALL_READ,
//      zxMsg('LBL_NOT_mark_all_as_read'),
//      zxMsg('LBL_NOT_mark_all_as_read'),
//      'RightObject', 'RightObject',
//      new AjxListener(this._item, this._item.markAllAsRead)),
//    op2 = new ZaOperation(ZaOperation.ZE_SHOW_OP_QUEUE,
//      zxMsg('LBL_NOT_op_queue'),
//      zxMsg('LBL_NOT_op_queue'),
//      'ZxOpQueue', 'ZxOpQueue',
//      new AjxListener(this._item, this._item.showOpQueue));
//
//  this._popupOperations[ZaOperation.ZE_MARK_ALL_READ] = op1;
//  this._popupOperations[ZaOperation.ZE_SHOW_OP_QUEUE] = op2;
//};

ZxNotifications.prototype._loadNotifications = function(ignoreErrors) {
  ignoreErrors = ignoreErrors || false;

  var errFcn = function(){};
  if (!ignoreErrors) {
    errFcn = this._getDefaultCatchErrorFcn();
  }

  ZxAdminRequest.sendRequest(
    this.getCurrentServer(),
    'ZxCore',
    'getAllNotifications'
  ).then(
    AjxCallback.simpleClosure(
      function(resp) {
        var response = resp.response,
          controller = this.getCurrentController(),
          notificationsArray = [],
          i;

        for (i = 0; i < response.notifications.length; i += 1) {
          var row = {};
          row[ZxNotifications.KEY_DATE] = response.notifications[i].date;
          row[ZxNotifications.KEY_LEVEL] = ZxNotifications.SEVERITY_TO_CODE[response.notifications[i].level];
          row[ZxNotifications.KEY_SUBJECT] = response.notifications[i].subject;
          row[ZxNotifications.KEY_TEXT] = response.notifications[i].text;
          row[ZxNotifications.KEY_ID] = response.notifications[i].id;
          row[ZxNotifications.KEY_READ] = response.notifications[i].ack;

          notificationsArray.push(row);
        }
        notificationsArray.reverse();

        this.attrs[ZxNotifications.KEY_NOTIFICATIONS] = notificationsArray;
        if (ZeXtras && ZeXtras.getNotificationsDialog()) {
          ZeXtras.getNotificationsDialog().notificationsLoaded(this);
        }
        controller.setInstanceValue(
          ZxNotifications.KEY_NOTIFICATIONS,
          notificationsArray
        );
      },
      this
    ),
    errFcn
  );
};

/**
 * Method called when new data need to be loaded.
 */
ZxNotifications.prototype.loadMethod = function(by, val) {
  this.attrs[ZxNotifications.KEY_NOTIFICATIONS] = [];

  this.attrs[ZxModule.KEY_ENABLED_ON_STARTUP] = true;
  this.attrs[ZxModule.KEY_RUNNING] = true;

  this._loadNotifications(false);
  var controller = this.getCurrentController();
  if (controller) {
    controller.setInstanceValue(
      'attrs/' + ZxModule.KEY_ENABLED_ON_STARTUP,
      true
    );
    controller.setInstanceValue(
      'attrs/' + ZxModule.KEY_RUNNING,
      true
    );
  }
};

/**
 * Get the count of the notifications filtered by level.
 * @param {number} level
 * @return {number}
 */
ZxNotifications.prototype.getCountByLevel = function(level) {
  if (!this.attrs[ZxNotifications.KEY_NOTIFICATIONS]) { return 0; }
  var array = this.attrs[ZxNotifications.KEY_NOTIFICATIONS],
    i = 0,
    count = 0;

  for (i = 0; i < array.length; i += 1) {
    if (array[i][ZxNotifications.KEY_LEVEL] === level) {
      count += 1;
    }
  }

  return count;
};

/**
 * Get the count of the unread notifications filtered by level.
 * @param {number} level
 * @return {number}
 */
ZxNotifications.prototype.getCountUnreadByLevel = function(level) {
  if (!this.attrs[ZxNotifications.KEY_NOTIFICATIONS]) { return 0; }
  var array = this.attrs[ZxNotifications.KEY_NOTIFICATIONS],
    i = 0,
    count = 0;

  for (i = 0; i < array.length; i += 1) {
    if (array[i][ZxNotifications.KEY_LEVEL] === level &&
        array[i][ZxNotifications.KEY_READ] === false) {
      count += 1;
    }
  }

  return count;
};

/**
 * Get if the warning message should be shown for whis server.
 * @return {boolean}
 */
ZxNotifications.prototype.shouldShowNotifications = function() {
  return (this.getCountUnreadByLevel(ZxNotifications.SEVERITY_TO_CODE.Critical) +
    this.getCountUnreadByLevel(ZxNotifications.SEVERITY_TO_CODE.Error) +
    this.getCountUnreadByLevel(ZxNotifications.SEVERITY_TO_CODE.Warning)) !== 0;
};

/**
 * Show the information for a given operation, in a dialog.
 * @param {ZxOperation} operation
 */
ZxNotifications.showOpInfo = function(operation) {
  var msgDialog = new ZxOperationDialog(
    ZaApp.getInstance().getCurrentController().getShell(),
    operation
  );
  msgDialog.setObject(operation);
  msgDialog.popup();
};

/**
 *
 * @param {{}} obj
 */
ZxNotifications.prototype.onInit = function(obj) {
  Q.delay(3000).then(
    AjxCallback.simpleClosure(
      this._loadNotifications,
      this,
      true
    )
  );
};
