/*
 * ***** BEGIN LICENSE BLOCK *****
 * Copyright (C) 2011-2016 ZeXtras
 *
 * The contents of this file are subject to the ZeXtras EULA;
 * you may not use this file except in compliance with the EULA.
 * You may obtain a copy of the EULA at
 * http://www.zextras.com/zextras-eula.html
 * ***** END LICENSE BLOCK *****
 */

/**
 * @param {number} modCount
 * @constructor
 */
function ZxNotificationsDialog(modCount) {
  this._remainsToLoad = modCount || 0;
  this._poppedUp = false;
  this._message = '';
  this._modules = {};
}

/**
 * Register a module have loaded it informations.
 * @param {ZxNotifications} module
 */
ZxNotificationsDialog.prototype.notificationsLoaded = function(module) {
  if (this._modules.hasOwnProperty(module.getServerName())) {
    return;
  }
  this._modules[module.getServerName()] = module;
  this._remainsToLoad -= 1;

  if (this._remainsToLoad === 0 && !this._poppedUp) {
    this._decideToNotify();
  }
};

ZxNotificationsDialog.prototype.popup = function() {
  var actxt = ZaApp.getInstance().getAppCtxt();

  if (!actxt) {
    Q.delay(500).then(
      AjxCallback.simpleClosure(
        this.popup,
        this
      )
    );
  } else {
    var msgDialog,
      showDetailsButton = new DwtDialog_ButtonDescriptor(
        ZaErrorDialog.DETAIL_BUTTON,
        ZaMsg.showDetails,
        DwtDialog.ALIGN_LEFT,
        new AjxCallback(
          this,
          this.showDetails
        )
      );

    msgDialog = new DwtDialog({
      standardButtons: [DwtDialog.DISMISS_BUTTON],
      extraButtons: [showDetailsButton],
      title: ZxUtils.getTitleWithIcon('ZxNotifications', zxMsg('LBL_ZeXtras_Notifications')),
      parent: actxt.getShell(),
      mode: DwtDialog.MODAL
    });

    msgDialog.registerCallback(
      DwtDialog.DISMISS_BUTTON,
      new AjxCallback(
        msgDialog,
        msgDialog.popdown
      )
    );

    msgDialog.registerCallback(
      ZaErrorDialog.DETAIL_BUTTON,
      new AjxCallback(
        this,
        this.showDetails,
        msgDialog
      )
    );

    msgDialog.setContent(this._message);

    msgDialog.popup();
  }
};

ZxNotificationsDialog.prototype.showDetails = function(msgDialog) {
  msgDialog.popdown();
  ZeXtras.followMenuItems([
    ZxMenuIds.ZEXTRAS,
    ZxMenuIds.ZEXTRAS_NOTIFICATION
  ]);
};

ZxNotificationsDialog.prototype._decideToNotify = function() {
  this._poppedUp = true;
  var i = 0,
    server,
    tmpMod,
    tmpRow,
    table = [],
    show = false,
    cols = ['<colgroup>',
      '<col width="16" />',
      '<col width="200" />',
      '<col width="16" />',
      '<col width="100" />',
      '<col width="16" />',
      '<col width="100" />',
      '</colgroup>'].join('');

  table.push(['<table style="padding: 10px 5px 0px 5px; cellspacing="0" cellpadding="0" border="0" width="100%" height="100%">',
    cols,
    '<tr style="background-color: rgb(185, 185, 185);"><th width="16px"></th><th width="200px">', zxMsg('LBL_SERVER_LIST_Server'),
    '</th><th width="16px"></th><th width="100px">', zxMsg('LBL_Errors'),
    '</th><th width="16px"></th><th width="100px">', zxMsg('LBL_Warnings'),
    '</th></tr></table>'].join(''));

  table.push('<div style="padding: 0px 5px 10px 5px; display: block; max-height: 200px; overflow-y: auto;">');
  table.push('<table cellspacing="0" cellpadding="0" border="0" width="100%" height="100%">');
  table.push(cols);
  for (server in this._modules) {
    i += 1;

    if (!this._modules.hasOwnProperty(server)) { continue; }
    tmpMod = this._modules[server];
    tmpRow = [];

    if (tmpMod.shouldShowNotifications()) {
      show = show || true;
      var redCount = tmpMod.getCountByLevel(ZxNotifications.SEVERITY_TO_CODE.Critical) +
          tmpMod.getCountByLevel(ZxNotifications.SEVERITY_TO_CODE.Error),
        redUnreadCount = tmpMod.getCountUnreadByLevel(ZxNotifications.SEVERITY_TO_CODE.Critical) +
          tmpMod.getCountUnreadByLevel(ZxNotifications.SEVERITY_TO_CODE.Error),
        yellowCount = tmpMod.getCountByLevel(ZxNotifications.SEVERITY_TO_CODE.Warning),
        yellowUnreadCount = tmpMod.getCountUnreadByLevel(ZxNotifications.SEVERITY_TO_CODE.Warning);

      tmpRow.push('<td><div class="ImgServer"></div></td>');
      tmpRow.push('<td>');
      tmpRow.push(AjxStringUtil.clip(server, 200));
      tmpRow.push('</td>');

      tmpRow.push('<td><div class="ImgCritical"></div></td>');
      tmpRow.push('<td>');
      if (redUnreadCount > 0) {
        tmpRow.push(zxMsg('MSG_Count_with_unread', [redCount, redUnreadCount]));
      } else {
        tmpRow.push(redCount);
      }

      tmpRow.push('<td><div class="ImgWarning"></div></td>');
      tmpRow.push('<td>');
      if (yellowUnreadCount > 0) {
        tmpRow.push(zxMsg('MSG_Count_with_unread', [yellowCount, yellowUnreadCount]));
      } else {
        tmpRow.push(yellowCount);
      }
      tmpRow.push('</td>');

      table.push('<tr class="Row ' + ((i % 2 === 0) ? 'RowOdd' : 'RowOven') + '">' + tmpRow.join('') + '</tr>');
    }
  }

  table.push('</table>');
  table.push('</div>');

  this._message = '<div style="padding: 5px;">' + zxMsg('MSG_NOT_DLG_Startup') + '</div>' + table.join('');

  if (show) {
    this.popup();
  }
};
