/*
 * ***** BEGIN LICENSE BLOCK *****
 * Copyright (C) 2011-2016 ZeXtras
 *
 * The contents of this file are subject to the ZeXtras EULA;
 * you may not use this file except in compliance with the EULA.
 * You may obtain a copy of the EULA at
 * http://www.zextras.com/zextras-eula.html
 * ***** END LICENSE BLOCK *****
 */

function ZxNotificationsListView() {
  DwtListView.apply(this, arguments);
  this.addSelectionListener(new AjxListener(this, this._listSelectionListener));
}

ZxNotificationsListView.KEY_COL_DATE = 'date';
ZxNotificationsListView.KEY_COL_ICON = 'icon';
ZxNotificationsListView.KEY_COL_SEVERITY = 'severity';
ZxNotificationsListView.KEY_COL_DESCRIPTION = 'description';

ZxNotificationsListView.prototype = new DwtListView();
ZxNotificationsListView.prototype.constructor = ZxNotificationsListView;

ZxNotificationsListView.prototype._getCellContents = function (htmlArr, idx, item, field, colIdx, params) {
  if (field === ZxNotificationsListView.KEY_COL_DATE) {
    htmlArr[idx++] = ZxUtils.computeWordyDateStr(new Date(), item[ZxNotifications.KEY_DATE]);

  } else if (field === ZxNotificationsListView.KEY_COL_ICON) {
    switch (item[ZxNotifications.KEY_LEVEL]) {
    case ZxNotifications.SEVERITY_TO_CODE.Debug:
      htmlArr[idx++] = AjxImg.getImageHtml('ZxNotifications');
      break;
    case ZxNotifications.SEVERITY_TO_CODE.Information:
      htmlArr[idx++] = AjxImg.getImageHtml('Information');
      break;

    case ZxNotifications.SEVERITY_TO_CODE.Warning:
      htmlArr[idx++] = AjxImg.getImageHtml('Warning');
      break;

    case ZxNotifications.SEVERITY_TO_CODE.Error:
    case ZxNotifications.SEVERITY_TO_CODE.Critical:
      htmlArr[idx++] = AjxImg.getImageHtml('Critical');
      break;

    default:
      htmlArr[idx++] = '';
    }
  } else if (field === ZxNotificationsListView.KEY_COL_SEVERITY) {
    htmlArr[idx++] = ZxNotifications.CODE_TO_LEVEL[item[ZxNotifications.KEY_LEVEL]];

  } else if (field === ZxNotificationsListView.KEY_COL_DESCRIPTION) {
    htmlArr[idx++] = item[ZxNotifications.KEY_SUBJECT].toString ?
        item[ZxNotifications.KEY_SUBJECT].toString() : item[ZxNotifications.KEY_SUBJECT];
  }

  return idx;
};

ZxNotificationsListView.prototype._getRowClass = function(item, params) {
  var className = 'ZX' + (item[ZxNotifications.KEY_LEVEL] ?
      ZxNotifications.CODE_TO_SEVERITY[item[ZxNotifications.KEY_LEVEL]].toLowerCase() :
      'information');

  if (!item[ZxNotifications.KEY_READ]) {
    className += ' ZXNotificationUnread';
  }
  return className;
};

ZxNotificationsListView.prototype._listSelectionListener = function(ev) {
  if (ev.detail === DwtListView.ITEM_SELECTED) {
    var item = ev.item,
      date = ZxUtils.computeWordyDateStr(new Date(), item[ZxNotifications.KEY_DATE]),
      level = ZxNotifications.CODE_TO_LEVEL[item[ZxNotifications.KEY_LEVEL]] || zxMsg('LBL_information'),
      subject = item[ZxNotifications.KEY_SUBJECT] || zxMsg('LBL_NOT_zx_notifications'),
      text = item[ZxNotifications.KEY_TEXT] || zxMsg('LBL_NOT_no_details'),
      title = date + ' - ' + level + ': ' + subject,
      title_item = this.parent.getItemsById('ZXNoteTitle')[0],
      detail_item = this.parent.getItemsById('ZXNoteDetail')[0];

    this.markAsRead(ev);

    title_item.updateElement(title);
    detail_item.updateElement(['<pre>', ZxUtils.FindURLs(text), '</pre>'].join(''));
    this.parent._setAllowSelection();
  }
};

/**
 * Mark a notification as read.
 */
ZxNotificationsListView.prototype.markAsRead = function(ev) {
  var item = ZaApp.getInstance().getCurrentController()._currentObject,
    id = ev.item[ZxNotifications.KEY_ID],
    ack = ev.item[ZxNotifications.KEY_READ],
    div = this.getSelectedItems().get(0);

  if (ack) {
    return;
  }

  ZxAdminRequest.sendRequest(
    item.getCurrentServer(),
    'ZxCore',
    'setNotificationAttr',
    {
      notificationId: id,
      key: 'ack',
      value: 'true'
    }
  ).then(
    AjxCallback.simpleClosure(
      function(ev, resp) {
        ev.item[ZxNotifications.KEY_READ] = true;
        var row = div.getElementsByTagName('tr')[0];
        row.className = row.className.replace('ZXNotificationUnread', '');
      },
      this,
      ev
    ),
    ZxUtils.catchError
  );
};

/**
 * Mark selected notifications as unread.
 * @this XFormItemFactory|ZxNotificationsListView
 */
ZxNotificationsListView.markAsUnread = function() {
  var controller = this.getForm().getController().getCurrentController(),
    childrens = controller._view._localXForm.getChildren(),
    listView,
    selection,
    rows,
    item,
    i;

  for (i = 0; i < childrens.length; i = i + 1) {
    if (childrens[i] instanceof ZxNotificationsListView) {
      listView = childrens[i];
      break;
    }
  }
  if (!listView) {
    return;
  }

  item = controller._currentObject;
  selection = listView.getSelection();
  rows = listView.getSelectedItems();
  if (selection.length < 1) {
    return;
  }

  for (i = 0; i < selection.length; i = i + 1) {
    var id = selection[i][ZxNotifications.KEY_ID],
      row = rows.get(i).getElementsByTagName('tr')[0];

    if (!selection[i][ZxNotifications.KEY_READ]) {
      continue;
    }

    ZxAdminRequest.sendRequest(
      item.getCurrentServer(),
      'ZxCore',
      'setNotificationAttr',
      {
        notificationId: id,
        key: 'ack',
        value: false
      }
    ).then(
      AjxCallback.simpleClosure(
        function(obj, row, resp) {
          obj[ZxNotifications.KEY_READ] = false;
          row.className += ' ZXNotificationUnread';
        },
        this,
        selection[i],
        row
      ),
      ZxUtils.catchError
    );
  }
};

/**
 * Get the header list for the ZxNotificationListView object.
 * @return {Array}
 */
ZxNotificationsListView.getHeaderList = function() {
  var headerList = [];
  // function(idPrefix, text, iconInfo, width, sortable, sortField, resizeable, visible)
  headerList.push(new ZaListHeaderItem(ZxNotificationsListView.KEY_COL_DATE,
    zxMsg('LBL_Date'), null, '130px', false, ZxNotificationsListView.KEY_COL_DATE, false, true));
  headerList.push(new ZaListHeaderItem(ZxNotificationsListView.KEY_COL_ICON,
    '', null, '22px', false, null, false, true));
  headerList.push(new ZaListHeaderItem(ZxNotificationsListView.KEY_COL_SEVERITY,
    zxMsg('LBL_NOT_severity'), null, '118px', false, ZxNotificationsListView.KEY_COL_SEVERITY, false, true));
  headerList.push(new ZaListHeaderItem(ZxNotificationsListView.KEY_COL_DESCRIPTION,
    zxMsg('LBL_NOT_subject'), null, 'auto', false, ZxNotificationsListView.KEY_COL_DESCRIPTION, false, true));
  // headerList.push(new ZaListHeaderItem('ZXNText', 'Detail', null, 'auto', null, 'ZXNText', false, true));
  return headerList;
};
