/*
 * ***** BEGIN LICENSE BLOCK *****
 * Copyright (C) 2011-2016 ZeXtras
 *
 * The contents of this file are subject to the ZeXtras EULA;
 * you may not use this file except in compliance with the EULA.
 * You may obtain a copy of the EULA at
 * http://www.zextras.com/zextras-eula.html
 * ***** END LICENSE BLOCK *****
 */

/**
 * Create the ZxPowerstore object.
 * @constructor
 */
function ZxPowerstore() {
  ZxModule.call(this, ZxPowerstore.MOD_ID, zxMsg('LBL_ZxPowerstore'), 'ZxPowerstore');

  this.registerLoadMethod(this.loadMethod, 'ZxPowerstore');
  this.registerModifyMethod(this.modifyMethod, 'ZxPowerstore');

  this._helpURL = ZxLinks.HELP_HSM_Powerstore;
}

ZxPowerstore.prototype = new ZxModule();
ZxPowerstore.prototype.constructor = ZxPowerstore;
ZxPowerstore.MOD_ID = 'ZxPowerstore';


ZxPowerstore.COL_ID = 0;
ZxPowerstore.COL_NAME = 1;
ZxPowerstore.COL_PATH = 2;
ZxPowerstore.COL_COMPRESSED = 3;
ZxPowerstore.COL_CURRENT = 4;
ZxPowerstore.COL_THRESHOLD = 5;
ZxPowerstore.COL_VOLUME_TYPE = 6;

ZxPowerstore.KEY_PRIMARIES = 'primaries';
ZxPowerstore.KEY_SECONDARIES = 'secondaries';
ZxPowerstore.KEY_S3_STORES = 's3_stores';
ZxPowerstore.KEY_INDEXES = 'indexes';
ZxPowerstore.KEY_HSMS = 'hsms';
ZxPowerstore.KEY_HSM_LIST = 'hsm_list';
ZxPowerstore.KEY_CACHE_PATH = 'cache_path';
ZxPowerstore.KEY_CACHE_SIZE = 'cache_size';
ZxPowerstore.KEY_HSM_MOVE_SCHEDULING_ENABLED = 'ZxPowerstore_MoveSchedulingEnabled';
ZxPowerstore.KEY_HSM_SCHEDULED_TIME = 'ZxPowerstore_MoveStartTime';

ZxPowerstore.STORAGE_TYPE_VOLUME = 'Volume';
ZxPowerstore.STORAGE_TYPE_S3 = 'S3';

ZxPowerstore.panelToVolume = {
  primaryVolumes: 1,
  secondaryVolumes: 2,
  indexVolumes: 10
};
ZxPowerstore.idToVolumeName = {
  1: 'Primary Store',
  2: 'Secondary Store',
  10: 'Index Store'
};
ZxPowerstore.idToVolumeType = {
  1: 'primary',
  2: 'secondary',
  10: 'index'
};

ZxPowerstore.prototype.loadMethod = function(by, val) {
  var server = this.getCurrentServer();

  this.attrs[ZxPowerstore.KEY_HSMS] = [];
  this.attrs[ZxPowerstore.KEY_HSM_LIST] = [];
  this.attrs[ZxPowerstore.KEY_PRIMARIES] = [];
  this.attrs[ZxPowerstore.KEY_SECONDARIES] = [];
  this.attrs[ZxPowerstore.KEY_INDEXES] = [];

  ZxAdminRequest.sendRequest(
    server,
    'ZxPowerstore',
    'getAllVolumes'
  ).then(AjxCallback.simpleClosure(
    function(resp) {
      var response = resp.response,
        controller = this.getCurrentController(),
        i = 0,
        primaries = [],
        secondaries = [],
        indexes = [],
        s3Stores = [];

      primaries = this.getLegacyConnector().Powerstore.convertGetAllVolumesResponseArray(response.primaries, true);
      this.attrs[ZxPowerstore.KEY_PRIMARIES] = primaries;
      controller.setInstanceValue(
        ZxPowerstore.KEY_PRIMARIES,
        primaries
      );

      secondaries = this.getLegacyConnector().Powerstore.convertGetAllVolumesResponseArray(response.secondaries, false);
      for (i = 0; i < secondaries.length; i++) {
        if (this.getLegacyConnector().Powerstore.isS3Store(secondaries[i])) {
          s3Stores.push(secondaries[i]);
        }
      }
      this.attrs[ZxPowerstore.KEY_S3_STORES] = s3Stores;
      controller.setInstanceValue(
        ZxPowerstore.KEY_S3_STORES,
        s3Stores
      );
      this.attrs[ZxPowerstore.KEY_SECONDARIES] = secondaries;
      controller.setInstanceValue(
        ZxPowerstore.KEY_SECONDARIES,
        secondaries
      );
    },
    this
  ), this._getDefaultCatchErrorFcn());

  ZxAdminRequest.sendRequest(
    server,
    'ZxPowerstore',
    'getHSMPolicy'
  ).then(AjxCallback.simpleClosure(
    function(resp) {
      var policies = resp.response.policies,
        controller = this.getCurrentController(),
        i,
        policy,
        row,
        hsms = [];

      this.attrs[ZxPowerstore.KEY_HSMS] = policies;
      controller.setInstanceValue(
        ZxPowerstore.KEY_HSMS,
        policies
      );

      for (i = 0; i < policies.length; i = i + 1) {
        policy = policies[i];
        if (policy.isValid) {
          row = [
            ZxUtils.itemTypeToString(policy.hsmType) + ':' + policy.hsmQuery,
            policy.isValid
          ];
        } else {
          row = [
            '<span style="color:red">' + policy.hsmQuery + '</span>',
            policy.isValid
          ];
        }
        hsms.push(row);
      }

      this.attrs[ZxPowerstore.KEY_HSM_LIST] = hsms;
      controller.setInstanceValue(
        ZxPowerstore.KEY_HSM_LIST,
        hsms
      );
    },
    this
  ), this._getDefaultCatchErrorFcn());

  ZxAdminRequest.sendRequest(
    server,
    'ZxPowerstore',
    'getPublicProperties'
  ).then(AjxCallback.simpleClosure(
    function(resp) {
      var properties = resp.response.properties,
        controller = this.getCurrentController();

      if (!this.attrs.properties) {
        this.attrs.properties = {};
      }

      this.attrs.properties[ZxPowerstore.KEY_HSM_SCHEDULED_TIME] = properties.ZxPowerstore_MoveStartTime;
      this.attrs.properties[ZxPowerstore.KEY_HSM_MOVE_SCHEDULING_ENABLED] =
        (properties.ZxPowerstore_MoveSchedulingEnabled === 'true');

      controller.setInstanceValue(
        'properties/' + ZxPowerstore.KEY_HSM_SCHEDULED_TIME,
        properties.ZxPowerstore_MoveStartTime
      );

      controller.setInstanceValue(
        'properties/' + ZxPowerstore.KEY_HSM_MOVE_SCHEDULING_ENABLED,
        (properties.ZxPowerstore_MoveSchedulingEnabled === 'true')
      );

    },
    this
  ), this._getDefaultCatchErrorFcn());

  this.getLegacyConnector().getServerConfig(
    val,
    this.getLegacyConnector().createCallback(
      this,
      function(config) {
        var controller = this.getCurrentController(),
          cachePath = config.get("localStoreCachePath").getValueOrInherited(),
          cacheSize = config.get("localStoreCacheSize").getValueOrInherited();

        this.attrs[ZxPowerstore.KEY_CACHE_PATH] = cachePath;
        controller.setInstanceValue(
          ZxPowerstore.KEY_CACHE_PATH,
          cachePath
        );
        this.attrs[ZxPowerstore.KEY_CACHE_SIZE] = ZxUtils.bytesToSize(cacheSize);
        controller.setInstanceValue(
          ZxPowerstore.KEY_CACHE_SIZE,
          ZxUtils.bytesToSize(cacheSize)
        );
      }
    )
  );
};

/**
 * Enable or disable the Powerstore Module.
 * @param {boolean} enable
 */
ZxPowerstore.prototype._doEnableDisableModule = function(enable) {
  var controller = this.getCurrentController(),
    msgDialog = new DwtMessageDialog({
      buttons: [DwtDialog.YES_BUTTON, DwtDialog.NO_BUTTON],
      title: '',
      parent: controller.getShell()
    }),
    license = ZeXtras.getLicense();

  if (!ZaSettings.isNetworkVersion()) {
    ZxModule.prototype.__doEnableDisableModuleOnStartup.call(this, enable);
    return;
  }

  if (license.getModules(this.getCurrentServer())[ZxCore.KEY_LICENSE_ZXPOWERSTORE].enabledOnNE) {
    ZxModule.prototype.__doEnableDisableModuleOnStartup.call(this, enable);
    return;
  }

  if (!enable) {
    return;
  }
  msgDialog.setMessage(zxMsg('MSG_COR_question_enable_powerstore_on_network'), null, ZxUtils.getTitleWithIcon(
    'ZxPowerstore',
    'ZxPowerstore'
  ));

  msgDialog.registerCallback(
    DwtDialog.YES_BUTTON,
    new AjxCallback(
      this,
      function(ev) {
        var opts = ZxUtils.createObjectFromRef('properties/' + ZxCore.KEY_OVERRIDE_NE_POWERSTORE, true);

        ZxAdminRequest.sendRequest(this.getCurrentServer(), 'ZxPowerstore', 'setPublicProperties', opts).then(
          AjxCallback.simpleClosure(
            function(resp) {
              ZxModule.prototype.__doEnableDisableModuleOnStartup.call(this, true);
            },
            this
          ),
          ZxUtils.catchError
        );
        msgDialog.popdown();
      }
    )
  );

  msgDialog.popup();
};

/**
 * Save the modifications of the module's interface.
 * @param {{}} mods
 */
ZxPowerstore.prototype.modifyMethod = function(mods) {
  if (ZxUtils.isObjectEmpty(mods)) {
    return;
  }

  if (mods[ZxPowerstore.KEY_CACHE_SIZE]) {
    if (!this.getLegacyConnector().Utils.StringUtils.stringIsSize(mods[ZxPowerstore.KEY_CACHE_SIZE])) {
      ZaApp.getInstance().getCurrentController().popupWarningDialog(
        ZxMsg.getMessage('MSG_Value_of_is_not_valid', [ZxMsg.getMessage('LBL_HSM_cache_size')])
      );
      return;
    }
    this.getLegacyConnector().modifyServerConfig(
      this.id,
      'localStoreCacheSize',
      'NUMBER',
      this.getLegacyConnector().Utils.StringUtils.sizeToByte(mods[ZxPowerstore.KEY_CACHE_SIZE])
    );
    delete mods[ZxPowerstore.KEY_CACHE_SIZE];
  }
  if (mods[ZxPowerstore.KEY_CACHE_PATH]) {
    // TODO: Improve the store of the hazelcast settings on the legacy code.
    this.getLegacyConnector().modifyServerConfig(
      this.id,
      'localStoreCachePath',
      'STRING',
      mods[ZxPowerstore.KEY_CACHE_PATH]
    );
    delete mods[ZxPowerstore.KEY_CACHE_PATH];
  }
  if (mods[ZxPowerstore.KEY_INDEXES]) {
    delete mods[ZxPowerstore.KEY_INDEXES];
  }
  if (mods[ZxPowerstore.KEY_PRIMARIES]) {
    delete mods[ZxPowerstore.KEY_PRIMARIES];
  }
  if (mods[ZxPowerstore.KEY_SECONDARIES]) {
    delete mods[ZxPowerstore.KEY_SECONDARIES];
  }
  if (mods[ZxPowerstore.KEY_HSMS]) {
    delete mods[ZxPowerstore.KEY_HSMS];
  }
  if (mods[ZxPowerstore.KEY_HSM_LIST]) {
    delete mods[ZxPowerstore.KEY_HSM_LIST];
  }
  if (mods[ZxPowerstore.KEY_S3_STORES]) {
    delete mods[ZxPowerstore.KEY_S3_STORES];
  }

  ZxModule.prototype.modifyMethod.call(this, mods);
};

/**
 * Trigger the change the type of the store selected in the panelIDFrom
 * @param {Dwt_Button_XFormItem} button
 * @param {DwtEvent} ev
 * @param {string} panelIDFrom
 * @param {string} paneIDTo
 */
ZxPowerstore.doMoveVolume = function(button, ev, panelIDFrom, paneIDTo) {
  var form = button.getForm(),
    fromList = form.getItemsById(panelIDFrom)[0],
    volumeData,
    item = ZeXtras.getModuleItem(ZxPowerstore, form.getInstance().id);

  if (fromList.getSelection().length === 0) {
    return;
  }
  volumeData = fromList.getSelection();

  if (item.getLegacyConnector().Powerstore.isS3Store(volumeData[0])) {
    // S3 Volumes cannot be set as primary
    return;
  }
  item._doMoveVolume(volumeData[0], paneIDTo);
};

/**
 * Change the volume to another type.
 * @param {PhysicalStoreDescriptor} volData
 * @param {string} destination
 */
ZxPowerstore.prototype._doMoveVolume = function(volData, destination) {
  var volumeID = volData.getId(),
    volumeName = volData.getName(),
    volumePath = volData.getPath(),
    isCompressed = volData.isCompressed(),
    isCurrent = volData.isCurrent(),
    threshold = volData.getCompressionThreshold(),
    dest = ZxPowerstore.panelToVolume[destination],
    reqParams = {},
    msgDialog;

  if (isCurrent) {
    ZaApp.getInstance().getCurrentController().popupWarningDialog(
      zxMsg('MSG_HSM_Cannot_move_current_volume', [volumeName, '<br>'])
    );
    return;
  }

  reqParams.volumeType = ZxPowerstore.idToVolumeType[dest];
  reqParams.volumeName = volumeName;
  reqParams.volumePath = volumePath;
  reqParams.volumeCompressed = isCompressed;
  reqParams.volumeThreshold = threshold;
  reqParams.volumeId = volumeID;

  ZxUtils.log(reqParams, 'Moving volume');

  msgDialog = new DwtMessageDialog({
    buttons: [DwtDialog.YES_BUTTON, DwtDialog.NO_BUTTON],
    title: '',
    parent: ZaApp.getInstance().getCurrentController().getShell()
  });

  msgDialog.setMessage(
    zxMsg('MSG_HSM_Set_volume_as',
      [
        volumeName,
        ZxPowerstore.idToVolumeName[dest]
      ]
      ),
    DwtMessageDialog.WARNING_STYLE,
    ZxUtils.getTitleWithIcon('ZxPowerstore', 'ZxPowerstore')
  );

  msgDialog.registerCallback(DwtDialog.YES_BUTTON, new AjxCallback(
    this,
    function(reqParams, msgDialog) {
      msgDialog.popdown();

      ZxAdminRequest.sendRequest(
        this.getCurrentServer(),
        'ZxPowerstore',
        'doUpdateVolume',
        reqParams
      ).then(AjxCallback.simpleClosure(function(resp) {
        ZaApp.getInstance().getCurrentController().popupMsgDialog(
          resp.response.message
        );
        this.fireLoadMethods();
      }, this), ZxUtils.catchError);

    },
    [reqParams, msgDialog]
  ));

  msgDialog.popup();
};

/**
 * Check if we have to show a waring in the case of invalid hsm rule found.
 * @return {boolean}
 * @this XFormItem
 */
ZxPowerstore.checkInvalidHSMPresence = function() {
  var hsms = this.getInstanceValue('attrs/' + ZxPowerstore.KEY_HSMS),
    i;

  for (i = 0; i < hsms.length; i = i + 1) {
    if (!hsms[i].isValid) {
      return true;
    }
  }
  return false;
};

ZxPowerstore.doStartOperations = function(ev) {
  var item = ZeXtras.getModuleItem(ZxPowerstore, this.getForm().getInstance().id);
  item._doStartOperations();
};

ZxPowerstore.prototype._doStartOperations = function() {
  var controller = this.getCurrentController(),
    msgDialog = new DwtMessageDialog({
      buttons: [DwtDialog.YES_BUTTON, DwtDialog.NO_BUTTON],
      title: zxMsg('MSG_HSM_Apply_now'),
      parent: controller.getShell()
    });

  msgDialog.setMessage(
    zxMsg('MSG_HSM_Wuold_you_apply_policies')
  );

  msgDialog.registerCallback(
    DwtDialog.YES_BUTTON,
    new AjxCallback(
      this,
      function(controller, msgDialog) {
        ZxAdminRequest.sendRequest(
          this.getCurrentServer(),
          'ZxPowerstore',
          'doMoveBlobs'
        ).then(
          AjxCallback.simpleClosure(
            function(controller, resp) {
              controller.popupMsgDialog(zxMsg('MSG_Operation_queued'));
              ZeXtras.getOperationsManager().requestUpdate();
            },
            this,
            controller
          ),
          ZxUtils.catchError
        );
        msgDialog.popdown();
      },
      [controller, msgDialog]
    )
  );

  msgDialog.registerCallback(
    DwtDialog.NO_BUTTON,
    new AjxCallback(
      this,
      function(controller, msgDialog) {
        controller.popupMsgDialog(zxMsg('MSG_HSM_Operation_aborted'));
        ZeXtras.getOperationsManager().requestUpdate();
        msgDialog.popdown();
      },
      [controller, msgDialog]
    )
  );

  msgDialog.popup();
};

/**
 * Trigger to stop the operations.
 */
ZxPowerstore.doStopOperations = function(ev) {
  var item = ZeXtras.getModuleItem(ZxPowerstore, this.getForm().getInstance().id);
  item._doStopOperations();
};

/**
 *
 */
ZxPowerstore.prototype._doStopOperations = function() {
  var msgDialog = new DwtMessageDialog({
    buttons: [DwtDialog.YES_BUTTON, DwtDialog.NO_BUTTON],
    title: '',
    parent: this.getCurrentController().getShell()
  });

  msgDialog.setMessage(
    zxMsg('MSG_HSM_Would_you_stop_operations'),
    DwtMessageDialog.WARNING_STYLE,
    ZxUtils.getTitleWithIcon('ZxPowerstore', 'ZxPowerstore')
  );
  msgDialog.registerCallback(
    DwtDialog.YES_BUTTON,
    new AjxCallback(
      this,
      function(dialog, controller) {
        ZxAdminRequest.sendRequest(
          this.getCurrentServer(),
          'ZxPowerstore',
          'doStopAllOperations'
        ).then(
          AjxCallback.simpleClosure(
            function(controller, resp) {
              controller.popupMsgDialog(
                zxMsg('MSG_HSM_Termination_signal_sent')
              );
              ZeXtras.getOperationsManager().requestUpdate();
            },
            this,
            controller
          ),
          ZxUtils.catchError
        );

        dialog.popdown();
      },
      [msgDialog, this.getCurrentController()]
    )
  );
  msgDialog.popup();
};
