/*
 * ***** BEGIN LICENSE BLOCK *****
 * Copyright (C) 2011-2016 ZeXtras
 *
 * The contents of this file are subject to the ZeXtras EULA;
 * you may not use this file except in compliance with the EULA.
 * You may obtain a copy of the EULA at
 * http://www.zextras.com/zextras-eula.html
 * ***** END LICENSE BLOCK *****
 */

function ZxAddModifyHSMDialog(controller, item, oldPolicy, oldPolicyString, newMode) {
  if (arguments.length === 0) {
    return;
  }
  this._standardButtons = [DwtDialog.OK_BUTTON, DwtDialog.CANCEL_BUTTON];
  this._newMode = newMode;
  this._item = item;
  ZaXDialog.call(
    this,
    controller.getShell(),
    null,
    ZxUtils.getTitleWithIcon(
      'ZxPowerstore',
      newMode ? zxMsg('LBL_HSM_Add_new_policy') : zxMsg('LBL_HSM_Modify_policy')
    ),
    void 0,
    '170px',
    void 0,
    void 0
  );
  this.initForm([], this.getMyXForm());
  this.setObject(oldPolicy);

  this.registerCallback(
    DwtDialog.OK_BUTTON,
    new AjxCallback(
      this,
      function(controller, newMode) {

        var policy,
          item = controller.getItem(),
          server = item.getCurrentServer(),
          reqParams,
          setCallback;

        if (this._containedObject.hsmType.length === 0) {
          controller.popupWarningDialog(
            zxMsg('MSG_HSM_No_item_types_defined')
          );
          return;
        }
        this.popdown();

        policy = [ZxUtils.itemTypeToString(this._containedObject.hsmType),
          ':', this._containedObject.hsmQuery].join('');

        setCallback = AjxCallback.simpleClosure(
          function(controller, server, newMode, resp) {
            if (resp.ok) {
              if (newMode) {
                controller.popupMsgDialog(zxMsg('MSG_HSM_Policy_added_successfully'));
              } else {
                controller.popupMsgDialog(zxMsg('MSG_HSM_Policy_edited_successfully'));
              }
            } else {
              if (newMode) {
                controller.popupWarningDialog(zxMsg('MSG_HSM_Error_adding_policy'));
              } else {
                controller.popupWarningDialog(zxMsg('MSG_HSM_Error_editing_policy'));
              }
            }
            controller.getItem().fireLoadMethods();
          },
          this,
          controller,
          server,
          newMode
        );

        if (!newMode) {
          if (oldPolicyString !== policy) {
            ZxAdminRequest.sendRequest(
              server,
              'ZxPowerstore',
              'setHSMPolicy',
              {
                hsmPolicy: policy,
                policyToAdd: true
              }
            ).then(AjxCallback.simpleClosure(
              function (server, response) {
                ZxAdminRequest.sendRequest(
                  server,
                  'ZxPowerstore',
                  'removeHSMPolicy',
                  {
                    hsmPolicy: oldPolicyString
                  }
                ).then(setCallback, ZxUtils.catchError);
              },
              this,
              server
            ), ZxUtils.catchError);
          }
        } else {

          ZxAdminRequest.sendRequest(
            server,
            'ZxPowerstore',
            'setHSMPolicy',
            {
              hsmPolicy: policy,
              policyToAdd: true
            }
          ).then(setCallback, ZxUtils.catchError);
        }
      },
      [controller, newMode]
    )
  );

  this._helpURL = ZxLinks.HELP_HSM_Dialog_AddModify_HSM;
}

ZxAddModifyHSMDialog.prototype = new ZaXDialog();
ZxAddModifyHSMDialog.prototype.constructor = ZxAddModifyHSMDialog;

ZxAddModifyHSMDialog.prototype.getMyXForm = function() {
  var xFormObject = {
    numCols: 2,
    colSizes: ['150px', '*'],
    items: [
      {
        type: _ZE_HSM_TYPES_,
        label: zxMsg('LBL_HSM_Items_to_move') + ': ',
        ref: 'hsmType'
      },
      {
        colSpan: 2,
        type: _ZE_HSM_QUERY_,
        ref: 'hsmQuery'
      }
    ]
  };
  return xFormObject;
};
