/*
 * ***** BEGIN LICENSE BLOCK *****
 * Copyright (C) 2011, 2012, 2013, 2014 ZeXtras
 *
 * The contents of this file are subject to the ZeXtras EULA;
 * you may not use this file except in compliance with the EULA.
 * You may obtain a copy of the EULA at
 * http://www.zextras.com/zextras-eula.html
 * ***** END LICENSE BLOCK *****
 */
/**
 * The dialog shown to allow to add or edit a volume.
 * @param {} parent
 * @param {string} title
 * @param {LegacyConnector} legacyConnector
 * @param {StoreDescriptor} item
 * @param {S3BucketConfiguration[]} s3BucketConfigurations
 * @param {string} serverId
 * @constructor
 */
function ZxModifyVolumeDialog(parent, title, legacyConnector, item, s3BucketConfigurations, serverId) {
  if (arguments.length === 0) {
    return;
  }
  this.mLegacyConnector = legacyConnector;
  this._standardButtons = [DwtDialog.OK_BUTTON, DwtDialog.CANCEL_BUTTON];
  this._data = {};
  this._buckets = s3BucketConfigurations;
  this._serverId = serverId || 'sad-asd-asd-asd-asd';
  ZaXDialog.call(this, parent, null, title, '450px', '250px', null, null);

  this.initForm(this.mLegacyConnector.Powerstore.getStoreDescriptorXModel(item), this.getMyXForm());
  this.setObject(item.clone());
  this._original = item;

  this.registerCallback(
    DwtDialog.OK_BUTTON,
    new AjxCallback(this, this._doSaveOrCreateVolume)
  );

  this._localXForm.addListener(
    DwtEvent.XFORMS_FORM_DIRTY_CHANGE,
    new AjxListener(
      this,
      this.handleXFormChange
    )
  );
  if (item.getTypeName() === "S3") {
    this._helpURL = ZxLinks.HELP_HSM_Dialog_AddModify_S3_Volume;
  } else {
    this._helpURL = ZxLinks.HELP_HSM_Dialog_AddModify_Volume;
  }
}
ZxModifyVolumeDialog.prototype = new ZaXDialog();
ZxModifyVolumeDialog.prototype.constructor = ZxModifyVolumeDialog;

/**
 * When a value is changed, check if we have to enable/disable the OK button
 * of the dialog.
 */
ZxModifyVolumeDialog.prototype.handleXFormChange = function() {
  var obj = this._containedObject;

  if (obj.getTypeName() === "LOCAL") {
    if (obj.getName() !== "" && obj.getPath() !== "" && (this.getObject().getValidatedCompressionThreshold() || !this.getObject().isCompressed())) {
      this._button[DwtDialog.OK_BUTTON].setEnabled(true);
    } else {
      this._button[DwtDialog.OK_BUTTON].setEnabled(false);
    }
  }

  if (obj.getTypeName() === "S3") {
    if (obj.getName() !== "") {
      this._button[DwtDialog.OK_BUTTON].setEnabled(true);
    } else {
      this._button[DwtDialog.OK_BUTTON].setEnabled(false);
    }
  }
};

/**
 * Save the modifications or create the new volume.
 * @private
 */
ZxModifyVolumeDialog.prototype._doSaveOrCreateVolume = function() {
  var storeDescriptor /** @type {StoreDescriptor} */ = this.getObject();

  if (storeDescriptor.getTypeName() === "LOCAL") {
    this._saveModifiedPhysicalVolume(
      ZaApp.getInstance().getCurrentController().getItem().getCurrentServer(),
      storeDescriptor
    );
    this.popdown();
    return;
  }

  if (storeDescriptor.getTypeName() === "S3") {
    this.mLegacyConnector.Powerstore.editS3StoreDescriptor(
      ZaApp.getInstance().getCurrentController().getItem().id,
      this._original,
      storeDescriptor,
      this.mLegacyConnector.createCallback(this, function(response) {
        ZaApp.getInstance().getCurrentController().getItem().fireLoadMethods();
      }),
      this.mLegacyConnector.createCallback(this, ZxUtils.catchError)
    );
    this.popdown();
    return;
  }
};

/**
 * @param {PhysicalStoreDescriptor} descriptor
 * @private
 */
ZxModifyVolumeDialog.prototype._saveModifiedPhysicalVolume = function(server, descriptor) {
  var reqParams = {
    volumeId: descriptor.getId(),
    volumeType: descriptor.getOrderName().toLowerCase(),
    volumeName: descriptor.getName(),
    volumePath: descriptor.getPath(),
    volumeCompressed: descriptor.isCompressed(),
    volumeThreshold: descriptor.getCompressionThreshold(),
    volumeCurrent: descriptor.isCurrent()
  };
  ZxUtils.log(reqParams, 'Editing Volume with properties:');

  ZxAdminRequest.sendRequest(
    server,
    'ZxPowerstore',
    'doUpdateVolume',
    reqParams
  ).then(
    AjxCallback.simpleClosure(function(controller, server, item, resp) {
      controller.popupMsgDialog(
        zxMsg('LBL_HSM_Volume_edited', [item.getName()])
      );
      controller.getItem().fireLoadMethods();
    }, this, ZaApp.getInstance().getCurrentController(), server, descriptor),
    AjxCallback.simpleClosure(function(controller, server, error) {
      ZxUtils.catchError(error);
      //controller.popupWaringDialog(error);
    }, this, ZaApp.getInstance().getCurrentController(), server));
};

/**
 * Return the XFormItem of the dialog.
 * @return {{numCols: number, colSizes: Array, items: Array}}
 */
ZxModifyVolumeDialog.prototype.getMyXForm = function() {

  var isPhysicalVolume = function() {
    return this.getInstanceValue('storeType') === "LOCAL";
  }, isS3Volume = function() {
    return this.getInstanceValue('storeType') === "S3";
  }, isS3InfrequentAccessEnabled = function () {
    return this.getInstanceValue('enableInfrequentAccess') === "TRUE";
  }, testS3Bucket = (function(_this) {
    return function (ev) {
      var bucketId = this.getInstance().getBucketConfigurationId();
      _this.mLegacyConnector.Powerstore.testS3BucketById(bucketId);
    };
  })(this);


  var credentialChoices = [];
  for(var i = 0; i < this._buckets.length; i++) {
    credentialChoices.push({
      label: this._buckets[i].getBucketName(),
      value: this._buckets[i].getId()
    });
  }

  return {
    numCols: 1,
    colSizes: ['*'],
    items: [
      {
          type: _SPACER_,
          align: _CENTER_,
          valign: _TOP_
        },
        {
          colSpan: 2,
          type: _OUTPUT_,
          align: _CENTER_,
          value: zxMsg('MSG_HSM_Edit_current_settings')
        },
        {
        type: _GROUP_,
        numCols: 2,
        colSizes: [150, '*'],
        cssStyle: 'margin-top: 5px; margin-left: 5px',
        isTabGroup: true,
        items: [
          {
            type: _SPACER_,
            align: _CENTER_,
            valign: _TOP_
          },
          {
            type: _TEXTFIELD_,
            ref: 'name',
            width: '280px',
            label: zxMsg('LBL_HSM_Volume_name') + ': ',
            labelLocation: _LEFT_
          },
          {
            type: _TEXTFIELD_,
            ref: 'volumePrefix',
            width: '280px',
            label: zxMsg('LBL_HSM_Volume_prefix') + ': ',
            labelLocation: _LEFT_,
            visibilityChangeEventSources: ['storeType'],
            visibilityChecks: [isS3Volume],
            enableDisableChangeEventSources: ['storeType'],
            enableDisableChecks: [function() { return false; }]
          },
          {
            type: _CHECKBOX_,
            labelCssClass: 'ZxCheckboxLabel',
            labelLocation: _LEFT_,
            ref: 'enableInfrequentAccess',
            label: zxMsg('LBL_S3_Enable_infrequent_access') + ': ',
            visibilityChangeEventSources: ['storeType'],
            visibilityChecks: [isS3Volume],
            trueValue: "TRUE",
            falseValue: "FALSE"
          },
          {
            type: _TEXTFIELD_,
            ref: 'infrequentAccessThresholdSize',
            width: '280px',
            label: zxMsg('LBL_S3_Infrequent_access_threshold') + ': ',
            labelLocation: _LEFT_,
            visibilityChangeEventSources: ['storeType'],
            visibilityChecks: [isS3Volume],
            enableDisableChangeEventSources: ['enableInfrequentAccess'],
            enableDisableChecks: [isS3InfrequentAccessEnabled]
          },
          {
            type: _OSELECT1_,
            ref: 'bucketConfigurationId',
            label: zxMsg('LBL_HSM_Store_Credential') + ': ',
            width: '280px',
            labelLocation: _LEFT_,
            visibilityChangeEventSources: ['storeType'],
            visibilityChecks: [isS3Volume],
            enableDisableChecks: [function() { return false; }],
            choices: credentialChoices,
            getDisplayValue: (function(_buckets) {
              return function(value) {
                if (typeof value === 'undefined') { return value; }
                for (var i = 0; i < _buckets.length; i++) {
                  if (_buckets[i].getId() === value) {
                    return _buckets[i].getBucketName()
                  }
                }
                return value;
              }
            }(this._buckets)),
            onChange: (function(_buckets) {
              return function(value, ev, form) {
                for (var i = 0; i < _buckets.length; i++) {
                  if (_buckets[i].getBucketName() === value) {
                    form.notifyListeners(DwtEvent.XFORMS_VALUE_CHANGED, ev);
                    this.setInstanceValue(_buckets[i].getId());
                    return;
                  }
                }
                form.notifyListeners(DwtEvent.XFORMS_VALUE_CHANGED, ev);
                this.setInstanceValue(value);
              }
            }(this._buckets))
          },
          {
            type: _TEXTFIELD_,
            ref: 'path',
            label: zxMsg('LBL_HSM_Path_for_the_volume') + ': ',
            width: '280px',
            labelLocation: _LEFT_,
            visibilityChangeEventSources: ['storeType'],
            visibilityChecks: [isPhysicalVolume]
          },
          {
            type: _CHECKBOX_,
            labelCssClass: 'ZxCheckboxLabel',
            labelLocation: _LEFT_,
            ref: 'compressed',
            label: zxMsg('LBL_HSM_Enable_compression') + ': ',
            visibilityChangeEventSources: ['storeType'],
            visibilityChecks: [isPhysicalVolume],
            trueValue: "TRUE",
            falseValue: "FALSE"
          },
          {
            type: _TEXTFIELD_,
            ref: 'compressionThreshold',
            label: zxMsg('LBL_HSM_Compression_Threshold') + ': ',
            width: '280px',
            labelLocation: _LEFT_,
            visibilityChangeEventSources: ['storeType'],
            visibilityChecks: [isPhysicalVolume],
            enableDisableChangeEventSources: ['compressed'],
            enableDisableChecks: [function() {
              return this.getInstanceValue('compressed') === "TRUE";
            }],
            onChange: function(value) {
              this.setInstanceValue(value)
            }
          },
          {
            type: _CHECKBOX_,
            labelCssClass: 'ZxCheckboxLabel',
            labelLocation: _LEFT_,
            ref: 'current',
            label: zxMsg('LBL_HSM_set_as_current') + ': ',
            trueValue: "TRUE",
            falseValue: "FALSE"
          },
          {
            type: _CELL_SPACER_
          },
          {
            type: _DWT_BUTTON_,
            width: '280px',
            label: zxMsg('LBL_Test_s3_credentials'),
            visibilityChangeEventSources: ['storeType'],
            visibilityChecks: [isS3Volume],
            onActivate: testS3Bucket
          },
          {
            type: _TEXTFIELD_,
            width: '280px',
            label: zxMsg('LBL_S3_AMZN_ARN') + ': ',
            visibilityChangeEventSources: ['storeType'],
            visibilityChecks: [isS3Volume],
            enableDisableChecks: [function () { return false; }],
            valueChangeEventSources: ['path', 'bucketConfigurationId'],
            getDisplayValue: (function(_buckets, _serverId) {
              return function(wtf) {
                if (this.getInstanceValue('storeType') !== "S3") {
                  return '';
                }
                var descriptor = this.getInstance();
                var bucket = void 0;
                for (var i = 0; i < _buckets.length; i++) {
                  if (_buckets[i].getId() === descriptor.getBucketConfigurationId()) {
                    bucket = _buckets[i];
                  }
                }
                if (typeof bucket === 'undefined') {
                  return '';
                }
                return descriptor.getARN(bucket, _serverId);
              }
            }(this._buckets, this._serverId))
          }
        ]
      }
    ]
  };
};

