/*
 * ***** BEGIN LICENSE BLOCK *****
 * Copyright (C) 2011-2016 ZeXtras
 *
 * The contents of this file are subject to the ZeXtras EULA;
 * you may not use this file except in compliance with the EULA.
 * You may obtain a copy of the EULA at
 * http://www.zextras.com/zextras-eula.html
 * ***** END LICENSE BLOCK *****
 */

/**
 * Dialog used to display informations about a powerstore volume.
 * @param {DwtShell} parent
 * @param {String} w
 * @param {String} h
 * @param {String} volName
 * @param {{}} statData
 * @constructor
 */
function ZxPowerstoreStatsDialog(parent, volName, statData) {
  if (arguments.length === 0) {
    return;
  }
  this._standardButtons = [DwtDialog.OK_BUTTON];

  var title = [zxMsg('LBL_HSM_Stats_for_volume'), ': ', volName].join('');

  ZaXDialog.call(this, parent, null, title, void 0, void 0, void 0, void 0);
  this.initForm([], this.getMyXForm());
  this.setObject(statData);

  this._helpURL = ZxLinks.HELP_HSM_Dialog_Stats;
}

ZxPowerstoreStatsDialog.prototype = new ZaXDialog();
ZxPowerstoreStatsDialog.prototype.constructor = ZxPowerstoreStatsDialog;

ZxPowerstoreStatsDialog.prototype.getMyXForm = function() {
  var xForm = {
    numCols: 2,
    width: '100%',
    colSizes: ['*', '*'],
    items: [
      {type: _OUTPUT_, align: _LEFT_, ref: 'id', label: zxMsg('LBL_HSM_Volume_id') + ': '},
      {type: _OUTPUT_, align: _LEFT_, ref: 'name', label: zxMsg('LBL_HSM_Volume_name') + ': '},
      {type: _OUTPUT_, align: _LEFT_, ref: 'path', label: zxMsg('LBL_Path') + ': '},
      {type: _OUTPUT_, align: _LEFT_, ref: 'compressed', label: zxMsg('LBL_HSM_Compressed') + ': '},
      {type: _OUTPUT_, align: _LEFT_, ref: 'threshold', label: zxMsg('LBL_HSM_Threshold') + ': '},
      {type: _OUTPUT_, align: _LEFT_, ref: 'current', label: zxMsg('LBL_HSM_Current') + ': '},
      {type: _OUTPUT_, align: _LEFT_, ref: 'bytesSaved', label: zxMsg('LBL_HSM_Disk_space_saved') + ': '},
      {type: _OUTPUT_, align: _LEFT_, ref: 'bytesSavedLast', label: zxMsg('LBL_HSM_Disk_space_saved_last') + ': '},
      {type: _OUTPUT_, align: _LEFT_, ref: 'lastItemMovedCount', label: zxMsg('LBL_HSM_Blob_moved_last') + ': '},
      {type: _OUTPUT_, align: _LEFT_, ref: 'lastMoveTimestamp', label: zxMsg('LBL_HSM_Last_move_time') + ': '},
      {type: _OUTPUT_, align: _LEFT_, ref: 'lastMoveDuration', label: zxMsg('LBL_HSM_Last_move_duration') + ': '}
    ]
  };
  return xForm;
};

/**
 * Display the statistics of a volume of the store.
 * @param {string} serverName
 * @param {PhysicalStoreDescriptor} volume
 * @param {boolean} showSize
 * @param {boolean} showNumBlobs
 */
ZxPowerstoreStatsDialog.showStats = function(serverName, volume, showSize, showNumBlobs) {
  ZxAdminRequest.sendRequest(
    serverName,
    'ZxPowerstore',
    'getVolumeStats',
    {
      volumeId: volume.getId(),
      showSize: showSize,
      showNumBlobs: showNumBlobs
    }
  ).then(
    AjxCallback.simpleClosure(function(tmpVolume, shell, resp) {
      var data = resp.response.volume,
        dialog;

      data.current = tmpVolume.isCurrent();
      dialog = new ZxPowerstoreStatsDialog(
        shell,
        tmpVolume.getName(),
        data
      );
      dialog.popup();
    }, this, volume, ZaApp.getInstance().getAppCtxt().getShell()),
    ZxUtils.catchError
  );
};
