/*
 * ***** BEGIN LICENSE BLOCK *****
 * Copyright (C) 2011-2016 ZeXtras
 *
 * The contents of this file are subject to the ZeXtras EULA;
 * you may not use this file except in compliance with the EULA.
 * You may obtain a copy of the EULA at
 * http://www.zextras.com/zextras-eula.html
 * ***** END LICENSE BLOCK *****
 */

function ZxHSMQueryWidget() {}

ZxHSMQueryWidget.REGEXP = /(before|after):[\+\-]?([0-9]*)(minutes?|hours?|days?|weeks?|months?|years?)\s?(.*)?/;
ZxHSMQueryWidget.TOKEN_NUM = 2;
ZxHSMQueryWidget.TOKEN_UNIT = 3;
ZxHSMQueryWidget.TOKEN_OPTIONS = 4;

XFormItemFactory.createItemType('_ZE_HSM_QUERY_', 'zx_hsm_query', ZxHSMQueryWidget, Composite_XFormItem);

ZxHSMQueryWidget.parseQuery = function(value, token) {
  var tokens = ZxHSMQueryWidget.REGEXP.exec(value);

  if (!tokens) {
    return '';
  }
  if (tokens[3].slice(-1) !== 's') {
    tokens[3] += "s";
  }
  return tokens[token];
};

ZxHSMQueryWidget.handleChange = function(obj, value, instance, event, token) {
  ZxUtils.log(value, instance, token);

  var tokens = ZxHSMQueryWidget.REGEXP.exec(instance),
    new_instance;
  tokens[token] = value;
  new_instance = tokens[1] + ':-' + tokens[2].replace(/^((?!0$)0)+/, '') + tokens[3] +
    ((typeof tokens[4] !== 'undefined') ? (' ' + tokens[4]) : '');

  ZxUtils.log(new_instance);
  if (ZxHSMQueryWidget.REGEXP.exec(new_instance) !== null) {
    obj.getForm().itemChanged(obj.getParentItem(), new_instance, event);
  } else {
    obj.getForm().refresh();
  }
};

ZxHSMQueryWidget.prototype.initializeItems = function() {
  this.items = [
    {
      type: _GROUP_,
      ref: '.',
      numCols: 3,
      colSizes: ['150px', '*', '*'],
      isTabGroup: true,
      items: [
        {
          type: _TEXTFIELD_,
          ref: '.',
          getDisplayValue: function(value) {
            return ZxHSMQueryWidget.parseQuery(value, ZxHSMQueryWidget.TOKEN_NUM);
          },
          elementChanged: function(value, instanceValue, event) {
            return ZxHSMQueryWidget.handleChange(this,
              value,
              instanceValue,
              event,
              ZxHSMQueryWidget.TOKEN_NUM);
          },
          label: zxMsg('LBL_HSM_Move_items_older_then') + ': '
        },
        {
          type: _OSELECT1_,
          ref: '.',
          labelLocation: _NONE_,
          getDisplayValue: function(value) {
            return ZxHSMQueryWidget.parseQuery(value, ZxHSMQueryWidget.TOKEN_UNIT);
          },
          elementChanged: function(value, instanceValue, event) {
            return ZxHSMQueryWidget.handleChange(this,
              value,
              instanceValue,
              event,
              ZxHSMQueryWidget.TOKEN_UNIT);
          },
          choices: [
            {value: 'years', label: AjxMsg.years},
            {value: 'months', label: AjxMsg.months},
            {value: 'weeks', label: AjxMsg.weeks},
            {value: 'days', label: AjxMsg.days},
            {value: 'hours', label: AjxMsg.hours},
            {value: 'minutes', label: AjxMsg.minutes}
          ]
        },
        {
          type: _TEXTFIELD_,
          ref: '.',
          getDisplayValue: function(value) {
            return ZxHSMQueryWidget.parseQuery(value, ZxHSMQueryWidget.TOKEN_OPTIONS);
          },
          elementChanged: function(value, instanceValue, event) {
            return ZxHSMQueryWidget.handleChange(this,
              value,
              instanceValue,
              event,
              ZxHSMQueryWidget.TOKEN_OPTIONS);
          },
          label: zxMsg('LBL_HSM_Additional_options') + ': '
        },
        { type: _SPACER_ }
      ]
    }
  ];

  Composite_XFormItem.prototype.initializeItems.call(this);
};
