/*
 * ***** BEGIN LICENSE BLOCK *****
 * Copyright (C) 2011-2016 ZeXtras
 *
 * The contents of this file are subject to the ZeXtras EULA;
 * you may not use this file except in compliance with the EULA.
 * You may obtain a copy of the EULA at
 * http://www.zextras.com/zextras-eula.html
 * ***** END LICENSE BLOCK *****
 */

function ZxPowerstoreHSMListView() {
  DwtListView.apply(this, arguments);
  this.addSelectionListener(new AjxListener(this, this._listSelectionListener));
}

ZxPowerstoreHSMListView.prototype = new DwtListView();
ZxPowerstoreHSMListView.prototype.constructor = ZxPowerstoreHSMListView;

ZxPowerstoreHSMListView.prototype._getCellContents = function(htmlArr, idx, item, field, colIdx, params) {
  htmlArr[idx++] = item[colIdx].toString ? item[colIdx].toString() : item[colIdx];
  return idx;
};

ZxPowerstoreHSMListView.prototype._listSelectionListener = function (ev) {
  var item = ev.item,
    hsm_list = this.parent.getItemsById('hsmList')[0];

  if (ev.detail === DwtListView.ITEM_DBL_CLICKED) {
    hsm_list.widget._doAddModifyRow(false);
  }
};

ZxPowerstoreHSMListView.getHeaderList = function() {
  var headerList = [];
  headerList.push(new DwtListHeaderItem({field:'ZXHSMPolicy', text:zxMsg('LBL_HSM_Policy'), width:'auto', resizeable:false, visible:true}));
  headerList.push(new DwtListHeaderItem({field:'ZXHSMValidity', text:zxMsg('LBL_HSM_Validity'), width:'auto', resizeable:false, visible:false}));
  return headerList;
};

/**
 * Add/Modify a row of the HSM list.
 * @param {DwtEvent} ev
 * @param {String} panelID
 * @param {boolean} newMode
 */
ZxPowerstoreHSMListView.doAddModifyRow = function(ev, panelID, newMode) {
  var list = ev.dwtObj.parent.getChildren(),
    i = 0,
    panel;

  for (i = 0; i < list.length; i = i + 1) {
    if (list[i] instanceof ZxPowerstoreHSMListView &&
        ZxUtils.indexOf(list[i]._className, panelID) !== -1) {
      panel = list[i];
      break;
    }
  }

  if (!panel) {
    return;
  }

  panel._doAddModifyRow(newMode);
};

ZxPowerstoreHSMListView.prototype._doAddModifyRow = function(newMode) {
  var m = this.firstSelIndex,
    elementSelected = [],
    controller = this.parent.getController().getCurrentController(),
    item = controller.getItem(),
    oldPolicy = {},
    oldPolicyString,
    dialog;

  if (!newMode && m === -1) {
    if (this._list.getArray().length === 1) {
      m = 0;
    } else {
      return;
    }
  }

  oldPolicy = {
    hsmType: [],
    hsmQuery: 'before:-0days'
  };

  if (!newMode) {
    if (!this._list.get(m)[1]) {
      controller.popupWarningDialog(
        zxMsg('MSG_HSM_Cannot_modify_invalid_policy')
      );
      return;
    }
    oldPolicy = controller._currentObject.attrs.hsms[m];
    oldPolicyString = this._list.get(m)[0];
  }

  dialog = new ZxAddModifyHSMDialog(
    controller,
    item,
    oldPolicy,
    oldPolicyString,
    newMode
  );
  dialog.popup();
};

ZxPowerstoreHSMListView.doDeleteRow = function(ev, panelID) {
  var list = ev.dwtObj.parent.getChildren(),
    i = 0,
    panel;

  for (i = 0; i < list.length;  i += 1) {
    if (list[i] instanceof ZxPowerstoreHSMListView &&
        ZxUtils.indexOf(list[i]._className, panelID) !== -1) {
      panel = list[i];
      break;
    }
  }

  if (!panel) {
    return;
  }

  panel._doDeleteRow();
};

ZxPowerstoreHSMListView.prototype._doDeleteRow = function(newMode) {
  var m = this.firstSelIndex,
    elementSelected = [],
    controller = this.parent.getController().getCurrentController(),
    item = controller.getItem(),
    validity,
    policy,
    dialog;

  if (m === -1) {
    return;
  }

  validity = this._list.get(m)[1];
  policy = controller._currentObject.attrs.hsm_list[m][0];
  if (!validity) {
    policy = policy.slice(24, -7);
  }

  dialog = new ZxDeleteHSMDialog(controller.getShell(), item, policy);
  dialog.popup();
};
