/*
 * ***** BEGIN LICENSE BLOCK *****
 * Copyright (C) 2011-2016 ZeXtras
 *
 * The contents of this file are subject to the ZeXtras EULA;
 * you may not use this file except in compliance with the EULA.
 * You may obtain a copy of the EULA at
 * http://www.zextras.com/zextras-eula.html
 * ***** END LICENSE BLOCK *****
 */

/**
 * List view used in the ZeXtras Powerstore module.
 * @constructor
 */
function ZxPowerstoreVolumeListView() {
  DwtListView.apply(this, arguments);
  this.addSelectionListener(new AjxListener(this, this._listSelectionListener));
}

ZxPowerstoreVolumeListView.prototype = new DwtListView();
ZxPowerstoreVolumeListView.prototype.constructor = ZxPowerstoreVolumeListView;

ZxPowerstoreVolumeListView.FIELD_ICON = 'icon';
ZxPowerstoreVolumeListView.FIELD_ID = 'id';
ZxPowerstoreVolumeListView.FIELD_NAME = 'name';
ZxPowerstoreVolumeListView.FIELD_PATH = 'path';
ZxPowerstoreVolumeListView.FIELD_COMPRESSED = 'compressed';
ZxPowerstoreVolumeListView.FIELD_CURRENT = 'current';
ZxPowerstoreVolumeListView.FIELD_THRESHOLD = 'threshold';

ZxPowerstoreVolumeListView.PANEL_PRIMARY = 'primaryVolumes';
ZxPowerstoreVolumeListView.PANEL_SECONDARY = 'secondaryVolumes';

ZxPowerstoreVolumeListView.prototype._getCellContents = function(htmlArr, idx, item, field, colIdx, params) {
  var modItem = ZaApp.getInstance().getCurrentController().getItem();
  var legacyConnector = modItem.getLegacyConnector();

  if (field === ZxPowerstoreVolumeListView.FIELD_COMPRESSED) {
    if (!legacyConnector.Powerstore.isS3Store(item)) {
      htmlArr[idx++] = item.isCompressed() ? '<div class="ImgZipDoc" style="margin: 0 auto"></div>' : '';
    }
    htmlArr[idx++] = '';

  } else if (field === ZxPowerstoreVolumeListView.FIELD_CURRENT) {
    htmlArr[idx++] = item.isCurrent() ? '<div class="ImgCheck" style="margin: 0 auto"></div>' : '';

  } else if (field === ZxPowerstoreVolumeListView.FIELD_ICON) {
    htmlArr[idx++] = legacyConnector.Powerstore.isS3Store(item) ?
      '<div class="ImgZXS3Storage" style="margin: 0 9px"></div>' :
      '<div class="ImgServer" style="margin: 0 auto"></div>';

  } else if (field === ZxPowerstoreVolumeListView.FIELD_NAME) {
    htmlArr[idx++] = item.getName();

  } else {
    htmlArr[idx++] = '';

  }
  return idx;
};

/**
 * When a row is selected in the list view, this method is invoked by the listview itself.
 * @param {DwtSelectionEvent} ev
 */
ZxPowerstoreVolumeListView.prototype._listSelectionListener = function(ev) {
  var panelID = this.getClassName().split(' ')[1];
//    vol_list = ev.dwtObj.parent.getItemsById(panelID)[0],
//    volumeType = ZxPowerstore.panelToVolume[panelID];

  if (ev.detail === DwtListView.ITEM_DBL_CLICKED) {
    this._doModifyRow(panelID, false);
  }
};

/**
 * Delete a row of the powerstore table.
 */
ZxPowerstoreVolumeListView.prototype._doDeleteRow = function() {
  var position = this.firstSelIndex,
    controller = this.parent.getController().getCurrentController(),
    item = void 0,
    msgDialog = new DwtMessageDialog({
      buttons: [DwtDialog.YES_BUTTON, DwtDialog.NO_BUTTON],
      title: '',
      parent: controller.getShell()
    });

  if (this.firstSelIndex === -1) {
    if (this.getList().size() === 1) {
      position = 0;
    } else {
      return;
    }
  }

  item = this.getList().getArray()[position];

  if (item.isCurrent()) {
    controller.popupWarningDialog(
      zxMsg('MSG_HSM_Cannot_delete_current_volume')
    );
    return;
  }

  msgDialog.setMessage(
    zxMsg(
      'MSG_HSM_confirm_delete_volume',
      [item.getName()]
    ),
    DwtMessageDialog.WARNING_STYLE,
    ZxUtils.getTitleWithIcon('ZxPowerstore', 'ZxPowerstore')
  );
  msgDialog.registerCallback(
    DwtDialog.YES_BUTTON,
    new AjxCallback(
      this,
      this._deleteItem,
      [
        msgDialog,
        item,
        controller
      ]
    )
  );

  msgDialog.popup();
};

ZxPowerstoreVolumeListView.prototype._deleteItem = function(dialog, item, controller) {
  var modItem = ZaApp.getInstance().getCurrentController().getItem();

  if (item.getTypeName() === "LOCAL") {
    ZxAdminRequest.sendRequest(
      controller.getItem().getCurrentServer(),
      'ZxPowerstore',
      'doDeleteVolume',
      {
        volumeId: item.getId(),
        name: item.getName(),
        current: item.isCurrent()
      }
    ).then(
      AjxCallback.simpleClosure(
        function(controller, resp) {
          controller.popupMsgDialog(resp.response);
          controller.getItem().fireLoadMethods();
        },
        this,
        controller
      ),
      ZxUtils.catchError
    );
  }

  if (item.getTypeName() === "S3") {
    modItem.getLegacyConnector().Powerstore.deleteS3StoreDescriptor(
      modItem.id,
      item,
      modItem.getLegacyConnector().createCallback(this, function(args, response) {
        args[0].getItem().fireLoadMethods();
      }, controller),
      modItem.getLegacyConnector().createCallback(void 0, ZxUtils.catchError)
    )
  }

  dialog.popdown();
};

/**
 * Trigger the action for deleting a row from the volume list.
 * @param {DwtEvent} ev
 * @param {String} panelID
 */
ZxPowerstoreVolumeListView.doDeleteRow = function(ev, panelID) {
  var list = ev.dwtObj.parent.getChildren(),
    i = 0,
    panel;

  for (i = 0; i < list.length; i = i + 1) {
    if (list[i] instanceof ZxPowerstoreVolumeListView &&
        ZxUtils.indexOf(list[i]._className, panelID) !== -1) {
      panel = list[i];
      break;
    }
  }

  if (!panel) {
    return;
  }

  panel._doDeleteRow();
};

/**
 * Add or modify a row of the specified panel.
 * @private
 */
ZxPowerstoreVolumeListView.prototype._doModifyRow = function() {
  var position = this.firstSelIndex;
  var modItem = ZaApp.getInstance().getCurrentController().getItem();

  if (this.firstSelIndex === -1) {
    if (this.getList().size() === 1) {
      position = 0;
    } else {
      return;
    }
  }

  modItem.getLegacyConnector().Powerstore.getS3BucketConfigurations(
    modItem.getLegacyConnector().createCallback(this, this._popupModifyVolume, this.getList().getArray()[position])
  );
};

ZxPowerstoreVolumeListView.prototype._popupModifyVolume = function(args, s3BucketConfigurations) {
  var controller = this.parent.getController().getCurrentController();
  var modItem = ZaApp.getInstance().getCurrentController().getItem();

  var dialog = new ZxModifyVolumeDialog(
    controller.getShell(),
    ZxUtils.getTitleWithIcon(
      'ZxPowerstore',
      zxMsg('LBL_HSM_Modify_volume_setname', [args[0].getName()])
    ),
    modItem.getLegacyConnector(),
    args[0],
    s3BucketConfigurations,
    this.parent.getInstance().id // server id
  );
  dialog.popup();
};

/**
 * Add o modify the row selected when a button "modify" or "add" is selected.
 * @param {DwtEvent} ev
 * @param {string} panelID
 * @this {Dwt_Button_XFormItem}
 */
ZxPowerstoreVolumeListView.doModifyRow = function(ev, panelID) {
  var list = ev.dwtObj.parent.getChildren(),
    i = 0,
    panel;

  for (i = 0; i < list.length; i = i + 1) {
    if (list[i] instanceof ZxPowerstoreVolumeListView &&
        ZxUtils.indexOf(list[i]._className, panelID) !== -1) {
      panel = list[i];
      break;
    }
  }

  if (!panel) {
    return;
  }

  panel._doModifyRow();
};

/**
 * Add o modify the row selected when a button "modify" or "add" is selected.
 * @param {DwtEvent} ev
 * @param {string} panelID
 * @this {Dwt_Button_XFormItem}
 */
ZxPowerstoreVolumeListView.doAddRow = function(ev, panelID) {
  var list = ev.dwtObj.parent.getChildren(),
    i = 0,
    panel;

  for (i = 0; i < list.length; i = i + 1) {
    if (list[i] instanceof ZxPowerstoreVolumeListView &&
      ZxUtils.indexOf(list[i]._className, panelID) !== -1) {
      panel = list[i];
      break;
    }
  }

  if (!panel) {
    return;
  }

  panel._doAddRow(panelID === ZxPowerstoreVolumeListView.PANEL_PRIMARY);
};

ZxPowerstoreVolumeListView.prototype._doAddRow = function(isPrimary) {
  var modItem = ZaApp.getInstance().getCurrentController().getItem();
  modItem.getLegacyConnector().Powerstore.startAddPowerstoreVolumeWizard(
    ZaApp.getInstance().getCurrentController().getShell(),
    isPrimary,
    this.parent.getController().getCurrentController().getItem().id,
    this.parent.getController().getCurrentController().getItem().name
  );
};

ZxPowerstoreVolumeListView.prototype._doStartDeduplicate = function() {
  var position = this.firstSelIndex,
    elementSelected = {},
    controller = this.parent.getController().getCurrentController();

  if (this.firstSelIndex === -1) {
    if (this.getList().size() === 1) {
      position = 0;
    } else {
      return;
    }
  }

  elementSelected = this.getList().getArray()[position];

  if (elementSelected.isLocal()) {
    var dialog = new ZxStartDeduplicateVolumeDialog(
      controller.getShell(),
      controller.getItem(),
      elementSelected.getId(),
      elementSelected.getName()
    );
    dialog.popup();
  } else {
    controller.popupMsgDialog(
      zxMsg("MSG_HSM_S3Stores_does_not_support_deduplication")
    );
  }
};

/**
 * Start the deduplicate process on the selected volume.
 * @param {DwtEvent} ev
 * @param {string} panelID
 * @this {Dwt_Button_XFormItem}
 */
ZxPowerstoreVolumeListView.doStartDeduplicate = function(ev, panelID) {
  var list = ev.dwtObj.parent.getChildren(),
    i = 0,
    panel;

  for (i = 0; i < list.length; i = i + 1) {
    if (list[i] instanceof ZxPowerstoreVolumeListView &&
      ZxUtils.indexOf(list[i]._className, panelID) !== -1) {
      panel = list[i];
      break;
    }
  }

  if (!panel) {
    return;
  }

  panel._doStartDeduplicate();
};

ZxPowerstoreVolumeListView.doShowStatistics = function (ev, panelID) {
  var list = ev.dwtObj.parent.getChildren(),
    i = 0,
    panel;

  for (i = 0; i < list.length; i = i + 1) {
    if (list[i] instanceof ZxPowerstoreVolumeListView &&
      ZxUtils.indexOf(list[i]._className, panelID) !== -1) {
      panel = list[i];
      break;
    }
  }

  if (!panel) {
    return;
  }

  panel._doShowStatistics(panelID);
};

ZxPowerstoreVolumeListView.prototype._doShowStatistics = function (panelID, showSize, showNumBlobs) {
  var position = this.firstSelIndex,
    elementSelected = {},
    controller = this.parent.getController().getCurrentController();

  if (this.firstSelIndex === -1) {
    if (this.getList().size() === 1) {
      position = 0;
    } else {
      return;
    }
  }

  elementSelected = this.getList().getArray()[position];

  if (elementSelected.isLocal()) {
    ZxPowerstoreStatsDialog.showStats(this.parent.getInstance().name, elementSelected, false, false);
  } else {
    controller.popupMsgDialog(
      zxMsg("MSG_HSM_S3Stores_does_not_support_statistics")
    );
  }
};

/**
 * Return the header list for ZxPowerstoreVolumeListView
 * @return {ZaListHeaderItem[]}
 */
ZxPowerstoreVolumeListView.getHeaderList = function() {
  var headerList = [];
  // function(idPrefix, text, iconInfo, width, sortable, sortField, resizeable, visible)
  headerList.push(new DwtListHeaderItem({field:ZxPowerstoreVolumeListView.FIELD_ICON, text:'Type', width:'35px', resizeable:false, visible:true}));
  headerList.push(new DwtListHeaderItem({field:ZxPowerstoreVolumeListView.FIELD_CURRENT, text:'Current', width:'50px', resizeable:false, visible:true}));
  headerList.push(new DwtListHeaderItem({field:ZxPowerstoreVolumeListView.FIELD_COMPRESSED, text:'Compressed', width:'75px', resizeable:false, visible:true}));
  headerList.push(new DwtListHeaderItem({field:ZxPowerstoreVolumeListView.FIELD_NAME, text:'Name', resizeable:false, visible:true}));
  return headerList;
};
