/*
 * ***** BEGIN LICENSE BLOCK *****
 * Copyright (C) 2011-2016 ZeXtras
 *
 * The contents of this file are subject to the ZeXtras EULA;
 * you may not use this file except in compliance with the EULA.
 * You may obtain a copy of the EULA at
 * http://www.zextras.com/zextras-eula.html
 * ***** END LICENSE BLOCK *****
 */

function ZxListView() {}
ZxListView.KEY_HIDE_ROW = '_hideFromList';
ZxListView.KEY_FILTERED_ROW = '_filterPass';

function ZxListView_XFormItem() {
  this._filter = '';
  this._lastFilter = '';
  this._lastFilteredValue = null;
  this._page = 0;
  this.pageSize = ZxListView_XFormItem.MAX_ROWS_PER_PAGE;
}
XFormItemFactory.createItemType('_ZX_LIST_VIEW_', 'zx_list', ZxListView_XFormItem, Group_XFormItem);

ZxListView_XFormItem.prototype.numCols = 1;
ZxListView_XFormItem.prototype.colSizes = ['*'];
ZxListView_XFormItem.prototype.width = '100%';
ZxListView_XFormItem.prototype.bmolsnr = true;
if (typeof appNewUI === 'undefined' || appNewUI) {
  ZxListView_XFormItem.prototype.tableCssClass = 'grid_xform_table';
}

ZxListView_XFormItem.MAX_ROWS_PER_PAGE = 40;
ZxListView_XFormItem.ID_BTN_PREV = '_BTNPrev';
ZxListView_XFormItem.ID_BTN_NEXT = '_BTNNext';
ZxListView_XFormItem.ID_OUT_COUNT = '_OUTCount';
ZxListView_XFormItem.ID_OUT_TOTAL = '_OUTTotal';
ZxListView_XFormItem.ID_GROUP = '_GROUP';

ZxListView_XFormItem.prototype.initFormItem = function() {
  Group_XFormItem.prototype.initFormItem.call(this);

  var ref = this.getInheritedProperty('ref'),
    headerList = this.getInheritedProperty('headerList'),
    filterEnabled = this.getInheritedProperty('filterEnabled') || false,
    navigationEnabled = this.getInheritedProperty('navigationEnabled') || false,
    optionalButtons = this.getInheritedProperty('optionalButtons') || [],
    multiselect = this.getInheritedProperty('multiselect') || false,
//    hideIfEmpty = this.getInheritedProperty('hideIfEmpty'),
    onSelectionFcn = this.getInheritedProperty('onSelection'),
    i = 0,
    hideHeader = (headerList ? headerList.length <= 0 : true),
    optBtnsGrp = {
      type: _GROUP_,
      cssStyle: 'margin: 3px;',
      numCols: 1,
      colSizes: [],
      items: []
    };

  if (typeof onSelectionFcn !== 'function') {
    onSelectionFcn = function(ev) {
      /* Hello! I'm a stub function, replace me with your favourite function,
       * use the onSelection parameter.
       */
      return;
    };
  }

  this.pageSize = this.getInheritedProperty('pageSize') || ZxListView_XFormItem.MAX_ROWS_PER_PAGE;

  this.items = [];

  if (filterEnabled) {
    this.items.push(
      {
        type: _GROUP_,
        numCols: 2,
        width: '100%',
        colSizes: ['80px', '*'],
        items: [
          {
            type: _INPUT_,
            cssStyle: 'width: 90%',
            label: ZaMsg.search_option_filter,
            onChange: AjxCallback.simpleClosure(this._setFilter, this),
            getDisplayValue: AjxCallback.simpleClosure(this._getFilterValue, this),
            visibilityChecks: [function() { return true; }],
            enableDisableChecks: [function() { return true; }]
          }
        ]
      }
    );
  }

  if (navigationEnabled) {
    this.items.push(
      {
        type: _GROUP_,
        numCols: 6,
        width: '100%',
        colSizes: ['*', 100, 80, 10, 80, 100],
        items: [
          {type: _CELL_SPACER_},
          {
            type: _DWT_BUTTON_,
            label: ZaMsg.Previous,
            width: 100,
            icon: 'LeftArrow',
            disIcon: 'LeftArrowDis',
            onActivate: AjxCallback.simpleClosure(this._goPrev, this)
          },
          {
            type: _OUTPUT_,
            align: _RIGHT_,
            width: 80
          },
          {
            type: _OUTPUT_,
            align: _CENTER_,
            value: '  /'
          },
          {
            type: _OUTPUT_,
            align: _LEFT_,
            width: 80
          },
          {
            type: _DWT_BUTTON_,
            label: ZaMsg.Next,
            width: 100,
            icon: 'RightArrow',
            disIcon: 'RightArrowDis',
            onActivate: AjxCallback.simpleClosure(this._goNext, this)
          }
        ]
      }
    );
    this.items.push(
      {type: _SPACER_, height: 3}
    );
  }

  this.items.push(
    {
      type: _GROUP_,
      numCols: 1,
      colSizes: ['*'],
      items: [
        {
          parentWidget: this,
          type: __ZX_LIST_VIEW__,
          width: '100%',
          hideHeader: hideHeader,
          multiSelect: multiselect,
          headerList: headerList,
          onSelection: AjxCallback.simpleClosure(onSelectionFcn, this)
        }
      ]
    }
  );

  if (optionalButtons.length > 0) {
    optBtnsGrp.numCols = optionalButtons.length * 2;
    for (i = 0; i < optionalButtons.length; i += 1) {
      var tmpBtn = optionalButtons[i],
        tmpCallBack;
      // Wrap the callback within the right context, also pass the selected items.
      tmpCallBack = AjxCallback.simpleClosure(
        function(cbk, ev) {
          cbk(ev, this.getDwtList().getSelection());
        },
        this,
        AjxCallback.simpleClosure(tmpBtn.callback, this)
      );

      optBtnsGrp.items.push(
        {
          type: _DWT_BUTTON_,
          icon: tmpBtn.icon,
          label: tmpBtn.label,
          onActivate: tmpCallBack
        },
        { type: _CELLSPACER_ }
      );

      if (i === optionalButtons.length - 1) {
        optBtnsGrp.colSizes.push(120, 3);
      } else {
        optBtnsGrp.colSizes.push(120, '0');
      }
    }
    this.items.push(optBtnsGrp);
  }
};

ZxListView_XFormItem.prototype.updateElement = function (val) {
  XFormItem.prototype.updateElement.call(this);
  if (this._lastFilter !== '') {
    this._setFilter(this._lastFilter, true);
  } else {
    this.goToPage(0);
  }
};

ZxListView_XFormItem.prototype.getDwtList = function() {
  var filterEnabled = this.getInheritedProperty('filterEnabled') || false,
    navigationEnabled = this.getInheritedProperty('navigationEnabled') || false,
    listGroupIndex = 0,
    listView;

  if (filterEnabled === true) {
    listGroupIndex += 1;
  }
  if (navigationEnabled === true) {
    listGroupIndex += 2;
  }

  listView = this.getItems()[listGroupIndex].items[0].getWidget();
  if (!(listView instanceof _ZxListInternalClass)) {
    // Error! The expected object is a _ZxListInternalClass
    return;
  }

  return listView;
};

/**
 * Get the selected items of the list.
 * @return {{}[]|Selection}
 */
ZxListView_XFormItem.prototype.getSelection = function() {
  return this.getDwtList().getSelection();
};

ZxListView_XFormItem.prototype.getNavigationButtons = function() {
  var filterEnabled = this.getInheritedProperty('filterEnabled') || false,
    navigationEnabled = this.getInheritedProperty('navigationEnabled') || false,
    listGroupIndex = 0,
    prev = null,
    next = null;

  if (filterEnabled === true) {
    listGroupIndex += 1;
  }

  prev = this.getItems()[listGroupIndex].getItems()[1];
  next = this.getItems()[listGroupIndex].getItems()[5];

  if (!(prev instanceof Dwt_Button_XFormItem && next instanceof Dwt_Button_XFormItem)) {
    // Error! The expected prev and next must be Dwt_Button_XFormItem
    return;
  }

  return {prev: prev, next: next};
};

ZxListView_XFormItem.prototype.getNavigationLabels = function() {
  var filterEnabled = this.getInheritedProperty('filterEnabled') || false,
    navigationEnabled = this.getInheritedProperty('navigationEnabled') || false,
    listGroupIndex = 0,
    displayed = null,
    total = null;

  if (filterEnabled === true) {
    listGroupIndex += 1;
  }

  displayed = this.getItems()[listGroupIndex].getItems()[2];
  total = this.getItems()[listGroupIndex].getItems()[4];

  if (!(displayed instanceof Output_XFormItem && total instanceof Output_XFormItem)) {
    // Error! The expected displayed and total must be Output_XFormItem
    return;
  }

  return {displayed: displayed, total: total};
};

ZxListView_XFormItem.prototype._setFilter = function(value, forceUpdate) {
  this._filter = value || '';
  forceUpdate = forceUpdate || false;

  //If the filter is the same of the last one, exits.
  if (this._filter === this._lastFilter && !forceUpdate) {
    return;
  }

  var listView = this.getDwtList(),
    i = 0,
    list = this.getInstanceValue(),
    filtered = [],
    remove = false,
    item,
    filterRegex = new RegExp(this._filter, 'ig'),
    prop;

  list = list || [];

  if (this._filter === '') {
    filtered = list.slice(0);
  } else {
    for (i = 0; i < list.length; i += 1) {
      item = list[i];
      try { // If the pattern is not valid, ignore the error and go to next element.
        if (typeof item === 'string') {
          if (filterRegex.test(item.toLowerCase())) {
            filtered.push(item);
          }
        }

        if (ZxUtils.isArray(item)) {
          if (filterRegex.test(item.toString().toLowerCase())) {
            filtered.push(item);
          }
        } else if (ZxUtils.isObject(item)) {
          remove = true;
          prop = null;

          for (prop in item) {
            if (!item.hasOwnProperty(prop)) {
              continue;
            }

            if (typeof item[prop] === 'string') {
              if (filterRegex.test(item[prop].toLowerCase())) {
                remove = false;
              }

            } else if (typeof item[prop] === 'undefined') {

            } else if (typeof item[prop] === 'number') {
              if (filterRegex.test('' + item[prop])) {
                remove = false;
              }

            } else {
              if (filterRegex.test(item[prop].toString().toLowerCase())) {
                remove = false;
              }
            }
          }

          if (!remove) {
            filtered.push(item);
          }
        }
      } catch (ignored) {
      }
    }
  }
  listView.set(AjxVector.fromArray(filtered));
  this._lastFilter = this._filter;
  this._lastFilteredValue = filtered.slice(0);
  this.goToPage(0);
//  this._updateNavigation();
};

ZxListView_XFormItem.prototype.isFiltered = function() {
  return !!this._filter && this._filter !== '';
};

ZxListView_XFormItem.prototype.getFilteredList = function() {
  return this._lastFilteredValue;
};

ZxListView_XFormItem.prototype._getFilterValue = function() {
  return this._filter;
};

ZxListView_XFormItem.prototype._goPrev = function() {
  this.goToPage(this._page - 1);
};
ZxListView_XFormItem.prototype._goNext = function() {
  this.goToPage(this._page + 1);
};

ZxListView_XFormItem.prototype.goToPage = function(page) {
  this._page = page;

  var navigationEnabled = this.getInheritedProperty('navigationEnabled') || false,
    filterEnabled = this.getInheritedProperty('filterEnabled') || false,
    listView = this.getDwtList(),
    list = (
      (filterEnabled && this._lastFilteredValue !== null) ? this._lastFilteredValue : this.getInstanceValue()
      ) || [],
    limited = [],
    start = (navigationEnabled) ? this._page * this.pageSize : 0,
    end = (navigationEnabled) ? (this._page + 1) * this.pageSize : list.length;

  limited = list.slice(start, (end > list.length) ? list.length : end);

  if (!!listView) {
    listView.set(AjxVector.fromArray(limited));
  }

  this._updateNavigation();
};

ZxListView_XFormItem.prototype._updateNavigation = function() {
  var navigationEnabled = this.getInheritedProperty('navigationEnabled') || false,
    filterEnabled = this.getInheritedProperty('filterEnabled') || false,
    value = (
      (filterEnabled && this._lastFilteredValue !== null) ? this._lastFilteredValue : this.getInstanceValue()
      ) || [],
    total = value.length,
    navButtons,
    navLabels,
    index = (navigationEnabled) ? this._page * this.pageSize : 0,
    howMany = (navigationEnabled) ? this.pageSize : total,
    start = (total > 0) ? index + 1 : 0,
    end = (total < index + howMany) ? total : index + howMany,
    hasPrev = this._page > 0,
    hasNext = total > end;

  if (navigationEnabled !== true) {
    return;
  }

  navLabels = this.getNavigationLabels();
  navLabels.displayed.updateElement(start + ' - ' + end);
  navLabels.total.updateElement(total);

  navButtons = this.getNavigationButtons();
  navButtons.prev.setElementEnabled(hasPrev);
  navButtons.next.setElementEnabled(hasNext);
};

/**
 * Defines the optional button associated to a ZxListView.
 * @param {String} icon
 * @param {String} label
 * @param {AjxCallback} callback
 * @constructor
 */
function ZxListViewButton(icon, label, callback) {
  this.icon = icon;
  this.label = label;
  this.callback = callback;
}

/**
 * Internal class which display the real list view.
 * @constructor
 */
function _ZxListInternalView() {
  this.widgetClass = _ZxListInternalClass;
}
XFormItemFactory.createItemType('__ZX_LIST_VIEW__', 'zx_list_component', _ZxListInternalView, Dwt_List_XFormItem);

_ZxListInternalView.prototype.constructWidget = function() {
  var widget = Dwt_List_XFormItem.prototype.constructWidget.call(this);
  widget._parentWidget = this.getInheritedProperty('parentWidget');
  return widget;
};

_ZxListInternalView.prototype.updateWidget = function (newValue) {
  if (typeof (newValue) === 'undefined' || newValue === null) {
    var parent = this.getInheritedProperty('parentWidget');
    newValue = parent.getInstanceValue();
  }
  if (typeof (newValue) != 'undefined') {
    this.setItems(newValue);
  }
};

_ZxListInternalView.prototype.setItems = function (itemArray){
  var list = this.widget.getList(),
    existingArr = [],
    tmpArr = [],
    defaultColumnSort = this.getInheritedProperty('defaultColumnSortable'),
    preserveSelection = this.getInheritedProperty('preserveSelection'),
    selection = null,
    i = 0;

  if (list) {
    existingArr = list.getArray();
  }

  if (itemArray && itemArray.length > 0) {
    if (this.getForceUpdate() || Dwt_List_XFormItem.isItemsChanged(itemArray, existingArr)) {
      if (preserveSelection) {
        selection = this.widget.getSelection();
      }
      for (i = 0; i < itemArray.length; i += 1) {
        tmpArr.push(itemArray[i]);
      }

      this.widget.set(AjxVector.fromArray(tmpArr), defaultColumnSort);
      if (typeof itemArray._version !== 'undefined' && itemArray._version !== null) {
        this.widget.getList().getArray()._version = itemArray._version;
      }

      if (preserveSelection && selection) {
        this.widget.setSelectedItems(selection);
      }
    }
  } else {
    this.widget.set(AjxVector.fromArray([]), defaultColumnSort);
  }
};

function _ZxListInternalClass(form, cssClass, posStyle, headerList) {
  DwtListView.call(this, form, cssClass, posStyle, headerList);
  this._parentWidget = null;
}
_ZxListInternalClass.prototype = new DwtListView;
_ZxListInternalClass.prototype.constructor = _ZxListInternalClass;

_ZxListInternalClass.prototype._createItemHtml = function(item) {
  if (item.hasOwnProperty(ZxListView.KEY_HIDE_ROW) && item[ZxListView.KEY_HIDE_ROW] === true) {
    return null;
  }

  var div = document.createElement("div"),
    html = [],
    id;

  div[DwtListView._STYLE_CLASS] = "Row";
  div[DwtListView._SELECTED_STYLE_CLASS] = div[DwtListView._STYLE_CLASS] + "-" + DwtCssStyle.SELECTED;
  div.className = div[DwtListView._STYLE_CLASS];

  id = this.associateItemWithElement(item, div, DwtListView.TYPE_LIST_ITEM);

  html.push('<table width="100%" cellspacing="2" cellpadding="0">');
  html.push('<tr>');

  if (this._headerList) {
    var i = 0;
    for (i = 0; i < this._headerList.length; i++) {
      var header = this._headerList[i],
        headerWidth = header._width,
        field = header._field,
        objToRender,
        iconTrue,
        hoverTrue,
        iconFalse,
        hoverFalse,
        rendered = '',
        hint;

      if (header._visible === false) {
        continue;
      }

      if (header.icons && header.icons.trueIcon) {
        iconTrue = header.icons.trueIcon;
        hoverTrue = header.icons.trueHint || field;
        iconFalse = header.icons.falseIcon || iconTrue;
        hoverFalse = header.icons.falseHint || field;
      }

      objToRender = item[field];
      if (typeof objToRender === 'undefined') {
        html.push("<td width='" + headerWidth + "'></td>");
        continue;
      }
      rendered = objToRender.toString ? objToRender.toString() : objToRender;

      if (header.hintField && item[header.hintField]) {
        hint = item[header.hintField];
      } else {
        if (typeof objToRender === 'string') {
          hint = objToRender;
        } else {
          hint = rendered;
        }
      }

      html.push("<td width='" + headerWidth + "'>");

      if (typeof objToRender === 'boolean' && iconTrue) {
        if (objToRender === true) {
          html.push(
            "<div class='" + AjxImg.getClassForImage(iconTrue, false) + "' title='" + hoverTrue + "'></div>"
          );
        } else {
          html.push(
            "<div class='" + AjxImg.getClassForImage(iconTrue, iconTrue === iconFalse) + "' title='" + hoverFalse + "'></div>"
          );
        }
      } else if (typeof objToRender === 'string') {
        html.push("<div title='" + hint + "'>" + objToRender + "</div>");
      } else {
        html.push("<div title='" + hint + "'>" + rendered + "</div>");
      }
    }

    html.push("</td>");

  } else {
    var itemString = item.toString ? item.toString() : item;
    html.push("<td width='auto'>");
    html.push(itemString);
    html.push("</td>");
  }

  html.push("</tr></table>");
  div.innerHTML = html.join("");
  return div;
};

//_ZxListInternalClass.prototype._createHeader = function (htmlArr, idx, headerCol, i, numCols, id, defaultColumnSort) {
//
//};

_ZxListInternalClass.prototype._sortColumn = function(columnItem, bSortAsc) {
  this._resetListView();
  this._list.sort(
    AjxCallback.simpleClosure(
      function(col, asc, a, b) {
        if (!a.hasOwnProperty(col) || !b.hasOwnProperty(col)) {
          return 0;
        }

        var tmpA = a[col],
          tmpB = b[col];

        if (typeof tmpA === 'string') {
          tmpA = tmpA.toLowerCase();
        }
        if (typeof tmpB === 'string') {
          tmpB = tmpB.toLowerCase();
        }

        if (tmpA < tmpB) {
          return (asc) ? -1 : 1;
        }
        if (tmpA > tmpB) {
          return (asc) ? 1 : -1;
        }
        return 0;
      },
      this,
      columnItem.getSortField(),
      bSortAsc
    )
  );
  this._renderList(this._list, false, null);
};
