/*
 * ***** BEGIN LICENSE BLOCK *****
 * Copyright (C) 2011-2016 ZeXtras
 *
 * The contents of this file are subject to the ZeXtras EULA;
 * you may not use this file except in compliance with the EULA.
 * You may obtain a copy of the EULA at
 * http://www.zextras.com/zextras-eula.html
 * ***** END LICENSE BLOCK *****
 */

function ZxTimeWidget() {
  this.useParentTable = true;
  this.numCols = 2;
}

XFormItemFactory.createItemType('_ZX_TIME_', 'zextras_time_widget', ZxTimeWidget, Composite_XFormItem);

ZxTimeWidget.TIME_HOUR_CHOICES = [
  '00', '01', '02', '03', '04', '05',
  '06', '07', '08', '09', '10', '11',
  '12', '13', '14', '15', '16', '17',
  '18', '19', '20', '21', '22', '23'
];

ZxTimeWidget.prototype.items = [
  {
    type: _DWT_SELECT_,
    ref: '.',
    width: 70,
    cssStyle: 'overflow-y: hidden;',
    valign: _MIDDLE_,
    label: zxMsg('LBL_Hour') + ': ',
    labelLocation: _LEFT_,
    choices: ZxTimeWidget.TIME_HOUR_CHOICES,
    getDisplayValue: function(newValue) {
      if (!(newValue instanceof Date)) {
        newValue = new Date();
      }
      var hours;
      if (newValue.getHours() < 10) {
        hours = '0' + newValue.getHours();
      } else {
        hours = newValue.getHours();
      }
      return hours;
    },
    elementChanged: function(hoursStr, currentDate, event) {
      if (!(currentDate instanceof Date)) {
        currentDate = new Date();
      }  //??? should get values of other fields???
      if (!this.__dummyDate) {
        this.__dummyDate = new Date();
      }
      this.__dummyDate.setTime(currentDate.getTime());

      var hours = Number(hoursStr, 10),
        parentItem,
        elementChangedMethod;

      if (!isNaN(parseFloat(hours)) && isFinite(hours)) {
        this.__dummyDate.setHours(hours);
      }

      parentItem = this.getParentItem();
      elementChangedMethod = parentItem.getElementChangedMethod();

      if (elementChangedMethod) {
        elementChangedMethod.call(this.getParentItem(), this.__dummyDate, currentDate, event);
      } else {
        this.getForm().itemChanged(this.getParentItem(), this.__dummyDate, event);
      }
    }
  },
  {
    type: _INPUT_,
    ref: '.',
    width: 48,
    valign: _MIDDLE_,
    label: zxMsg('LBL_Minute') + ': ',
    labelLocation: _LEFT_,
    getDisplayValue: function(newValue) {
      if (!(newValue instanceof Date)) {
        newValue = new Date();
      }
      var minutes = newValue.getMinutes();
      return minutes;
    },
    elementChanged: function(minutesStr, currentDate, event) {
      if (!(currentDate instanceof Date)) {
        currentDate = new Date();
      }  //??? should get values of other fields???
      if (!this.__dummyDate) {
        this.__dummyDate = new Date();
      }
      this.__dummyDate.setTime(currentDate.getTime());

      if (minutesStr === '') {
        return;
      }

      var minutes = Number(minutesStr),
        parentItem,
        elementChangedMethod;
      if (minutes >= 0 && minutes < 60) {
        this.__dummyDate.setMinutes(minutes);
      }

      this.updateElement(this.__dummyDate.getMinutes());
      parentItem = this.getParentItem();
      elementChangedMethod = parentItem.getElementChangedMethod();

      if (elementChangedMethod) {
        elementChangedMethod.call(this.getParentItem(), this.__dummyDate, currentDate, event);
      } else {
        this.getForm().itemChanged(this.getParentItem(), this.__dummyDate, event);
      }
    }
  },
  {
    type: _INPUT_,
    ref: '.',
    width: 48,
    label: zxMsg('LBL_Second') + ': ',
    labelLocation: _LEFT_,
    valign: _MIDDLE_,
    getDisplayValue: function(newValue) {
      if (!(newValue instanceof Date)) {
        newValue = new Date();
      }
      var seconds = newValue.getSeconds();
      return seconds;
    },
    elementChanged: function(secondStr, currentDate, event) {
      var secs = this,
        seconds,
        parentItem,
        elementChangedMethod;

      if (!currentDate) {
        currentDate = new Date();
      }  //??? should get values of other fields???
      if (!this.__dummyDate) {
        this.__dummyDate = new Date();
      }
      this.__dummyDate.setTime(currentDate.getTime());

      if (secondStr === '') {
        return;
      }

      seconds = Number(secondStr);
      if (seconds >= 0 && seconds < 60) {
        this.__dummyDate.setSeconds(seconds);
      }

      this.updateElement(this.__dummyDate.getSeconds());

      parentItem = this.getParentItem();
      elementChangedMethod = parentItem.getElementChangedMethod();
      if (elementChangedMethod) {
        elementChangedMethod.call(this.getParentItem(), this.__dummyDate, currentDate, event);
      } else {
        this.getForm().itemChanged(this.getParentItem(), this.__dummyDate, event);
      }
    }
  }
];
